{
        Program Name    : Mem_Watchpoints_Dialog
        Package Name    : CEF
        Purpose         : Memory watchpoints edit dialog
        Institution     : Conroy & Conroy Co.
        Date written    : 10-Apr-2005
        Written by      : Alan Conroy
        Version         : 1.0

        Copyright (C) 2005 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

      *******************************************************
      *                                                     *
      *      M O D I F I C A T I O N   H I S T O R Y        *
      *                                                     *
      *    DATE        BY            REASON                 *
      *                                                     *
      *******************************************************

      *******************************************************
      *                                                     *
      *          P R O G R A M   P U R P O S E              *
      *                                                     *
      *******************************************************

        This dialog is used to edit breakpoints and watchpoints for CEF.
}

unit Mem_Watchpoints_Dialog ;

interface

uses // Borland...
     Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
     Dialogs, StdCtrls, Buttons, ExtCtrls, Grids,
     Menus,

     // C&C...
     CommonUt, // TInteger_List
     Standard, // TInteger_List64

     // CEF...
     CEF ; // TComponent

type
  TMemory_Watchpoints_Dialog = class(TForm)
    Button_Panel: TPanel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    Grid: TStringGrid;
    PopupMenu1: TPopupMenu;
    Delete1: TMenuItem;
    Properties1: TMenuItem;
    Add1: TMenuItem;
    DeleteAll1: TMenuItem;
    Help_Button: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure GridKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure DeleteAll1Click(Sender: TObject);
    procedure Add1Click(Sender: TObject);
    procedure Properties1Click(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
    procedure Help_ButtonClick(Sender: TObject);

  private // Instance data...
      _Watchpoints : TInteger64_List ;
      _Watchpoint_Size, _Watchpoint_Access : TInteger_List ;
      Term : string ; // "breakpoint" or "watchpoint", as appropriate
      _Port : boolean ;

  private // Internal utility routines...
      procedure Delete_Watchpoint ;
      procedure Insert_Watchpoint ;
      procedure Modify_Watchpoint ;

  public // API...
      CPU : TComponent ;

      procedure Set_Data( Watchpoints : TInteger64_List ;
          Watchpoint_Size, Watchpoint_Access : TInteger_List ; Port : boolean ) ;
  end ;

procedure Add_Breakpoint( Component : TComponent ;
    Watchpoints : TInteger64_List ; Accesses : TInteger_List ; Base : int64 ;
    Size : longint ; Port : boolean ) ;

procedure Add_Watchpoint( Component : TComponent ;
    Watchpoints : TInteger64_List ;
    Watchpoint_Size, Watchpoint_Access : TInteger_List ; Base : int64 ;
    Size : longint ; Memory : boolean ) ;

var Memory_Watchpoints_Dialog : TMemory_Watchpoints_Dialog ;

implementation

uses // C&C...
     ASCIIDef, // CR
     CVT, // CVTB

     // CEF...
     CEFMain,
     Create_Watchpoint ;

{$R *.dfm}

procedure Add_Watchpoint( Component : TComponent ;
    Watchpoints : TInteger64_List ;
    Watchpoint_Size, Watchpoint_Access : TInteger_List ; Base : int64 ;
    Size : longint ; Memory : boolean ) ;

var Access : integer ;
    Address : int64 ;
    Count, Loop : integer ;
    S : string ;

begin
    New_Watchpoint.Base := Base ;
    New_Watchpoint.Size.Value := Size ;
    New_Watchpoint.GroupBox1.Visible := True ;
    if (
         ( Component.Component_Type = Component_Type_CPU )
         and
         ( Watchpoint_Size = nil )
         and
         ( not Memory )
       ) then
    begin
        if( New_Watchpoint.Size.Visible ) then // Was a memory watchpoint
        begin
            New_Watchpoint.Address.Text := '' ;
        end ;
        New_Watchpoint.Size.Visible := False ;
        New_Watchpoint.Label2.Visible := False ;
        New_Watchpoint.GroupBox1.Visible := True ;
        New_Watchpoint.Label1.Caption := 'Register:' ;
        New_Watchpoint.Caption := 'Create CPU Register Breakpoint' ;
        New_Watchpoint.CPU := Component ;
    end else
    begin
        New_Watchpoint.Address.Enabled := True ;
        if( not New_Watchpoint.Size.Visible ) then // Was a CPU breakpoint
        begin
            New_Watchpoint.Address.Text := '' ;
        end ;
        New_Watchpoint.GroupBox1.Visible := True ;
        New_Watchpoint.Size.Visible := True ;
        New_Watchpoint.Label2.Visible := True ;
        New_Watchpoint.Label1.Caption := 'Address:' ;
        New_Watchpoint.Caption := 'Create Memory Watchpoint' ;
        New_Watchpoint.CPU := nil ;
        New_Watchpoint.Low := 0 ;
    end ;
    New_Watchpoint.Read.Caption := 'Read' ;
    New_Watchpoint.Write.Caption := 'Write' ;
    if( New_Watchpoint.ShowModal = mrOK ) then
    begin
        Access := 0 ;
        if( New_Watchpoint.Read.Checked ) then
        begin
            Access := Access or Access_Read ;
        end ;
        if( New_Watchpoint.Write.Checked ) then
        begin
            Access := Access or Access_Write ;
        end ;
        if (
             ( Component.Component_Type = Component_Type_CPU )
             and
             ( Watchpoint_Size = nil )
             and
             ( not Memory )
           ) then
        begin
            Address := 0 ;
            while( True ) do
            begin
                Size := Component.CPU.Register_Size( Address ) ;
                if( Size = 0 ) then
                begin
                    break ;
                end ;
                S := Component.CPU.Register_Name( Address ) ;
                if( uppercase( S ) = uppercase( New_Watchpoint.Address.Text ) ) then
                begin
                    break ;
                end ;
                inc( Address ) ;
            end ;
        end else
        begin
            Address := strtoint( Cvtb( Base, 10, New_Watchpoint.Address.Text ) ) ;
        end ;
        if(
            ( Component.Component_Type = Component_Type_CPU )
            and
            ( Watchpoint_Size = nil )
          ) then
        begin
            New_Watchpoint.Size.Value :=
                Component.CPU.Register_Size( Address ) ;
        end ;
	for Loop := 0 to Watchpoints.Count - 1 do
        begin
            if( Watchpoints[ Loop ] = Address ) then
            begin
                if(
                    ( Watchpoint_Size = nil )
                    or
                    ( Watchpoint_Size[ Loop ] = New_Watchpoint.Size.Value )
                  ) then
                begin
                    if( Watchpoint_Access[ Loop ] = Access ) then
                    begin
                        exit ; // Duplicate watchpoint
                    end ;
                end ;
            end ;
        end ;
        Watchpoints.Add( Address ) ;
        if( Watchpoint_Size <> nil ) then
        begin
            Watchpoint_Size.Add( New_Watchpoint.Size.Value ) ;
        end ;
        Watchpoint_Access.Add( Access ) ;
        if (
             ( Component.Component_Type = Component_Type_CPU )
             and
             ( Watchpoint_Size = nil )
             and
             ( not Memory )
           ) then
        begin
            Component.CPU.Set_Internal_Watchpoint( Address, Memory, Access ) ;
        end else
        begin
            Count := New_Watchpoint.Size.Value ;
            for Loop := 0 to Count do
            begin
                Component.Set_Watchpoint( Address, Memory, Access ) ;
                inc( Address ) ;
            end ;
        end ;
    end ; // if( New_Watchpoint.ShowModal = mrOK )
end ; // Add_Watchpoint


procedure Add_Breakpoint( Component : TComponent ;
    Watchpoints : TInteger64_List ; Accesses : TInteger_List ; Base : int64 ;
    Size : longint ; Port : boolean ) ;

var Access : integer ;
    Address : int64 ;
    Loop : integer ;

begin
    New_Watchpoint.Base := Base ;
    New_Watchpoint.Size.Value := Size ;
    New_Watchpoint.Address.Text := '' ;
    New_Watchpoint.Size.Visible := False ;
    New_Watchpoint.Label2.Visible := False ;
    if( Port ) then
    begin
        New_Watchpoint.Label1.Caption := 'Port:' ;
        New_Watchpoint.Caption := 'Create Port Breakpoint' ;
        New_Watchpoint.Low := Component.CPU.Get_Low_Port ;
        New_Watchpoint.High := Component.CPU.Get_High_Port ;
        New_Watchpoint.Read.Caption := 'Input' ;
        New_Watchpoint.Write.Caption := 'Output' ;
        New_Watchpoint.GroupBox1.Visible := True ;
    end else
    begin
        New_Watchpoint.Label1.Caption := 'Address:' ;
        New_Watchpoint.Caption := 'Create Execution Breakpoint' ;
        New_Watchpoint.Low := Component.CPU.Get_Low_Memory ;
        New_Watchpoint.High := Component.CPU.Get_High_Memory ;
        New_Watchpoint.GroupBox1.Visible := False ;
    end ;
    if( New_Watchpoint.ShowModal = mrOK ) then
    begin
        Address := strtoint( Cvtb( Base, 10, New_Watchpoint.Address.Text ) ) ;
	for Loop := 0 to Watchpoints.Count - 1 do
        begin
            if( Watchpoints[ Loop ] = Address ) then
            begin
                exit ; // Duplicate watchpoint
            end ;
        end ;
        Watchpoints.Add( Address ) ;
        if( Port ) then
        begin
            Access := 0 ;
            if( New_Watchpoint.Read.Checked ) then
            begin
                Access := Access or Access_Input ;
            end ;
            if( New_Watchpoint.Write.Checked ) then
            begin
                Access := Access or Access_Output ;
            end ;
            Component.Set_Watchpoint( Address, False, Access )
        end else
        begin
            Component.CPU.Set_Breakpoint( Address, 0, True ) ;
        end ;
    end ; // if( New_Watchpoint.ShowModal = mrOK )
end ; // Add_Breakpoint


// TMemory_Watchpoints_Dialog...

// Internal utility routines...

procedure TMemory_Watchpoints_Dialog.Delete_Watchpoint ;

var Loop, Loop1 : integer ;
    Rect : TGridRect ;

begin
    Rect := Grid.Selection ;
    if( Rect.Top > 0 ) then // Not the header
    begin
        for Loop := Rect.Top + 1 to Grid.RowCount - 1 do
        begin
            for Loop1 := 0 to 2 do
            begin
                Grid.Cells[ Loop1, Loop - 1 ] := Grid.Cells[ Loop1, Loop ] ;
            end ;
        end ;
        if( Grid.RowCount = 2 ) then // Never less than 2 rows
        begin
            for Loop1 := 0 to 2 do
            begin
                Grid.Cells[ Loop1, 1 ] := '' ;
            end ;
        end else
        begin
            Grid.RowCount := Grid.RowCount - 1 ;
        end ;
    end ;
end ;


procedure TMemory_Watchpoints_Dialog.Insert_Watchpoint ;

var Count : integer ;
    S : string ;

begin
    Count := _Watchpoints.Count ;
    if( _Watchpoint_Size = nil ) then
    begin
        if( _Port ) then
        begin
            Main_Form.AddPortBreakpoint1Click( nil ) ;
        end else
        if( _Watchpoint_Access = nil ) then
        begin
            Main_Form.AddBreakpoint1Click( nil ) ;
        end else
        begin
            Main_Form.AddRegisterBreakpoint1Click( nil ) ;
        end ;
    end else
    begin
        Main_Form.Createnew1Click( nil ) ;
    end ;
    if( Count <> _Watchpoints.Count ) then // Watchpoint added
    begin
        Count := _Watchpoints.Count ;
        Grid.RowCount := Count + 1 ;
        if( _Watchpoint_Size <> nil ) then
        begin
            Grid.Cells[ 0, Count ] := inttostr( _Watchpoints[ Count - 1 ] ) ;
            Grid.Cells[ 1, Count ] := inttostr( _Watchpoint_Size[ Count - 1 ] ) ;
        end else
        begin
            if( _Port or ( _Watchpoint_Access = nil ) ) then
            begin
                Grid.Cells[ 0, Count ] := inttostr( _Watchpoints[ Count - 1 ] ) ;
            end else
            begin
                Grid.Cells[ 0, Count ] := CPU.CPU.Register_Name( _Watchpoints[ Count - 1 ] ) ;
            end ;
            Grid.Cells[ 1, Count ] := '' ;
        end ;
        S := '' ;
        if( _Watchpoint_Access <> nil ) then
        begin
            if( ( _Watchpoint_Access[ Count - 1 ] and Access_Read ) <> 0 ) then
            begin
                if( _Port ) then
                begin
                    S := 'input' ;
                end else
                begin
                    S := 'read' ;
                end ;
            end ;
            if( ( _Watchpoint_Access[ Count - 1 ] and Access_Write ) <> 0 ) then
            begin
                if( length( S ) > 0 ) then
                begin
                    S := S + ', ' ;
                end ;
                if( _Port ) then
                begin
                    S := S + 'output' ;
                end else
                begin
                    S := S + 'write' ;
                end ;
            end ;
            if( ( _Watchpoint_Access[ Count - 1 ] and Access_Execute ) <> 0 ) then
            begin
                if( length( S ) > 0 ) then
                begin
                    S := S + ', ' ;
                end ;
                S := 'execute' ;
            end ;
        end ; // if( _Watchpoint_Access <> nil )
        Grid.Cells[ 2, Count ] := S ;
    end ;
end ; // TMemory_Watchpoints_Dialog.Insert_Watchpoint


procedure TMemory_Watchpoints_Dialog.Modify_Watchpoint ;

var Loop : integer ;
    Rect : TGridRect ;
    S : string ;

begin
    Rect := Grid.Selection ;
    if( Rect.Top > 0 ) then // Not the header
    begin
        if( _Port ) then // Port breakpoint
        begin
            New_Watchpoint.Caption := 'Port Breakpoint' ;
            New_Watchpoint.Address.Text := inttostr( _Watchpoints[ Rect.Top - 1 ] ) ;
            if( _Watchpoint_Size <> nil ) then
            begin
                New_Watchpoint.Size.Value := _Watchpoint_Size[ Rect.Top - 1 ] ;
            end ;
            New_Watchpoint.Address.Enabled := False ;
            New_Watchpoint.Size.Visible := False ;
            New_Watchpoint.Read.Checked :=
                ( ( _Watchpoint_Access[ Rect.Top - 1 ] and Access_Read ) <> 0 ) ;
            New_Watchpoint.Write.Checked :=
                ( ( _Watchpoint_Access[ Rect.Top - 1 ] and Access_Write ) <> 0 ) ;
            New_Watchpoint.Read.Caption := 'Input' ;
            New_Watchpoint.Write.Caption := 'Output' ;
            if( New_Watchpoint.ShowModal = mrOK ) then
            begin
                Loop := 0 ;
                if( New_Watchpoint.Read.Checked ) then
                begin
                    Loop := Access_Read ;
                end ;
                if( New_Watchpoint.Write.Checked ) then
                begin
                    Loop := Loop or Access_Write ;
                end ;
                _Watchpoint_Access[ Rect.Top - 1 ] := Loop ;
            end ;
            New_Watchpoint.Address.Enabled := True ;
            New_Watchpoint.Size.Enabled := True ;
            New_Watchpoint.Size.Visible := True ;
        end else
        if( _Watchpoint_Access = nil ) then // Execution breakpoint
        begin
            // Nothing to edit on execution breakpoints
        end else
        if( _Watchpoint_Size = nil ) then // Register breakpoint
        begin
            New_Watchpoint.Caption := 'CPU Register Breakpoint' ;
            New_Watchpoint.Address.Text :=
                CPU.CPU.Register_Name( _Watchpoints[ Rect.Top - 1 ] ) ;
            if( _Watchpoint_Size <> nil ) then
            begin
                New_Watchpoint.Size.Value := _Watchpoint_Size[ Rect.Top - 1 ] ;
            end ;
            New_Watchpoint.Address.Enabled := False ;
            New_Watchpoint.Size.Visible := False ;
            New_Watchpoint.Read.Checked :=
                ( ( _Watchpoint_Access[ Rect.Top - 1 ] and Access_Read ) <> 0 ) ;
            New_Watchpoint.Write.Checked :=
                ( ( _Watchpoint_Access[ Rect.Top - 1 ] and Access_Write ) <> 0 ) ;
            New_Watchpoint.Read.Caption := 'Read' ;
            New_Watchpoint.Write.Caption := 'Write' ;
            if( New_Watchpoint.ShowModal = mrOK ) then
            begin
                Loop := 0 ;
                if( New_Watchpoint.Read.Checked ) then
                begin
                    Loop := Access_Read ;
                end ;
                if( New_Watchpoint.Write.Checked ) then
                begin
                    Loop := Loop or Access_Write ;
                end ;
                _Watchpoint_Access[ Rect.Top - 1 ] := Loop ;
            end ;
            New_Watchpoint.Address.Enabled := True ;
            New_Watchpoint.Size.Visible := True ;
        end else // Memory watchpoint
        begin
            New_Watchpoint.Caption := 'Memory Watchpoint' ;
            New_Watchpoint.Address.Text :=
                inttostr( _Watchpoints[ Rect.Top - 1 ] ) ;
            if( _Watchpoint_Size <> nil ) then
            begin
                New_Watchpoint.Size.Value := _Watchpoint_Size[ Rect.Top - 1 ] ;
            end ;
            New_Watchpoint.Address.Enabled := False ;
            New_Watchpoint.Size.Visible := True ;
            New_Watchpoint.Size.Enabled := False ;
            New_Watchpoint.Read.Checked :=
                ( ( _Watchpoint_Access[ Rect.Top - 1 ] and Access_Read ) <> 0 ) ;
            New_Watchpoint.Write.Checked :=
                ( ( _Watchpoint_Access[ Rect.Top - 1 ] and Access_Write ) <> 0 ) ;
            New_Watchpoint.Read.Caption := 'Read' ;
            New_Watchpoint.Write.Caption := 'Write' ;
            if( New_Watchpoint.ShowModal = mrOK ) then
            begin
                Loop := 0 ;
                if( New_Watchpoint.Read.Checked ) then
                begin
                    Loop := Access_Read ;
                end ;
                if( New_Watchpoint.Write.Checked ) then
                begin
                    Loop := Loop or Access_Write ;
                end ;
                _Watchpoint_Access[ Rect.Top - 1 ] := Loop ;
            end ;
            New_Watchpoint.Address.Enabled := True ;
            New_Watchpoint.Size.Enabled := True ;
        end ;
        if( _Watchpoint_Size <> nil ) then
        begin
            Grid.Cells[ 1, Rect.Top ] := inttostr( _Watchpoint_Size[ Rect.Top - 1 ] ) ;
        end else
        begin
            Grid.Cells[ 1, Rect.Top ] := '' ;
        end ;
        S := '' ;
        if( _Watchpoint_Access <> nil ) then
        begin
            if( ( _Watchpoint_Access[ Rect.Top - 1 ] and Access_Read ) <> 0 ) then
            begin
                S := 'read' ;
            end ;
            if( ( _Watchpoint_Access[ Rect.Top - 1 ] and Access_Write ) <> 0 ) then
            begin
                if( length( S ) > 0 ) then
                begin
                    S := S + ', ' ;
                end ;
                S := 'write' ;
            end ;
            if( ( _Watchpoint_Access[ Rect.Top - 1 ] and Access_Execute ) <> 0 ) then
            begin
                if( length( S ) > 0 ) then
                begin
                    S := S + ', ' ;
                end ;
                S := 'execute' ;
            end ;
        end ; // if( _Watchpoint_Access <> nil )
        Grid.Cells[ 2, Rect.Top ] := S ;
    end ; // if( Rect.Top > 0 )
end ; // TMemory_Watchpoints_Dialog.Modify_Watchpoint


// API...

procedure TMemory_Watchpoints_Dialog.Set_Data( Watchpoints : TInteger64_List ;
    Watchpoint_Size, Watchpoint_Access : TInteger_List ; Port : boolean ) ;

var Loop : integer ;
    S : string ;

begin
    _Port := Port ;
    _Watchpoints := Watchpoints ;
    _Watchpoint_Size := Watchpoint_Size ;
    _Watchpoint_Access := Watchpoint_Access ;
    if( _Watchpoint_Size = nil ) then
    begin
        if( Port ) then
        begin
            Caption := 'Port breakpoints' ;
        end else
        if( _Watchpoint_Access = nil ) then
        begin
            Caption := 'Execution breakpoint' ;
        end else
        begin
            Caption := 'Register breakpoints' ;
        end ;
        Term := 'breakpoint' ;
        Grid.Cells[ 1, 0 ] := '' ;
    end else
    begin
        Caption := 'Memory watchpoints' ;
        Term := 'watchpoint' ;
        Grid.Cells[ 1, 0 ] := 'Size' ;
    end ;

    Grid.RowCount := _Watchpoints.Count + 1 ;
    for Loop := 0 to _Watchpoints.Count - 1 do
    begin
        if( _Watchpoint_Size <> nil ) then
        begin
            Grid.Cells[ 0, Loop + 1 ] := inttostr( _Watchpoints[ Loop ] ) ;
            Grid.Cells[ 1, Loop + 1 ] := inttostr( _Watchpoint_Size[ Loop ] ) ;
        end else
        begin
            if( Port or ( _Watchpoint_Access = nil ) ) then
            begin
                Grid.Cells[ 0, Loop + 1 ] := inttostr( _Watchpoints[ Loop ] ) ;
            end else
            begin
                Grid.Cells[ 0, Loop + 1 ] := CPU.CPU.Register_Name( _Watchpoints[ Loop ] ) ;
            end ;
            Grid.Cells[ 1, Loop + 1 ] := '' ;
        end ;
        S := '' ;
        if( _Watchpoint_Access <> nil ) then
        begin
            if( ( _Watchpoint_Access[ Loop ] and Access_Read ) <> 0 ) then
            begin
                S := 'read' ;
            end ;
            if( ( _Watchpoint_Access[ Loop ] and Access_Write ) <> 0 ) then
            begin
                if( length( S ) > 0 ) then
                begin
                    S := S + ', ' ;
                end ;
                S := 'write' ;
            end ;
            if( ( _Watchpoint_Access[ Loop ] and Access_Execute ) <> 0 ) then
            begin
                if( length( S ) > 0 ) then
                begin
                    S := S + ', ' ;
                end ;
                S := 'execute' ;
            end ;
        end ; // if( _Watchpoint_Access <> nil )
        Grid.Cells[ 2, Loop + 1 ] := S ;
    end ;
end ; // TMemory_Watchpoints_Dialog.Set_Data


procedure TMemory_Watchpoints_Dialog.FormShow( Sender : TObject ) ;

begin
    Grid.ColWidths[ 0 ] := ClientWidth div 3 - 2 ;
    Grid.ColWidths[ 1 ] := ClientWidth div 3 - 2 ;
    Grid.ColWidths[ 2 ] := ClientWidth div 3 - 2 ;
    Grid.Cells[ 0, 0 ] := 'Address' ;
    if( _Watchpoint_Size = nil ) then
    begin
        Grid.Cells[ 1, 0 ] := '' ;
    end else
    begin
        Grid.Cells[ 1, 0 ] := 'Size' ;
    end ;
    Grid.Cells[ 2, 0 ] := 'Access' ;
end ;


procedure TMemory_Watchpoints_Dialog.GridKeyUp( Sender : TObject ;
    var Key : Word ; Shift : TShiftState ) ;

begin
    if( ( Key = VK_RETURN ) or ( Key = VK_INSERT ) ) then // New watchpoint
    begin
        Insert_Watchpoint ;
        Key := 0 ;
    end else
    if( ( Key = ord( 'D' ) ) and ( ssCtrl in Shift ) ) then // Delete
    begin
        Delete_Watchpoint ;
        Key := 0 ;
    end else
    if( ( Key = ord( 'E' ) ) and ( ssCtrl in Shift ) ) then // Modify
    begin
        Modify_Watchpoint ;
        Key := 0 ;
    end ;
end ; // TMemory_Watchpoints_Dialog.GridKeyUp


procedure TMemory_Watchpoints_Dialog.DeleteAll1Click( Sender : TObject ) ;

var Loop : integer ;

begin
    if( MessageBox( 0, PChar( 'Really delete all ' + Term + 's?' ),
        'Delete all', MB_YESNO or MB_ICONQUESTION ) = mrOK ) then
    begin
       _Watchpoints.Clear ;
       if( _Watchpoint_Size <> nil ) then
       begin
          _Watchpoint_Size.Clear ;
       end ;
       if( _Watchpoint_Access <> nil ) then
       begin
           _Watchpoint_Access.Clear ;
       end ;
       Grid.RowCount := 2 ;
       for Loop := 0 to 2 do
       begin
           Grid.Cells[ Loop, 1 ] := '' ;
       end ;
    end ;
end ;


procedure TMemory_Watchpoints_Dialog.Add1Click( Sender : TObject ) ;

begin
    Insert_Watchpoint ;
end ;


procedure TMemory_Watchpoints_Dialog.Properties1Click( Sender : TObject ) ;

begin
    Modify_Watchpoint ;
end ;


procedure TMemory_Watchpoints_Dialog.Delete1Click(Sender: TObject);

begin
    Delete_Watchpoint ;
end ;


procedure TMemory_Watchpoints_Dialog.Help_ButtonClick( Sender : TObject ) ;

begin
    Application.HelpContext( HelpContext ) ;
end ;



end.
