{
              Program Name : BACKUPS
              Package Name : LB
              Purpose      : Extended Backup
              Institution  : Conroy & Conroy Co.
              Date Written : 10-Jul-1991
	      Last Modified: 31-Dec-1999
              Written By   : Alan Conroy
              Version      : 2.1D

              Copyright (C) 1991-1999 by Alan Conroy.  Released to the public domain.

	      TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *********************************************************
        *                                                       *
        *        M O D I F I C A T I O N   H I S T O R Y        *
        *                                                       *
        *     DATE      BY             REASON                   *
        *                                                       *
	* 04-Jan-1994	EAC	Allow use of command file.      *
	* 27-Dec-1994	EAC	Skip directory files on backup. *
	* 13-Apr-1996	EAC	Added /START.                   *
	* 31-Dec-1999	EAC	Allow specification of log file.*
	* 03-Jan-1999	EAC	Use PSFile for file access.     *
        *                                                       *
        *********************************************************

        *********************************************************
        *                                                       *
        *           P R O G R A M   P U R P O S E               *
        *                                                       *
        *********************************************************

	  This program is a callable backup utility.

        *********************************************************
        *                                                       *
        *            F I L E   D E S C R I P T I O N S          *
        *                                                       *
        *********************************************************

}

unit Backups ;

interface

{$I EDEFINES.INC}

type Terror_Procedure = procedure( S : string ) ;
     Trequest_Procedure = procedure( S : string ; T : Integer ) ;

     Buf = array[ 0..65519 ] of Byte ;
     Buffer_Ptr = ^Buf ;
     Volume_Header =	record
			    Saveset : string[ 12 ] ; { Name of saveset }
			    Version : Integer ; { Version of saveset }
			    Volume : Integer ; { Volume of saveset }
			    Date_Y, Date_M, Date_D : Word ; { Date of saveset }
			    Expire_Y, Expire_M, Expire_D : Word ;
			    { Expiration date }
			    Filler : array[ 1..15 ] of Byte { Reserved }
			end ;
     File_Header =	record
			    Compression : Byte ; { Type of compression.  0 = none }
			    Attributes : Byte ; { DOS File attributes }
			    Date_Time : Longint ; { Update date/time }
			    Size : Longint ; { DOS file size }
			    Saveset_Size : Longint ; { Size on saveset }
			    Start : Longint ;
			    { Where in DOS file the following data bytes begin }
			    Flags : Longint ; { Reserved }
			    Filename : string { Name }
			end ;

var Abort : Boolean ; { True to abort operation }
    EP : Terror_Procedure ;
    Request : Trequest_Procedure ;
    FH : File_Header ;

procedure Backup_Backup( Command0 : string ; E_P : Terror_Procedure ;
    R_P : Trequest_Procedure ) ; {Backup interface entry point}

implementation

uses { Borland... }
{$IFDEF DELPHI}
     SysUtils, { strpas }
{$ELSE}
     Strings, { strpas }
{$ENDIF}

     { C&C... }
     ASCIIDef, { CRLF }
     CommonUt, { FEdit }
     Dates, { Date }
     ErrMastr, {Get_Error_Text }
     TypeDefs, { TRecord_Size - Must be before Files}
     Files, { PSFile }
     GetChars,
     Parse,
     Misc,
     Num1S,
     XLate,
     SMU,
     Standard,
     O_S, { OS }
     _UEHDefs ; { tUEC }

var  Buffer : Buffer_Ptr ;
     Command : string ; { Input or command line }
     Command_File : string ; { Command file name }
     Control : Integer ; { Loop control variable }
     Count : TRecord_Size ; { Count of bytes to read or write }
     Current_File : string ; { Name of current file being processed }
     Dummy : Integer ; { Dummy loop variable }
     Dummy1 : Word ;
     Fcommand : PSFile ; { Command file }
     Ignore : Boolean ; { True if ignoring creation errors }
{$ifNDEF DELPHI}
     Image : array[ 0..16383 ] of Byte ; { Image of FAT/directory to write }
{$ENDIF}
     Initialize : Boolean ; { True if initializing output device }
     Input_Spec, Output_Spec : string ;
     Input_File, Output_File : PSFile ;
     Log_File : array[ 1..64 ] of PSFile ; { Log files }
     Log_File_Count : integer ;
     No_Warnings : Boolean ;
     offset : Word ; { Offset in buffer }
     Restore : Boolean ; {True if doing a restore operation}
     Result : Word ;
     Select : Boolean ; {True if /SELECT used}
     Selection : string ; { Files to restore or list }
     Start : string ;
     Temp : string ;
     Tt : Text ;
     Vh : Volume_Header ;
     Volume : Integer ; {Current backup disk volume number}


procedure Show_Error( S : string ) ;

begin
    if( @Ep = nil )
	then
	    writeln( Tt, S )
	else
	    Ep( S ) ;
end ;


procedure Write_Log( X : string ) ;

var Dummy : integer ;
    UEC : tUEC ;

begin
    for Dummy := 1 to Log_File_Count do
    begin
	if( X = '' ) then
	begin
	    Log_File[ Dummy ]^.Writeln( '' ) ;
	end else
	begin
	    Log_File[ Dummy ]^.Write( X ) ;
	end ;
	UEC := Log_File[ Dummy ]^.IO_Error ;
	if( UEC.COde <> 0 ) then
	begin
	    Show_Error( strpas( Get_Error_Text( UEC, -1 ) ) + ' - writing to log file' ) ;
	    halt ;
	end ;
    end ;
end ;


procedure Show_Label ;

var Find_Record : TFind_Record ;
    Dummy : integer ;
    FS : PFile_System ;
    UEC : TUEC ;

begin
    FS := Open_File_System( Command[ 1 ] + ':', 0, UEC ) ;
{$IFDEF DELPHI}
  {$WARNINGS OFF}
    Dummy := FS^.FindFirst( Command[ 1 ] + ':\*.*', faVolumeID, Find_Record ) ;
  {$WARNINGS ON}
{$ELSE}
    {$IFDEF WINDOWS}
    Dummy := FS^.FindFirst( Command[ 1 ] + ':\*.*', faVolumeID, Find_Record ) ;
    {$ELSE}
    Dummy := FS^.FindFirst( Command[ 1 ] + ':\*.*', VolumeID, Find_Record ) ;
    {$ENDIF}
{$ENDIF}
    while( Dummy = 0 ) do
    begin
	if( ( Find_Record.Found_Attributes and FSFF_VolumeID ) <> 0 ) then
	begin
	    Write_Log( 'Volume ID is ' + Find_Record.Found_Name + CRLF ) ;
	    exit ; { Done if we found the disk label }
	end ;
	Dummy := FS^.FindNext( Find_Record ) ;
    end ;
    Close_File_System( Command[ 1 ] + ':' ) ;
end ; { Show_Label }


procedure Do_Write( offset, Count : Word ; var UEC : tUEC ) ;

var Adjust : Boolean ; {True if adjusting amount to output}
    Number : Longint ;
    Res : TRecord_Size ;

begin
    Number := Diskfree( Ord( Command[ 1 ] ) - 64 ) ; { Room left on output disk }
    if( Number = 0 ) then { Disk is full }
    begin
	UEC.Facility := 1 ;
	UEC.Code := 101 ; { Disk full error }
	Exit
    end ;

    Adjust := False ;
    if( Number < Count ) then { Not enough room on disk to fit entire block }
    begin
	Count := Number ; { Write only what fits }
	Adjust := True ;
    end ;
    Output_File^.Blockwrite( Buffer^[ offset ], Count, Res ) ; { Write to saveset }
    UEC := Output_File^.IO_Error ; { Get error, if any }
    if( UEC.Code = 0 ) then
    begin
	if Adjust then
	begin
	    UEC.Facility := 1 ;
	    UEC.Code := 101 ; { Didn't all fit }
	end ;
    end ;
end ;


procedure Do_Read( Var Count : Word ; var UEC : tUEC ) ;

var Number : Longint ;
    Res : TRecord_Size ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Number := Input_File^.Get_size - Input_File^.Filepos ;
    { Room left in saveset }
    if( Number = 0 ) then { Eof }
    begin
	Count := 0 ; { Nothing read }
	exit ;
    end ;

    if( Number < Count ) then { Not enough room on volume for entire block }
    begin
	Count := Number ; { Read only what is there }
    end ;
    Input_File^.Blockread( Buffer^, Count, Res ) ; { Read from saveset }
    UEC := Input_File^.IO_Error ; { Get error, if any }
end ;


procedure Format( Drive : Char ) ; { Format a floppy }
{ NOTE: It is assumed that Reg contains drive format information from DOS }

{$ifNDEF DELPHI}
label Loop ;

var A, B, C, Dummy : Integer ; { Dummy variables }
    Buffer : array[ 0..511 ] of Byte ; { Buffer for DISKREAD }
    Dumm : Real ;
    Dummie : Word ;
    Fat : Byte ; { FAT entry size }
    Fat_Root : Integer ; { Offset of FAT entry 0 }
    Fat_Sector : Integer ; { First sector of FAT }
    Max_Fat : Longint ; { Maximum cluster number }
    Media_Id : Byte ; { Type of media }
    Flags, Result : Integer ; { Result and flags }
    S : string ;
    Sector_Size : Integer ; { Size of sectors in bytes }

    function Next_Fat( Cluster : Word ) : Word ;

    var offset : Integer ;
        Temp : Word ;

    begin
        if( Fat = 12 ) then { 12-bit FAT entries? }
        begin
            offset := Fat_Root + ( Cluster * 3 Div 2 ) ; { Byte offset into FAT }
            Temp := Word( Image[ offset ] ) + Word( Image[ offset + 1 ] ) Shl 8 ;
            if( ( Cluster and 1 ) = 1 ) { Odd cluster }
                then
                    Temp := Temp div 16 ;
            Next_Fat := Temp and $Fff
        end
        else { 16 bit entries }
            Next_Fat := Word( Image[ Fat_Root + Cluster * 2 ] ) +
                Word( Image[ Fat_Root + Cluster * 2 + 1 ] ) shl 8
    end ;


    procedure Clear_Fat( Cluster : Word ) ;

    var offset : Integer ;
        Temp : Word ;

    begin
        if( Fat = 12 ) then { 12-bit FAT entries? }
        begin
            offset := Fat_Root + ( Cluster * 3 div 2 ) ; { Byte offset into FAT }
            Temp := Word( Image[ offset ] ) + Word( Image[ offset + 1 ] ) shl 8 ;
            if( ( Cluster and 1 ) = 1 ) { Odd cluster }
                then
                    Temp := Temp and $F
                else
                    Temp := Temp and ( not $Fff ) ;
            Image[ offset ] := Lo( Temp ) ;
            Image[ offset + 1 ] := Hi( Temp ) ;
        end else { 16 bit entries }
        begin
            Image[ Fat_Root + Cluster * 2 ] := 0 ;
            Image[ Fat_Root + Cluster * 2 + 1 ] := 0 ;
        end
    end ;
{$ENDIF}

begin { Format }
{$ifNDEF DELPHI}
    if( not OS^.Valid_Device( Drive ) ) then { Error }
    begin
	Show_Error( 'Invalid output drive: ' + Command + ':' ) ;
	Halt ;
    end ;
    Fillchar( Image, Sizeof( Image ), 0 ) ; { Zero out image }
    Media_Id := OS^.Media_ID( Drive ) ; { Get media ID byte }
    Max_Fat := OS^.Disk_Clusters( Drive ) + 2 ; { Maximum cluster number }
    if( Max_Fat < 4081 )
	then
	    Fat := 12 { 12 bit FAT entry size }
	else
	    Fat := 16 ; { 16 bit FAT entry size }
    Sector_Size := OS^.Disk_Sector_Size( Drive ) ;
    Dummy := Mem[ $F000 : $Fffe ] ; { Get machine type }
    if( ( ( Dummy = 6 ) Or ( Dummy = $94 ) ) And ( Media_Id = $Fa ) ) then { Rainbow RX50 }
    begin
        Fat_Sector := 20 ; { FAT begins at sector 20, but offset 0 in Image }
        Fat_Root := 0
    end else { Non-rainbow }
    begin
        A := Ord( Drive ) - 65 ;
        Diskread( A, 0, 1, Buffer, Result, Flags ) ;
        Fat_Sector := Buffer[ $0E ] + Buffer[ $0F ] * 256 ;
        { FAT 1 begins after reserved sectors }
        Fat_Root := Fat_Sector * Sector_Size
    end ;
    if( Fat = 12 ) { Get bytes in first part of FAT }
	then
	    Dumm := Max_Fat * 1.5
	else
	    Dumm := Max_Fat * 2.0 ;
    if( Dumm * 2 + Fat_Root > 16383 ) then { FAT is too big for Image }
    begin
	Show_Error( 'FAT is too big' ) ;
	Halt ;
    end ;
    Dumm := Dumm / Sector_Size ; { Convert to sectors }
    if Dumm <> Int( Dumm ) then
    begin
        Dumm := Int( Dumm ) + 1 ;
    end ;
    Dummy := Trunc( Dumm ) ; { Round and integerize for sector count }
    A := Ord( Drive ) - 65 ; { Get drive number for diskread/diskwrite }
    Diskread( A, Fat_Sector, Dummy * 2, Image[ Fat_Root ], Result, Flags ) ;

    { Now set-up FAT #1 }
    for Dummy := 2 to Max_Fat do { Check all clusters }
    begin
	Dummie := Next_Fat( Dummy ) ; { Get FAT value for cluster }
        if( Fat = 12 ) then
        begin
            if( Dummie = $Ff7 ) then
            begin
                Dummie := $Fff7 ;
            end ;
        end ;
	if( Dummie <> $Fff7 ) then
        begin
            Clear_Fat( Dummy ) ; { Zero out non-bad clusters }
        end ;
    end ;

    { Now set-up FAT #2 }
    Fat_Root := Fat_Root + Trunc( Dumm * Sector_Size ) ;
    for Dummy := 2 to Max_Fat do { Check all clusters }
    begin
	Dummie := Next_Fat( Dummy ) ; { Get FAT value for cluster }
        if( Fat = 12 ) then
        begin
            if( Dummie = $Ff7 ) then
            begin
                Dummie := $Fff7 ;
            end ;
        end ;
	if( Dummie <> $Fff7 ) then
        begin
            Clear_Fat( Dummy ) ; { Zero out non-bad clusters }
        end ;
    end ;
    if(
        ( Media_Id = 250 )
        and
        (
          ( Mem[ $F000 : $Fffe ] = 6 )
          or
          ( Mem[ $F000 : $Fffe ] = $94 )
        )
      ) then { RX50 on a Rainbow }
    begin
	B := 20 ; { LSN of FAT }
	C := 7 ; { Count }
    end Else
    begin
	B := 1 ; { LSN }
	C := 32 ; { Count }
    end ;
Loop :
    Diskwrite( A, B, C, Image, Dummy, Flags ) ;
    if( ( Flags and 1 ) = 1 ) then { Error }
    begin
	A := 1 ;
	Flags := Hi( Dummy ) ;
	if( ( Flags and 3 ) = 3 ) then
        begin
            Flags := 9
        end else
        begin
	    while( ( Flags and 1 ) = 0 ) do { Find first error code }
	    begin
		A := A + 1 ;
		Flags := Flags div 2
	    end ;
        end ;
	case A of
	    1 : S := 'Unknown command or invalid drvie' ;
	    2 : S := 'Bad address mark' ;
	    3 : S := 'Sector not found' ;
	    4 : S := 'DMA failure' ;
	    5 : S := 'Data error (CRC)' ;
	    6 : S := 'Controller failure' ;
	    7 : S := 'Seek failure' ;
	    8 : S := 'Drive does not respond' ;
	    9 : S := 'Drive is write locked' ;
	end ;
	case Lo( Dummy ) of
	   0 : S := S + '.  Drive is write locked' ;
	   1 : S := S + '.  Unknown unit' ;
	   2 : S := S + '.  Drive not ready' ;
	   3 : S := S + '.  Unknown command in ABSOLUTE SECTOR WRITE' ;
	   4 : S := S + '.  Data error (CRC)' ;
	   5 : S := S + '.  Bad request structure length' ;
	   6 : S := S + '.  Seek error' ;
	   7 : S := S + '.  Unknown media type' ;
	   8 : S := S + '.  Sector not found' ;
	   9 : S := S + '.  Printer out of paper' ;
	  10 : S := S + '.  Write fault' ;
	  11 : S := S + '.  Read fault' ;
	  12 : S := S + '.  General failure' ;
	  13 : S := S + '.  Error 13 in ABSOLUTE SECTOR WRITE' ;
	  14 : S := S + '.  Error 14 in ABSOLUTE SECTOR WRITE' ;
	  15 : S := S + '.  Invalid disk change' ;
	end ;
	Show_Error( S ) ;
	if( @Request = nil )
	    then
		Writeln( Tt,
		    'Initialization error.  Correct and press any key to continue' )
	    else
		Request( 'Initialization error', 0 ) ;
	Command := Inkey ; { Wait for keypress }
	goto Loop ;
    end ; { if Dummy <> 0 }
    Sleep( 2 ) ;
    { This is settle-down time for DOS to recognize free space on disk }
{$endif}
end ;


procedure Write_Header ; { Write volume header to disk }

var Res : TRecord_Size ;
    UEC : tUEC ;

begin
    Control := 1 ; { Execute loop at least once }
    while( Control <> 0 ) do
    begin
	if( @Request = nil ) then
        begin
            Writeln( Tt, 'Place saveset volume ', Vh.Volume,
                ' into drive ', Command, ': and press RETURN' ) ;
            Default_Input.Clear ;
            readln( Temp ) ;
        end else
        begin
            Request( 'Place saveset volume ' + Num1( Vh.Volume ) +
                ' into drive ' + Command + ':', 1 ) ;
        end ;
	if( Initialize )
	    then
		Format( Command[ 1 ] ) ; { Format disk }
	Show_Label ;
	Writeln( Scr ) ;
	Output_File := Open( Command + ':' + Output_Spec, FSOM_Output, UEC ) ;
	if( UEC.Code = 0 ) then { File was opened OK }
	begin
	    OS.Get_Date( Vh.Date_Y, Vh.Date_M, Vh.Date_D ) ;
	    Output_File^.Blockwrite( VH, Sizeof( Vh ), Res ) ;
	    UEC := Output_File^.IO_Error ;
	end ;
	if( UEC.Code <> 0 ) then
	begin
	    Show_Error( Ert( Control ) + ' while writing volume header to ' +
		Command + ':.  Please use another disk.' )
	end ;
    end ;
end ;


procedure Do_Next_Volume ; { Handle next volume for output }

begin
    dispose( Output_File, Done ) ;
    Vh.Volume := Vh.Volume + 1 ; { Next volume }
    Write_Header ;
    Write_Log( '' ) ;
    Write_Log( 'Starting volume ' + Num1( Vh.Volume ) + '  ' ) ;
    Show_Label ;
    Write_Log( '' ) ;
end ;


procedure Get_Next_Volume ; { Handle next volume for input }

label Loop ;

Var Dummy, Dummy1 : Integer ;
    Res : TRecord_Size ;
    UEC : tUEC ;
    Work : string ; { Work variable }

begin
    dispose( Input_File, Done ) ;
    Dummy := Vh.Volume + 1 ;
    Sleep( 2 ) ; { Necessary for SMARTDRV to notice change of disks }
Loop:
    if( @Request = nil ) then
    begin
        writeln( Tt, 'Place volume ', Dummy, ' in drive ',
            Command[ 1 ], ': and press RETURN' ) ;
        Default_Input.Clear ;
        readln( Temp ) ;
    end else
    begin
        Request( 'Place volume ' + Num1( Dummy ) + ' in drive ' +
            Command[ 1 ] + ':', 1 ) ;
    end ;
    if Abort
        then
            exit ;
    Work := Input_Spec ;
    if( pos( ':', Work ) = 0 )
	then
	    Work := Command + ':' + Work ;
    Input_File := Open( Work, FSOM_Input, UEC ) ;
    Filemode := 0 ; { Read only }
    Dummy1 := Ioresult ;
    if( Dummy1 <> 0 ) then
    begin
	Show_Error( Ert( Dummy1 ) + ' opening ' + Input_Spec ) ;
	if( Dummy = 1 )
            then { First volume }
	        Abort := True ;
        if( Abort )
            then
                exit ;
	Goto Loop ;
    end ;
    Filemode := 2 ; { Read/Write }
    if( Input_File^.Get_Size < Sizeof( Vh ) ) then
    begin
	Show_Error( 'Invalid saveset format' ) ;
	if( Dummy = 1 )
            then { First volume }
	        Abort := True ;
        if( Abort )
            then
                exit ;
	goto Loop ;
    end ;
    Input_File^.Blockread( Vh, Sizeof( Vh ), Res ) ; { Get volume header }
    if( ( Vh.Date_D < 1 ) or ( Vh.Date_D > 31 ) or ( Vh.Date_M < 1 ) or ( Vh.Date_M > 12 ) ) then
    begin
	Show_Error( 'Invalid saveset format' ) ;
	if( Dummy = 1 )
	    then { First volume }
	        Abort := True ;
        if( Abort )
            then
                exit ;
	goto Loop ;
    end ;
    if( Vh.Version <> 0 ) then
    begin
	Show_Error(
	  'Saveset was created by a later version of XBACKUP - cannot read saveset' ) ;
	if( Dummy = 1 )
            then { First volume }
	        Abort := True ;
        if Abort
            then
                exit ;
	goto Loop ;
    end ;
    if( ( Vh.Saveset <> copy( Work, 3, 254 ) ) and ( No_Warnings = false ) )
	then
	    Show_Error( 'WARNING: Internal saveset name of ' +
                Vh.Saveset + ' does not match external name' ) ;
    if Abort
        then
            exit ;
    if( ( Vh.Volume <> Dummy ) and ( No_Warnings = false ) ) then
    begin
	Show_Error( 'WARNING: Not the next volume of the saveset' ) ;
    end ;
    Write_Log( 'Volume ' + Num1( Vh.Volume ) + '  ' ) ;
    Show_Label ;
    Write_Log( '' ) ;
end ; { Get_Next_Volume }


procedure Write_File_Header ; { Write file header to saveset }

var Control : Integer ; { Loop control variable }
    Res : TRecord_Size ;
    UEC : TUEC ;

begin
    Control := 1 ;
    while( Control <> 0 ) do
    begin
	Output_File^.Blockwrite( Fh, 23 + Length( Fh.Filename ), Res ) ;
        { Write file header }
	UEC := Output_File^.IO_Error ;
	if( UEC.Code <> 0 ) then { Error }
	begin
	    if( Control = 101 ) { Disk full }
		then
		    if Diskfree( Ord( Command[ 1 ] ) - 64 ) <> 0
			then
			    Show_Error( 'Disk write error while writing to ' +
				Command + ':.  Skipping to next volume.' )
			Else
		Else
		    Show_Error( Ert( Control ) + ' while writing to ' +
			Command + ':.  Skipping to next volume.' ) ;
		Do_Next_Volume ;
	end ;
    end ;
end ;


procedure Do_List ; { List a saveset }

label Loop ;

var Printed : Boolean ; {True if printed name of current file}
    Res : TRecord_Size ;
    UEC : tUEC ;

begin
    Abort := False ;
    if( Length( Input_Spec ) = 2 )
	then
	    Input_Spec := Input_Spec + 'BACKUP.' ;
    Filemode := 0 ; {Read only}
    Input_File := Open( Input_Spec, FSOM_Input, UEC ) ;
    Filemode := 2 ; {Read/Write}
    if( UEC.Code <> 0 ) then
    begin
	Show_Error( strpas( Get_Error_Text( UEC, -1 ) ) +
	    ' while opening saveset ' + Input_Spec ) ;
	exit ;
    end ;
    if( Input_File^.Get_Size < Sizeof( Vh ) ) then
    begin
	Show_Error( 'Invalid saveset format' ) ;
	exit ;
    end ;
    Writeln ;
    Writeln( 'List of saveset ', Input_Spec ) ;
    Writeln ;
    Input_File^.Blockread( Vh, Sizeof( Vh ), Res ) ; {Get volume header}
    if( ( Vh.Date_D < 1 ) Or ( Vh.Date_D > 31 ) Or ( Vh.Date_M < 1 ) Or ( Vh.Date_M > 12 ) ) then
    begin
	Show_Error( 'Invalid saveset format' ) ;
	Exit ;
    end ;
    if( Vh.Version <> 0 ) then
    begin
	Show_Error( 'Saveset was created by a later version of XBACKUP - cannot read saveset' ) ;
	Exit ;
    end ;
    if( ( Vh.Saveset <> Copy( Input_Spec, 3, 254 ) ) and ( No_Warnings = false ) )
	then
	    Show_Error( 'WARNING: Internal saveset name of ' + Vh.Saveset +
		' does not match external name' ) ;
    if( ( Vh.Volume <> 1 ) and ( No_Warnings = false ) )
	then
	    Show_Error( 'WARNING: Not beginning with first volume of saveset' ) ;
    Writeln( 'Volume ', Vh.Volume ) ;
    Printed := False ; { No filenames printed yet }
Loop:
    Input_File^.Blockread( Fh, 23, Res ) ; { Get first part of file header }
    if( Result < 23 ) then { Need next volume }
    begin
	Get_Next_Volume ;
	if( Abort ) then
        begin
            Exit ; { Exit on error }
        end ;
	goto Loop ; { Try again }
    end ;
    if( Fh.Filename = '' )
	then
	    Exit ; { end of saveset }
    if( Input_File^.Filepos + length( Fh.Filename ) > Input_File^.Get_Size ) then
    begin
	Get_Next_Volume ;
	if( Abort ) then
        begin
            Exit ; { Exit on error }
        end ;
	goto Loop ; { Try again }
    end ;
    Input_File^.Blockread( Fh.Filename[ 1 ], length( Fh.Filename ), Res ) ;
    { Read name }
    if( Result < length( Fh.Filename ) ) then
    begin
	Get_Next_Volume ;
	if( Abort ) then
        begin
            Exit ; { Exit on error }
        end ;
	goto Loop ; { Try again }
    end ;
    if( Length( Start ) <> 0 ) then
    begin
	if( WC_Match( Start, FH.Filename ) ) then
	begin
	    Start := '' ;
	end ;
    end ;
    if( ( Wc_Match( Selection, Fh.Filename ) ) and ( length( Start ) = 0 ) ) then
    begin
	if( not Printed ) then
        begin
            Writeln( '    ', Fh.Filename, '  ', Fh.Size ) ;
	    { Show name and size }
        end ;
	Printed := True ;
    end ;
    if( Input_File^.Filepos + Fh.Size - Fh.Start > Input_File^.Get_Size ) then
    begin
	Get_Next_Volume ;
	if( Abort ) then
        begin
            Exit ; { Exit on error }
        end ;
	goto Loop ; { Try again }
    end ;
    Printed := False ;
    Input_File^.Seek( Input_File^.Filepos + Fh.Size - Fh.Start ) ; { Jump to next file }
    goto Loop ;
end ; {Do_Restore}


function Create_Dir( X : string ) : Boolean ;

var Dummy : Integer ;
    Work : string ; {Work variable}

begin
    Dummy := Length( X ) ;
    while( ( X[ Dummy ] <> '\' ) and ( Dummy > 0 ) ) do
    begin
        Dummy := Dummy - 1 ;
    end ;
    X := Copy( X, 1, Dummy - 1 ) ; { Strip filename, etc. }
    if( Length( X ) < 4 ) then { Only root directory }
    begin
	Create_Dir := True ; { OK }
	Exit
    end ;
    Dummy := Pos( '\', X ) ;
    Work := Copy( X, 1, Dummy ) ; { Nothing so far }
    while( Length( Work ) < Length( X ) ) do { Until full path exists }
    begin
	Dummy := Length( Work ) + 1 ;
	while( ( X[ Dummy ] <> '\' ) and ( Dummy <= Length( X ) ) ) do
        begin
            Dummy := Dummy + 1 ; { Find end of next directory in path }
        end ;
	Work := Copy( X, 1, Dummy - 1 ) ; { Get partial path }
        Dummy := Mkdir( Work ) ; { Create the directory }
	if( ( Dummy <> 0 ) and ( Dummy <> 5 ) ) then
        { Error other than already exists }
	begin
	    Show_Error( Ert( Dummy ) + ' while creating directory path: ' +
		Work ) ;
	    Create_Dir := False ; {Failure}
	    exit ;
	end ;
        Work := Work + '\' ;
    end ;
    Create_Dir := True ; { OK }
end ;


Procedure Qualify_Name( var X : string ) ; { Fully qualify a filename }

begin
    if X[ Length( X ) ] = '\' { No filename at all }
	then
	    X := X + '*.*'
	else
            if Extension_Pos( X ) = 0 { No extension }
                then
                    X := X + '.*' ; { Default extension }

end ; { Qualify_Name }


procedure Do_Restore ; { Restore files from a saveset }

label Loop, Next_Selection, Skip_File ;

var Count : Word ; { Number of bytes to read/write }
    Fh_Temp : File_Header ; { Temporary file header }
    Files_Matched : longint ; { Number of matches found to restore }
    Output_File : PSFile ; { File being created }
    Output_Name : string ; { Name of output file }
    Res : TRecord_Size ;
    UEC : tUEC ;

begin
    { Validate output drive }
    if( Switch_Present( Output_Spec ) > 0 ) then
    begin
	Show_Error( 'Invalid switch: ' + Output_Spec ) ;
	Exit ;
    end ;
    Files_Matched := 0 ; { No files thus far }
    if( Pos( ':', Output_Spec ) = 0 ) then
    begin
	Getdir( 0, Current_File ) ;
	Output_Spec := Current_File[ 1 ] + ':' + Output_Spec ;
    end else
    begin
	if( Pos( ':', Output_Spec ) <> 2 ) then
	begin
	    Show_Error( 'Invalid output drive: ' + Output_Spec ) ;
	    Exit ;
	end ;
    end ;
    Qualify_Name( Output_Spec ) ; { Fully qualify name }

    { Validate input drive(s) }
    if( ( Command[ 1 ] < 'A' ) Or ( Command[ 1 ] > 'Z' ) ) then
    begin
	Show_Error( 'Invalid input drive: ' + Command + ':' ) ;
	Exit ;
    end ;

    if( Diskfree( ord( Output_Spec[ 1 ] ) - 64 ) = -1 ) then
    begin
	Show_Error( 'Invalid output drive: ' + Output_Spec[ 1 ] + ':' ) ;
	exit ;
    end ;

    if( Diskfree( ord( Command[ 1 ] ) - 64 ) = -1 ) then
    begin
	Show_Error( 'Invalid input drive: ' + Command + ':' ) ;
	exit ;
    end ;

    Write_Log( 'beginning restore of ' + Command + ':' +
	Input_Spec + ' to ' + Output_Spec + ' on ' + Dates.Date + ' at ' +
        Dates.Time + CRLF ) ;
    if( Length( Input_Spec ) = 2 )
	then
	    Input_Spec := Input_Spec + '*.*' ;

    if( Output_Spec[ 1 ] = Command ) then { From/to same disk }
    begin
	if( @Request = nil ) then
        begin
            Writeln( Tt, 'Place source disk into drive ', Command,
                ': and press RETURN' ) ;
            Default_Input.Clear ;
            readln( Temp ) ;
        end else
            Request( 'Place source disk into drive ' + Command + ':',
                1 ) ;
    end ;
    Abort := False ;
    if( Length( Input_Spec ) = 2 )
	then
	    Input_Spec := Input_Spec + 'BACKUP.' ;
    Filemode := 0 ; { Read only }
    Input_File := Open( Command + ':' + Input_Spec, FSOM_Input, UEC ) ;
    Filemode := 2 ; { Read/Write }
    if( UEC.Code <> 0 ) then
    begin
	Show_Error( strpas( Get_Error_Text( UEC, -1 ) ) +
	    ' while opening saveset ' + Command + ':' + Input_Spec ) ;
	exit ;
    end ;
    if( Input_File^.Get_Size < Sizeof( Vh ) ) then
    begin
	Show_Error( 'Invalid saveset format' ) ;
	Exit ;
    end ;
    Input_File^.Blockread( Vh, Sizeof( Vh ), Res ) ; { Get volume header }
    if( ( Vh.Date_D < 1 ) Or ( Vh.Date_D > 31 ) Or ( Vh.Date_M < 1 ) Or ( Vh.Date_M > 12 ) ) then
    begin
	Show_Error( 'Invalid saveset format' ) ;
	Exit ;
    end ;
    if( Vh.Version <> 0 ) then
    begin
	Show_Error(
	    'Saveset was created by a later version of XBACKUP - cannot read saveset' ) ;
	Exit ;
    end ;
    if( ( Vh.Saveset <> Input_Spec ) and ( No_Warnings = false ) )
	then
	    Show_Error( 'WARNING: Internal saveset name of ' + Vh.Saveset +
		' does not match external name' ) ;
    if( ( Vh.Volume <> 1 ) and ( No_Warnings = false ) )
	then
	    Show_Error( 'WARNING: Not beginning with first volume of saveset' ) ;
    Write_Log( 'Volume ' + Num1( Vh.Volume ) + CRLF ) ;


    { Code to actually do restore follows... }
Next_Selection:
    if( Length( Command_File ) > 0 ) then
    begin
	if( FCommand^.eof )
	    then
		exit ;
	FCommand^.Readln( Selection ) ;
    end ;

Loop:
    if( Input_File^.Filepos + 23 > Input_File^.Get_Size ) then { Eof }
    begin
	Get_Next_Volume ;
	if Abort
	    then
		Exit ; { Exit on error }
	goto Loop ; { Try again }
    end ; { if Input_File^.FILEPOS + 23 > Input_File^.Get_Size }

    Input_File^.Blockread( Fh, 23, Res ) ; { Get first part of file header }
    if( Result < 23 ) then { Need next volume }
    begin
	Get_Next_Volume ;
	if Abort
	    then
		Exit ; { Exit on error }
	goto Loop ; { Try again }
    end ; { if Result < 23 }
    if( Fh.Filename = '' ) then
    begin
	if( Files_Matched = 0 )
	    then
		Show_Error( 'No files found' ) ;
	exit ; { end of saveset }
    end ;

    if( Input_File^.Filepos + length( Fh.Filename ) > Input_File^.Get_Size ) then
    begin
	Get_Next_Volume ;
	if( Abort )
	    then
		Exit ; { Exit on error }
	Goto Loop ; { Try again }
    end ;

    Input_File^.Blockread( Fh.Filename[ 1 ], length( Fh.Filename ), Res ) ;
    { Read name }
    if( Result < length( Fh.Filename ) ) then
    begin
	Get_Next_Volume ;
	if( Abort )
	    then
		Exit ; { Exit on error }
	goto Loop ; { Try again }
    end ; { if( Result < length( Fh.Filename ) )  }

    if( length( Start ) <> 0 ) then
    begin
	if( WC_Match( Start, FH.Filename ) ) then
	begin
	    Start := '' ;
	end ;
    end ;
    if(
	Wc_Match( Selection, Fh.Filename )
	and
	( length( Start ) = 0 )
	and
	( Fh.Start = 0 )
      ) then { if a selected file and starting at beginning of file }
    begin
        inc( Files_Matched ) ; { Count matches }
        Output_Name := WC_Xlate( Selection, Fh.Filename, Output_Spec ) ;
        Write_Log( Output_Name + CRLF ) ; { Show name }
        if( not Create_Dir( Output_Name ) )
            then
                Exit ; { Create any needed sub-directories, exit if error }

        Output_File := Open( Output_Name, FSOM_Output, UEC ) ;
        if( UEC.Code <> 0 ) then
        begin
            if( Ignore ) then { Ignoring creation errors }
            begin
                Write_Log( strpas( Get_Error_Text( UEC, -1 ) ) +
                    ' while creating file ' +
                    Output_Name + ' - skipping file' + CRLF ) ;
                goto Skip_File ;
            end ;
            Show_Error( Ert( Dummy ) + ' while creating file ' + Output_Name ) ;
            Exit ;
        end ; { if( Dummy <> 0 ) }

        while( Output_File^.Get_Size < Fh.Size ) do { Until entire file restored }
        begin
            if Fh.Size - Output_File^.Get_Size > 65520
                then
                    Count := 65520
                Else
                    Count := Fh.Size - Output_File^.Get_Size ;
            Do_Read( Count, UEC ) ; { Read into buffer }
            if( UEC.Code <> 0 ) then
            begin
                Show_Error( strpas( Get_Error_Text( UEC, -1 ) ) + ' while reading saveset' ) ;
                Exit ;
            end ; { if UEC.Code <> 0 }

            if( Count > 0 ) then { if anything was read in from saveset }
            begin
                Output_File^.Blockwrite( Buffer^, Count, Res ) ; { Write to output }
                UEC := Output_File^.IO_Error ;
                if( UEC.Code <> 0 ) then { Handle error }
                begin
                    Show_Error( strpas( Get_Error_Text( UEC, -1 ) ) + ' while writing file ' +
                        Output_Name ) ;
                    Exit ;
                end ; {if Dummy<>0}
            end ; { if Count > 0 }

            if( Input_File^.Filepos = Input_File^.Get_Size ) then
            { Need next volume }
            begin
                Fh_Temp := Fh ;
                SetLength( Fh_Temp.Filename, length( Fh.Filename ) + 1 ) ;
                while(
                       ( Fh_Temp.Filename <> Fh.Filename )
                       or
                       ( Fh_Temp.Size <> Fh.Size )
                     ) do
                { We must do this to prevent synchronization problems (ie
                  a matching saveset name/volume number, but not really
                  from the same saveset). }
                begin
                    Dummy := Vh.Volume ;
                    Get_Next_Volume ;
                    if( Abort ) then
                    begin
                        Exit ; { Exit on error }
                    end ;
                    Input_File^.Blockread( Fh_Temp, 23, Res ) ;
                    { Get first part of file header }
                    if( Result = 23 ) then { File header was read }
                    begin
                        if( Input_File^.Filepos + length( Fh_Temp.Filename ) <=
                                    Input_File^.Get_Size ) then
                        begin
                            Input_File^.Blockread( Fh_Temp.Filename[ 1 ],
                                length( Fh_Temp.Filename ), Res ) ;
                            { Read file name }
                            if( Result = length( Fh_Temp.Filename ) ) then { File name was read }
                            begin
                                if(
                                    ( Fh_Temp.Filename <> Fh.Filename )
                                    or
                                    ( Fh_Temp.Size <> Fh.Size )
                                  ) then
                                begin
                                    Vh.Volume := Dummy ;
                                    Show_Error(
                                        'Not the next volume of the saveset' ) ;
                                end else
                                begin
                                    Fh := Fh_Temp ;
                                end ;
                            end else
                            begin
                                set_length( Fh_Temp.Filename, length( Fh.Filename ) + 1 ) ;
                            end ;
                        end ; { if... }
                    end ;
                end ; { WHILE... }
            end ; { if Input_File^.FILEPOS = Input_File^.Get_Size }
        end ; { WHILE Output_File^.Get_Size < FH.Size }

        Output_File.SetFTime( Fh.Date_Time ) ; { Set file date/time }
        Output_File.Set_Attributes( Fh.Attributes ) ; { Set file attributes }
        dispose( Output_File, Done ) ; { Done restoring file }
        if(
            ( pos( '*', Selection ) = 0 )
            and
            ( pos( '...', Selection ) = 0 )
            and
            ( pos( '?', Selection ) = 0 )
          ) then  { No wildcards }
        begin
            if Length( Command_File ) = 0 { No command file }
                then
                    exit
                else
                    goto Next_Selection ;
        end ;
        goto Loop ;
    end ;
Skip_File:
    if( Input_File^.Filepos + Fh.Size - Fh.Start > Input_File^.Get_Size ) then
    begin
	Get_Next_Volume ;
	if( Abort )
	    then
		Exit ; { Exit on error }
	Goto Loop ; { Try again }
    end ;
    Input_File^.Seek( Input_File^.Filepos + Fh.Size - Fh.Start ) ;
    { Jump to next file }
    goto Loop ;
end ;


procedure Do_Backup ; { Perform backup }

    function Do_Lookup( X : string ; var Find : TFind_Record ) : string ;
    { Do a wildcard lookup, but ignore certain files }

    label Loop ;

    var D : Integer ;
        Filename, Res : string ;

    begin
Loop:
        Res := Lookup( X, Find ) ; {Lookup first/next file}
        X := '' ; {In case we loop back}
        if( Length( Res ) = 0 ) then {Quick exit if no more matches}
        begin
            Do_Lookup := '' ; {Return Null}
            exit
        end ;
        D := Length( Res ) ;
        while( ( D > 0 ) and ( Res[ D ] <> '\' ) ) do
        begin
            D := D - 1 ;
        end ;
        Filename := Copy( Res, D + 1, 253 ) ;
        if( ( Filename = '.' ) or ( Filename = '..' ) ) then
        begin
            goto Loop ; { Skip . and .. }
        end ;
        if( length( Start ) <> 0 ) then
        begin
            if( WC_Match( Start, Res ) ) then
            begin
                Start := '' ;
            end else
            begin
                goto Loop ;
            end ;
        end ;
        Do_Lookup := Result ; { Return result }
    end ; { Do_Backup.Do_Lookup }


    procedure end_Saveset ; { end a saveset }

    begin
        Fh.Filename := '' ; { This signifies the EOS }
        Filemode := 2 ; { Read/Write }
        Write_File_Header ;
    end ; { Do_Backup.end_Saveset }


    procedure Validate_Input ; {Validate input drive}

    begin
        if( Pos( ':', Input_Spec ) = 0 ) then
        begin
            Getdir( 0, Current_File ) ;
            Input_Spec := Current_File[ 1 ] + ':' + Input_Spec ;
        end else
        begin
            if( Pos( ':', Input_Spec ) <> 2 ) then
            begin
                Show_Error( 'Invalid input drive: ' + Input_Spec ) ;
                Abort := True ;
                Exit ;
            end ;
        end ;
        if( Pos( '\', Input_Spec ) = 0 )
            then
                Input_Spec := Copy( Input_Spec, 1, 2 ) + '\...\' + Copy( Input_Spec, 3, 254 ) ;
        Qualify_Name( Input_Spec ) ;
        if( Diskfree( ord( Input_Spec[ 1 ] ) - 64 ) = -1 ) then
        begin
            Show_Error( 'Invalid input drive: ' + Input_Spec[ 1 ] + ':' ) ;
            Abort := True ;
            Exit ;
        end ;
        if( Length( Input_Spec ) = 2 )
            then
                Input_Spec := Input_Spec + '*.*' ;
    end ; {Do_Backup.Validate_Input}

label Skip_File, Next_Selection ;

var Find : TFind_Record ;
    UEC : tUEC ;

begin { Do_Backup }
    if( Select ) then {/SELECT is only allowed on restores and lists}
    begin
	Show_Error( '/SELECT is only allowed on restores and listings' ) ;
	Exit ;
    end ;
    if( Switch_Present( Input_Spec ) > 0 ) then
    begin
	Show_Error( 'Invalid switch: ' + Input_Spec ) ;
	Exit ;
    end ;

    { Validate output drive(s) }
    if( ( Command[ 1 ] < 'A' ) Or ( Command[ 1 ] > 'Z' ) ) then
    begin
	Show_Error( 'Invalid output drive: ' + Command + ':' ) ;
	Exit ;
    end ;

    if( Diskfree( ord( Command[ 1 ] ) - 64 ) = -1 ) then
    begin
	Show_Error( 'Invalid output drive: ' + Command + ':' ) ;
	Exit ;
    end ;

    if( OS^.Device_Manager^.Fixed_Device( Command ) ) then { Fixed drive }
    begin
	Show_Error( 'Cannot initialize a non-removable disk' ) ;
	Exit ;
    end ;

    Vh.Saveset := Output_Spec ; { Saveset name }
    Vh.Version := 0 ; { Version 0 of saveset format }
    Vh.Volume := 1 ; { First volume }
    Vh.Expire_Y := 9999 ; { Expiration date }
    Vh.Expire_M := 1 ;
    Vh.Expire_D := 1 ;
    Write_Header ; { Write header to disk }

    if( Length( Command_File ) = 0 ) then
    begin
	Validate_Input ;
	if( Abort )
	    then
		exit ;
    end ;

    Write_Log( 'beginning backup of ' + Input_Spec + ' to ' + Command +
	':' + Output_Spec + ' on ' + Dates.Date + ' at ' + Dates.Time +
        CRLF ) ;

    if( Input_Spec[ 1 ] = Command ) then { From/to same disk }
    begin
	if( @Request = nil ) then
        begin
            Writeln( Tt, 'Place source disk into drive ',
                Command, ': and press RETURN' ) ;
            Default_Input.Clear ;
            readln( Temp ) ;
        end else
        begin
            Request( 'Place source disk into drive ' + Command + ':', 1 ) ;
        end ;
    end ;
Next_Selection:
    if( Length( Command_File ) > 0 ) then
    begin
	if( FCommand^.Eof ) then
	begin
	    end_Saveset ;
	    exit ; { Done }
	end ;
	FCommand^.Readln( Input_Spec ) ;
	Validate_Input ;
	if( Abort )
	    then
		exit ;
    end ;

    Initialize_Find_Record( Find ) ;
    Current_File := Do_Lookup( Input_Spec, Find ) ; { Find first file }
    while( Length( Current_File ) <> 0 ) do { Until no more files to process }
    begin
	Dummy1 := Input_File^.Get_Attributes ; { DOS File attributes }
	if( ( Dummy1 and FSFF_Directory ) <> 0 ) then { Skip directories }
	begin
	    goto Skip_File ;
	end ;
	Write_Log( Current_File + CRLF ) ; { Show file being backed up }
	Fh.Attributes := Dummy1 ;
	Filemode := 0 ; { Read-only }
	Input_File := Open( Current_File, FSOM_Input, UEC ) ;
	Filemode := 2 ; { Read/Write }
	if( UEC.Code <> 0 ) then { Error on open }
	begin
	    if( Ignore ) then
	    begin
		Write_Log( strpas( Get_Error_Text( UEC, -1 ) ) + ' while opening file ' +
		    Current_File + ' - skipping file' + CRLF ) ;
		Goto Skip_File ;
	    end ;
	    Show_Error( strpas( Get_Error_Text( UEC, -1 ) ) + ' while opening file ' + Current_File ) ;
	    Exit ;
	end ;
	Fh.Compression := 0 ; { Compression not supported yet }
	Input_File^.Getftime( Fh.Date_Time  ) ;
	Fh.Size := Input_File^.Get_Size ; { Get DOS file size }
	Fh.Saveset_Size := Fh.Size ; { Same as DOS with no compression }
	Fh.Flags := 0 ; { Not used yet }
	Fh.Filename := Current_File ;
	Fh.Start := 0 ; { offset 0 }
	if( Input_Spec[ 1 ] = Command ) then { From/to same disk }
	begin
	    if( @Request = nil ) then
            begin
                Writeln( Tt, 'Place saveset volume ', Vh.Volume, ' into drive ',
                    Command, ': and press RETURN' ) ;
                Default_Input.Clear ;
                readln( Temp ) ;
            end else
            begin
                Request( 'Place saveset volume ' + Num1( Vh.Volume ) +
                    ' into drive ' + Command + ':', 1 ) ;
            end ;
	end ;
	Write_File_Header ;

	if( Input_Spec[ 1 ] = Command ) then { From/to same disk }
	begin
	    if( @Request = nil ) then
            begin
                Writeln( Tt, 'Place source disk into drive ',
                    Command, ': and press RETURN' ) ;
                Default_Input.Clear ;
                readln( Temp ) ;
            end else
            begin
                Request( 'Place source disk into drive ' + Command + ':', 1 ) ;
            end ;
	end ;
	while( Input_File^.Filepos < Input_File^.Get_Size ) do
	begin
	    Input_File^.Blockread( Buffer^, 65520, Count ) ;
	    { Read as much as we can }

	    if( Input_Spec[ 1 ] = Command ) then { From/to same disk }
	    begin
		if @Request = nil then
                begin
                    Writeln( Tt, 'Place saveset volume ', Vh.Volume,
                        ' into drive ', Command,
                        ': and press RETURN' ) ;
                    Default_Input.Clear ;
                    readln( Temp ) ;
                end else
                begin
                    Request( 'Place saveset volume ' + Num1( Vh.Volume ) +
                        ' into drive ' + Command + ':', 1 ) ;
                end ;
	    end ;
	    Result := 0 ;
	    offset := 0 ;
	    while( Result < Count ) do
	    begin
		Result := Output_File^.Filepos ;
		Do_Write( offset, Count, UEC ) ; { Write to saveset }
		if( UEC.Code = 0 ) then
		begin
		    Result := Count ;
		end else
		begin
		    if( UEC.Code = 101 ) { Disk full }
			then
			    if( Diskfree( Ord( Command[ 1 ] ) - 64 ) <> 0 )
				then
				    Show_Error(
					'Disk write error while writing to ' +
					Command +
					':.  Skipping to next volume.' )
				else
			else
			    Show_Error( strpas( Get_Error_Text( UEC, -1 ) ) + ' while writing to ' +
				Command + ':.  Skipping to next volume.' ) ;
		    Result := Output_File^.Filepos - Result ;
		    { Bytes actually written }

		    Do_Next_Volume ;
		    Fh.Start := Input_File^.Filepos - Count + Result ;
		    offset := Result ;
		    Count := Count - Result ; { Number of bytes left to transfer }
		    Write_File_Header ;
		    Result := 0 ; { Force loop to keep executing }
		end { if Control = 0 }
	    end { WHILE Result < Count }
	end ; { WHILE Input_File^.FILEPOS < Input_File^.Get_Size }
        dispose( Input_File, Done ) ; { Done with file }

Skip_File :
	Current_File := Do_Lookup( '', Find ) ; { Find next file }
    end ; { while LENGTH(Current_File) <> 0 }
    if Length( Command_File ) > 0
	then
	    goto Next_Selection ;
    end_Saveset ;
end ;


procedure Backup_Backup( Command0 : string ; E_P : Terror_Procedure ;
    R_P : Trequest_Procedure ) ; { Backup interface entry point }

var Dummy : integer ;
    UEC : tUEC ;

begin
    Getmem( Buffer, 65520 ) ; { Allocate memory for buffer }
    Ep := E_P ;
    Request := R_P ;
    Assign( Tt, '' ) ;
    Rewrite( Tt ) ;

    { Parse switches... }

    if( Parse_Switch( 'NOW|ARNINGS', '', Command0, Temp ) = 1 ) then
    begin
	if length( Temp ) > 0 then { Parameter to switch }
	begin
	    Show_Error( 'Illegal parameter on /NOWARNINGS' ) ;
	    exit ;
	end ;
	No_Warnings := True ; { No warnings }
    end else No_Warnings := False ;

    if( Parse_Switch( 'ST|ART', '', Command0, Start ) = 1 ) then
    begin
	if length( Start ) = 0 then { Parameter to switch }
	begin
	    Show_Error( 'Missing file specification on /START' ) ;
	    exit ;
	end ;
    end else
    begin
        Start := '' ;
    end ;

    if Parse_Switch( 'I|GNORE', '', Command0, Temp ) = 1 then
    begin
	if Length( Temp ) > 0 then { Parameter to switch }
	begin
	    Show_Error( 'Illegal parameter on /IGNORE' ) ;
	    Exit ;
	end ;
	Ignore := True ; { No warning logging }
    end Else Ignore := False ;

    Log_File_Count := 0 ;
    if( Parse_Switch( 'NOLO|G', 'LO|G', Command0, Temp ) = 1 ) then
    begin
	if( Length( Temp ) > 0 ) then { Parameter to switch }
	begin
	    Show_Error( 'Illegal parameter on /NOLOG' ) ;
	    Exit ;
	end ;
	Log_File_Count := -1 ;
    end else
    begin
	if( Length( Temp ) > 0 ) then { Parameter to switch }
	begin
	    while( length( Temp ) > 0 ) do
	    begin
		Dummy := pos( ',', Temp + ',' ) ;
		Log_File_Count := Log_File_Count + 1 ;
		if( Log_File_Count > 64 ) then
		begin
		    Show_Error( 'Too many log files' ) ;
		    exit ;
		end ;
		Log_File[ Log_File_Count ] :=
		    Open( copy( Temp, 1, Dummy - 1 ),
		    FSOM_Output or FSOM_Text, UEC ) ;
		if( UEC.Code <> 0 ) then
		begin
		    Show_Error( strpas( Get_Error_Text( UEC, -1 ) ) +
			' while opening log file ' +
			copy( Temp, 1, Dummy - 1 ) ) ;
		    exit ;
		end ;
		Temp := copy( Temp, Dummy + 1, length( Temp ) ) ;
	    end ;
	end else
	begin
	    Log_File_Count := 1 ;
	    Log_File[ 1 ] := Open( 'CON', FSOM_Output or FSOM_Text, UEC ) ;
	    if( UEC.Code <> 0 ) then
	    begin
		Show_Error( strpas( Get_Error_Text( UEC, -1 ) ) +
		    ' while opening log file' ) ;
	    end ;
	end ;
    end ;

    if Parse_Switch( 'R|ESTORE', '', Command0, Temp ) = 1 then
    begin
	if( Length( Temp ) > 0 ) then { Parameter to switch }
	begin
	    Writeln( 'Illegal parameter on /RESTORE' ) ;
	    Exit ;
	end ;
	Restore := True ; { Restore from a saveset }
    end else
    begin
        Restore := False ;
    end ;

    Command_File := '' ;
    Dummy := Parse_Switch( 'C|OMMAND', '', Command0, Temp ) ;
    if( Dummy = 1 ) then { /COMMAND }
    begin
	if( Length( Temp ) = 0 ) then { No parameter }
	begin
	    Show_Error( 'Missing parameter for /COMMAND' ) ;
	    exit ;
	end ;
	Command_File := Temp ;
	FCommand := Open( Command_File, FSOM_Input or FSOM_Text, UEC ) ;
	if( UEC.Code <> 0 ) then
	begin
	    Show_Error( Ert( Dummy ) + ' while opening command file' ) ;
	    exit ;
	end ;
    end ;

    Dummy := Parse_Switch( 'NOI|NITIALIZE', 'I|NITIALIZE', Command0, Temp ) ;
    if Dummy = 1 then { /NOINITIALIZE }
    begin
	if( Length( Temp ) > 0 ) then { Parameter to switch }
	begin
	    Show_Error( 'Illegal parameter on /INITIALIZE or /NOINITIALIZE' ) ;
	    Exit ;
	end ;
	Initialize := False ; { No initialization }
    end else
    begin
	if( Restore ) then
	begin
	    if( Dummy = 2 ) then { /INITIALIZE }
	    begin
		Show_Error( 'Incompatible switches' ) ;
		exit ;
	    end ;
	    Initialize := False ;
	end else
	begin
	    Initialize := True ;
	end ;
    end ;

    FEdit( Command0, Command0, 8 + 128 ) ; { No leading/trailing spaces }

    Input_Spec := Parse_Parameter( ' ', Command0 ) ;
    { Input specification is the first parameter }
    Selection := '*:\...\*.*' ; { Default to restoring/listing everything }
    if( Parse_Switch( 'SE|LECT', '', Input_Spec, Temp ) = 1 ) then
    begin
	if( length( Command_File ) > 0 ) then
	begin
	    Show_Error( 'Incompatible switches' ) ;
	    exit ;
	end ;
	if( Length( Temp ) > 0 )
	    then
		Selection := Temp ;
	Select := True ; { Indicate that the switch was used }
	if( Pos( ':', Selection ) = 0 )
	    then
		Selection := '*:' + Selection ;
	{ Default to all devices }
	if( Pos( '\', Selection ) = 0 )
	    then { Default to all directories }
		Selection := Copy( Selection, 1, 2 ) + '\' + Copy( Selection, 3, 254 ) ;
	Qualify_Name( Selection ) ;
    end else
    begin
        Select := False ; { No selection specified }
    end ;
    Fedit( Command0, Command0, 32 ) ;
    Fedit( Input_Spec, Input_Spec, 32 ) ;
    if( Restore ) then
    begin
	if( Switch_Present( Input_Spec ) > 0 ) then
	begin
	    Show_Error( 'Unknown switch: ' + Input_Spec ) ;
	end ;
	Output_Spec := Command0 ;
	Command0 := Input_Spec[ 1 ] ;
	Input_Spec := Copy( Input_Spec, 3, 254 ) ;
	if( Length( Input_Spec ) = 0 ) then
	begin
	    Input_Spec := 'BACKUP.' ;
	end ;

	{ Input_Spec now contains the saveset name }
	if( Length( Input_Spec ) > 12 ) then
	begin
	    Show_Error( 'Invalid saveset name: ' + Input_Spec ) ;
	    Exit ;
	end ;
    end ;

    if( Length( Command0 ) = 0 ) then { Only one parameter }
    begin
	if( Parse_Switch( 'LI|ST', '', Input_Spec, Temp ) = 1 ) then
	begin
	    if( Ignore or Restore or ( Log_File_Count < 0 ) or ( Length( Command_File ) > 0 ) ) then
	    { Cannot do /LIST and /RESTORE or /LIST and /NOLOG or /LIST and
	      /IGNORE or /LIST and /COMMAND }
	    begin
		Show_Error( 'Incompatible switches' ) ;
		Exit ;
	    end ;
	    if Length( Temp ) > 0
		then { Parameter to switch }
		    Show_Error( 'Illegal parameter on /LIST' )
		Else
		    Do_List ; { List a saveset }
	    Exit
	end ;
	Show_Error( 'Missing output device' ) ;
	Exit ;
    end ;
    if( not Restore )
	then
	    Output_Spec := Command0 ;
	    { Output specification is the second parameter }

    { Parse output specification }

    if( not Restore ) then
    begin
	if( Output_Spec[ 2 ] <> ':' ) then
	begin
	    Show_Error( 'Invalid output drive: ' + Output_Spec ) ;
	    Exit ;
	end ;
	Command0 := Output_Spec[ 1 ] ;
	Output_Spec := Copy( Output_Spec, 3, 254 ) ;
	if( Length( Output_Spec ) = 0 )
	    then
		Output_Spec := 'BACKUP.' ;
	{ Output_Spec now contains the saveset name }
        if( Extension_Pos( Output_Spec ) = 0 )
            then
                Output_Spec := Output_Spec + '.' ;
	if( Length( Output_Spec ) > 12 ) then
	begin
	    Show_Error( 'Invalid saveset name: ' + Output_Spec ) ;
	    Exit ;
	end ;
    end ;
    if( Switch_Present( Output_Spec ) > 0 ) then
    begin
	Show_Error( 'Invalid switch: ' + Output_Spec ) ;
	Exit ;
    end ;
    Command := Command0 ;

    if Restore
	then
	    Do_Restore
	Else
	    Do_Backup ;

    { Now free up any allocated memory... }
    dispose( Input_File, Done ) ;
    dispose( Output_File, Done ) ;
    Freemem( Buffer, 65520 ) ; { Deallocate memory for buffer }
end ;


{$IFDEF DELPHI}
initialization
    System.Initialize( FH.Filename ) ;

finalization
    Finalize( FH.Filename ) ;
{$ENDIF}
end.
