{
        Program Name : DPropert
        Package Name : LB
        Purpose      : Standard dynamic properties
        Institution  : Conroy & Conroy Co.
        Date Written : 14-February-2005
        Written By   : Alan Conroy
        Version      : 1.0

	Copyright (C) 2005 by Alan Conroy.  Released to the Public Domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TDynamic_Properties is an abstract base class definition for objects
        which provide dynamic object properties.
}

unit DPropert ;

interface

{$I EDEFINES.INC}

uses { Borland... }
{$IFDEF DELPHI}
     Classes, { TStringList }
{$ENDIF}

     { Layer 0... }
     _Propert, { TSDynamic_Properties }
     _GenVal, { TGeneric_Value }

     { Layer 1... }
     Collect ; { TCollection }


type TSStandard_Dynamic_Properties = object( TSDynamic_Properties )
                                         public { Construictors and destructors... }
                                             constructor Init ;
                                             destructor Done ; virtual ;

                                         private { Instance data... }
                                             Names : TString_Collection ;
                                             Values : TCollection ;

                                         public { API... }
                                             { Adds a new property, returning the index.  Returns
                                               -1 if a property already exists by that name. }
                                             function Add( const Nam : string ;
                                                 Value : PSGeneric_Value ) : longint ;
                                                 virtual ;

                                             { Deletes all properties. }
                                             procedure Clear ; virtual ;

                                             { Count of defined properties... }
                                             function Count : longint ;
                                                 virtual ;

                                             { Returns the index of the property with the
                                               specified name.  Returns -1 if the named
                                               property doesn't exist. }
                                             function Index_Of( const Nam : string ) : longint ;
                                                 virtual ;

                                             { Retrieves string name of property with
                                               given index.  If index is invalid, returns
                                               a null string. }
                                             function Get_Name( Index : longint ) : string ;
                                                 virtual ;

                                             { Gets the value for the specified index.  Returns
                                               nil if the index is invalid. }
                                             function Get_Value( Index : longint ) : PSGeneric_Value ;
                                                 virtual ;

                                             { Sets the value for the specified index. }
                                             procedure Set_Value( Index : longint ;
                                                 Value : PSGeneric_Value ) ;
                                                 virtual ;
                                     end ;
     PSStandard_Dynamic_Properties = ^TSStandard_Dynamic_Properties ;

{$IFDEF DELPHI}
type TStandard_Dynamic_Properties = class( TDynamic_Properties )
                                        public { Constructors and destructors... }
                                            constructor Create ;
                                            destructor Destroy ; override ;

                                        private { Instance data... }
                                             _Reference_Count : integer ;
                                             Names : TStringList ;
                                             Values : TList ;

                                        public { API... }
                                            function Add( const Nam : string ;
                                                 Value : TGeneric_Value ) : longint ;
                                                 override ;

                                             procedure Attach ; override ;

                                             procedure Clear ; override ;

                                             function Count : longint ;
                                                 override ;

                                             procedure Detach ; override ;

                                             function Index_Of( const Nam : string ) : longint ;
                                                 override ;

                                             function Get_Name( Index : longint ) : string ;
                                                 override ;

                                             function Get_Value( Index : longint ) : TGeneric_Value ;
                                                 override ;

                                             procedure Set_Value( Index : longint ;
                                                 Value : TGeneric_Value ) ;
                                                 override ;
                                    end ;

type TStandard_COM_Dynamic_Properties = class( TCOM_Dynamic_Properties )
                                            public { Constructors and destructors... }
                                                constructor Create ;
                                                destructor Destroy ; override ;

                                            private { Instance data... }
                                                 Names : TStringList ;
                                                 Values : TList ;

                                            public { API... }
                                                function Add( Nam : PChar ;
                                                     Value : TCOM_Generic_Value ) : longint ;
                                                     override ;

                                                 procedure Clear ; override ;

                                                 function Count : longint ;
                                                     override ;

                                                 function Index_Of( Nam : PChar ) : longint ;
                                                     override ;

                                                 function Get_Name( Index : longint ) : PChar ;
                                                     override ;

                                                 function Get_Value( Index : longint ) : TCOM_Generic_Value ;
                                                     override ;

                                                 procedure Set_Value( Index : longint ;
                                                     Value : TCOM_Generic_Value ) ;
                                                     override ;
                                        end ;
{$ENDIF}

implementation

uses { Layer1... }
     CommonUt ; { Edit }

{ TSStandard_Dynamic_Properties methods... }

{ Construictors and destructors... }

constructor TSStandard_Dynamic_Properties.Init ;

begin
    inherited Init ;

    Names.Init( 0, 4 ) ;
    Values.Init( 0, 4 ) ;
end ;


destructor TSStandard_Dynamic_Properties.Done ;

begin
    Names.Done ;
    Values.Done ;

    inherited Done ;
end ;


{ API... }

function TSStandard_Dynamic_Properties.Add( const Nam : string ;
    Value : PSGeneric_Value ) : longint ;

var S : string ;

begin
    S := Edit( Nam, 2 or 32 ) ; { Uppercase, no spaces ]
    if( Index_Of( S ) <> -1 ) then
    begin
        Add := -1 ;
        exit ;
    end ;
    Add := Names.Count ;
    Names.Insert( S ) ;
    Values.Insert( Value ) ;
    Value^.Attach ;
end ;


procedure TSStandard_Dynamic_Properties.Clear ;

var Loop : integer ;
    Temp : PSGeneric_Value ;

begin
    for Loop := 0 to Count - 1 do
    begin
        Temp := PSGeneric_Value( Values.At( Loop ) ) ;
        if( Temp <> nil ) then
        begin
            Temp^.Detach ;
        end ;
    end ;
    Names.DeleteAll ;
    Values.DeleteAll ;
end ;


function TSStandard_Dynamic_Properties.Count : longint ;

begin
    Count := Names.Count ;
end ;


function TSStandard_Dynamic_Properties.Index_Of( const Nam : string ) : longint ;

var S : string ;

begin
    S := Edit( Nam, 2 or 32 ) ;
    Index_Of := Names.IndexOf( S ) ;
end ;


function TSStandard_Dynamic_Properties.Get_Name( Index : longint ) : string ;

begin
    if( ( Index < 0 ) or ( Index >= Names.Count ) ) then
    begin
        Get_Name := '' ;
        exit ;
    end ;
    Get_Name := Names.At( Index ) ;
end ;


function TSStandard_Dynamic_Properties.Get_Value( Index : longint ) : PSGeneric_Value ;

begin
    if( ( Index < 0 ) or ( Index >= Names.Count ) ) then
    begin
        Get_Value := nil ;
        exit ;
    end ;
    Get_Value := PSGeneric_Value( Values.At( Index ) ) ;
end ;


procedure TSStandard_Dynamic_Properties.Set_Value( Index : longint ;
    Value : PSGeneric_Value ) ;

var Temp : PSGeneric_Value ;

begin
    if( ( Index < 0 ) or ( Index >= Names.Count ) ) then
    begin
        exit ;
    end ;
    if( Value <> nil ) then
    begin
        Value^.Attach ;
    end ;
    Temp := PSGeneric_Value( Values.At( Index ) ) ;
    if( Temp <> nil ) then
    begin
        Temp^.Detach ;
    end ;
    Values.AtPut( Index, Value ) ;
end ;



{$IFDEF DELPHI}
{ TStandard_Dynamic_Properties methods... }

{ Constructors and destructors... }

constructor TStandard_Dynamic_Properties.Create ;

begin
    inherited Create ;

    Names := TStringList.Create ;
    Values := TList.Create ;
end ;


destructor TStandard_Dynamic_Properties.Destroy ;

begin
    Names.Free ;
    Names := nil ;
    Values.Free ;
    Values := nil ;

    inherited Destroy ;
end ;


{ API... }

function TStandard_Dynamic_Properties.Add( const Nam : string ;
    Value : TGeneric_Value ) : longint ;

var S : string ;

begin
    S := Edit( Nam, 2 or 32 ) ; { Uppercase, no spaces ]
    if( Index_Of( S ) <> -1 ) then
    begin
        Add := -1 ;
        exit ;
    end ;
    Add := Names.Count ;
    Names.Add( S ) ;
    Values.Add( Value ) ;
    if( Value <> nil ) then
    begin
        Value.Attach ;
    end ;
end ;


procedure TStandard_Dynamic_Properties.Attach ;

begin
    inc( _Reference_Count ) ;
end ;


procedure TStandard_Dynamic_Properties.Clear ;

var Loop : integer ;
    Temp : TGeneric_Value ;

begin
    for Loop := 0 to Count - 1 do
    begin
        Temp := TGeneric_Value( Values[ Loop ] ) ;
        if( Temp <> nil ) then
        begin
            Temp.Detach ;
        end ;
    end ;
    Names.Clear ;
    Values.Clear ;
end ;


function TStandard_Dynamic_Properties.Count : longint ;

begin
    Count := Names.Count ;
end ;


procedure TStandard_Dynamic_Properties.Detach ;

begin
    dec( _Reference_Count ) ;
    if( _Reference_Count <= 0 ) then
    begin
        Free ;
    end ;
end ;


function TStandard_Dynamic_Properties.Index_Of( const Nam : string ) : longint ;

begin
    Index_Of := Names.IndexOf( Edit( Nam, 2 or 32 ) ) ;
end ;


function TStandard_Dynamic_Properties.Get_Name( Index : longint ) : string ;

begin
    if( ( Index < 0 ) or ( Index >= Count ) ) then
    begin
        Get_Name := '' ;
        exit ;
    end ;
    Get_Name := Names[ Index ] ;
end ;


function TStandard_Dynamic_Properties.Get_Value( Index : longint ) : TGeneric_Value ;

begin
    if( ( Index < 0 ) or ( Index >= Count ) ) then
    begin
        Get_Value := nil ;
        exit ;
    end ;
    Get_Value := TGeneric_Value( Values[ Index ] ) ;
end ;


procedure TStandard_Dynamic_Properties.Set_Value( Index : longint ;
     Value : TGeneric_Value ) ;

begin
    if( ( Index < 0 ) or ( Index >= Count ) ) then
    begin
        exit ;
    end ;
    if( Value <> nil ) then
    begin
        Value.Attach ;
    end ;
    if( Values[ Index ] <> nil ) then
    begin
        TGeneric_Value( Values[ Index ] ).Detach ;
    end ;
    Values[ Index ] := Value ;
end ;



{ TStandard_COM_Dynamic_Properties methods... }

{ Constructors and destructors... }

constructor TStandard_COM_Dynamic_Properties.Create ;

begin
    inherited Create ;

    Names := TStringList.Create ;
    Values := TList.Create ;
end ;


destructor TStandard_COM_Dynamic_Properties.Destroy ;

begin
    Names.Free ;
    Names := nil ;
    Values.Free ;
    Values := nil ;

    inherited Destroy ;
end ;


{ API... }

function TStandard_COM_Dynamic_Properties.Add( Nam : PChar ;
    Value : TCOM_Generic_Value ) : longint ;

var S : string ;

begin
    S := Edit( string( Nam ), 2 or 32 ) ; { Uppercase, no spaces }
    if( Index_Of( PChar( S ) ) <> -1 ) then
    begin
        Add := -1 ;
        exit ;
    end ;
    Add := Names.Count ;
    Names.Add( S ) ;
    Values.Add( Value ) ;
    if( Value <> nil ) then
    begin
        Value.Attach ;
    end ;
end ;


procedure TStandard_COM_Dynamic_Properties.Clear ;

var Loop : integer ;
    Temp : TGeneric_Value ;

begin
    for Loop := 0 to Count - 1 do
    begin
        Temp := TGeneric_Value( Values[ Loop ] ) ;
        if( Temp <> nil ) then
        begin
            Temp.Detach ;
        end ;
    end ;
    Names.Clear ;
    Values.Clear ;
end ;


function TStandard_COM_Dynamic_Properties.Count : longint ;

begin
    Count := Names.Count ;
end ;


function TStandard_COM_Dynamic_Properties.Index_Of( Nam : PChar ) : longint ;

begin
    Index_Of := Names.IndexOf( Edit( string( Nam ), 2 or 32 ) ) ;
end ;


function TStandard_COM_Dynamic_Properties.Get_Name( Index : longint ) : PChar ;

begin
    if( ( Index < 0 ) or ( Index >= Count ) ) then
    begin
        Get_Name := '' ;
        exit ;
    end ;
    Get_Name := PChar( Names[ Index ] ) ;
end ;


function TStandard_COM_Dynamic_Properties.Get_Value( Index : longint ) : TCOM_Generic_Value ;

begin
    if( ( Index < 0 ) or ( Index >= Count ) ) then
    begin
        Get_Value := nil ;
        exit ;
    end ;
    Get_Value := TCOM_Generic_Value( Values[ Index ] ) ;
end ;


procedure TStandard_COM_Dynamic_Properties.Set_Value( Index : longint ;
    Value : TCOM_Generic_Value ) ;

begin
    if( ( Index < 0 ) or ( Index >= Count ) ) then
    begin
        exit ;
    end ;
    if( Value <> nil ) then
    begin
        Value.Attach ;
    end ;
    if( Values[ Index ] <> nil ) then
    begin
        TCOM_Generic_Value( Values[ Index ] ).Detach ;
    end ;
    Values[ Index ] := Value ;
end ;
{$ENDIF}



end.
