{$N+}
{$E+}
{
       Program Name    : DATES
       Package Name    : LB
       Purpose         : Various date and time functions
       Institution     : Conroy & Conroy Company
       Date written    : 24-Nov-82
       Written by      : Alan Conroy
       Version         : 1.1D

       Copyright (C) 1982 by Alan Conroy.  Released to the public domain.

       TO THE GLORY OF GOD THROUGH JESUS CHRIST

       *******************************************************
       *                                                     *
       *       M O D I F I C A T I O N   H I S T O R Y       *
       *                                                     *
       *       DATE    BY      REASON                        *
       *                                                     *
       *  6-Nov-2002   EAC     Added constants and time      *
       *                       functions.                    *
       * 15-Nov-2002   EAC     Fixed error in                *
       *                       Sirius_Timestamp.             *
       * 16-Nov-2002   EAC     Fix BP compilation error.     *
       * 20-Dec-2002   EAC     Fix bug in Format_Time and add*
       *                       Format_Date.                  *
       *  2-Jul-2003   EAC     Enhanced Format_Date.         *
       * 10-Jul-2004   EAC     Added Sirius date/time format *
       *                       functions.                    *
       *                                                     *
       *******************************************************

       *******************************************************
       *                                                     *
       *       P R O G R A M   P U R P O S E                 *
       *                                                     *
       *******************************************************

         These functions are used to manipulate dates and times.
}

unit Dates ;

interface

{$I EDEFINES.INC}

uses TypeDefs ; { int64 }

function Convert_Time_To_Seconds( S : string ) : longint ;
function Date : string ; { Current date in VMS style }
function Day( const Da : string ) : string ;
{ This function will return the name of the day for any given date.  NOTE: The
  date passed must be in a valid RSTS/E format. }

function Dayval( const Da : String ) : Integer ;
function Encode_Sirius_Timestamp( Y, Mo, D, H, M, S : word ; NS : longint ) : int64 ;
function Format_Date( Format : string ;
    Year, Month, Day : integer ) : string ;
function Format_Sirius_Date( const Format : string ; Value : int64 ) : string ;
function Format_Sirius_Time( const Format : string ; Value : int64 ) : string ;
function Format_Time( Format : string ;
    Days, Hours, Minutes, Seconds, Hundreths : integer ) : string ;
function Julian( M, D, Y : integer ) : integer ;
function Leap_Year( Year : longint ) : integer ;
procedure Parse_Sirius_Timestamp( TS : int64 ; var Y, Mo, D, H, M, S : word ;
    var NS : longint ) ;
procedure Parse_Time( T : longint ;
    var Days, Hours, Minutes, Seconds, Hundreths : integer ) ;
function Sirius_Timestamp : int64 ;
function Time : string ; { Current time in VMS style }
function Time_Seconds : longint ; { Returns time since Midnight in 100th seconds }
function Time_To_String( const Format : string ; T : longint ) : string ;
function XTime : string ; { Extended version of TIME }

{$IFDEF Test}
procedure Test_Unit ;
{$ENDIF}

const Month_January = 1 ;
      Month_February = 2 ;
      Month_March = 3 ;
      Month_April = 4 ;
      Month_May = 5 ;
      Month_June = 6 ;
      Month_July = 7 ;
      Month_August = 8 ;
      Month_September = 9 ;
      Month_October = 10 ;
      Month_November = 11 ;
      Month_December = 12 ;

      Day_Sunday = 1 ;
      Day_Monday = 2 ;
      Day_Tuesday = 3 ;
      Day_Wednesday = 4 ;
      Day_Thursday = 5 ;
      Day_Friday = 6 ;
      Day_Saturday = 7 ;

implementation

uses O_S ; { OS }

const Months : array[ Month_January..Month_December ] of string[ 3 ] =
      ( 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec' ) ;
const Months_Full : array[ Month_January..Month_December ] of string[ 9 ] =
      ( 'January', 'February', 'March', 'April', 'May', 'June', 'July',
        'August', 'September', 'October', 'November', 'December' ) ;

function Convert_Time_To_Seconds( S : string ) : longint ;

var Dummy : integer ;
    Multiplier : longint ;
    Res : longint ;
    Temp : string ;
    This_Time : longint ;

begin
    Res := 0 ;
    Multiplier := 1 ;
    Dummy := length( S ) ;
    while( length( S ) > 0 ) do
    begin
        while( ( Dummy > 0 ) and ( S[ Dummy ] <> ':' ) ) do
        begin
            dec( Dummy ) ;
        end ;
        Temp := copy( S, Dummy + 1, length( S ) ) ;
        S := copy( S, 1, Dummy - 1 ) ;
        val( Temp, This_Time, Dummy ) ;
        case Multiplier of
            1 : Res := Res + This_Time ;
            2 : Res := Res + This_Time * 60 ; { Minutes }
            3 : Res := Res + This_Time * 24 * 60 ; { Hours }
        end ;
        inc( Multiplier ) ;
    end ;
    Convert_Time_To_Seconds := Res ;
end ;


function Date : string ;

var Day_String, Year_String : string[ 4 ] ;
    Year, Month, Day : Word ;

begin
    OS^.Get_Date( Year, Month, Day ) ;
    Str( Year, Year_String ) ;
    Str( Day, Day_String ) ; { Convert day and year to strings }
    if( Length( Day_String ) = 1 ) then
    begin
        Day_String := '0' + Day_String ;
    end ;
    Date := Day_String + '-' + Months[ Month ] + '-' + Year_String ;
end ;


function Day( const Da : string ) : string ;

type Day_Array = array[ Day_Sunday..Day_Saturday ] of string[ 9 ] ;

const Day_Name : Day_Array = ( 'Sunday', 'Monday', 'Tuesday', 'Wednesday',
                               'Thursday', 'Friday', 'Saturday' ) ;

begin
    Day := Day_Name[ Dayval( Da ) ] ;
end ;


function Leap_Year( Year : longint ) : integer ;

begin
    Leap_Year := 0 ;
    if( ( Year and 3 ) = 0 ) then { evenly divisible by 4 }
    begin
        Leap_Year := 1 ;
        if( ( Year mod 100 ) = 0 ) then { ...but not by 100 }
        begin
            Leap_Year := 0 ;
            if( ( Year mod 400 ) = 0 ) then { ... except every 400 }
            begin
                Leap_Year := 1 ;
                if( ( Year mod 3200 ) = 0 ) then { ...but not by 3,200 }
                begin
                    Leap_Year := 0 ;
                    if( ( Year mod 80000 ) = 0 ) then { ... except every 80,000 }
                    begin
                        Leap_Year := 1 ;
                    end ;
                end ;
            end ;
        end ;
    end ;
end ;


type Month_Array = array[ 1..12 ] of Byte ;

const Month : Month_Array = ( 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ) ;

function Encode_Sirius_Timestamp( Y, Mo, D, H, M, S : word ; NS : longint ) : int64 ;

var Days, Res : int64 ;
    Loop : integer ;

begin
    if( Y = 0 ) then
    begin
        Days := D ;
    end else
    begin
        Days := Y * 365 + ( ( Y - 1 ) div 4 ) - ( ( Y - 1 ) div 100 ) +
            ( ( Y - 1 ) div 400 ) - ( ( Y - 1 ) div 3200 ) - 1 + D ;
            { - 1 to not count today }
    end ;
    if( ( Mo > Month_February ) or ( ( Mo = Month_February ) and ( D = 29 ) ) ) then
    begin
        Days := Days + Leap_Year( Y ) ;
        { Add 1 day if this is a leap year, and after February 29 }
    end ;
    for Loop := 1 to Mo - 1 do
    begin
        Days := Days + Month[ Loop ] ;
    end ;
    Res := ( ( Days * 24 * 60 * 60 ) + ( H * 60 * 60 ) + ( M * 60 ) + S ) ;
    { Each day = 24 hours of 60 minutes of 60 seconds }
    Encode_Sirius_Timestamp := Res * 10000000 + NS ;
    { Convert from seconds to 100 ns and add the 100 ns }
end ;


function Format_Date( Format : string ;
    Year, Month, Day : integer ) : string ;

var BC : boolean ;
    Dummy : integer ;
    _Result, Temp : string ;

begin
    { Setup... }
    if( length( Format ) = 0 ) then { Use default format }
    begin
        Format := 'DD-MMM-YYYY' ;
    end ;

    if( pos( 'C', Format ) > 0 ) then
    begin
        BC := ( Year < 0 ) ;
        if( BC ) then
        begin
            Year := -Year ;
        end ;
    end else
    begin
        BC := False ;
    end ;

    { Normalize values }
    while( Month > 12 ) do
    begin
        Month := Month - 12 ;
        Year := Year + 1 ;
    end ;

    { Parse format... }
    _Result := '' ;
    while( length( Format ) > 0 ) do
    begin
	if( pos( Format[ 1 ], 'DMYC' ) = 0 ) then
        begin
            _Result := _Result + Format[ 1 ] ;
            Format := copy( Format, 2, length( Format ) ) ;
        end else
        begin
            { A formatting specification... }
            Dummy := 2 ;
            while(
                   ( Dummy <= length( Format ) )
                   and
                   ( Format[ Dummy ] = Format[ 1 ] )
                 ) do
            begin
                inc( Dummy ) ; { Find end of this run }
            end ;
            Temp := copy( Format, 1, Dummy - 1 ) ;
            Format := copy( Format, Dummy, length( Format ) ) ;
            if( Temp = 'C' ) then
            begin
                if( BC ) then
                begin
                    _Result := _Result + 'BC' ;
                end else
                begin
                    _Result := _Result + 'AD' ;
                end ;
            end ;
            if( Temp = 'CC' ) then
            begin
                _Result := _Result + 'C' ;
            end ;
            if( Temp = 'D' ) then
            begin
                str( Day, Temp ) ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'DD' ) then
            begin
                str( Day, Temp ) ;
                if( length( Temp ) < 2 ) then
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'DDD' ) then
            begin
                _Result := _Result + 'D' ;
            end ;
            if( Temp = 'M' ) then
            begin
                str( Month, Temp ) ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'MM' ) then
            begin
                str( Month, Temp ) ;
                if( length( Temp ) < 2 ) then
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'MMM' ) then
            begin
                _Result := _Result + Months[ Month ] ;
            end ;
            if( Temp = 'MMMM' ) then
            begin
                _Result := _Result + Months_Full[ Month ] ;
            end ;
            if( Temp = 'MMMMM' ) then
            begin
                _Result := 'M' ;
            end ;
            if( Temp = 'Y' ) then
            begin
                str( Year, Temp ) ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'YY' ) then
            begin
                str( Year, Temp ) ;
                if( length( Temp ) < 2 ) then
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + copy( Temp, length( Temp ) - 1, 2 ) ;
            end ;
            if( Temp = 'YYY' ) then
            begin
                _Result := _Result + 'Y' ;
            end ;
            if( Temp = 'YYYY' ) then
            begin
                str( Year, Temp ) ;
                while( length( Temp ) < 4 ) do
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + Temp ;
            end ;
        end ;
    end ; { while( length( Format ) > 0 ) do }
    Format_Date := _Result ;
end ; { Format_Date }


function Format_Sirius_Date( const Format : string ; Value : int64 ) : string ;

var Year, Month, Day, H, M, S : word ;
    NS : longint ;

begin
    Parse_Sirius_Timestamp( Value, Year, Month, Day, H, M, S, NS ) ;
    Format_Sirius_Date := Format_Date( Format, Year, Month, Day ) ;
end ;


function Format_Sirius_Time( const Format : string ; Value : int64 ) : string ;

var Days, Hours, Minutes, Seconds, Y, Mo : word ;
    NS : longint ;

begin
    Parse_Sirius_Timestamp( Value, Y, Mo, Days, Hours, Minutes, Seconds, NS ) ;
    Format_Sirius_Time :=
        Format_Time( Format, Days, Hours, Minutes, Seconds, NS div 100000 ) ;
end ;


function Format_Time( Format : string ;
    Days, Hours, Minutes, Seconds, Hundreths : integer ) : string ;

var Dummy : integer ;
    _Result, Temp : string ;
    PM : boolean ;

begin
    { Setup... }
    if( length( Format ) = 0 ) then { Use default format }
    begin
        Format := 'DDDHHHMM:SS.1' ;
    end ;

    { Normalize passed values... }
    Seconds := Seconds + Hundreths div 100 ;
    Hundreths := Hundreths mod 100 ;

    Minutes := Minutes + Seconds div 60 ;
    Seconds := Seconds mod 60 ;

    Hours := Hours + Minutes div 60 ;
    Minutes := Minutes mod 60 ;

    Days := Days + Hours div 24 ;
    Hours := Hours mod 24 ;

    if( pos( 'A', Format ) > 0 ) then
    begin
        PM := ( Hours > 11 ) ;
        if( PM ) then
        begin
            Hours := Hours - 12 ;
            if( Hours = 0 ) then
            begin
                Hours := 12 ;
            end ;
        end ;
    end else
    begin
        PM := False ;
    end ;

    { Parse format... }
    _Result := '' ;
    while( length( Format ) > 0 ) do
    begin
	if( pos( Format[ 1 ], 'DHMS1A' ) = 0 ) then
        begin
            _Result := _Result + Format[ 1 ] ;
            Format := copy( Format, 2, length( Format ) ) ;
        end else
        begin
            { A formatting specification... }
            Dummy := 2 ;
            while(
                   ( Dummy <= length( Format ) )
                   and
                   ( Format[ Dummy ] = Format[ 1 ] )
                 ) do
            begin
                inc( Dummy ) ; { Find end of this run }
            end ;
            Temp := copy( Format, 1, Dummy - 1 ) ;
            Format := copy( Format, Dummy, length( Format ) ) ;
            if( Temp = 'A' ) then
            begin
                if( PM ) then
                begin
                    _Result := _Result + 'p' ;
                end else
                begin
                    _Result := _Result + 'a' ;
                end ;
            end ;
            if( Temp = 'AA' ) then
            begin
                if( PM ) then
                begin
                    _Result := _Result + 'pm' ;
                end else
                begin
                    _Result := _Result + 'am' ;
                end ;
            end ;
            if( Temp = 'AAA' ) then
            begin
                _Result := _Result + 'A' ;
            end ;
            if( Temp = '1' ) then
            begin
                str( Hundreths, Temp ) ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = '11' ) then
            begin
                str( Hundreths, Temp ) ;
                if( Hundreths > 9 ) then
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = '111' ) then
            begin
                if( Hundreths > 0 ) then
                begin
                    str( Hundreths, Temp ) ;
                    _Result := _Result + '.' + Temp ;
                end ;
            end ;
            if( Temp = '1111' ) then
            begin
                if( Hundreths > 0 ) then
                begin
                    str( Hundreths, Temp ) ;
                    if( Hundreths > 9 ) then
                    begin
                        Temp := '0' + Temp ;
                    end ;
                    _Result := _Result + '.' + Temp ;
                end ;
            end ;
            if( Temp = '11111' ) then
            begin
                _Result := _Result + '1' ;
            end ;
            if( Temp = 'S' ) then
            begin
                str( Seconds, Temp ) ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'SS' ) then
            begin
                str( Seconds, Temp ) ;
                if( Seconds <= 9 ) then
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'SSS' ) then
            begin
                if( Seconds > 0 ) then
                begin
                    str( Seconds, Temp ) ;
                    _Result := _Result + ':' + Temp ;
                end ;
            end ;
            if( Temp = 'SSSS' ) then
            begin
                if( Seconds > 0 ) then
                begin
                    str( Seconds, Temp ) ;
                    if( Seconds > 9 ) then
                    begin
                        Temp := '0' + Temp ;
                    end ;
                    _Result := _Result + ':' + Temp ;
                end ;
            end ;
            if( Temp = 'SSSSS' ) then
            begin
                _Result := _Result + 'S' ;
            end ;
            if( Temp = 'M' ) then
            begin
                str( Minutes, Temp ) ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'MM' ) then
            begin
                str( Minutes, Temp ) ;
                if( Minutes <= 9 ) then
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'MMM' ) then
            begin
                if( Minutes > 0 ) then
                begin
                    str( Minutes, Temp ) ;
                    _Result := _Result + ':' + Temp ;
                end ;
            end ;
            if( Temp = 'MMMM' ) then
            begin
                if( Minutes > 0 ) then
                begin
                    str( Minutes, Temp ) ;
                    if( Minutes > 9 ) then
                    begin
                        Temp := '0' + Temp ;
                    end ;
                    _Result := _Result + ':' + Temp ;
                end ;
            end ;
            if( Temp = 'MMMMM' ) then
            begin
                _Result := _Result + 'M' ;
            end ;
            if( Temp = 'H' ) then
            begin
                str( Hours, Temp ) ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'HH' ) then
            begin
                str( Hours, Temp ) ;
                if( Hours <= 9 ) then
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'HHH' ) then
            begin
                if( Hours > 0 ) then
                begin
                    str( Hours, Temp ) ;
                    _Result := _Result + ':' + Temp ;
                end ;
            end ;
            if( Temp = 'HHHH' ) then
            begin
                if( Hours > 0 ) then
                begin
                    str( Hours, Temp ) ;
                    if( Hours > 9 ) then
                    begin
                        Temp := '0' + Temp ;
                    end ;
                    _Result := _Result + ':' + Temp ;
                end ;
            end ;
            if( Temp = 'HHHHH' ) then
            begin
                _Result := _Result + 'H' ;
            end ;
            if( Temp = 'D' ) then
            begin
                str( Days, Temp ) ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'DD' ) then
            begin
                str( Days, Temp ) ;
                if( Hours > 9 ) then
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'DDD' ) then
            begin
                str( Days, Temp ) ;
                while( length( Temp ) < 3 ) do
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'DDDD' ) then
            begin
                str( Days, Temp ) ;
                while( length( Temp ) < 4 ) do
                begin
                    Temp := '0' + Temp ;
                end ;
                _Result := _Result + Temp ;
            end ;
            if( Temp = 'DDDDD' ) then
            begin
                _Result := _Result + 'D' ;
            end ;
        end ;
    end ; { while( length( Format ) > 0 ) do }
    Format_Time := _Result ;
end ; { Format_Time }


function Julian( M, D, Y : integer ) : integer ;

var C, _Result : integer ;

begin
    _Result := 0 ;
    for C := 1 to M - 1 do
    begin
	_Result := _Result + Month[ C ] ; { Add up days for each month up to M }
    end ;
    _Result := _Result + D ;
    if( M > 2 ) then { After February }
    begin
	_Result := _Result + Leap_Year( Y ) ;
    end ;
    Julian := _Result ;
end ;


function Dayval( const Da : string ) : Integer ;
{ NOTE: The date passed must be in a valid RSTS/E or VAX format:
  d-mmm-yy, d-mmm-yyyy, dd-mmm-yy, or dd-mmm-yyyy, or yy.mm.dd }

type Month_Array = array[ 1..13 ] of string[ 3 ] ;

const Month_Name : Month_Array = ( 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
				   'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec',
				   '   ' ) ;

var Days : Real ;
    Day, Month, Year : longint ;
    Dummy : integer ;
    C, Y : longint ;
    Dash : integer ;

begin
    Dash := pos( '-', Da ) ;
    if( Dash > 0 ) then
    begin
	Val( Copy( Da, Dash + 5, 4 ), Year, Dummy )
    end else
    begin
        Val( Copy( Da, 1, 2 ), Year, Dummy ) ;
    end ;
    if( Year < 200 ) then
    begin
	Year := Year + 1900 ;
    end ;

    { Get month }
    if( Dash > 0 ) then
    begin
        Month := 1 ;
        While(
               ( Month < 13 )
               and
               ( Month_Name[ Month ] <> Copy( Da, Dash + 1, 3 ) )
             ) do
	begin
	    Month := Month + 1 ;
	end ;
    end else
    begin
	Val( Copy( Da, 4, 2 ), Month, Dummy ) ;
    end ;

    { Get day of month }
    if( Dash > 0 ) then
    begin
        Val( Copy( Da, 1, Dash - 1 ), Day, Dummy ) ;
    end else
    begin
        Val( Copy( Da, 7, 2 ), Day, Dummy ) ;
    end ;

    { Determine day with Key Value method... }
    Y := ( ( Year mod 100 ) div 4 ) + Day ;
    case Month of
        1, 10 : Y := Y + 1 ;
        2, 3, 11 : Y := Y + 4 ;
        5 : Y := Y + 2 ;
        6 : Y := Y + 5 ;
        8 : Y := Y + 3 ;
        9, 12 : Y := Y + 6 ;
    end ;
    if( Month < 3 ) then
    begin
        Y := Y - Leap_Year( Year ) ;
    end ;
    C := Year ;
    while( C < 1700 ) do
    begin
        C := C + 400 ;
    end ;
    while( C > 2099 ) do
    begin
        C := C - 400 ;
    end ;
    case C div 100 of
        17 : Y := Y + 4 ;
        18 : Y := Y + 2 ;
        20 : Y := Y + 6 ;
    end ;
    Y := Y + ( Year mod 100 ) ;
    Days := Y mod 7 ;

    if( Days = 0 ) then
    begin
	Days := 7 ;
    end ;
    Dayval := Trunc( Days )
end ;


procedure Parse_Sirius_Timestamp( TS : int64 ; var Y, Mo, D, H, M, S : word ;
    var NS : longint ) ;

var BC : boolean ;
    Work : int64 ;

begin
    { Handle BC... }
    BC := ( TS < 0 ) ;
    if( BC ) then
    begin
        TS := TS - 1 ;
    end ;

    { Figure out time of day stuff... }
{$IFDEF DELPHI}
    Work := TS div 10000000 ;
    NS := TS - ( Work * 10000000 ) ;
    TS := Work ;
    Work := TS div 60 ;
    S := TS - ( Work * 60 ) ;
    TS := Work ;
    Work := TS div 60 ;
    M := TS - ( Work * 60 ) ;
    TS := Work ;
    Work := TS div 24 ;
    H := TS - ( Work * 24 ) ;

    { Now convert from days to Y, M, D... }
    TS := Work ;
    Y := TS div 365 ;
{$ELSE}
    Work := TS / 10000000. ;
    NS := trunc( TS - ( Work * 10000000 ) ) ;
    TS := Work ;
    Work := TS / 60 ;
    S := trunc( TS - ( Work * 60 ) ) ;
    TS := Work ;
    Work := TS / 60 ;
    M := trunc( TS - ( Work * 60 ) ) ;
    TS := Work ;
    Work := TS / 24 ;
    H := trunc( TS - ( Work * 24 ) ) ;

    { Now convert from days to Y, M, D... }
    TS := Work ;
    Y := trunc( TS / 365 ) ;
{$ENDIF}
    TS := TS - ( Y * 365 ) + 1 ; { Days into year Y +1 because <1 day into year = day 1 (Jan 1)}
    if( Y <> 0 ) then
    begin
         TS := TS - ( ( Y - 1 ) div 4 ) + ( ( Y - 1 ) div 100 ) - ( ( Y - 1 ) div 400 ) + ( ( Y - 1 ) div 3200 ) ;
    end ;
    while( TS < 0 ) do { Leap year adjustments took us back into previous year }
    begin
        Y := Y - 1 ;
        TS := TS + 365 + Leap_Year( Y ) ;
    end ;
    Mo := 1 ;
    while( TS > Month[ Mo ] ) do
    begin
        TS := TS - Month[ Mo ] ;
        inc( Mo ) ;
    end ;
    if( ( Mo > 2 ) and ( Leap_Year( Y ) = 1 ) ) then
    begin
        TS := TS - 1 ;
        if( TS < 1 ) then
        begin
            dec( Mo ) ;
            if( Mo = 2 ) then
            begin
                TS := 29 ;
            end else
            begin
                TS := Month[ Mo ] ;
            end ;
        end ;
    end ;
{$IFDEF DELPHI}
    D := TS ;
{$ELSE}
    D := trunc( TS ) ;
{$ENDIF}
    if( BC ) then
    begin
        Y := -Y ;
    end ;
end ; { Parse_Sirius_Timestamp }


procedure Parse_Time( T : longint ; var Days, Hours, Minutes, Seconds, Hundreths : integer ) ;

begin
    Hundreths := T mod 100 ;
    T := T div 100 ;
    Seconds := T mod 60 ;
    T := T div 60 ;
    Minutes := T mod 60 ;
    T := T div 60 ;
    Hours := T mod 24 ;
    T := T div 24 ;
    Days := T ;
end ;


{$IFDEF DELPHI}
var Last_Sirius_Timestamp : int64 = 0 ;
{$ELSE}
var Last_Sirius_Timestamp : int64 ;
{$ENDIF}
{$IFDEF TEST}
var Last_Y, Last_M, Last_D : word ;
    Last_H, Last_Min, Last_S, Last_X : word ;
{$ENDIF}

function Sirius_Timestamp : int64 ;

var Y, M, D : word ;
    H, Min, S, X : word ;
    Res : int64 ;

begin
    OS^.Get_Date( Y, M, D ) ;
    OS^.Get_Time( H, Min, S, X ) ;
    Res := Encode_Sirius_Timestamp( Y, M, D, H, Min, S, X * 10000 ) ; { X is in ms, so convert to 100 ns }
    if( Res < Last_Sirius_Timestamp ) then { Due to a bug in either our code or Windows }
    begin
        Res := Last_Sirius_Timestamp + 1 ;
    end ;
{$IFDEF Test}
    Last_Y := Y ;
    Last_M := M ;
    Last_D := D ;
    Last_H := H ;
    Last_Min := Min ;
    Last_S := S ;
    Last_X := X ;
{$ENDIF}
    Last_Sirius_Timestamp := Res ;
    Sirius_Timestamp := Res ;
end ;


function Time ;

var Am_Pm : string[ 2 ] ;
    Hour, Minute, Second, Hundreths : Word ;
    Hour_String, Minute_String : string[ 2 ] ;

begin
    OS^.Get_Time( Hour, Minute, Second, Hundreths ) ;
    Am_Pm := 'AM' ;
    if( Hour > 11 ) then
    begin
        Am_Pm := 'PM' ;
        if( Hour > 12 ) then
        begin
            Hour := Hour - 12 ;
        end ;
    end ;
    if( Hour = 0 ) then
    begin
        Hour := 12 ;
    end ;
    Str( Hour, Hour_String ) ;
    Str( Minute, Minute_String ) ; { Convert hour and minute to strings }
    if( Length( Minute_String ) = 1 ) then
    begin
        Minute_String := '0' + Minute_String ;
    end ;
    Time := Hour_String + ':' + Minute_String + ' ' + Am_Pm ; { Now return value }
end ;


function Time_Seconds : longint ; { Returns time since Midnight in 100th seconds }

var Hour, Minute, Second, Hundreths : Word ;
    Res : longint ;

begin
    OS^.Get_Time( Hour, Minute, Second, Hundreths ) ;
    Res := Hour ;
    Res := Res * 60 + Minute ;
    Res := Res * 60 + Second ;
    Res := Res * 100 + Hundreths ;
    Time_Seconds := Res ;
end ;


function Time_To_String( const Format : string ; T : longint ) : string ;

var Days, Hours, Minutes, Seconds, Hundreths : integer ;

begin
    Parse_Time( T, Days, Hours, Minutes, Seconds, Hundreths ) ;
    Time_To_String := Format_Time( Format, Days, Hours, Minutes, Seconds, Hundreths ) ;
end ;


function XTime : string ;

var Am_Pm : string[ 2 ] ;
    Hour, Minute, Second, Hundreths : Word ;
    Hour_String, Minute_String, Second_String, Hundreths_String : string[ 2 ] ;

begin
    OS^.Get_Time( Hour, Minute, Second, Hundreths ) ;
    Am_Pm := 'AM' ;
    if( Hour > 11 ) then
    begin
        Am_Pm := 'PM' ;
        if( Hour > 12 ) then
        begin
            Hour := Hour - 12 ;
        end ;
    end ;
    if( Hour = 0 ) then
    begin
        Hour := 12 ;
    end ;
    Str( Hour, Hour_String ) ;
    Str( Minute, Minute_String ) ; { Convert hour and minute to strings }
    Str( Second, Second_String ) ; { Convert seconds to string }
    Str( Hundreths, Hundreths_String ) ; { Convert hundreths to string }
    if( Length( Minute_String ) = 1 ) then
    begin
        Minute_String := '0' + Minute_String ;
    end ;
    if( Length( Second_String ) = 1 ) then
    begin
        Second_String := '0' + Second_String ;
    end ;
    if( Length( Hundreths_String ) = 1 ) then
    begin
        Hundreths_String := '0' + Hundreths_String ;
    end ;
    Xtime := Hour_String + ':' + Minute_String + ':' + Second_String +
        '.' + Hundreths_String + ' ' + Am_Pm ; { Now return value }
end ;


{$IFDEF Test}
procedure Test_Sirius_Timestamp ;

var Loop : longint ;

begin
    for Loop := 0 to 1000000 do
    begin
        Sirius_Timestamp ;
    end ;
end ;


procedure Test_Encode_Sirius_Timestamp ;

var TS : int64 ;
    Y, Mo, D, H, M, S : word ;
    NS : longint ;

begin
    if( Encode_Sirius_Timestamp( 2006, 12, 6, 13, 34, 37, 9840000 ) >
        Encode_Sirius_Timestamp( 2006, 12, 6, 13, 34, 38, 0 ) ) then
    begin
        writeln( 'Encode_Sirius_Timestamp error' ) ;
    end ;
    Parse_Sirius_Timestamp( Encode_Sirius_Timestamp( 2006, 12, 6, 13, 34, 37, 9840000 ),
        Y, Mo, D, H, M, S, NS ) ;
    if( ( Y <> 2006 ) or ( Mo <> 12 ) or ( D <> 6 ) or ( H <> 13 ) or ( M <> 34 ) or ( S <> 37 ) or ( NS <> 9840000 ) ) then
    begin
        writeln( 'Encode/decode error for Sirius timestamp' ) ;
    end ;
end ;


procedure Test_Unit ;

begin
    Test_Sirius_Timestamp ;
    Test_Encode_Sirius_Timestamp ;

    { Julian tests... }
    if( Julian( 1, 1, 2000 ) <> 1 ) then
    begin
        writeln( 'Julian failed' ) ;
    end ;
    if( Julian( 1, 31, 2000 ) <> 31 ) then
    begin
        writeln( 'Julian failed' ) ;
    end ;
    if( Julian( 3, 1, 2000 ) <> 31 + 29 + 1 ) then
    begin
        writeln( 'Julian failed' ) ;
    end ;
    if( Julian( 3, 1, 1996 ) <> 31 + 29 + 1 ) then
    begin
        writeln( 'Julian failed' ) ;
    end ;
    if( Julian( 1, 1, 1900 ) <> 1 ) then
    begin
        writeln( 'Julian failed' ) ;
    end ;
    if( Julian( 1, 31, 1900 ) <> 31 ) then
    begin
        writeln( 'Julian failed' ) ;
    end ;
    if( Julian( 3, 1, 1900 ) <> 31 + 28 + 1 ) then
    begin
        writeln( 'Julian failed' ) ;
    end ;

    { Dayval tests... }
    if( Dayval( '1-Jan-1996' ) <> 2 ) then
    begin
        writeln( 'Dayval failed' ) ;
    end ;
    if( Dayval( '11-Feb-1996' ) <> 1 ) then
    begin
        writeln( 'Dayval failed' ) ;
    end ;
    if( Dayval( '1-Mar-1996' ) <> 6 ) then
    begin
        writeln( 'Dayval failed' ) ;
    end ;
    if( Dayval( '1-Jan-2002' ) <> 3 ) then
    begin
        writeln( 'Dayval failed' ) ;
    end ;
    if( Dayval( '11-Feb-2002' ) <> 2 ) then
    begin
        writeln( 'Dayval failed' ) ;
    end ;
    if( Dayval( '1-Mar-2002' ) <> 6 ) then
    begin
        writeln( 'Dayval failed' ) ;
    end ;
    if( Dayval( '1-Jan-2001' ) <> 2 ) then
    begin
        writeln( 'Dayval failed' ) ;
    end ;
    if( Dayval( '1-Dec-2001' ) <> 7 ) then
    begin
        writeln( 'Dayval failed' ) ;
    end ;
end ;
{$ENDIF}

begin
    Last_Sirius_Timestamp := 0 ;
end.
