{
        Program Name : Decorations
        Package Name : LB
        Purpose      : VCL decoration objects
        Institution  : Conroy & Conroy Co.
        Date Written : 10-July-2004
        Written By   : Alan Conroy
        Version      : 1.0

        Copyright (C) 2004 by Alan Conroy.  All rights reserved.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          Definition of non-window-handle objects that can draw themselves on
        other controls.
}

unit Decorations ;

interface

uses // Borland...
     Windows, // TRect

     Classes, // TNotifyEvent
     Controls, // TControl
     Graphics, // TColor

     // Other...
     Common ; // TBase_Delphi_Object

type TDecoration = class( TBase_Delphi_Object )
                       protected // Instance data...
                           _Rect : TRect ; // Location where last painted

                       public // API...
                           Align : TAlign ;
                           Color : TColor ;
                           Disabled_Help_Context : integer ;
                           Enabled : boolean ;
                           Enabled_Help_Context : integer ;
                           _Enabled_Hint : string ;
                           Flat : boolean ;
                           Height : integer ;
                           OnClick : TNotifyEvent ;
                           Parent : TControl ;
                           Transparent : boolean ;
                           Visible : boolean ;

                           { If < 0 then use height to determine width.  In such
                             case, the width is Height / ((-Width) / 100 ).  For
                             instance, if Width is -66, then the width is 66% of
                             the height.  If Width is -200 then width is 200% of
                             height. }
                           Width : integer ;

                       protected // Property handlers...
                           function Get_Enabled_Hint : string ; virtual ;
                           procedure Set_Enabled_Hint( Value : string ) ;
                               virtual ;

                       public // API...
                           function ClientRect : TRect ;
                           function ClientHeight( W, H : integer ) : integer ;
                               virtual ;
                           function ClientWidth( W, H : integer ) : integer ;
                               virtual ;
                           property Enabled_Hint : string
                               read Get_Enabled_Hint
                               write Set_Enabled_Hint ;
                           procedure Invalidate ;
                           procedure Paint( Canvas : TCanvas ;
                               X, Y, W, H : integer ) ; virtual ;
                           property Rect : TRect
                               read _Rect ;
                   end ; // TDecoration

type TFrame_Decoration = class( TDecoration )
                             public // API...
                                 Decoration_Category : integer ;
                                 Decoration_Type : integer ;

                             public // Constructors and destructors...
                                 constructor Create ;

                             public // API...
                                 function ClientRect( W, H : integer ) : TRect ;
                                 function ClientHeight( W, H : integer ) : integer ;
                                     override ;
                                 function ClientWidth( W, H : integer ) : integer ;
                                     override ;
                                 procedure Paint( Canvas : TCanvas ;
                                     X, Y, W, H : integer ) ; override ;
                         end ;

implementation

uses // C&C...
     Types ; // Rect()

// TDecoration methods...

// Property handlers...

function TDecoration.Get_Enabled_Hint : string ;

begin
    Result := _Enabled_Hint ;
end ;


procedure TDecoration.Set_Enabled_Hint( Value : string ) ;

begin
    _Enabled_Hint := Value ;
end ;


// API...

function TDecoration.ClientRect : TRect ; //TODO:REFACTOR: Shouldn't this be virtual  - EAC, 1-Dec-2006

begin
    Result := Types.Rect( 0, 0, Width, Height ) ;
end ;


function TDecoration.ClientHeight( W, H : integer ) : integer ;

begin
    Result := Height ;
end ;


function TDecoration.ClientWidth( W, H : integer ) : integer ;

begin
    Result := Width ;
end ;


procedure TDecoration.Invalidate ;

begin
    if( Parent <> nil ) then
    begin
        Parent.Invalidate ;
    end ;
end ;


procedure TDecoration.Paint( Canvas : TCanvas ;
    X, Y, W, H : integer ) ;

begin
    // This routine intentionally left blank
end ;



type TLocal_Control = class( TControl )
			  public
                              property Color ;
                      end ;

// TFrame_Decoration methods...

// Constructors and destructors...

constructor TFrame_Decoration.Create ;

begin
    inherited Create ;

    Color := clBtnFace ;
    Enabled := True ;
    Visible := True ;
    Height := -1 ; // Height of parent
    Width := -100 ; // 100% of height
end ;


// API...

function TFrame_Decoration.ClientRect( W, H : integer ) : TRect ;

begin
    // Setup...
    if( Height > -1 ) then
    begin
        H := Height ;
    end ;
    if( Width > -1 ) then
    begin
        W := Width ;
    end ;
    if( Width < 0 ) then
    begin
        W := ( H * -Width ) div 100 ;
    end ;
    Result := Bounds( 0, 0, W, H ) ;
end ;


function TFrame_Decoration.ClientHeight( W, H : integer ) : integer ;

var Rect : TRect ;

begin
    Rect := ClientRect( W, H ) ;
    Result := Rect.Bottom - Rect.Top + 1 ;
end ;


function TFrame_Decoration.ClientWidth( W, H : integer ) : integer ;

var Rect : TRect ;

begin
    Rect := ClientRect( W, H ) ;
    Result := Rect.Right - Rect.Left + 1 ;
end ;


procedure TFrame_Decoration.Paint( Canvas : TCanvas ;
    X, Y, W, H : integer ) ;

var Bitmap : TBitmap ;
    Flags : integer ;
    ParentColor : TColor ;

begin
    // Setup...
    if( Height > -1 ) then
    begin
        H := Height ;
    end ;
    if( Width > -1 ) then
    begin
        W := Width ;
    end ;
    if( Width < 0 ) then
    begin
        W := ( H * -Width ) div 100 ;
    end ;

    if( Transparent and ( Parent <> nil ) ) then
    begin
        ParentColor := TLocal_Control( Parent ).Color ;
    end else
    begin
        ParentColor := Color ;
    end ;

    Flags := Decoration_Type ;
    if( not Enabled ) then
    begin
        Flags := Flags or DFCS_INACTIVE ;
    end ;
//    if( Flat ) then
    begin
        Flags := Flags or DFCS_FLAT ;
    end ;

    // Draw image to bitmap...
    Bitmap := TBitmap.Create ;
    Bitmap.Height := H ;
    Bitmap.Width := W ;
    Bitmap.Canvas.Brush.Color := ParentColor ;
    Bitmap.Canvas.FillRect( ClientRect( W, H ) ) ;
    DrawFrameControl( Bitmap.Canvas.Handle, ClientRect( W, H ), Decoration_Category, Flags or DFCS_TRANSPARENT ) ;

    // Get rid of border that DrawFrameControl adds...

    Bitmap.Canvas.Pen.Color := Bitmap.Canvas.Pixels[ 2, 2 ] ;
    Bitmap.Canvas.MoveTo( 0, 0 ) ;
    Bitmap.Canvas.LineTo( W - 1, 0 ) ;
    Bitmap.Canvas.LineTo( W - 1, H - 1 ) ;
    Bitmap.Canvas.LineTo( 0, H - 1 ) ;
    Bitmap.Canvas.LineTo( 0, 0 ) ;

    // Draw result on canvas...
    Bitmap.Transparent := True ;
    Bitmap.TransparentColor := Bitmap.Canvas.Pixels[ 2, 2 ] ;
    Bitmap.TransparentMode := tmFixed ;
    Canvas.Draw( X, Y, Bitmap ) ;
    _Rect.Top := Y ;
    _Rect.Left := X ;
    _Rect.Bottom := Y + H - 1 ;
    _Rect.Right := X + W - 1 ;

    // Clean up...
    Bitmap.Free ;
end ; // TMenu_Caption_Decoration.Paint



end.
