{
        Package Name : Editor_Panel
        Purpose      : Editor interface
        Institution  : Conroy & Conroy Co.
        Date Written : 10-Nov-1992
        Written By   : Alan Conroy
        Version      : 1.3

        Copyright (C) 1992 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TEditor_Panel is an editor interface.
}

unit Editor_Panel ;

interface

uses // Borland...
     Classes,   // TComponent
     ExtCtrls,  // TPanel
     Graphics,  // clWhite
     Messages,  // TMessage

     // C&C...
     Operatio,  // POperation_Logger
     UEditor ;  // PEditor

type TKeyEvent = procedure( Sender : Tobject ; var Key : word ; Shift : TShiftState ) of object ;

type TEditor_Panel = class( TPanel )
                         public // Public instance data...
                             Control_K : boolean ; // True if in Control-K mode
                             Control_O : boolean ; // True if in Control-O mode
                             Control_Q : boolean ; // True if in Control-Q mode
                             Editor : PEditor ;
                             Mouse_Selection : boolean ; // True if selection via mouse
                             Undo_List : POperation_Logger ;

                         public // Constructors and destructors...
                             constructor Create( AOwner : TComponent ) ;
                                 override ;
                             destructor Destroy ; override ;

                         private // Message handlers...
                             procedure GetDlgCode( var Message : tWMGetDlgCode ) ;
                                 message wm_GetDlgCode ;
                             procedure Kill_Focus( var Message : tWMKillFocus ) ;
                                 message wm_KillFocus ;
                             procedure Set_Focus( var Message : tWMSetFocus ) ;
                                 message wm_SetFocus ;
                             procedure wmLButtonDblclk( var Message : tWMLButtonDblclk ) ;
                                 message wm_LButtonDblclk ;
                             procedure wmLButtonDown( var Message : tWMLButtonDown ) ;
                                 message wm_LButtonDown ;
                             procedure wmLButtonUp( var Message : tWMLButtonUp ) ;
                                 message wm_LButtonUp ;
                             procedure wmMouseMove( var Message : tWMMouseMove ) ;
                                 message wm_MouseMove ;

                         private // Internal utility routines...
                             procedure Cancel_Selection ;
                             procedure Cursor_Save( New_Transaction : boolean ) ;
                             procedure Delete ;
                             procedure Delete_Current_Selection ;
                             procedure Delete_Line ;
                             procedure Delete_Word ;
                             function Get_Current_Word : string ;
                             procedure Handle_Horizontal ;
                             procedure Indent ;
                             procedure Insert( S : string ) ;
                             procedure Lowercase_Word ;
                             procedure Mouse_Move( var Message : tWMMouseMove ) ;
                             procedure On_KeyDown( Sender : TObject ;
                                 var Key : word ; Shift : TShiftState ) ;
                             procedure Outdent ;
                             procedure Select_Current_Word ;
                             procedure Select_Text( Start_Line : longint ;
                                 Start_Column : byte ; Finish_Line : longint ;
                                 Finish_Column : byte ) ;
                             procedure Start_Selection ;
                             procedure Synchronize_Caret_To_XY( X, Y : integer ) ;
                             procedure Uppercase_Word ;

                         public // Overrides...
                             procedure KeyDown( var Key : word ;
                                 Shift : TShiftState ) ; override ;
                             procedure KeyPress( var Key : char ) ;
                                 override ;
                             procedure Paint ; override ;
                             procedure SetBounds( ALeft, ATop, AWidth, AHeight : integer ) ;
                                 override ;

                         public // API...
                             _OnKeyDown : TKeyEvent ;
                             On_Focus : TNotifyEvent ;

                             procedure Bring_Into_View ;
                             procedure Cursor_Down ;
                             procedure Cursor_Left ;
                             procedure Cursor_Up ;
                             procedure Cursor_Right ;
                             function Open( S : string ) : boolean ;
                             procedure Delete_Text ;
                             procedure Insert_Buffer( Buffer : pLine_Buffer ) ;
                             procedure Position_Cursor( Line, Column : integer ) ;
                             function Search( X : string ;
                                 Case_Sensitive : boolean ) : boolean ;
                             procedure Undo ;
                     end ;

implementation

uses // Borland...
     Windows,   // GetFocus
     Controls,  // crIBeam
     Dialogs,   // ShowMessage
     SysUtils,  // strcopy

     // Sirius...
     ASCIIDef,
     EditPref, // Editor_Preferences
     EditWin, // TEditor_Window
     SMU,       // Create_Window_Wrapper
     Standard ; // ERT

var Mouse_Control : TEditor_Panel ;
    Timer : integer ; // Autoscroll timer ID


type PInsert_Record = ^TInsert_Record ;
     TInsert_Record = record
                          This : TEditor_Panel ;
                          Size : integer ;
                      end ;
     PDelete_Record = ^TDelete_Record ;
     TDelete_Record = record
                          This : TEditor_Panel ;
                          Size : integer ;
                          Data : PChar ;
                      end ;
     PMovement_Record = ^TMovement_Record ;
     TMovement_Record = record
                            This : TEditor_Panel ;
                            Line, Column : integer ;
                        end ;
     PSelection_Record = ^TSelection_Record ;
     TSelection_Record = record
                             This : TEditor_Panel ;
                             Start_Line, End_Line : longint ;
                             Start_Column, End_Column : integer ;
                         end ;
     PIndent_Record = ^TIndent_Record ;
     TIndent_Record = record
                          This : TEditor_Panel ;
                          Line : integer ;
                      end ;
     POutdent_Record = ^TOutdent_Record ;
     TOutdent_Record = record
                          This : TEditor_Panel ;
                          Line : integer ;
                      end ;
     PChange_Record = ^TChange_Record ;
     TChange_Record = record
                          This : TEditor_Panel ;
                          Size : integer ;
                          Data : PChar ;
                      end ;
     PInsert_File_Record = ^TInsert_File_Record ;
     TInsert_File_Record = record
                               This : TEditor_Panel ;
                               Line : integer ; // First line of insertion
                               First : integer ; // Number of old bytes in first line
                               Count : integer ; // Number of inserted lines
                           end ;

const TRANSACTION_Insert = 0 ;
      TRANSACTION_Delete = 1 ;
      TRANSACTION_Movement = 2 ;
      TRANSACTION_Selection = 3 ;
      TRANSACTION_Indent = 4 ;
      TRANSACTION_Outdent = 5 ;
      TRANSACTION_Change = 6 ;
      TRANSACTION_Insert_File = 7 ;

// Unit routines...

procedure CB_Change( ID : integer ; Data : pointer ) ;

var Editor : PEditor ;

begin
    // Setup
    Editor := PChange_Record( Data )^.This.Editor ;

    // Restore original line
    Editor^.Set_Line( -1, strpas( PChar( PChange_Record( Data )^.Data ) ) ) ;
    freemem( PChange_Record( Data )^.Data ) ;
end ;


procedure CB_Insert_File( ID : integer ; Data : pointer ) ;

var Dummy : integer ;
    Editor : PEditor ;
    Line : integer ;

begin
    // Setup...
    Editor := PInsert_File_Record( Data )^.This.Editor ;
    Line := PInsert_File_Record( Data )^.Line ;

    if( PInsert_File_Record( Data )^.Count = -1 ) then // Delete entire buffer
    begin
        Editor^.File_Root.Zero ;
    end else
    begin
        if( PInsert_File_Record( Data )^.Count > 0 ) then // Something inserted
        begin
            if( PInsert_File_Record( Data )^.Count > 2 ) then
            begin
                Editor^.Delete_Lines( Line + 1, PInsert_File_Record( Data )^.Count - 2 ) ;
            end ;
            Editor^.Set_Line( Line, copy( Editor^.Fetch_Line( Line ), 1,
                PInsert_File_Record( Data )^.First ) ) ;
            Editor^.Position_Cursor( Line + 1, 1 ) ;
            Dummy := Editor^.Options ;
            Editor^.Options := Editor^.Options and ( not eds_NoCursorWrap ) ;
            Editor^.Delete_Char( True ) ;
            Editor^.Options := Dummy ;
        end ;
    end ;
end ; // CB_Insert_File


procedure CB_Insert( ID : integer ; Data : pointer ) ;

var Dummy : integer ;
    Editor : PEditor ;

begin
    // Setup
    Editor := PInsert_Record( Data )^.This.Editor ;

    // Insert new line
    if( PInsert_Record( Data )^.Size = 0 ) then
    begin
        Editor^.Position_Cursor( Editor^.Screen_Line + Editor^.Cursor_Row, 1 ) ;
        Editor^.Delete_Char( True ) ;
        exit ;
    end ;

    // Normal insert
    for Dummy := 1 to PInsert_Record( Data )^.Size do
    begin
        Editor^.Delete_Char( True ) ;
    end ;
end ;


procedure CB_Delete( ID : integer ; Data : pointer ) ;

var Dummy : integer ;
    Editor : PEditor ;
    S : string ;

begin
    // Setup
    Editor := PDelete_Record( Data )^.This.Editor ;

    // Handle selection deletions
    if( PDelete_Record( Data )^.Size = -1 ) then
    begin
        Editor^.Paste( PLine_Buffer( PDelete_Record( Data )^.Data ) ) ;
        dispose( PLine_Buffer( PDelete_Record( Data )^.Data ), Done ) ;
        exit ;
    end ;

    // Normal deletes...
    setlength( S, PDelete_Record( Data )^.Size ) ;
    for Dummy := 1 to length( S ) do
    begin
        S[ Dummy ] := PDelete_Record( Data )^.Data[ Dummy - 1 ] ;
    end ;
    freemem( PDelete_Record( Data )^.Data ) ;
    Editor^.Insert_Chars( S ) ;
end ; // CB_Delete


procedure CB_Indent( ID : integer ; Data : pointer ) ;

var Editor : PEditor ;
    S : string ;

begin
    // Setup
    Editor := PIndent_Record( Data )^.This.Editor ;
    PIndent_Record( Data )^.This.Bring_Into_View ;

    // Outdent...
    S := Editor^.Fetch_Line( PIndent_Record( Data )^.Line ) ;
    Editor^.Set_Line( PIndent_Record( Data )^.Line, copy( S, 2, 254 ) ) ;
end ;


procedure CB_Movement( ID : integer ; Data : pointer ) ;

begin
    PMovement_Record( Data )^.This.Editor^.Position_Cursor( PMovement_Record( Data )^.Line, PMovement_Record( Data )^.Column ) ;
    PMovement_Record( Data )^.This.Bring_Into_View ;
end ;


procedure CB_Outdent( ID : integer ; Data : pointer ) ;

var Editor : PEditor ;
    S : string ;

begin
    // Setup
    Editor := POutdent_Record( Data )^.This.Editor ;
    POutdent_Record( Data )^.This.Bring_Into_View ;

    // Outdent...
    S := Editor^.Fetch_Line( POutdent_Record( Data )^.Line ) ;
    Editor^.Set_Line( POutdent_Record( Data )^.Line, ' ' + S ) ;
end ;


procedure CB_Selection( ID : integer ; Data : pointer ) ;

var Editor : PEditor ;

begin
    // Setup
    Editor := PSelection_Record( Data )^.This.Editor ;
    PSelection_Record( Data )^.This.Bring_Into_View ;

    // Handle removing selection
    if( PSelection_Record( Data )^.Start_Line < 0 ) then
    begin
        PSelection_Record( Data )^.This.Editor^.Unselect ;
        Editor^.Notification_Routine1( 0 ) ;
        exit ;
    end ;

    // Set selection
    Editor^.Select_Text( PSelection_Record( Data )^.Start_Line,
         PSelection_Record( Data )^.Start_Column,
         PSelection_Record( Data )^.End_Line,
         PSelection_Record( Data )^.End_Column ) ;
end ;


procedure CB_Timer( W : HWND ; Message, ID, Tim : integer ) ;

var Msg : TwmMouseMove ;
    P : TPoint ;

begin
    KillTimer( 0, ID ) ;
    Timer := 0 ;
    GetCursorPos( P ) ;
    ScreenToClient( Mouse_Control.Handle, P ) ;
    Msg.XPos := P.X ;
    Msg.YPos := P.Y ;
    Mouse_Control.Mouse_Move( Msg ) ;
end ;


// Constructors and destructors...

constructor TEditor_Panel.Create( AOwner : TComponent ) ;

begin
    Editor := nil ;
    inherited Create( AOwner ) ;

    Cursor := crIBeam ;
    ControlStyle := ControlStyle - [csOpaque] + [csCaptureMouse] ;
    Editor := new( PEditor, Init ) ;
    Editor.End_Of_Buffer_String := '' ;
    Editor.End_Of_Line_Character := ' ' ;
    Enabled := True ;
    Color := clWhite ;
    OnKeyDown := On_KeyDown ;
    Undo_List := new( POperation_Logger, Init( $3FFFFFFF ) ) ;
end ;


destructor TEditor_Panel.Destroy ;

begin
    if( Timer <> 0 ) then
    begin
        KillTimer( 0, Timer ) ;
    end ;

    dispose( Editor, Done ) ;
    dispose( Undo_List, Done ) ;

    inherited Destroy ;
end ;


// Message handlers...

procedure TEditor_Panel.GetDlgCode( var Message : tWMGetDlgCode ) ;

begin
    Message.Result := -1 ;
end ;


procedure TEditor_Panel.Kill_Focus( var Message : tWMKillFocus ) ;

begin
    inherited ;

    Clear_Cursor( 0, 0 ) ;
end ;


procedure TEditor_Panel.Set_Focus( var Message : tWMSetFocus ) ;

begin
    inherited ;

    Select_Window( Editor^.Screen_Window ) ;
    Set_Cursor( Editor^.Cursor_Row,
        Editor^.Cursor_Column - Editor^.Screen_Column + 1 ) ;
    if( assigned( On_Focus ) ) then
    begin
        On_Focus( self ) ;
    end ;
end ;


procedure TEditor_Panel.Select_Current_Word ;

var Start, Finish : integer ;
    S : string ;

begin
    // Get line and exit if out of range
    S := Editor^.Fetch_Line( Editor^.Screen_Line + Editor^.Cursor_Row - 1 ) ;
    if( Editor^.Physical_Column > length( S ) ) then
    begin
        exit ;
    end ;

    // Determine bounds of word...
    Start := Editor^.Physical_Column ;
    while(
           ( Start > 0 )
           and
           ( pos( S[ Start ], 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890_$' ) > 0 )
         ) do
    begin
        dec( Start ) ;
    end ;
    inc( Start ) ;
    Finish := Start ;
    while(
           ( Finish <= length( S ) )
           and
           ( pos( S[ Finish ], 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890_$' ) > 0 )
         ) do
    begin
        inc( Finish ) ;
    end ;
    dec( Finish ) ;

    // Exit if nothing selected
    if( Start = Finish ) then
    begin
        exit ;
    end ;

    // Select text
    Select_Text( Editor^.Screen_Line + Editor^.Cursor_Row - 1, Start,
        Editor^.Screen_Line + Editor^.Cursor_Row - 1, Finish ) ;
end ; // TEditor_Panel.Select_Current_Word


procedure TEditor_Panel.Select_Text( Start_Line : longint ; Start_Column : byte ;
    Finish_Line : longint ; Finish_Column : byte ) ;

var X : PSelection_Record ;

begin
    // Cancel previous selection and set new one...
    Cancel_Selection ;
    Editor^.Select_Text( Start_Line, Start_Column, Finish_Line, Finish_Column ) ;
    Invalidate ;
    X := AllocMem( sizeof( X^ ) ) ;
    X^.This := self ;
    X^.Start_Line := -1 ; // Undo selection code
    Undo_List.New_Transaction( TRANSACTION_Selection ) ;
    Undo_List.New_Operation( CB_Selection, X, sizeof( X^ ) ) ;
end ; // TEditor_Panel.Select_Text


function TEditor_Panel.Get_Current_Word : string ;

var Start, Finish : integer ;
    S : string ;

begin
    // Get line and exit if out of range
    S := Editor^.Fetch_Line( Editor^.Screen_Line + Editor^.Cursor_Row - 1 ) ;
    if( Editor^.Physical_Column > length( S ) ) then
    begin
        Result := '' ;
        exit ;
    end ;

    // Determine bounds of word...
    Start := Editor^.Physical_Column ;
    while(
           ( Start > 0 )
           and
           ( pos( S[ Start ], 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890_$' ) > 0 )
         ) do
    begin
        dec( Start ) ;
    end ;
    inc( Start ) ;
    Finish := Start ;
    while(
           ( Finish <= length( S ) )
           and
           ( pos( S[ Finish ], 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890_$' ) > 0 )
         ) do
    begin
        inc( Finish ) ;
    end ;
    dec( Finish ) ;

    // Exit if nothing selected
    if( Start = Finish ) then
    begin
        exit ;
    end ;

    Result := copy( S, Start, Finish - Start + 1 ) ;
end ;


procedure TEditor_Panel.wmLButtonDblclk( var Message : tWMLButtonDblclk ) ;

begin
    inherited ;

    Select_Current_Word ;
end ;


procedure TEditor_Panel.wmLButtonDown( var Message : tWMLButtonDown ) ;

    procedure Set_Caret ;

    begin
        Synchronize_Caret_To_XY( Message.XPos, Message.YPos ) ;
    end ;

begin
    Cursor_Save( True ) ;
    if( ( Message.Keys and MK_SHIFT ) <> 0 ) then
    begin
        Set_Caret ;
        Start_Selection ;
    end else
    begin
        Cancel_Selection ;
        Set_Caret ;
    end ;

    if( Timer <> 0 ) then
    begin
        KillTimer( 0, Timer ) ;
    end ;

    inherited ;
end ;


procedure TEditor_Panel.wmLButtonUp( var Message : tWMLButtonUp ) ;

begin
    inherited ;

    SetFocus ;
    if( Mouse_Selection ) then
    begin
        Editor^.Freeze_Selection ;
        Mouse_Selection := False ;
    end ;
    if( Timer <> 0 ) then
    begin
        KillTimer( 0, Timer ) ;
    end ;
end ;


procedure TEditor_Panel.wmMouseMove( var Message : tWMMouseMove ) ;

begin
    inherited ;
    Mouse_Move( Message ) ;
end ;


// Internal utility routines...

procedure TEditor_Panel.Cancel_Selection ; // Cancels selection due to keyboard/mouse input

begin
    if( Editor^.Selection ) then
    begin
        if( not Editor_Preferences.Persistant_Selection.Checked ) then
        begin
            Editor^.Selection := False ;
            Invalidate_Window( Editor^.Screen_Window ) ;
            Editor^.Redraw( 1 ) ;
        end else
        begin
            Editor^.Freeze_Selection ;
        end ;
    end ;
    Editor^.Notification_Routine1( 0 ) ;
end ; // TEditor_Panel.Cancel_Selection


procedure TEditor_Panel.Bring_Into_View ; // Make sure caret is visible

begin
    if(
        ( Editor^.Cursor_Row < 1 )
        or
        ( Editor^.Cursor_Row > Editor^.Screen_Window^.Rows )
      ) then
    begin
        Editor^.Screen_Line := Editor^.Screen_Line +
            Editor^.Cursor_Row - ( Editor^.Screen_Window^.Rows div 2 ) - 1 ;
        Editor^.Cursor_Row := ( Editor^.Screen_Window^.Rows div 2 ) + 1 ;
        Invalidate_Window( Editor^.Screen_Window ) ;
        Editor^.Redraw( 1 ) ;
    end ;
end ;


procedure TEditor_Panel.Handle_Horizontal ;

var Scroll_Amount : integer ;

begin
    Scroll_Amount := Editor^.Screen_Column - Editor^.Cursor_Column ;
    if( Scroll_Amount > 0 ) then // Need to scroll left
    begin
	Editor^.Scroll_Columns( -Scroll_Amount - 5 ) ;
        if( assigned( Editor^.Notification_Routine1 ) ) then
        begin
            Editor^.Notification_Routine1( edn_Scroll ) ;
        end ;
    end ;
    Scroll_Amount := ( Editor^.Cursor_Column - Editor^.Screen_Column + 1 ) -
	Editor^.Screen_Window^.Columns ;
    if( Scroll_Amount >= 0 ) then // Need to scroll right
    begin
	Editor^.Scroll_Columns( Scroll_Amount + 5 ) ;
        if( assigned( Editor^.Notification_Routine1 ) ) then
        begin
            Editor^.Notification_Routine1( edn_Scroll ) ;
        end ;
    end ;
end ;


procedure TEditor_Panel.Indent ; // Indent selected lines

var Dummy, Dummy1 : integer ;
    First, Last : longint ;
    S : string ;
    X : PIndent_Record ;

begin
    Undo_List.New_Transaction( TRANSACTION_Indent ) ;
    if( not Editor^.Something_Selected ) then
    begin
        exit ;
    end ;
    Editor^.Get_Selection_Range( First, Last, Dummy, Dummy1 ) ;
    if( Dummy1 = 0 ) then // Last line not selected
    begin
        dec( Last ) ;
    end ;
    for Dummy := First to Last do
    begin
        S := Editor^.Fetch_Line( Dummy ) ;
        if( ( length( S ) > 0 ) and ( length( S ) < 254 ) ) then
        begin
            Editor^.Set_Line( Dummy, ' ' + S ) ;
            X := AllocMem( sizeof( X^ ) ) ;
            X.This := self ;
            X.Line := Dummy ;
            Undo_List.New_Operation( CB_Indent, X, sizeof( X^ ) ) ;
        end ;
    end ;
    Invalidate ;
end ; // TEditor_Panel.Indent


// Insert the passed text at the current position.
procedure TEditor_Panel.Insert( S : string ) ;

var X : PInsert_Record ;

begin
    X := AllocMem( sizeof( X^ ) ) ;
    X^.Size := length( S ) ;
    X^.This := self ;
    Undo_List.New_Transaction( TRANSACTION_Insert ) ;
    Undo_List.New_Operation( CB_Insert, X, sizeof( X^ ) ) ;
    if( length( S ) = 0 ) then
    begin
        Editor^.Insert_Blank_Line ;
    end else
    begin
        Editor^.Insert_Chars( S ) ;
    end ;
    if( assigned( Editor^.Notification_Routine1 ) ) then
    begin
        Editor^.Notification_Routine1( 0 ) ;
    end ;
end ;


procedure TEditor_Panel.KeyDown( var Key : word ; Shift : TShiftState ) ;

var C : char ;
    Dummy : integer ; // Dummy variable
    Something_Was_Selected : boolean ;

begin
    if( assigned( _OnKeyDown ) ) then
    begin
        _OnKeyDown( self, Key, Shift ) ;
        if( Key = 0 ) then
        begin
            exit ;
        end ;
    end ;

    // Ignore special keys
    if( ( Key = VK_MENU ) or ( Key = VK_SHIFT ) or ( Key = VK_CONTROL ) ) then
    begin
        exit ;
    end ;

    // Check meaningful keys...
    If( Key = VK_END ) then
    Begin
        if( ssShift in Shift ) then
        begin
            Start_Selection ;
        end else
        begin
            Cancel_Selection ;
        end ;
        Cursor_Save( True ) ;
	if( ssCtrl in Shift ) then
        begin
            Editor^.Bottom ;
        end else
        begin
            Editor^.Advance ;
            Dummy := Editor^.Options ;
            Editor^.Options := Editor^.Options and ( not eds_NoCursorWrap ) ;
            Editor^.Cursor_Left ;
            Editor^.Options := Dummy ;
	end ;
        Key := 0 ;
    End ;
    If( Key = VK_HOME ) Then {Top of buffer}
    Begin
        if( ssShift in Shift ) then
        begin
            Start_Selection ;
        end else
        begin
            Cancel_Selection ;
        end ;
        Cursor_Save( True ) ;
	if( ssCtrl in Shift ) then
        begin
            Editor^.Top ;
        end else
        begin
            Editor^.Advance ;
            Editor^.Cursor_Up ;
	end ;
	Key := 0 ;
    End ;
    If( Key = VK_UP ) Then
    Begin
        if( ssShift in Shift ) then
        begin
            Start_Selection ;
        end else
        begin
            Cancel_Selection ;
            Undo_List.New_Transaction( TRANSACTION_Movement ) ;
        end ;
	if( ssCtrl in Shift ) then
        begin
            if( Editor^.Screen_Line > 1 ) then
            begin
                dec( Editor^.Screen_Line ) ;
                inc( Editor^.Cursor_Row ) ;
                Invalidate ;
            end ;
        end else
        begin
            Bring_Into_View ;
            Cursor_Save( True ) ;
	    Editor^.Cursor_Up ;
        end ;
        Key := 0 ;
    End ;
    If( Key = VK_DOWN ) Then
    Begin
        if( ssShift in Shift ) then
        begin
            Start_Selection ;
        end else
        begin
            Cancel_Selection ;
            Undo_List.New_Transaction( TRANSACTION_Movement ) ;
        end ;
	if( ssCtrl in Shift ) then
        begin
            if( Editor^.Screen_Line < Editor^.File_Root^.Last_Line ) then
            begin
                inc( Editor^.Screen_Line ) ;
                dec( Editor^.Cursor_Row ) ;
                Invalidate ;
            end ;
        end else
        begin
            Bring_Into_View ;
            Cursor_Save( False ) ;
	    Editor^.Cursor_Down ;
	end ;
        Key := 0 ;
    End ;
    If( Key = VK_LEFT ) Then
    Begin
        if( ssShift in Shift ) then
        begin
            Start_Selection ;
        end else
        begin
            Cancel_Selection ;
            Undo_List.New_Transaction( TRANSACTION_Movement ) ;
        end ;
        Cursor_Save( False ) ;
	if( ssCtrl in Shift ) then
        begin
            Editor^.Word_Backward ;
        end else
        begin
            Editor^.Cursor_Left ;
	end ;
	Bring_Into_View ;
        Key := 0 ;
    end ;
    if( Key = VK_RIGHT ) Then
    begin
        if( ssShift in Shift ) then
        begin
            Start_Selection ;
        end else
        begin
            Cancel_Selection ;
            Undo_List.New_Transaction( TRANSACTION_Movement ) ;
        end ;
        Cursor_Save( False ) ;
	if( ssCtrl in Shift ) then
        begin
            Editor^.Word_Forward ;
        end else
        begin
            Editor^.Cursor_Right ;
	end ;
	Bring_Into_View ;
        Key := 0 ;
    end ;
    if( Key = VK_NEXT ) then
    begin
        if( ssShift in Shift ) then
        begin
            Start_Selection ;
        end else
        begin
            Cancel_Selection ;
            Undo_List.New_Transaction( TRANSACTION_Movement ) ;
        end ;
        Cursor_Save( False ) ;
        if( ssCtrl in Shift ) then
        begin
            Editor^.Set_Cursor_Row( Editor^.Max_Row ) ;
            if( ssShift in Shift ) then
            begin
                Invalidate ;
            end ;
        end else
        begin
            Bring_Into_View ;
            Editor^.Page_Down ;
        end ;
        Key := 0 ;
    end ;
    if( Key = VK_PRIOR ) then
    begin
        if( ssShift in Shift ) then
        begin
            Start_Selection ;
        end else
        begin
            Cancel_Selection ;
            Undo_List.New_Transaction( TRANSACTION_Movement ) ;
        end ;
        Cursor_Save( False ) ;
        if( ssCtrl in Shift ) then
        begin
            Editor^.Set_Cursor_Row( 1 ) ;
            if( ssShift in Shift ) then
            begin
                Invalidate ;
            end ;
        end else
        begin
            Bring_Into_View ;
            Editor^.Page_Up ;
        end ;
        Key := 0 ;
    end ;
    if( Key = VK_DELETE ) then // Delete current character
    begin
	Bring_Into_View ;
        Something_Was_Selected := Editor^.Something_Selected ;
        if( not Editor_Preferences.Inactive_Selection.Checked ) then
        begin
            Delete_Current_Selection ;
        end ;
        if( Something_Was_Selected ) then
        begin
            Invalidate_Window( Editor^.Screen_Window ) ;
            Editor^.Redraw( 1 ) ;
            exit ;
        end ;
        If( Editor^.At_EOB ) Then
        begin
            exit ; {Do nothing if at EOB}
        end ;
        Cursor_Right ;
        Delete ;
        Key := 0 ;
        Cancel_Selection ;
    end ;
    if( Key = VK_RETURN ) then
    begin
        if( not ( ssAlt in Shift ) ) then
        begin
            if( ssCtrl in Shift ) then
            begin
                TEditor_Window( Parent ).Open_File( Get_Current_Word ) ;
            end else
            begin
                C := CR ;
                if( not Editor_Preferences.Inactive_Selection.Checked ) then
                begin
                    Delete_Current_Selection ;
                end ;
                Insert( C ) ;
            end ;
        end ;
        Key := 0 ;
    end ;
    if( ( not Control_K ) and ( not Control_O ) and ( not Control_Q ) ) then
    begin
        if( Key = ord( 'I' ) ) then
        begin
            if( not ( ssAlt in Shift ) ) then
            begin
                if( ( ssCtrl in Shift ) and ( ssShift in Shift ) ) then
                begin
                    Indent ;
                end else
                begin
                    if( ssCtrl in Shift ) then
                    begin
                        C := HT ;
                    end else
                    begin
                        if( ssShift in Shift ) then
                        begin
                            C := 'I' ;
                        end else
                        begin
                            C := 'i' ;
                        end ;
                    end ;
                    if( not Editor_Preferences.Inactive_Selection.Checked ) then
                    begin
                        Delete_Current_Selection ;
                    end ;
                    Insert( C ) ;
                end ;
            end ;
            Key := 0 ;
        end ;
        if( Key = ord( 'U' ) ) then
        begin
            if( not ( ssAlt in Shift ) ) then
            begin
                if( ( ssCtrl in Shift ) and ( ssShift in Shift ) ) then
                begin
                    Outdent ;
                end else
                begin
                    if( ssCtrl in Shift ) then
                    begin
                        C := #21 ;
                    end else
                    begin
                        if( ssShift in Shift ) then
                        begin
                            C := 'U' ;
                        end else
                        begin
                            C := 'u' ;
                        end ;
                    end ;
                    if( not Editor_Preferences.Inactive_Selection.Checked ) then
                    begin
                        Delete_Current_Selection ;
                    end ;
                    Insert( C ) ;
                end ;
            end ;
            Key := 0 ;
        end ;
    end ;
    Set_Cursor( Editor^.Cursor_Row,
        Editor^.Cursor_Column - Editor^.Screen_Column + 1 ) ;
    Handle_Horizontal ;
end ; // TEditor_Panel.KeyDown


procedure TEditor_Panel.Lowercase_Word ;

var S : string ;
    X : PChange_Record ;

begin
    X := AllocMem( sizeof( X^ ) ) ;
    X.This := self ;
    S := Editor^.Fetch_Line( -1 ) ;
    X.Size := length( S ) ;
    X.Data := AllocMem( X.Size + 1 ) ;
    strcopy( PChar( X.Data ), PChar( S ) ) ;
    Undo_List.New_Transaction( TRANSACTION_Change ) ;
    Undo_List.New_Operation( CB_Change, X, sizeof( X^ ) ) ;
    Editor^.Lowercase_Word ;
end ;


procedure TEditor_Panel.Mouse_Move( var Message : tWMMouseMove ) ;

begin
    if( MouseCapture ) then
    begin
        if( not Mouse_Selection ) then
        begin
            Cancel_Selection ;
            Start_Selection ;
            Mouse_Selection := True ;
        end ;

        Synchronize_Caret_To_XY( Message.XPos, Message.YPos ) ;

        { Handle autoscroll... }
        if(
            ( Message.YPos < 0 )
            or
            ( Message.YPos >= ClientHeight )
            or
            ( Message.XPos < 0 )
            or
            ( Message.XPos >= ClientWidth )
          ) then
        begin
            Mouse_Control := self ;
            Timer := SetTimer( 0, 0, 200, @CB_Timer ) ;
        end ;
    end else
    begin
        if( Timer <> 0 ) then
        begin
            KillTimer( 0, Timer ) ;
        end ;
    end ;
end ; // TEditor_Panel.Mouse_Move


procedure TEditor_Panel.Cursor_Save( New_Transaction : boolean ) ;

var X : PMovement_Record ;

begin
    X := AllocMem( sizeof( X^ ) ) ;
    X^.This := self ;
    X^.Line := Editor^.Screen_Line + Editor^.Cursor_Row - 1 ;
    X^.Column := Editor^.Cursor_Column ;
    if( New_Transaction ) then
    begin
        Undo_List.New_Transaction( TRANSACTION_Movement ) ;
    end ;
    Undo_List.New_Operation( CB_Movement, X, sizeof( X^ ) ) ;
end ;


procedure TEditor_Panel.Cursor_Down ;

begin
    Cursor_Save( True ) ;
    Editor^.Cursor_Down ;
end ;


procedure TEditor_Panel.Cursor_Left ;

begin
    Cursor_Save( True ) ;
    Editor^.Cursor_Left ;
end ;


procedure TEditor_Panel.Cursor_Right ;

begin
    Cursor_Save( True ) ;
    Editor^.Cursor_Right ;
end ;


procedure TEditor_Panel.Cursor_Up ;

begin
    Cursor_Save( True ) ;
    Editor^.Cursor_Up ;
end ;


procedure TEditor_Panel.Delete ;

var S : string ;
    X : PDelete_Record ;

begin
    if(
        ( Editor^.Physical_Column = 1 )
        and
        ( Editor^.Cursor_Row = 1 )
        and
        ( Editor^.Screen_Line = 1 )
      ) then
    begin
        exit ;
    end ;

    // Build delete undo record...
    S := Editor^.Fetch_Line( -1 ) ;
    X := AllocMem( sizeof( X^ ) ) ;
    X^.Data := AllocMem( 1 ) ;
    if( Editor^.Physical_Column = 1 ) then
    begin
        X^.Data[ 0 ] := CR ;
    end else
    begin
        S := copy( S, Editor^.Physical_Column - 1, 1 ) ;
        if( length( S ) > 0 ) then
        begin
            X^.Data[ 0 ] := S[ 1 ] ;
        end else
        begin
            X^.Data[ 0 ] := NUL ;
        end ;
    end ;
    X^.Size := 1 ;
    X^.This := self ;
    Undo_List.New_Transaction( TRANSACTION_Insert ) ;
    Undo_List.New_Operation( CB_Delete, X, sizeof( X^ ) ) ;

    // Do the deletion
    Editor^.Delete_Char( True ) ;

    // Notification
    if( assigned( Editor^.Notification_Routine1 ) ) then
    begin
        Editor^.Notification_Routine1( 0 ) ;
    end ;
end ; // TEditor_Panel.Delete


procedure TEditor_Panel.Delete_Current_Selection ;

var X : PDelete_Record ;
    Z : PLine_Buffer ;

begin
    // Check for selection
    if( not Editor^.Selection ) then // Nothing is selected
    begin
        exit ;
    end ;

    // Build the deletion undo record...
    Z := new( PLine_Buffer, Init ) ;
    Editor^.Copy_Text( Z ) ;
    X := AllocMem( sizeof( X^ ) ) ;
    X^.Data := PChar( Z ) ;
    X^.Size := -1 ;
    X^.This := self ;
    Undo_List.New_Transaction( TRANSACTION_Insert ) ;
    Undo_List.New_Operation( CB_Delete, X, sizeof( X^ ) ) ;

    // Do the deletion
    Editor^.Delete_Text ;
    Invalidate_Window( Editor^.Screen_Window ) ;
    Editor^.Redraw( 1 ) ;

    // Notification
    if( assigned( Editor^.Notification_Routine1 ) ) then
    begin
        Editor^.Notification_Routine1( 0 ) ;
    end ;
end ; // TEditor_Panel.Delete_Current_Selection


procedure TEditor_Panel.Delete_Line ;

var Dummy : integer ;
    S : string ;
    X : PDelete_Record ;

begin
    if(
        ( Editor^.Physical_Column = 1 )
        and
        ( Editor^.Cursor_Row = 1 )
        and
        ( Editor^.Screen_Line = 1 )
      ) then
    begin
        exit ;
    end ;

    // Build delete undo record...
    S := Editor^.Fetch_Line( -1 ) ;
    X := AllocMem( sizeof( X^ ) ) ;
    X^.Data := AllocMem( length( S ) ) ;
    for Dummy := 0 to length( S ) do
    begin
        X^.Data[ Dummy - 1 ] := S[ Dummy ] ;
    end ;
    X^.Size := length( S ) ;
    X^.This := self ;
    Undo_List.New_Transaction( TRANSACTION_Insert ) ;
    Undo_List.New_Operation( CB_Delete, X, sizeof( X^ ) ) ;

    // Do the deletion
    Editor^.Delete_Current_Line ;

    // Notification
    if( assigned( Editor^.Notification_Routine1 ) ) then
    begin
        Editor^.Notification_Routine1( 0 ) ;
    end ;
end ; // TEditor_Panel.Delete_Line


procedure TEditor_Panel.Delete_Word ;

var Dummy : integer ;
    S : string ;
    Starting, Ending : integer ;
    X : PDelete_Record ;

begin
    // Get deleted word...
    Editor^.Word_Range( Starting, Ending ) ;
    S := Editor^.Fetch_Line( -1 ) ;
    if( Starting = Ending ) then
    begin
        S := CR ;
    end else
    begin
        S := copy( S, Starting, Ending - Starting + 1 ) ;
    end ;

    // Build delete undo record...
    X := AllocMem( sizeof( X^ ) ) ;
    X^.Data := AllocMem( length( S ) ) ;
    for Dummy := 0 to length( S ) do
    begin
        X^.Data[ Dummy - 1 ] := S[ Dummy ] ;
    end ;
    X^.Size := length( S ) ;
    X^.This := self ;
    Undo_List.New_Transaction( TRANSACTION_Insert ) ;
    Undo_List.New_Operation( CB_Delete, X, sizeof( X^ ) ) ;

    // Do the deletion
    Editor^.Delete_Word ;

    // Notification
    if( assigned( Editor^.Notification_Routine1 ) ) then
    begin
        Editor^.Notification_Routine1( 0 ) ;
    end ;
end ; // TEditor_Panel.Delete_Word


procedure TEditor_Panel.KeyPress( var Key : char ) ;

var S : string ;
    Something_Was_Selected : boolean ;

begin
    inherited KeyPress( Key ) ;

    if( Control_K ) then
    begin
        if( ( Key = 'e' ) or ( Key = 'E' ) or ( Key = #5 ) ) then
        begin
            Lowercase_Word ;
            Cancel_Selection ;
        end ;
        if( ( Key = 'f' ) or ( Key = 'F' ) or ( Key = #6 ) ) then
        begin
            Uppercase_Word ;
            Cancel_Selection ;
        end ;
        if( ( Key = 'i' ) or ( Key = 'I' ) or ( Key = #9 ) ) then
        begin
            Indent ;
        end ;
        if( ( Key = 'l' ) or ( Key = 'L' ) or ( Key = #12 ) ) then
        begin
            S := Editor^.Fetch_Line( Editor^.Screen_Line + Editor^.Cursor_Row - 1 ) ;
            Select_Text( Editor^.Screen_Line + Editor^.Cursor_Row - 1, 1,
                Editor^.Screen_Line + Editor^.Cursor_Row - 1, length( S ) ) ;
        end ;
        if( ( Key = 'r' ) or ( Key = 'R' ) or ( Key = #18 ) ) then
        begin
            TEditor_Window( Parent ).Include_File ;
        end ;
        if( ( Key = 's' ) or ( Key = 'S' ) or ( Key = #19 ) ) then
        begin
            TEditor_Window( Parent ).Save_File ;
        end ;
        if( ( Key = 't' ) or ( Key = 'T' ) or ( Key = #20 ) ) then
        begin
            Select_Current_Word ;
        end ;
        if( ( Key = 'u' ) or ( Key = 'U' ) or ( Key = #21 ) ) then
        begin
            Outdent ;
        end ;
        if( ( Key = 'y' ) or ( Key = 'Y' ) or ( Key = #25 ) ) then
        begin
            Delete_Current_Selection ;
        end ;
        Control_K := False ;
        Key := #0 ;
    end ;
    if( Control_O ) then
    begin
        if( ( Key = 'a' ) or ( Key = 'A' ) or ( Key = #1 ) ) then
        begin
            TEditor_Window( Parent ).Open_File( Get_Current_Word ) ;
        end ;
        if( ( Key = 'u' ) or ( Key = 'U' ) or ( Key = #21 ) ) then
        begin
            Editor^.Change_Case ;
        end ;
        Control_O := False ;
        Key := #0 ;
    end ;
    if( Control_Q ) then
    begin
        if( ( Key = 'a' ) or ( Key = 'A' ) or ( Key = #1 ) ) then
        begin
            TEditor_Window( Parent ).Replace1Click( self ) ;
        end ;
        if( ( Key = 'c' ) or ( Key = 'C' ) or ( Key = #3 ) ) then
        begin
            Cursor_Save( True ) ;
            Editor^.Bottom ;
        end ;
        if( ( Key = 'd' ) or ( Key = 'D' ) or ( Key = #4 ) ) then
        begin
            Cursor_Save( True ) ;
            Editor^.EOL ;
        end ;
        if( ( Key = 'e' ) or ( Key = 'E' ) or ( Key = #5 ) ) then
        begin
            Cursor_Save( True ) ;
            Editor^.Position_Cursor( 1, Editor^.Cursor_Column ) ;
        end ;
        if( ( Key = 'f' ) or ( Key = 'F' ) or ( Key = #6 ) ) then
        begin
            TEditor_Window( Parent ).Find1Click( self ) ;
        end ;
        if( ( Key = 'r' ) or ( Key = 'R' ) or ( Key = #18 ) ) then
        begin
            Cursor_Save( True ) ;
            Editor^.Top ;
        end ;
        if( ( Key = 's' ) or ( Key = 'S' ) or ( Key = #19 ) ) then
        begin
            Editor^.Position_Cursor( Editor^.Cursor_Row, 1 ) ;
        end ;
        if( ( Key = 't' ) or ( Key = 'T' ) or ( Key = #20 ) ) then
        begin
            Cursor_Save( True ) ;
            Editor^.Screen_Line := Editor^.Screen_Line + Editor^.Cursor_Row - 1 ;
            Editor^.Set_Cursor_Row( 1 ) ;
            if( assigned( Editor^.Notification_Routine1 ) ) then
            begin
                Editor^.Notification_Routine1( edn_Scroll ) ;
            end ;
            Invalidate ;
        end ;
        if( ( Key = 'u' ) or ( Key = 'U' ) or ( Key = #21 ) ) then
        begin
            if( Editor^.Screen_Line + Editor^.Cursor_Row - Editor^.Max_Row >= 1 ) then
            begin
                Cursor_Save( True ) ;
                Editor^.Screen_Line := Editor^.Screen_Line + Editor^.Cursor_Row - Editor^.Max_Row ;
                Editor^.Set_Cursor_Row( Editor^.Max_Row ) ;
                if( assigned( Editor^.Notification_Routine1 ) ) then
                begin
                    Editor^.Notification_Routine1( edn_Scroll ) ;
                end ;
                Invalidate ;
            end ;
        end ;
        if( ( Key = 'x' ) or ( Key = 'X' ) or ( Key = #24 ) ) then
        begin
            Cursor_Save( True ) ;
            Editor^.Position_Cursor( Editor^.Max_Row, Editor^.Cursor_Column ) ;
        end ;
        if( ( Key = 'y' ) or ( Key = 'Y' ) or ( Key = #25 ) ) then
        begin
            Delete_Line ;
            Cancel_Selection ;
        end ;
        Control_Q := False ;
        Key := #0 ;
    end ;
    if(
        ( Key = CR )
        or
        ( Key = 'i' )
        or
        ( Key = 'I' )
        or
        ( Key = HT )
        or
        ( Key = 'u' )
        or
        ( Key = 'U' )
        or
        ( Key = #21 )
        or
        ( Key = CR )
      ) then
    begin
        Key := #0 ; // These were handled in KeyDown
    end ;
    if( Key <> #0 ) then
    begin
        if( Key = #11 ) then // Control-K
        begin
            Control_K := True ;
            Key := #0 ;
        end ;
        if( Key = #15 ) then // Control-O
        begin
            Control_O := True ;
            Key := #0 ;
        end ;
        if( Key = #17 ) then // Control-Q
        begin
            Control_Q := True ;
            Key := #0 ;
        end ;
    end ;
    if(
        ( not Editor_Preferences.Inactive_Selection.Checked )
        and
        ( Key <> #0 )
        and
        ( Key <> #26 )
      ) then
    begin
        Something_Was_Selected := Editor^.Something_Selected ;
        Delete_Current_Selection ;
        if( ( Key = #255 ) or ( Key = #8 ) ) then
        begin
            if( Something_Was_Selected ) then
            begin
                Key := #0 ;
            end ;
        end ;
    end ;
    if( ( Key = #255 ) or ( Key = #8 ) ) then
    begin
	Bring_Into_View ;
        Delete ;
        Key := #0 ;
    end ;
    if( ( Key <> #0 ) and ( Key <> #26 ) ) then // Any key (except ^Z for undo) cancels selection
    begin
        Cancel_Selection ;
    end ;
    if( Key = #14 ) then // Control-N
    begin
        Bring_Into_View ;
        Insert( '' ) ;
        Key := #0 ;
    end ;
    if( Key = #20 ) then // Control-T
    begin
        Bring_Into_View ;
        Delete_Word ;
        Key := #0 ;
    end ;
    if( Key = #25 ) then // Control-Y
    begin
        Bring_Into_View ;
        Delete_Line ;
        Key := #0 ;
    end ;
    if( Key = #26 ) then // Control-Z
    begin
        TEditor_Window( Parent ).Undo1Click( self ) ;
        Key := #0 ;
    end ;
    if( Key <> #0 ) then
    begin
	Bring_Into_View ;
        Insert( Key ) ; { Insert a character }
    end ;
    Set_Cursor( Editor^.Cursor_Row,
        Editor^.Cursor_Column - Editor^.Screen_Column + 1 ) ;
    Handle_Horizontal ;
end ; // TEditor_Panel.KeyPress


procedure TEditor_Panel.On_KeyDown( Sender : TObject ; var Key : word ;
    Shift : TShiftState ) ;

begin
    KeyDown( Key, Shift ) ;
end ;


procedure TEditor_Panel.Outdent ;

var Dummy, Dummy1 : integer ;
    First, Last : longint ;
    S : string ;
    X : POutdent_Record ;

begin
    Undo_List.New_Transaction( TRANSACTION_Outdent ) ;
    if( not Editor^.Something_Selected ) then
    begin
        exit ;
    end ;
    Editor^.Get_Selection_Range( First, Last, Dummy, Dummy1 ) ;
    if( Dummy1 = 0 ) then // Last line not selected
    begin
        dec( Last ) ;
    end ;
    for Dummy := First to Last do
    begin
        S := Editor^.Fetch_Line( Dummy ) ;
        if( ( length( S ) > 0 ) and ( S[ 1 ] = ' ' ) ) then
        begin
            Editor^.Set_Line( Dummy, copy( S, 2, 254 ) ) ;
            X := AllocMem( sizeof( X^ ) ) ;
            X.This := self ;
            X.Line := Dummy ;
            Undo_List.New_Operation( CB_outdent, X, sizeof( X^ ) ) ;
        end ;
    end ;
    Invalidate ;
end ; // TEditor_Panel.Outdent


procedure TEditor_Panel.Paint ;

begin
    if( Editor^.Screen_Window = nil ) then // Not created yet
    begin
        exit ;
    end ;
    Select_Window( Editor^.Screen_Window ) ;
    Editor^.Screen_Window^.ID := Handle ; // In case window handle changed
    Clear_Cursor( 0, 0 ) ;
    Invalidate_Window( Editor^.Screen_Window ) ;
    Editor^.Redraw( 1 ) ;
    if( GetFocus = Handle ) then
    begin
        Set_Cursor( Editor^.Cursor_Row,
            Editor^.Cursor_Column - Editor^.Screen_Column + 1 ) ;
    end ;
end ;


procedure TEditor_Panel.SetBounds( ALeft, ATop, AWidth, AHeight : integer ) ;

begin
    inherited SetBounds( ALeft, ATop, AWidth, AHeight ) ;
    if( Editor <> nil ) then
    begin
        if( Editor^.Screen_Window <> nil ) then
        begin
            Editor.Screen_Window.Id := Parent.Handle ;
            // Sometimes the parent handle changes - make sure we always sync with it

            Editor^.Calculate_Rows( ClientHeight + Editor^.Text_Height ) ;
            // Always show one more line than fits so partial lines show up
        end ;
    end ;
end ;


procedure TEditor_Panel.Start_Selection ;

var X : PSelection_Record ;

begin
    X := AllocMem( sizeof( X^ ) ) ;
    X^.This := self ;
    if( Editor^.Selection ) then // If something is already selected
    begin
        Editor^.Get_Selection_Range( X^.Start_Line, X^.End_Line,
            X^.Start_Column, X^.End_Column ) ;
    end else
    begin
        X^.Start_Line := -1 ;
    end ;
    Undo_List.New_Transaction( TRANSACTION_Selection ) ;
    Undo_List.New_Operation( CB_Selection, X, sizeof( X^ ) ) ;

    if( Editor^.Selection_Is_Frozen ) then // A search result exists
    begin
        Editor^.Unselect ;
    end ;
    Editor^.Select ;
    Editor^.Notification_Routine1( 0 ) ;
end ;


procedure TEditor_Panel.Synchronize_Caret_To_XY( X, Y : integer ) ;

var Column : integer ;
    Row : integer ;

begin
    Row := Y div Editor^.Text_Height ;
    if( Y < 0 ) then
    begin
        dec( Row ) ;
    end ;
    if( Y >= ClientHeight ) then
    begin
        inc( Row ) ;
    end ;

    { The following is needed to handle embedded tabs or other text
      translations where the number of characters displayed for a character
      do not equal 1.  First we determine the screen position, based on a
      nonproportional font.  Then we convert that to the actual character
      which is displaying at that position.  We position to that physical
      character, and then get the starting screen position of that character
      and convert from the physical character offset to the window position.
      Finally, we set the caret to that position. }
    Column := ( X + Editor^.Screen_Window^.Text_Width div 2 ) div Editor^.Screen_Window^.Text_Width + 1 ;
    Column := Editor^.Screen_To_Text_Column( Row + Editor^.Screen_Line, Column ) ;
    Editor^.Position_Cursor( Row + Editor^.Screen_Line, Column ) ;
    Column := Editor^.Text_To_Screen_Column( Row + Editor^.Screen_Line,
        Editor^.Cursor_Column ) ;
    Set_Cursor( Editor^.Cursor_Row, Column - Editor^.Screen_Column + 1 ) ;
end ;


procedure TEditor_Panel.Uppercase_Word ;

var S : string ;
    X : PChange_Record ;

begin
    X := AllocMem( sizeof( X^ ) ) ;
    X.This := self ;
    S := Editor^.Fetch_Line( -1 ) ;
    X.Size := length( S ) ;
    X.Data := AllocMem( X.Size + 1 ) ;
    strcopy( PChar( X.Data ), PChar( S ) ) ;
    Undo_List.New_Transaction( TRANSACTION_Change ) ;
    Undo_List.New_Operation( CB_Change, X, sizeof( X^ ) ) ;
    Editor^.Uppercase_Word ;
end ;


// API...

procedure TEditor_Panel.Delete_Text ; // Delete selected text

begin
    Delete_Current_Selection ;
    Set_Cursor( Editor^.Cursor_Row,
        Editor^.Cursor_Column - Editor^.Screen_Column + 1 ) ;
    Handle_Horizontal ;
end ;


function TEditor_Panel.Open( S : string ) : boolean ; // Open and read-in a file

var Dummy : integer ;
    Input_File : TextFile ;

begin
    Result := True ;
    assignFile( Input_File, S ) ;
    {$I-}
    reset( Input_File ) ;
    {$I+}
    Dummy := IOResult ;
    if( Dummy = 2 ) then
    begin
        Result := False ;
        exit ;
    end ;
    if( Dummy <> 0 ) then
    begin
        ShowMessage( ERT( Dummy ) + ' opening ' + S ) ;
        Result := False ;
        exit ;
    end ;
    while( not Eof( Input_File ) ) do
    begin
        Readln( Input_File, S ) ;
        Editor^.File_Root^.Append( S ) ;
    end ;
    closefile( Input_File ) ;
    Editor^.Position_Cursor( 1, 1 ) ;
end ;


procedure TEditor_Panel.Insert_Buffer( Buffer : pLine_Buffer ) ;

var X : PInsert_File_Record ;

begin
    Editor^.Do_Update ; // Update current line
    X := AllocMem( sizeof( X^ ) ) ;
    X^.This := self ;
    if( Editor^.File_Root^.Last_Line = 0 ) then // Pasting into empty buffer
    begin
        X^.Count := -1 ;
    end else
    begin
        X^.Line := Editor^.Screen_Line + Editor^.Cursor_Row - 1 ;
        X^.First := Editor^.Physical_Column - 1 ;
        X^.Count := Buffer.Last_Line ;
    end ;
    Undo_List.New_Transaction( TRANSACTION_Insert_File ) ;
    Cursor_Save( False ) ;
    Undo_List.New_Operation( CB_Insert_File, X, sizeof( X^ ) ) ;

    Editor^.Paste( Buffer ) ; // Append to existing buffer
    Editor^.Redraw( 1 ) ;
end ; // TEditor_Panel.Insert_Buffer


procedure TEditor_Panel.Position_Cursor( Line, Column : integer ) ;

begin
    Editor^.Position_Cursor( Line, Column ) ;
end ;


function TEditor_Panel.Search( X : string ; Case_Sensitive : boolean ) : boolean ;

begin
    if( Case_Sensitive ) then
    begin
        Editor^.Options := Editor^.Options or eds_Case_Sensitive ;
    end else
    begin
        Editor^.Options := Editor^.Options and ( not eds_Case_Sensitive ) ;
    end ;
    Search := Editor^.Do_Search( X ) ;
end ;


procedure TEditor_Panel.Undo ;

begin
    // Early exit if nothing to undo
    if( Undo_List.Count = 0 ) then
    begin
        exit ;
    end ;

    // Undo the operation
    Undo_List.Unwind( 0 ) ;

    // Make sure cursor is good...
    Set_Cursor( Editor^.Cursor_Row,
        Editor^.Cursor_Column - Editor^.Screen_Column + 1 ) ;
    Handle_Horizontal ;

    // Update the window
    Invalidate ;
end ;



end.
