{
        Program Name : FStreams
        Package Name : LB
        Purpose      : File Stream objects
        Institution  : Conroy & Conroy Co.
        Date Written :
        Written By   : Alan Conroy
        Version      : 1.0

	Copyright (C) 2005 by Alan Conroy.  All rights reserved.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          Provides stream access to a PFile.
}

unit FStreams ;

interface

{$I EDEFINES.INC}

uses _Streams, { TCOM_Stream }
     Files, { PFile }
     Streams, { TBase_Stream }
     _UEHDefs ; { TUEC }

type TSFile_Stream = object( TBase_Stream )
                           private { Instance data... }
                               Contents : string ;
                               _Position : integer ; { Always just before character to read/write (eg 0=position 1) }

                           public { Overrides... }
                               constructor Init ;
                               destructor Done ; virtual ;

                           public { API... }
                               function At_End : boolean ; virtual ;

                               function Position : integer ; virtual ;

                               procedure Read( var Buffer ; var _Size : longint ) ;
                                   virtual ;

                               procedure Seek( Value : longint ) ; virtual ;

                               function Size : longint ; virtual ;

                               procedure Write( var Buffer ; _Size : longint ) ;
                                   virtual ;
                       end ; { TSString_Stream }
      PSFile_Stream = ^TSFile_Stream ;

{$IFDEF DELPHI}
type TCOM_File_Stream = class( TCOM_Stream )
                               private { Instance data... }
                                   _File : PSFile ;
                                   Temp_As_String : string ;

                               public { API... }
                                  function As_String : PChar ;
                                      virtual ; stdcall ;

                                  function At_End : boolean ; override ;

                                  procedure Clear ; virtual ; stdcall ;

                                  { Returns last error code, and resets the error code. }
                                  procedure Last_Error( var UEC : TUEC ) ;
                                      override ;

                                  { Reads the specified number of bytes from the stream.
                                    Size is modified to be the actual bytes transferred. }
                                  procedure Read( var Buffer ;
                                      var _Size : longint ) ; override ;

                                  { Reads one line of input, up to the size of the buffer.
                                    The line is assumed to end at an ASCII 13 code.  The
                                    ASCII 13 code is not included in the returned data.
                                    Size is modified to be the actual bytes transferred. }
                                  procedure Read_Line( var Buffer ;
                                      var _Size : longint ) ; override ;

                                  { Position to specified byte within streamed data. }
                                  procedure Seek( Position : longint ) ;
                                      override ;

                                  { Returns size of stream data, in bytes.  -1 indicate that
                                    the size is unknown or larger than 2^31. }
                                  function Size : longint ; override ;

                                  { Writes the specified buffer, of the specified size in
                                    bytes, to the stream. }
                                  procedure Write( var Buffer ;
                                      _Size : longint ) ; override ;

                                  { Writes the specified null-terminated text to the stream.
                                    An ASCII code 13 is appended to the text on output. }
                                  procedure Write_Line( Buffer : PChar ) ;
                                      override ;
                          end ; { TCOM_File_Stream }
{$ENDIF}

function Create_File_Stream( const Name : string ;
    var UEC : TUEC ) : TCOM_File_Stream ;

{$IFDEF Test}
procedure Test_Unit ;
{$ENDIF}

implementation

uses { Borland... }
{$IFDEF DELPHI}
     SysUtils, { strlen }
{$ENDIF}

     { C&C... }
     ASCIIDef ; { CR }

{ TSFile_Stream methods... }

{ Overrides... }

constructor TSFile_Stream.Init ;

begin
    inherited Init ;

{$IFDEF DELPHI}
    Initialize( Contents ) ;
{$ENDIF}
    Contents := '' ;
    _Position := 0 ;
end ;


destructor TSFile_Stream.Done ;

begin
{$IFDEF DELPHI}
    Finalize( Contents ) ;
{$ENDIF}

    inherited Done ;
end ;


function TSFile_Stream.At_End : boolean ;

begin
    At_End := ( _Position >= length( Contents ) ) ;
end ;


function TSFile_Stream.Position : integer ;

begin
    Position := _Position ;
end ;


procedure TSFile_Stream.Read( var Buffer ; var _Size : longint ) ;

var Len : integer ; { Maximum possible length }

begin
    Len := length( Contents ) - _Position ;
    if( Len < _Size ) then
    begin
        _Size := Len ;
    end ;
    move( Contents[ _Position + 1 ], Buffer, _Size ) ;
    _Position := _Position + _Size ;
end ;


procedure TSFile_Stream.Seek( Value : longint ) ;

begin
    _Position := Value ;
    if( _Position > length( Contents ) ) then
    begin
        _Position := length( Contents ) ;
    end ;
end ;


function TSFile_Stream.Size : longint ;

begin
    Size := length( Contents ) ;
end ;


procedure TSFile_Stream.Write( var Buffer ; _Size : longint ) ;

begin
    setlength( Contents, _Position + _Size ) ;
    move( Buffer, PChar( Contents )[ _Position ], _Size ) ;
    _Position := _Position + _Size ;
end ;


{$IFDEF DELPHI}
{ TCOM_File_Stream methods... }

{ API... }

function TCOM_File_Stream.As_String : PChar ;

var UEC : TUEC ;

begin
    setlength( Temp_As_String, _File.Get_Size ) ;
    _File.Read_Data( Temp_As_String[ 1 ], 0, length( Temp_As_String ), UEC ) ;
    As_String := PChar( Temp_As_String ) ;
end ;


function TCOM_File_Stream.At_End : boolean ;

begin
    At_End := _File.EOF ;
end ;


procedure TCOM_File_Stream.Clear ;

begin
    _File.Seek( 0 ) ;
end ;


procedure TCOM_File_Stream.Last_Error( var UEC : TUEC ) ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
end ;


procedure TCOM_File_Stream.Read( var Buffer ; var _Size : longint ) ;

var R : longint ;

begin
    _File.BlockRead( Buffer, _Size, R ) ;
    _Size := R ;
end ;


procedure TCOM_File_Stream.Read_Line( var Buffer ; var _Size : longint ) ;

var S : string ;

begin
    _File.Readln( S ) ;
    if( length( S ) > _Size ) then
    begin
        setlength( S, _Size ) ;
    end ;
    move( S[ 1 ], Buffer, length( S ) ) ;
end ;


procedure TCOM_File_Stream.Seek( Position : longint ) ;

begin
    _File.Seek( Position ) ;
end ;


function TCOM_File_Stream.Size : longint ;

begin
    Size := _File.Get_Size ;
end ;


procedure TCOM_File_Stream.Write( var Buffer ; _Size : longint ) ;

var UEC : TUEC ;

begin
    _File.Write_Data( Buffer, _File.FilePos, _Size, UEC ) ;
end ;


procedure TCOM_File_Stream.Write_Line( Buffer : PChar ) ;

begin
    _File.Writeln( string( Buffer ) ) ;
end ;
{$ENDIF}


{$IFDEF Test}
procedure Error( const S : string ) ;

begin
    writeln( S ) ;
    halt ;
end ;


procedure Test_Unit ;

var S : string[ 255 ] ;
    Size : integer ;
    Test : TCOM_File_Stream ;

begin
    Test := TCOM_File_Stream.Create ;

    if( not Test.At_End ) then
    begin
        Error( 'At_End failure' ) ;
    end ;
    S := '1234567890ABCDEFGHIJKLMNOPQRSTUVWYZ' ;
    Test.Write( S[ 1 ], 10 ) ;
    if( not Test.At_End ) then
    begin
        Error( 'At_End failure' ) ;
    end ;
    Test.Write( S[ 1 ], 1 ) ;
    Test.Seek( 100 ) ;
    Test.Seek( 0 ) ;
    Size := 10 ;
    Test.Read( S[ 1 ], Size ) ;
    if( Size <> 10 ) then
    begin
        Error( 'Read failure' ) ;
    end ;
    S[ 0 ] := #10 ;
    if( S <> '1234567890' ) then
    begin
        Error( 'Read failure' ) ;
    end ;
    Test.Read( S[ 1 ], Size ) ;
    if( Size <> 1 ) then
    begin
        Error( 'Read failure' ) ;
    end ;

    Test.Free ;
end ;
{$ENDIF}


function Create_File_Stream( const Name : string ;
    var UEC : TUEC ) : TCOM_File_Stream ;

var F : PSFile ;

begin
    Create_File_Stream := nil ;
    fillchar( UEC, sizeof( UEC ), 0 ) ;
{$IFDEF DELPHI}
    F := Open( Name, FSOM_Input, UEC ) ;
    if( UEC.Code <> 0 ) then
    begin
        exit ;
    end ;
    Create_File_Stream := TCOM_File_Stream.Create ;
    TCOM_File_Stream( Result )._File := F ;
{$ENDIF}
end ;



end.
