{
        Program Name : GenVal
        Package Name : Subroutine Library
        Purpose      : Simple Generic Value class definitions.
        Institution  : Conroy & Conroy Co.
        Date Written : 28-February-2005
        Written By   : Alan Conroy
        Version      : 1.0

	Copyright (C) 2005 by Alan Conroy.  Released to the Public Domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TSimple_Generic_Value is a class that provides an implementation of
        simple generic values.
}

unit GenVal ;

interface

{$I EDEFINES.INC}

uses { Layer 0... }
     _Common, { TMemory_Type }
     _GenVal, { TGeneric_Value }
     _Types ; { int8 }

type TSSimple_Generic_Number = object( TSGeneric_Number )
                                  private
                                      _Fixed : boolean ;
                                      _Signed : boolean ;
                                      _Mantissa : integer ;
                                      _Exponent : integer ;

                                  public
                                      function Fixed : boolean ; virtual ;
                                      function Signed : boolean ; virtual ;
                                      function Mantissa_Size : longint ;
                                          virtual ;
                                      function Exponent_Size : longint ;
                                          virtual ;
                                      function Fractional_Size : longint ;
                                          virtual ;

                                      function Serialize : string ;
                                      procedure Deserialize( const Image : string ) ;
                              end ;
     PSSimple_Generic_Number = ^TSSimple_Generic_Number ;

     TSSimple_Generic_Character = object( TSGeneric_Character )
                                      private
                                          _Length : integer ;
                                          _Max : integer ;
                                          _Fixed : boolean ;

                                      public
                                          function Max_Length : longint ;
                                              virtual ;
                                          function Fixed : boolean ; virtual ;
                                          function Length : longint ; virtual ;
                                          function Logical_Length : longint ;
                                              virtual ;

                                          function Serialize : string ;
                                          procedure Deserialize( const Image : string ) ;
                                  end ;
     PSSimple_Generic_Character = ^TSSimple_Generic_Character ;

     TSSimple_Type_Information = object( TSType_Information )
                                     private
                                         _Memory_Type : TMemory_Type ;
                                         _Numeric_Information : TSSimple_Generic_Number ;
                                         _Character_Information : TSSimple_Generic_Character ;
                                         _Family : TData_Type_Family ;

                                     public
                                         function Is_Composite : boolean ;
                                             virtual ;
                                         function Is_Collection : boolean ;
                                             virtual ;
                                         function Memory_Type : TMemory_Type ;
                                             virtual ;

                                         function Parameter : PSGeneric_Parameter ;
                                             virtual ;
                                         function Numeric_Information : PSGeneric_Number ;
                                             virtual ;
                                         function Character_Information : PSGeneric_Character ;
                                             virtual ;
                                         function Composite_Information : PSGeneric_Composite ;
                                             virtual ;
                                         function Collection_Information : PSGeneric_Collection ;
                                             virtual ;
                                         function Method : PSGeneric_Method ;
                                             virtual ;
                                         function _Property : PSProperty_Information ;
                                             virtual ;
                                         function Address : longint ; virtual ;

                                         function Range_Limited : boolean ;
                                             virtual ;

                                         function Low_Range : PSGeneric_Value ;
                                             virtual ;
                                         function High_Range : PSGeneric_Value ;
                                             virtual ;

                                         function Value_Name( Value : longint ) : string ;
                                             virtual ;

                                         function Serialize : string ;

                                         procedure Deserialize( const Image : string ) ;
                                 end ; { TSSimple_Type_Information }
     PSSimple_Type_Information = ^TSSimple_Type_Information ;

     TSSimple_Generic_Value = object( TSGeneric_Value )
                                  public { Constructors and destructors... }
                                      constructor Init ;
                                      destructor Done ; virtual ;

                                  private { Instance data... }
                                      _Serialize : string ;

                                      _Type_Information : TSSimple_Type_Information ;
                                      _Constant : boolean ;
                                      _Size : integer ;
                                      _Type : TValue_Data_Type ;

                                      _Value_String : string ;
                                      _Value : record
                                          case integer of
                                              1 : ( _Boolean : boolean ) ;
                                              2 : ( _Char : char ) ;
{$IFDEF DELPHI}
                                              3 : ( _uint32 : uint32 ) ;
{$ENDIF}
                                              4 : ( _int8 : int8 ) ;
                                              5 : ( _uint8 : uint8 ) ;
                                              6 : ( _int16 : int16 ) ;
                                              7 : ( _uint16 : uint16 ) ;
                                              8 : ( _int32 : int32 ) ;
                                              9 : ( _Single : single ) ;
                                              10 : ( _Double : double ) ;
                                              11 : ( _Extended : extended ) ;
                                              12 : ( _PChar : pchar ) ;
                                              13 : ( _int64 : int64 ) ;
                                              14 : ( _bytes : array[ 0..9 ] of byte )
                                      end ;

                                  public { API... }
                                      function Family : TData_Type_Family ;
                                          virtual ;
                                      function Data_Type : TValue_Data_Type ;
                                          virtual ;
                                      function User_Data_Type_Name : string ;
                                          virtual ;
                                      function Name : string ; virtual ;
                                      function Type_Information : PSType_Information ;
                                          virtual ;

                                      function Constant : boolean ; virtual ;
                                      function Size : longint ; virtual ;
                                      function Total_Size : longint ; virtual ;

                                      function Dereference : PSGeneric_Value ;
                                          virtual ;

                                      function Can_Convert( Typ : TValue_Data_Type ) : boolean ;
                                          virtual ;
                                      function Convert( Class_Type : PSGeneric_Value ;
                                          var Loss : boolean ) : PSGeneric_Value ;
                                          virtual ;

				      function Clone : PSGeneric_Value ;
                                          virtual ;

                                      procedure Simple_Cast( Typ : TValue_Data_Type ) ;
                                          virtual ;
                                      procedure Cast( Typ : PSGeneric_Value ) ;
                                          virtual ;

                                      procedure Deserialize( Value : PChar ) ;
                                          virtual ;
                                      function Serialize : PChar ; virtual ;

                                      procedure Set_Null ; virtual ;

                                      procedure Get_Undefined( Buffer : pointer ;
                                          Len : longint ) ; virtual ;
                                      procedure Set_Undefined( Buffer : pointer ;
                                          Len : longint ) ; virtual ;

                                      function Get_Boolean : boolean ; virtual ;
                                      procedure Set_Boolean( B : boolean ) ;
                                          virtual ;

                                      function Get_ByteBool : bytebool ;
                                          virtual ;
                                      procedure Set_ByteBool( B : bytebool ) ;
                                          virtual ;

                                      function Get_WordBool : wordbool ;
                                          virtual ;
                                      procedure Set_WordBool( B : wordbool ) ;
                                          virtual ;

                                      function Get_LongBool : longbool ;
                                          virtual ;
                                      procedure Set_LongBool( B : longbool ) ;
                                          virtual ;

                                      function Get_Char : char ; virtual ;
                                      procedure Set_Char( C : char ) ; virtual ;

                                      function Get_String : string ; virtual ;
                                      procedure Set_String( const S : string ) ;
                                          virtual ;

                                      function Get_PChar : PChar ; virtual ;
                                      procedure Set_PChar( P : PChar ) ;
                                          virtual ;

                                      function Get_Integer8 : int8 ; virtual ;
                                      procedure Set_Integer8( I : int8 ) ;
                                          virtual ;

                                      function Get_Integer16 : int16 ; virtual ;
                                      procedure Set_Integer16( I : int16 ) ;
                                          virtual ;

                                      function Get_Integer32 : int32 ; virtual ;
                                      procedure Set_Integer32( I : int32 ) ;
                                          virtual ;

                                      function Get_Integer64 : int64 ;
                                          virtual ;
                                      procedure Set_Integer64( I : int64 ) ;
                                          virtual ;

                                      function Get_Unsigned_Integer8 : uint8 ;
                                          virtual ;
                                      procedure Set_Unsigned_Integer8( I : uint8 ) ;
                                          virtual ;

                                      function Get_Unsigned_Integer16 : uint16 ;
                                          virtual ;
                                      procedure Set_Unsigned_Integer16( I : uint16 ) ;
                                          virtual ;

{$IFDEF DELPHI}
                                      function Get_Unsigned_Integer32 : uint32 ;
                                          virtual ;
                                      procedure Set_Unsigned_Integer32( I : uint32 ) ;
                                          virtual ;
{$ENDIF}

                                      function Get_Single : single ; virtual ;
                                      procedure Set_Single( S : single ) ;
                                          virtual ;

                                      function Get_Double : double ; virtual ;
                                      procedure Set_Double( D : double ) ;
                                          virtual ;

                                      function Get_Extended : extended ;
                                          virtual ;
                                      procedure Set_Extended( E : extended ) ;
                                          virtual ;
                              end ;
     PSSimple_Generic_Value = ^TSSimple_Generic_Value ;

{$IFDEF DELPHI}
type TSimple_Generic_Number = class( TGeneric_Number )
                                  private
                                      _Reference_Count : integer ;

                                      _Number : PSGeneric_Number ;

                                      _Fixed : boolean ;
                                      _Signed : boolean ;
                                      _Mantissa : integer ;
                                      _Exponent : integer ;

                                  public
                                      procedure Attach ; override ;
                                      procedure Detach ; override ;
                                      function Fixed : boolean ; override ;
                                      function Signed : boolean ; override ;
                                      function Mantissa_Size : longint ;
                                          override ;
                                      function Exponent_Size : longint ;
                                          override ;
                                      function Fractional_Size : longint ;
                                          override ;
                              end ;

     TSimple_Generic_Character = class( TGeneric_Character )
                                     private
                                         _Reference_Count : integer ;

                                         _Char : PSGeneric_Character ;

                                         _Length : integer ;
                                         _Max : integer ;
                                         _Fixed : boolean ;

                                     public
                                         procedure Attach ; override ;
                                         procedure Detach ; override ;
                                         function Max_Length : longint ;
                                             override ;
                                         function Fixed : boolean ; override ;
                                         function Length : longint ; override ;
                                         function Logical_Length : longint ;
                                             override ;
                                 end ;

     TSimple_Type_Information = class( TType_Information )
                                    private
                                        _Reference_Count : integer ;
                                        _Type : PSSimple_Type_Information ;

                                    public
                                        procedure Attach ; override ;
                                        procedure Detach ; override ;
                                        function Is_Composite : boolean ;
                                            override ;
                                        function Is_Collection : boolean ;
                                            override ;
                                        function Memory_Type : TMemory_Type ;
                                            override ;

                                        function Parameter : TGeneric_Parameter ;
                                            override ;
                                        function Numeric_Information : TGeneric_Number ;
                                            override ;
                                        function Character_Information : TGeneric_Character ;
                                            override ;
                                        function Composite_Information : TGeneric_Composite ;
                                            override ;
                                        function Collection_Information : TGeneric_Collection ;
                                            override ;
                                        function Method : TGeneric_Method ;
                                            override ;
                                        function _Property : TProperty_Information ;
                                            override ;
                                        function Address : longint ; override ;

                                        function Range_Limited : boolean ;
                                            override ;

                                        function Low_Range : TGeneric_Value ;
                                            override ;
                                        function High_Range : TGeneric_Value ;
                                            override ;

                                        function Value_Name( Value : longint ) : string ;
                                            override ;
                                end ;

     TSimple_Generic_Value = class( TGeneric_Value )
                                 private { Instance data... }
                                     _Reference_Count : integer ;
                                     _Value : PSGeneric_Value ;

                                 public { API... }
                                     procedure Attach ; override ;
                                     procedure Detach ; override ;

                                     function Family : TData_Type_Family ;
                                         override ;
                                     function Data_Type : TValue_Data_Type ;
                                         override ;
                                     function User_Data_Type_Name : string ;
                                         override ;
                                     function Name : string ; override ;
                                     function Type_Information : TType_Information ;
                                         override ;

                                     function Constant : boolean ; override ;
                                     function Size : longint ; override ;
                                     function Total_Size : longint ; override ;

                                     function Dereference : TGeneric_Value ;
                                         override ;

                                     function Can_Convert( Typ : TValue_Data_Type ) : boolean ;
                                         override ;
                                     function Convert( Class_Type : TGeneric_Value ;
                                         var Loss : boolean ) : TGeneric_Value ;
                                         override ;

			             function Clone : TGeneric_Value ;
                                         override ;

                                     procedure Simple_Cast( Typ : TValue_Data_Type ) ;
                                         override ;
                                     procedure Cast( Typ : TGeneric_Value ) ;
                                         override ;

                                     procedure Deserialize( Value : PChar ) ;
                                         override ;
                                     function Serialize : PChar ; override ;

                                     procedure Set_Null ; override ;

                                     procedure Get_Undefined( Buffer : pointer ;
                                         Len : longint ) ; override ;
                                     procedure Set_Undefined( Buffer : pointer ;
                                         Len : longint ) ; override ;

                                     function Get_Boolean : boolean ; override ;
                                     procedure Set_Boolean( B : boolean ) ;
                                         override ;

                                     function Get_ByteBool : bytebool ;
                                         override ;
                                     procedure Set_ByteBool( B : bytebool ) ;
                                         override ;

                                     function Get_WordBool : wordbool ;
                                         override ;
                                     procedure Set_WordBool( B : wordbool ) ;
                                         override ;

                                     function Get_LongBool : longbool ;
                                         override ;
                                     procedure Set_LongBool( B : longbool ) ;
                                         override ;

                                     function Get_Char : char ; override ;
                                     procedure Set_Char( C : char ) ; override ;

                                     function Get_String : string ; override ;
                                     procedure Set_String( const S : string ) ;
                                         override ;

                                     function Get_PChar : PChar ; override ;
                                     procedure Set_PChar( P : PChar ) ;
                                         override ;

                                     function Get_Integer8 : int8 ; override ;
                                     procedure Set_Integer8( I : int8 ) ;
                                         override ;

                                     function Get_Integer16 : int16 ; override ;
                                     procedure Set_Integer16( I : int16 ) ;
                                         override ;

                                     function Get_Integer32 : int32 ; override ;
                                     procedure Set_Integer32( I : int32 ) ;
                                         override ;

                                     function Get_Integer64 : int64 ; override ;
                                     procedure Set_Integer64( I : int64 ) ;
                                         override ;

                                     function Get_Unsigned_Integer8 : uint8 ;
                                         override ;
                                     procedure Set_Unsigned_Integer8( I : uint8 ) ;
                                         override ;

                                     function Get_Unsigned_Integer16 : uint16 ;
                                         override ;
                                     procedure Set_Unsigned_Integer16( I : uint16 ) ;
                                         override ;

                                     function Get_Unsigned_Integer32 : uint32 ;
                                         override ;
                                     procedure Set_Unsigned_Integer32( I : uint32 ) ;
                                         override ;

                                     function Get_Single : single ; override ;
                                     procedure Set_Single( S : single ) ;
                                         override ;

                                     function Get_Double : double ; override ;
                                     procedure Set_Double( D : double ) ;
                                         override ;

                                     function Get_Extended : extended ;
                                         override ;
                                     procedure Set_Extended( E : extended ) ;
                                         override ;
                             end ;

     TCOM_Simple_Generic_Number = class( TCOM_Generic_Number )
                                     private
                                         _Reference_Count : integer ;

                                         _Number : PSGeneric_Number ;

                                         _Fixed : boolean ;
                                         _Signed : boolean ;
                                         _Mantissa : integer ;
                                         _Exponent : integer ;

                                      public
                                          procedure Attach ; override ;
                                          procedure Detach ; override ;
                                          function Fixed : boolean ; override ;
                                          function Signed : boolean ; override ;
                                          function Mantissa_Size : longint ;
                                              override ;
                                          function Exponent_Size : longint ;
                                              override ;
                                          function Fractional_Size : longint ;
                                              override ;
                                  end ;

     TCOM_Simple_Generic_Character = class( TCOM_Generic_Character )
                                         private
                                             _Reference_Count : integer ;

                                             _Char : PSGeneric_Character ;

                                             _Length : integer ;
                                             _Max : integer ;
                                             _Fixed : boolean ;

                                         public
                                             procedure Attach ; override ;
                                             procedure Detach ; override ;
                                             function Max_Length : longint ;
                                                 override ;
                                             function Fixed : boolean ;
                                                 override ;
                                             function Length : longint ;
                                                 override ;
                                             function Logical_Length : longint ;
                                                 override ;
                                     end ;

     TCOM_Simple_Type_Information = class( TCOM_Type_Information )
                                        private
                                            _Type : PSSimple_Type_Information ;

                                        public
                                            function Is_Composite : boolean ;
                                                override ;
                                            function Is_Collection : boolean ;
                                                override ;
                                            function Memory_Type : TMemory_Type ;
                                                override ;

                                            function Parameter : TCOM_Generic_Parameter ;
                                                override ;
                                            function Numeric_Information : TCOM_Generic_Number ;
                                                override ;
                                            function Character_Information : TCOM_Generic_Character ;
                                                override ;
                                            function Composite_Information : TCOM_Generic_Composite ;
                                                override ;
                                            function Collection_Information : TCOM_Generic_Collection ;
                                                override ;
                                            function Method : TCOM_Generic_Method ;
                                                override ;
                                            function _Property : TCOM_Property_Information ;
                                                override ;
                                            function Address : longint ;
                                                override ;

                                            function Range_Limited : boolean ;
                                                override ;

                                            function Low_Range : TCOM_Generic_Value ;
                                                override ;
                                            function High_Range : TCOM_Generic_Value ;
                                                override ;

                                            function Value_Name( Value : longint ) : string ;
                                                override ;
                                    end ;

     TCOM_Simple_Generic_Value = class( TCOM_Generic_Value )
                                     private { Instance data... }
                                         _Reference_Count : integer ;
                                         _Value : PSGeneric_Value ;

                                     public { API... }
                                         procedure Attach ; override ;
                                         procedure Detach ; override ;

                                         function Family : TData_Type_Family ;
                                             override ;
                                         function Data_Type : TValue_Data_Type ;
                                             override ;
                                         function User_Data_Type_Name : string ;
                                             override ;
                                         function Name : string ;
                                             override ;
                                         function Type_Information : TCOM_Type_Information ;
                                             override ;

                                         function Constant : boolean ;
                                             override ;
                                         function Size : longint ; override ;
                                         function Total_Size : longint ;
                                             override ;

                                         function Dereference : TCOM_Generic_Value ;
                                             override ;

                                         function Can_Convert( Typ : TValue_Data_Type ) : boolean ;
                                             override ;
                                         function Convert( Class_Type : TCOM_Generic_Value ;
                                             var Loss : boolean ) : TCOM_Generic_Value ;
                                             override ;

				         function Clone : TCOM_Generic_Value ;
                                             override ;

                                         procedure Simple_Cast( Typ : TValue_Data_Type ) ;
                                             override ;
                                         procedure Cast( Typ : TCOM_Generic_Value ) ;
                                             override ;

                                         procedure Deserialize( Value : PChar ) ;
                                             override ;
                                         function Serialize : PChar ;
                                            override ;

                                         procedure Set_Null ; override ;

                                         procedure Get_Undefined( Buffer : pointer ;
                                             Len : longint ) ; override ;
                                         procedure Set_Undefined( Buffer : pointer ;
                                             Len : longint ) ; override ;

                                         function Get_Boolean : boolean ;
                                             override ;
                                         procedure Set_Boolean( B : boolean ) ;
                                             override ;

                                         function Get_ByteBool : bytebool ;
                                             override ;
                                         procedure Set_ByteBool( B : bytebool ) ;
                                             override ;

                                         function Get_WordBool : wordbool ;
                                             override ;
                                         procedure Set_WordBool( B : wordbool ) ;
                                             override ;

                                         function Get_LongBool : longbool ;
                                             override ;
                                         procedure Set_LongBool( B : longbool ) ;
                                             override ;

                                         function Get_Char : char ; override ;
                                         procedure Set_Char( C : char ) ;
                                             override ;

                                         function Get_String : string ;
                                             override ;
                                         procedure Set_String( const S : string ) ;
                                             override ;

                                         function Get_PChar : PChar ; override ;
                                         procedure Set_PChar( P : PChar ) ;
                                             override ;

                                         function Get_Integer8 : int8 ;
                                             override ;
                                         procedure Set_Integer8( I : int8 ) ;
                                             override ;

                                         function Get_Integer16 : int16 ;
                                             override ;
                                         procedure Set_Integer16( I : int16 ) ;
                                             override ;

                                         function Get_Integer32 : int32 ;
                                             override ;
                                         procedure Set_Integer32( I : int32 ) ;
                                             override ;

                                         function Get_Integer64 : int64 ;
                                             override ;
                                         procedure Set_Integer64( I : int64 ) ;
                                             override ;

                                         function Get_Unsigned_Integer8 : uint8 ;
                                             override ;
                                         procedure Set_Unsigned_Integer8( I : uint8 ) ;
                                             override ;

                                         function Get_Unsigned_Integer16 : uint16 ;
                                             override ;
                                         procedure Set_Unsigned_Integer16( I : uint16 ) ;
                                             override ;

                                         function Get_Unsigned_Integer32 : uint32 ;
                                             override ;
                                         procedure Set_Unsigned_Integer32( I : uint32 ) ;
                                             override ;

                                         function Get_Single : single ;
                                             override ;
                                         procedure Set_Single( S : single ) ;
                                             override ;

                                         function Get_Double : double ;
                                             override ;
                                         procedure Set_Double( D : double ) ;
                                             override ;

                                         function Get_Extended : extended ;
                                             override ;
                                         procedure Set_Extended( E : extended ) ;
                                             override ;
                                 end ; { TCOM_Simple_Generic_Value }
{$ENDIF}

implementation

uses { Borland... }
{$IFDEF DELPHI}
     SysUtils, { strlen }
{$ELSE}
     Strings, { strlen }
{$ENDIF}

     { C&C... }
     ASCIIDef, { NUL }
     CommonUt, { Boolean_To_String }
     Compatib, { Set_Length }
     CVT, { CVTB }
     HTML, { TSXML_Parser }
     Num1s, { Num1 }
     UStrings ; { String_To_Boolean }

{ TSSimple_Generic_Number methods... }

function TSSimple_Generic_Number.Fixed : boolean ;

begin
    Fixed := _Fixed ;
end ;


function TSSimple_Generic_Number.Signed : boolean ;

begin
    Signed := _Signed ;
end ;


function TSSimple_Generic_Number.Mantissa_Size : longint ;

begin
    Mantissa_Size := _Mantissa ;
end ;


function TSSimple_Generic_Number.Exponent_Size : longint ;

begin
    Exponent_Size := _Exponent ;
end ;


function TSSimple_Generic_Number.Fractional_Size : longint ;

begin
    Fractional_Size := 0 ;
end ;


function TSSimple_Generic_Number.Serialize : string ;

var Res : string ;

begin
    if( _Fixed ) then
    begin
        Res := Res + '<fixed/>' ;
    end ;
    if( _Signed ) then
    begin
        Res := Res + '<signed/>' ;
    end ;
    Res := Res + '<mantissa>' + Num1( _Mantissa ) + '<mantissa/' ;
    Res := Res + '<exponent>' + Num1( _Exponent ) + '<exponent/>' ;
    Serialize := Res ;
end ;


procedure TSSimple_Generic_Number.Deserialize( const Image : string ) ;

var Dummy : integer ;
    Parser : TSXML_Parser ;
    S : string ;

begin
    { Default values... }
    _Fixed := False ;
    _Signed := False ;

    { Initialize parser... }
    Parser.Init ;
    Parser.Source.Set_String( Image ) ;

    { Deserialize... }
    while( not Parser.EOF ) do
    begin
        S := Edit( Parser.Get_Token_String, 32 ) ;
        if( S = '<FIXED/>' ) then
        begin
            _Fixed := True ;
        end else
        if( S = '<SIGNED/>' ) then
        begin
            _Signed := True ;
        end else
        if( S = '<MANTISSA>' ) then
        begin
            S := Parser.Get_Section_String( 'mantissa' ) ;
            val( S, _Mantissa, Dummy ) ;
        end else
        if( S = '<EXPONENT>' ) then
        begin
            S := Parser.Get_Section_String( 'exponent' ) ;
            val( S, _Exponent, Dummy ) ;
        end else
        begin { Something we don't know about }
            if( pos( '/>', S ) = 0 ) then
            begin
                S := Parser.Get_Section_String( S ) ;
            end ;
        end ;
    end ;

    { Clean up }
    Parser.Done ;
end ; { TSSimple_Generic_Number.Deserialize }



{ TSSimple_Generic_Character methods... }

function TSSimple_Generic_Character.Max_Length : longint ;

begin
    Max_Length := _Max ;
end ;


function TSSimple_Generic_Character.Fixed : boolean ;

begin
    Fixed := _Fixed ;
end ;


function TSSimple_Generic_Character.Length : longint ;

begin
    Length := _Length ;
end ;


function TSSimple_Generic_Character.Logical_Length : longint ;

begin
    Logical_Length := Length ;
end ;


function TSSimple_Generic_Character.Serialize : string ;

var Res : string ;

begin
    if( _Fixed ) then
    begin
        Res := '<fixed/>' ;
    end ;
    Res := Res + '<length>' + Num1( _Length ) + '<length/><max>' + Num1( _Max ) + '<max/>' ;
end ;


procedure TSSimple_Generic_Character.Deserialize( const Image : string ) ;

var Dummy : integer ;
    Parser : TSXML_Parser ;
    S : string ;

begin
    { Default values... }
    _Fixed := False ;

    { Initialize parser... }
    Parser.Init ;
    Parser.Source.Set_String( Image ) ;

    { Deserialize... }
    while( not Parser.EOF ) do
    begin
        S := Edit( Parser.Get_Token_String, 32 ) ;
        if( S = '<FIXED/>' ) then
        begin
            _Fixed := True ;
        end else
        if( S = '<LENGTH>' ) then
        begin
            S := Parser.Get_Section_String( 'length' ) ;
            val( S, _Length, Dummy ) ;
        end else
        if( S = '<MAX>' ) then
        begin
            S := Parser.Get_Section_String( 'max' ) ;
            val( S, _Max, Dummy ) ;
        end else
        begin { Something we don't know about }
            if( pos( '/>', S ) = 0 ) then
            begin
                S := Parser.Get_Section_String( S ) ;
            end ;
        end ;
    end ;

    { Clean up }
    Parser.Done ;
end ; { TSSimple_Generic_Character.Deserialize }



{ TSSimple_Type_Information methods... }

function TSSimple_Type_Information.Is_Composite : boolean ;

begin
    Is_Composite := False ;
end ;


function TSSimple_Type_Information.Is_Collection : boolean ;

begin
    Is_Collection := False ;
end ;


function TSSimple_Type_Information.Memory_Type : TMemory_Type ;

begin
    Memory_Type := _Memory_Type ;
end ;


function TSSimple_Type_Information.Parameter : PSGeneric_Parameter ;

begin
    Parameter := nil ;
end ;


function TSSimple_Type_Information.Numeric_Information : PSGeneric_Number ;

begin
    if( _Family = DTF_Numeric ) then
    begin
        Numeric_Information := @_Numeric_Information ;
    end else
    begin
        Numeric_Information := nil ;
    end ;
end ;


function TSSimple_Type_Information.Character_Information : PSGeneric_Character ;

begin
    if( _Family = DTF_Character ) then
    begin
        Character_Information := @_Character_Information ;
    end else
    begin
        Character_Information := nil ;
    end ;
end ;


function TSSimple_Type_Information.Composite_Information : PSGeneric_Composite ;

begin
    Composite_Information := nil ;
end ;


function TSSimple_Type_Information.Collection_Information : PSGeneric_Collection ;

begin
    Collection_Information := nil ;
end ;


function TSSimple_Type_Information.Method : PSGeneric_Method ;

begin
    Method := nil ;
end ;


function TSSimple_Type_Information._Property : PSProperty_Information ;

begin
    _Property := nil ;
end ;


function TSSimple_Type_Information.Address : longint ;

begin
    Address := 0 ;
end ;


function TSSimple_Type_Information.Range_Limited : boolean ;

begin
    Range_Limited := False ;
end ;


function TSSimple_Type_Information.Low_Range : PSGeneric_Value ;

begin
    Low_Range := nil ;
end ;


function TSSimple_Type_Information.High_Range : PSGeneric_Value ;

begin
    High_Range := nil ;
end ;


function TSSimple_Type_Information.Value_Name( Value : longint ) : string ;

begin
    Value_Name := '' ;
end ;


function TSSimple_Type_Information.Serialize : string ;

var Res : string ;

begin
    case _Memory_Type of
        MT_Heap : Res := '<memtype>heap<memtype/>' ;
        MT_Data : Res := '<memtype>data<memtype/>' ;
        MT_Stack : Res := '<memtype>stack<memtype/>' ;
        MT_Code : Res := '<memtype>code<memtype/>' ;
        else Res := '<memtype>unknown<memtype/>' ;
    end ;
    case _Family of
        DTF_Undefined : Res := Res + '<family>undefined<family/>' ;
        DTF_Logical : Res := Res + '<family>logical<family/>' ;
        DTF_Numeric : Res := Res + '<family>numeric<family/>' ;
        DTF_Character : Res := Res + '<family>character<family/>' ;
        DTF_Pointer : Res := Res + '<family>pointer<family/>' ;
        DTF_Composite : Res := Res + '<family>composite<family/>' ;
        DTF_Collection : Res := Res + '<family>collection<family/>' ;
        DTF_User_Defined : Res := Res + '<family>user<family/>' ;
        DTF_Meta : Res := Res + '<family>meta<family/>' ;
        DTF_Other : Res := Res + '<family>other<family/>' ;
        else Res := Res + '<family>unknown<family/>' ;
    end ;
    if( _Family = DTF_Numeric ) then
    begin
        Res := Res + '<info>' + _Numeric_Information.Serialize + '<info/>' ;
    end else
    if( _Family = DTF_Character ) then
    begin
        Res := Res + '<info>' + _Character_Information.Serialize + '<info/>' ;
    end ;
    Serialize := Res ;
end ;


procedure TSSimple_Type_Information.Deserialize( const Image : string ) ;

var Parser : TSXML_Parser ;
    S : string ;

begin
    { Initialize parser... }
    Parser.Init ;
    Parser.Source.Set_String( Image ) ;

    { Deserialize... }
    while( not Parser.EOF ) do
    begin
        S := Edit( Parser.Get_Token_String, 32 ) ;
        if( S = '<MEMTYPE>' ) then
        begin
            S := Edit( Parser.Get_Section_String( 'memtype' ), 32 ) ;
	    if( S = 'HEAP' ) then
            begin
                _Memory_Type := MT_Heap ;
            end else
	    if( S = 'DATA' ) then
            begin
                _Memory_Type := MT_Data ;
            end else
	    if( S = 'STACK' ) then
            begin
                _Memory_Type := MT_Stack ;
            end else
	    if( S = 'CODE' ) then
            begin
                _Memory_Type := MT_Code ;
            end else
            begin
                _Memory_Type := MT_Unknown ;
            end ;
        end else
        if( S = '<FAMILY>' ) then
        begin
            S := Edit( Parser.Get_Section_String( 'family' ), 32 ) ;
            if( S = 'UNDEFINED' ) then
            begin
                _Family := DTF_Undefined ;
            end else
            if( S = 'LOGICAL' ) then
            begin
                _Family := DTF_Logical ;
            end else
            if( S = 'NUMERIC' ) then
            begin
                _Family := DTF_Numeric ;
            end else
            if( S = 'CHARACTER' ) then
            begin
                _Family := DTF_Character ;
            end else
            if( S = 'POINTER' ) then
            begin
                _Family := DTF_Pointer ;
            end else
            if( S = 'COMPOSITE' ) then
            begin
                _Family := DTF_Composite ;
            end else
            if( S = 'COLLECTION' ) then
            begin
                _Family := DTF_Collection ;
            end else
            if( S = 'USER' ) then
            begin
                _Family := DTF_User_Defined ;
            end else
            if( S = 'META' ) then
            begin
                _Family := DTF_Meta ;
            end else
            if( S = 'OTHER' ) then
            begin
                _Family := DTF_Other ;
            end else
            begin
                _Family := DTF_Unknown ;
            end ;
        end else
        if( S = '<INFO>' ) then
        begin
            S := Edit( Parser.Get_Section_String( 'info' ), 32 ) ;
            if( _Family = DTF_Numeric ) then
            begin
                _Numeric_Information.Deserialize( S ) ;
            end else
            if( _Family = DTF_Character ) then
            begin
                _Character_Information.Deserialize( S ) ;
            end ;
        end else
        begin { Something we don't know about }
            if( pos( '/>', S ) = 0 ) then
            begin
                S := Parser.Get_Section_String( S ) ;
            end ;
        end ;
    end ; { while( not Parser.EOF ) }

    { Clean up }
    Parser.Done ;
end ; { TSSimple_Type_Information.Deserialize }



{ TSSimpleGeneric_Value methods... }

{ Cosntructors and destructors... }

constructor TSSimple_Generic_Value.Init ;

begin
    inherited Init ;

    _Type := VDT_Empty ;
    _Type_Information.Init ;
    _Type_Information._Numeric_Information.Init ;
    _Type_Information._Character_Information.Init ;
    _Type_Information._Family := DTF_Unknown ;
    _Constant := True ;
    _Size := 0 ;
{$IFDEF DELPHI}
    initialize( _Serialize ) ;
    initialize( _Value_String ) ;
{$ELSE}
    _Serialize := '' ;
    _Value_String := '' ;
{$ENDIF}
end ;


constructor TSSimple_Generic_Value.Done ;

begin
{$IFDEF DELPHI}
    finalize( _Serialize ) ;
    finalize( _Value_String ) ;
{$ENDIF}

    inherited Done ;
end ;


{ API... }

function TSSimple_Generic_Value.Family : TData_Type_Family ;

begin
    case _Type of
        VDT_Empty : Family := DTF_Unknown ;
        VDT_Null : Family := DTF_Other ;
        VDT_Boolean : Family := DTF_Logical ;
        VDT_Integer : Family := DTF_Numeric ;
        VDT_Real : Family := DTF_Numeric ;
        VDT_String : Family := DTF_Character ;
        else Family := DTF_Unknown ;
    end ;
end ;


function TSSimple_Generic_Value.Data_Type : TValue_Data_Type ;

begin
    Data_Type := _Type ;
end ;


function TSSimple_Generic_Value.User_Data_Type_Name : string ;

begin
    User_Data_Type_Name := '' ;
end ;


function TSSimple_Generic_Value.Name : string ;

begin
    Name := '' ;
end ;


function TSSimple_Generic_Value.Type_Information : PSType_Information ;

begin
    if( ( _Type = VDT_Empty ) or ( _Type = VDT_Unknown ) ) then
    begin
        Type_Information := nil ;
    end else
    begin
        Type_Information := @_Type_Information ;
    end ;
end ;


function TSSimple_Generic_Value.Constant : boolean ;

begin
    Constant := _Constant ;
end ;


function TSSimple_Generic_Value.Size : longint ; { Size in bytes }

begin
    Size := _Size ;
end ;


function TSSimple_Generic_Value.Total_Size : longint ;

begin
    Total_Size := _Size ;
    if( _Type = VDT_String ) then
    begin
{$IFDEF DELPHI}
        Total_Size := _Size + 4 ;
{$ELSE}
        Total_Size := _Size + 1 ; 
{$ENDIF}
    end else
    if( _Type = VDT_Pointer ) then { For now, the only pointer we support is PChar }
    begin
        Total_Size := _Size + strlen( _Value._PChar ) ;
    end ;
end ;


function TSSimple_Generic_Value.Dereference : PSGeneric_Value ;

begin
    Dereference := nil ;
end ;


var Supported_Conversion : array[ TValue_Data_Type, TValue_Data_Type ] of boolean ;

function TSSimple_Generic_Value.Can_Convert( Typ : TValue_Data_Type ) : boolean ;

var Res : boolean ;

begin
    Res := Supported_Conversion[ _Type, Typ ] ;
    if( Res ) then
    begin
        
    end ;
    Can_Convert := Res ;
end ;


function TSSimple_Generic_Value.Convert( Class_Type : PSGeneric_Value ;
    var Loss : boolean ) : PSGeneric_Value ;

var B : boolean ;
    Dummy : integer ;
    E : extended ;
    I : int64 ;
    Res : PSGeneric_Value ;
    S : string ;

begin
    Convert := nil ;
    Loss := False ;
    if( ( Class_Type <> nil ) and ( Can_Convert( Class_Type^.Data_Type ) ) ) then
    begin
        Res := new( PSSimple_Generic_Value, Init ) ;
        Convert := Res ;
        case Class_Type^.Data_Type of
            VDT_Empty : ; { Already empty - do nothing }
            VDT_Null : Res^.Set_Null ;
            VDT_Boolean :
                begin
                    case _Type of
                        VDT_Boolean : B := Get_Boolean ;
                        VDT_Integer : B := ( Get_Integer32 <> 0 ) ;
                        VDT_Real : B := ( Get_Extended <> 0.0 ) ;
                        VDT_String : B := String_To_Boolean( Get_String ) ;
                        VDT_Char : B := ( pos( Get_Char, '1THOXthox' ) > 0 ) ;
                    end ;
                    case Class_Type^.Size of
                        1 : Res.Set_ByteBool( B ) ;
                        2 : Res.Set_WordBool( B ) ;
                        4 : Res.Set_LongBool( B ) ;
                        else Res.Set_Boolean( B ) ;
                    end ;
                end ;
            VDT_Integer : { Converting to integer }
                begin
                    case _Type of
                        VDT_Boolean :
                            if( Get_LongBool ) then
                            begin
                                I := -1 ;
                            end else
                            begin
                                I := 0 ;
                            end ;
                        VDT_Integer : I := Get_Integer64 ;
                        VDT_Real : I := trunc( Get_Extended ) ;
                        VDT_String : val( Get_String, I, Dummy ) ;
                        VDT_Char : val( Get_Char, I, Dummy ) ;
                    end ;
                    case Class_Type^.Size of
                        1 : Res.Set_integer8( I ) ;
                        2 : Res.Set_integer16( I ) ;
                        4 : Res.Set_integer32( I ) ;
                        else Res.Set_integer64( I ) ;
                    end ;
                    Loss := ( I <> Get_Integer64 ) ;
                end ;
            VDT_Real : { Converting to real }
                begin
                    case _Type of
                        VDT_Boolean :
                            if( Get_LongBool ) then
                            begin
                                E := -1 ;
                            end else
                            begin
                                E := 0 ;
                            end ;
                        VDT_Integer : E := Get_Integer64 ;
                        VDT_Real : E := Get_Extended ;
                        VDT_String : val( Get_String, E, Dummy ) ;
                        VDT_Char : val( Get_Char, E, Dummy ) ;
                    end ;
                    case Class_Type^.Size of
                        4 : Res.Set_Single( E ) ;
                        8 : Res.Set_Double( E ) ;
                        else Res.Set_Extended( E ) ;
                    end ;
                    Loss := ( E <> Get_Extended ) ;
                end ;
            VDT_String :
                begin
                    case _Type of
                        VDT_Boolean : Res.Set_String( Boolean_To_String( Get_Boolean ) ) ;
                        VDT_Integer : if( Size = 64 ) then
                                      begin
{$IFDEF DELPHI}
                                          Res.Set_String( inttostr( Get_Integer64 ) ) ;
{$ELSE}
                                          Res.Set_String( Num2( Get_Integer64 ) ) ;
{$ENDIF}
                                      end else
                                      begin
                                          Res.Set_String( Num( Get_Integer32 ) ) ;
                                      end ;
                        VDT_Real : Res.Set_String( Num2( Get_Extended ) ) ;
                        VDT_String : Res.Set_String( Get_String ) ;
                        VDT_Char : Res.Set_String( Get_Char ) ;
                    end ;
                end ;
            VDT_Char : { Converting to char }
                begin
                    case _Type of
                        VDT_Boolean : S := Boolean_To_String( Get_Boolean ) ;
                        VDT_Integer : if( Size = 64 ) then
                                      begin
{$IFDEF DELPHI}
                                          S := inttostr( Get_Integer64 ) ;
{$ELSE}
                                          S := Num2( Get_Integer64 ) ;
{$ENDIF}
                                      end else
                                      begin
                                          S := Num( Get_Integer32 ) ;
                                      end ;
                        VDT_Real : S := Num2( Get_Extended ) ;
                        VDT_String : S := Get_String ;
                        VDT_Char : S := Get_Char ;
                    end ;
                    if( length( S ) = 0 ) then
                    begin
                        Res.Set_Null ;
                    end else
                    begin
                        Res.Set_Char( S[ 1 ] ) ;
                    end ;
                    Loss := ( length( S ) > 1 ) ;
                end ;
        end ; { case Class_Type^.Data_Type }
    end ; { if }
end ; { TSSimple_Generic_Value.Convert }


function TSSimple_Generic_Value.Clone : PSGeneric_Value ;

var Res : PSSimple_Generic_Value ;

begin
    Res := new( PSSimple_Generic_Value, Init ) ;
    case _Type of
        VDT_Null : Res^.Set_Null ;
        VDT_Boolean :
            begin
                case _Size of
                    1 : Res^.Set_ByteBool( _Value._Boolean ) ;
                    2 : Res^.Set_WordBool( _Value._Boolean ) ;
                    4 : Res^.Set_LongBool( _Value._Boolean ) ;
                end ;
            end ;
        VDT_Integer :
            begin
                if( _Type_Information._Numeric_Information.Signed ) then
                begin
                    case _Size of
                        1 : Res^.Set_Integer8( _Value._int8 ) ;
                        2 : Res^.Set_Integer16( _Value._int16 ) ;
                        4 : Res^.Set_Integer32( _Value._int32 ) ;
                        8 : Res^.Set_Integer64( _Value._int64 ) ;
                    end ;
                end else
                begin
                    case _Size of
                        1 : Res^.Set_Unsigned_Integer8( _Value._uint8 ) ;
                        2 : Res^.Set_Unsigned_Integer16( _Value._uint16 ) ;
                        4 : Res^.Set_Unsigned_Integer32( _Value._uint32 ) ;
                    end ;
                end ;
            end ;
        VDT_Real :
            begin
                case _Size of
                    4 : Res^.Set_Single( _Value._Single ) ;
                    8 : Res^.Set_Double( _Value._Double ) ;
                    10 : Res^.Set_Extended( _Value._Extended ) ;
                end ;
            end ;
        VDT_String : Res^.Set_String( _Value_String ) ;
        VDT_Char : Res^.Set_Char( _Value._Char ) ;
        VDT_Pointer : Res^.Set_PChar( _Value._PChar ) ;
    end ;
    Clone := Res ;
end ; { TSSimple_Generic_Value.Clone }


procedure TSSimple_Generic_Value.Simple_Cast( Typ : TValue_Data_Type ) ;

    procedure Adjust_Size( Max : integer ) ;

    begin
        case _Size of
            3 : _Size := 2 ;
            5..7 : _Size := 4 ;
            9 : _Size := 8 ;
            11..32767 : _Size := 10 ;
        end ;
        if( _Size > Max ) then
        begin
            _Size := Max ;
        end ;
    end ;

var Len : integer ;

begin
    if( Typ <> _Type ) then
    begin
        if( _Type = VDT_String ) then { Converting from string }
        begin
            Len := length( _Value_String ) ;
            if( Len > sizeof( _Value ) ) then
            begin
                Len := sizeof( _Value ) ;
            end ;
            move( _Value_String[ 1 ], _Value, Len ) ;
            _Value_String := '' ;
            _Size := Len ;
            case Typ of
                VDT_Boolean : Adjust_Size( 4 ) ;
                VDT_Integer : Adjust_Size( 8 ) ;
                VDT_Real :
                    begin
                        Adjust_Size( 10 ) ;
                    end ;
                VDT_Char :
                    begin
                        _Size := 1 ;
                    end ;
            end ;
        end ;
        if( Typ = VDT_String ) then { Converting to string }
        begin
            SetLength( _Value_String, _Size ) ;
            move( _Value, _Value_String[ 1 ], _Size ) ;
        end ;
        _Type := Typ ;
        case Typ of
            VDT_Boolean :
                begin
                    case _Size of
                        1 : Set_ByteBool( _Value._Boolean ) ;
                        2 : Set_WordBool( _Value._Boolean ) ;
                        4 : Set_LongBool( _Value._Boolean ) ;
                    end ;
                end ;
            VDT_Integer :
                begin
                    case _Size of
                        1 : Set_Integer8( _Value._int8 ) ;
                        2 : Set_Integer16( _Value._int16 ) ;
                        4 : Set_Integer32( _Value._int32 ) ;
                        8 : Set_Integer64( _Value._int64 ) ;
                    end ;
                end ;
            VDT_Real :
                begin
                    case _Size of
                        4 : Set_Single( _Value._Single ) ;
                        8 : Set_Double( _Value._Double ) ;
                        10 : Set_Extended( _Value._Extended ) ;
                    end ;
                end ;
            VDT_Char : Set_Char( _Value._Char ) ;
        end ;
    end ; { if( Typ <> _Type ) }
end ; { TSSimple_Generic_Value.Simple_Cast }


procedure TSSimple_Generic_Value.Cast( Typ : PSGeneric_Value ) ;

begin
    if( Can_Convert( Typ^.Data_Type ) ) then
    begin
        Simple_Cast( Typ^.Data_Type ) ;
        _Size := Typ^.Size ;
    end ;
end ;


procedure TSSimple_Generic_Value.Deserialize( Value : PChar ) ;

var Dummy : integer ;
    Parser : TSXML_Parser ;
    S : string ;
    V : string ;
    Temp, Work : integer ;

begin
    { Default values... }
    _Type := VDT_Unknown ;
    _Constant := False ;
    V := '' ;

    { Initialize parser... }
    Parser.Init ;
{$IFDEF DELPHI}
    Parser.Source.Set_String( string( Value ) ) ;
{$ELSE}
    Parser.Source.Set_String( strpas( Value ) ) ;
{$ENDIF}

    { Deserialize... }
    while( not Parser.EOF ) do
    begin
        S := Edit( Parser.Get_Token_String, 32 ) ;
        if( copy( S, 1, 1 ) <> '<' ) then
        begin
            V := V + Edit( S, 2 or 4 ) ; { No CRLF, spaces, or tabs }
        end else
        if( S = '<EMPTY/>' ) then
        begin
            _Type := VDT_Empty ;
        end else
        if( S = '<NULL/>' ) then
        begin
            _Type := VDT_Null ;
        end else
        if( S = '<BOOLEAN/>' ) then
        begin
            _Type := VDT_Boolean ;
        end else
        if( S = '<INTEGER/>' ) then
        begin
            _Type := VDT_Integer ;
        end else
        if( S = '<REAL/>' ) then
        begin
            _Type := VDT_Real ;
        end else
        if( S = '<STRING/>' ) then
        begin
            _Type := VDT_String ;
        end else
        if( S = '<CHAR/>' ) then
        begin
            _Type := VDT_Char ;
        end else
        if( S = '<POINTER/>' ) then
        begin
            _Type := VDT_Pointer ;
        end else
        if( S = '<VALUE/>' ) then
        begin
            _Type := VDT_Value ;
        end else
        if( S = '<CODE/>' ) then
        begin
            _Type := VDT_Code ;
        end else
        if( S = '<SCALAR/>' ) then
        begin
            _Type := VDT_Scalar ;
        end else
        if( S = '<USER/>' ) then
        begin
            _Type := VDT_User_Defined ;
        end else
        if( S = '<CONSTANT/>' ) then
        begin
            _Constant := True ;
        end else
        if( S = '<SIZE>' ) then
        begin
            S := Parser.Get_Section_String( 'size' ) ;
            val( S, _Size, Dummy ) ;
        end else
        if( S = '<TYPE>' ) then
        begin
            S := Parser.Get_Section_String( 'type' ) ;
            _Type_Information.Deserialize( S ) ;
        end else
        begin { Something we don't know about }
            if( pos( '/>', S ) = 0 ) then
            begin
                S := Parser.Get_Section_String( S ) ;
            end ;
        end ;
    end ; { while( not Parser.EOF ) }

    if( length( V ) > 0 ) then
    begin
        Setlength( S, length( V ) div 2 ) ;
        for Dummy := 1 to length( S ) do
        begin
            val( CVTB( 16, 10, copy( V, Dummy * 2 - 1, 2 ) ), Work, Temp ) ;
            S[ Dummy ] := chr( Work ) ;
        end ;
        if( _Type = VDT_String ) then
        begin
            _Value_String := S ;
        end else
        begin
            move( PChar( S )[ 0 ], _Value, _Size ) ;
        end ;
    end ;

    { Clean up }
    Parser.Done ;
end ; { TSSimple_Generic_Value.Deserialize }


function TSSimple_Generic_Value.Serialize : PChar ;

var Loop : integer ;

begin
    case Data_Type of
        VDT_Empty : _Serialize := '<empty/>' ;
        VDT_Null : _Serialize := '<null/>' ;
        VDT_Boolean : _Serialize := '<boolean/>' ;
        VDT_Integer : _Serialize := '<integer/>' ;
        VDT_Real : _Serialize := '<real/>' ;
        VDT_String : _Serialize := '<string/>' ;
        VDT_Char : _Serialize := '<char/>' ;
        VDT_Pointer : _Serialize := '<pointer/>' ;
        VDT_Value : _Serialize := '<value/>' ;
        VDT_Code : _Serialize := '<code/>' ;
        VDT_Scalar : _Serialize := '<scalar/>' ;
        VDT_User_Defined : _Serialize := '<user/>' ;
        else _Serialize := '<unknown/>' ;
    end ;
    if( _Constant ) then
    begin
        _Serialize := _Serialize + '<constant/>' ;
    end ;
    _Serialize := _Serialize + '<size>' + num1( Size ) + '<size/>' ;
    case Data_Type of
        VDT_Boolean,
          VDT_Integer,
          VDT_Real,
          VDT_Char,
          VDT_Scalar :
              begin
                  for Loop := 0 to _Size - 1 do
                  begin
                      _Serialize := _Serialize +
                          CVTB( 10, 16, Num1( _Value._Bytes[ Loop ] ) ) ;
                  end ;
              end ;
        VDT_String :
            begin
                for Loop := 1 to length( _Value_String ) do
                begin
                    _Serialize := _Serialize +
                        CVTB( 10, 16, Num1( ord( _Value_String[ Loop ] ) ) ) ;
                end ;
            end ;
     end ;
    _Serialize := _Serialize +
        '<type>' + _Type_Information.Serialize + '<type/>' ;
    Result := PChar( _Serialize ) ;
end ; { TSSimple_Generic_Value.Serialize }


procedure TSSimple_Generic_Value.Get_Undefined( Buffer : pointer ;
    Len : longint ) ;

begin
    if( Len > _Size ) then
    begin
        Len := _Size ;
    end ;
    case _Type of
        VDT_Empty, VDT_Null : ; { Do nothing }
        VDT_String : move( PChar( _Value_String )[ 0 ], Buffer^, Len ) ;
        else move( _Value, Buffer^, Len ) ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Null ;

begin
    _Type := VDT_Null ;
    _Size := 0 ;
    _Value_String := '' ;
end ;


procedure TSSimple_Generic_Value.Set_Undefined( Buffer : pointer ;
    Len : longint ) ;

begin
    _Type := VDT_Unknown ;
    _Value._PChar := PChar( Buffer ) ;
    _Size := Len ;
    _Value_String := '' ;
end ;


function TSSimple_Generic_Value.Get_Boolean : boolean ;

begin
    case _Type of
        VDT_Boolean : Get_Boolean := _Value._Boolean ;
        VDT_Char : Get_Boolean := ( pos( _Value._Char, '1THOXthox' ) > 0 ) ;
        VDT_Integer : Get_Boolean := ( Get_Integer64 <> 0 ) ;
        VDT_Real : Get_Boolean := ( Get_Extended <> 0.0 ) ;
        VDT_String : Get_Boolean := String_To_Boolean( _Value_String ) ;
        VDT_Pointer : Get_Boolean := String_To_Boolean( strpas( _Value._PChar ) ) ;
        else Get_Boolean := False ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Boolean( B : boolean ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Boolean ;
    _Size := sizeof( B ) ;
    _Value._Boolean := B ;
    _Value_String := '' ;
end ;


function TSSimple_Generic_Value.Get_ByteBool : bytebool ;

begin
    Get_ByteBool := Get_Boolean ;
end ;


procedure TSSimple_Generic_Value.Set_ByteBool( B : bytebool ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Boolean ;
    _Size := 1 ;
    _Value._Boolean := B ;
    _Value_String := '' ;
end ;


function TSSimple_Generic_Value.Get_WordBool : wordbool ;

begin
    Get_WordBool := Get_Boolean ;
end ;


procedure TSSimple_Generic_Value.Set_WordBool( B : wordbool ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Boolean ;
    _Size := 2 ;
    _Value._Boolean := B ;
    _Value_String := '' ;
end ;


function TSSimple_Generic_Value.Get_LongBool : longbool ;

begin
    Get_LongBool := Get_Boolean ;
end ;


procedure TSSimple_Generic_Value.Set_LongBool( B : longbool ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Boolean ;
    _Size := 4 ;
    _Value._Boolean := B ;
    _Value_String := '' ;
end ;


function TSSimple_Generic_Value.Get_Char : char ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_Char := 'T' ;
            end else
            begin
                Get_Char := 'F' ;
            end ;
        VDT_Integer : Get_Char := char( _Value._int8 ) ;
        VDT_Real : Get_Char := char( trunc( _Value._Extended ) ) ;
        VDT_String :
            if( length( _Value_String ) > 1 ) then
            begin
                Get_Char := _Value_String[ 1 ] ;
            end else
            begin
                Get_Char := NUL ;
            end ;
        VDT_Pointer : { For now, all pointers are PChars }
            if( _Value._PChar <> nil ) then
            begin
                Get_Char := _Value._PChar[ 0 ] ;
            end else
            begin
                Get_Char := NUL ;
            end ;
        VDT_Char : Get_Char := _Value._Char ;
        else Get_Char := NUL ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Char( C : char ) ;

var Temp : TSSimple_Generic_Character ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Char ;
    _Size := 1 ;
    _Value._Char := C ;
    _Value_String := '' ;
    Temp := _Type_Information._Character_Information ;
    Temp._Length := 1 ;
    Temp._Max := 1 ;
    Temp._Fixed := True ;
    _Type_Information._Family := Family ;
end ;


function TSSimple_Generic_Value.Get_String : string ;

begin
    Get_String := _Value_String ;
end ;


procedure TSSimple_Generic_Value.Set_String( const S : string ) ;

var Temp : TSSimple_Generic_Character ;

begin
    _Type := VDT_String ;
    _Size := length( S ) ;
    _Value_String := S ;
    Temp := _Type_Information._Character_Information ;
    Temp._Length := length( S ) ;
{$IFDEF DELPHI}
    Temp._Max := 1 ;
{$ELSE}
    Temp._Max := $7FFFFFFF ;
{$ENDIF}
    Temp._Fixed := False ;
    _Type_Information._Family := Family ;
end ;


function TSSimple_Generic_Value.Get_PChar : PChar ;

begin
    case _Type of
        VDT_String : Get_PChar := PChar( _Value_String ) ;
        VDT_pointer : Get_PChar := _Value._PChar ;
        VDT_Null, VDT_Empty : Get_PChar := nil ;
        else Get_PChar := PChar( @_Value ) ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_PChar( P : PChar ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Pointer ;
    _Size := 4 ;
    _Value._PChar := P ;
    _Value_String := '' ;
    _Type_Information._Family := Family ;
end ;


function TSSimple_Generic_Value.Get_Integer8 : int8 ;

var Dummy : integer ;
    Res : int8 ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_Integer8 := -1 ;
            end else
            begin
                Get_Integer8 := 0 ;
            end ;
        VDT_Integer : Get_Integer8 := _Value._int8 ;
        VDT_Real : Get_Integer8 := trunc( _Value._Extended ) and $FF ;
        VDT_String :
            begin
                val( _Value_String, Res, Dummy ) ;
                Get_Integer8 := Res ;
            end ;
        VDT_Char :
            begin
                val( _Value._Char, Res, Dummy ) ;
                Get_Integer8 := Res ;
            end ;
        VDT_Pointer :
            begin
                if( _Value._PChar = nil ) then
                begin
                    Get_Integer8 := 0 ;
                end else
                begin
                    val( strpas( _Value._PChar ), Res, Dummy ) ;
                    Get_Integer8 := Res ;
                end ;
            end ;
        else Get_Integer8 := 0 ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Integer8( I : int8 ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Integer ;
    _Size := 1 ;
    _Value._int8 := I ;
    _Value_String := '' ;
    _Type_Information._Numeric_Information._Signed := True ;
    _Type_Information._Numeric_Information._Fixed := True ;
    _Type_Information._Numeric_Information._Mantissa := _Size * 8 - 1 ;
    _Type_Information._Numeric_Information._Exponent := 0 ;
    _Type_Information._Family := Family ;
end ;


function TSSimple_Generic_Value.Get_Integer16 : int16 ;

var Dummy : integer ;
    Res : int16 ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_Integer16 := -1 ;
            end else
            begin
                Get_Integer16 := 0 ;
            end ;
        VDT_Integer : Get_Integer16 := _Value._int16 ;
        VDT_Real : Get_Integer16 := trunc( _Value._Extended ) and $FFFF ;
        VDT_String :
            begin
                val( _Value_String, Res, Dummy ) ;
                Get_Integer16 := Res ;
            end ;
        VDT_Char :
            begin
                val( _Value._Char, Res, Dummy ) ;
                Get_Integer16 := Res ;
            end ;
        VDT_Pointer :
            begin
                if( _Value._PChar = nil ) then
                begin
                    Get_Integer16 := 0 ;
                end else
                begin
                    val( strpas( _Value._PChar ), Res, Dummy ) ;
                    Get_Integer16 := Res ;
                end ;
            end ;
        else Get_Integer16 := 0 ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Integer16( I : int16 ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Integer ;
    _Size := 2 ;
    _Value._int16 := I ;
    _Value_String := '' ;
    _Type_Information._Numeric_Information._Signed := True ;
    _Type_Information._Numeric_Information._Fixed := True ;
    _Type_Information._Numeric_Information._Mantissa := _Size * 8 - 1 ;
    _Type_Information._Numeric_Information._Exponent := 0 ;
    _Type_Information._Family := Family ;
end ;


function TSSimple_Generic_Value.Get_Integer32 : int32 ;

var Dummy : integer ;
    Res : int32 ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_Integer32 := -1 ;
            end else
            begin
                Get_Integer32 := 0 ;
            end ;
        VDT_Integer : Get_Integer32 := _Value._int32 ;
        VDT_Real : Get_Integer32 := trunc( _Value._Extended ) and $FFFFFFFF ;
        VDT_String :
            begin
                val( _Value_String, Res, Dummy ) ;
                Get_Integer32 := Res ;
            end ;
        VDT_Char :
            begin
                val( _Value._Char, Res, Dummy ) ;
                Get_Integer32 := Res ;
            end ;
        VDT_Pointer :
            begin
                if( _Value._PChar = nil ) then
                begin
                    Get_Integer32 := 0 ;
                end else
                begin
                    val( strpas( _Value._PChar ), Res, Dummy ) ;
                    Get_Integer32 := Res ;
                end ;
            end ;
        else Get_Integer32 := 0 ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Integer32( I : int32 ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Integer ;
    _Size := 4 ;
    _Value._int32 := I ;
    _Value_String := '' ;
    _Type_Information._Numeric_Information._Signed := True ;
    _Type_Information._Numeric_Information._Fixed := True ;
    _Type_Information._Numeric_Information._Mantissa := _Size * 8 - 1 ;
    _Type_Information._Numeric_Information._Exponent := 0 ;
    _Type_Information._Family := Family ;
end ;


function TSSimple_Generic_Value.Get_Integer64 : int64 ;

var Dummy : integer ;
    Res : int64 ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_Integer64 := -1 ;
            end else
            begin
                Get_Integer64 := 0 ;
            end ;
        VDT_Integer : Get_Integer64 := _Value._int64 ;
        VDT_Real : Get_Integer64 := trunc( _Value._Extended ) ;
        VDT_String :
            begin
                val( _Value_String, Res, Dummy ) ;
                Get_Integer64 := Res ;
            end ;
        VDT_Char :
            begin
                val( _Value._Char, Res, Dummy ) ;
                Get_Integer64 := Res ;
            end ;
        VDT_Pointer :
            begin
                if( _Value._PChar = nil ) then
                begin
                    Get_Integer64 := 0 ;
                end else
                begin
                    val( strpas( _Value._PChar ), Res, Dummy ) ;
                    Get_Integer64 := Res ;
                end ;
            end ;
        else Get_Integer64 := 0 ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Integer64( I : int64 ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Integer ;
    _Size := 8 ;
    _Value._int32 := I ;
    _Value_String := '' ;
    _Type_Information._Numeric_Information._Signed := True ;
    _Type_Information._Numeric_Information._Fixed := True ;
    _Type_Information._Numeric_Information._Mantissa := _Size * 8 - 1 ;
    _Type_Information._Numeric_Information._Exponent := 0 ;
    _Type_Information._Family := Family ;
end ;


function TSSimple_Generic_Value.Get_Unsigned_Integer8 : uint8 ;

var Dummy : integer ;
    Res : uint8 ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_Unsigned_Integer8 := 1 ;
            end else
            begin
                Get_Unsigned_Integer8 := 0 ;
            end ;
        VDT_Integer : Get_Unsigned_Integer8 := _Value._uint8 ;
        VDT_Real : Get_Unsigned_Integer8 := uint8( trunc( _Value._Extended ) and $FF ) ;
        VDT_String :
            begin
                val( _Value_String, Res, Dummy ) ;
                Get_Unsigned_Integer8 := Res ;
            end ;
        VDT_Char :
            begin
                val( _Value._Char, Res, Dummy ) ;
                Get_Unsigned_Integer8 := Res ;
            end ;
        VDT_Pointer :
            begin
                if( _Value._PChar = nil ) then
                begin
                    Get_Unsigned_Integer8 := 0 ;
                end else
                begin
                    val( strpas( _Value._PChar ), Res, Dummy ) ;
                    Get_Unsigned_Integer8 := Res ;
                end ;
            end ;
        else Get_Unsigned_Integer8 := 0 ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Unsigned_Integer8( I : uint8 ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Integer ;
    _Size := 1 ;
    _Value._uint8 := I ;
    _Value_String := '' ;
    _Type_Information._Numeric_Information._Signed := False ;
    _Type_Information._Numeric_Information._Fixed := True ;
    _Type_Information._Numeric_Information._Mantissa := _Size * 8 ;
    _Type_Information._Numeric_Information._Exponent := 0 ;
    _Type_Information._Family := Family ;
end ;


function TSSimple_Generic_Value.Get_Unsigned_Integer16 : uint16 ;

var Dummy : integer ;
    Res : uint16 ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_Unsigned_Integer16 := 1 ;
            end else
            begin
                Get_Unsigned_Integer16 := 0 ;
            end ;
        VDT_Integer : Get_Unsigned_Integer16 := _Value._uint16 ;
        VDT_Real : Get_Unsigned_Integer16 := uint16( trunc( _Value._Extended ) and $FFFF ) ;
        VDT_String :
            begin
                val( _Value_String, Res, Dummy ) ;
                Get_Unsigned_Integer16 := Res ;
            end ;
        VDT_Char :
            begin
                val( _Value._Char, Res, Dummy ) ;
                Get_Unsigned_Integer16 := Res ;
            end ;
        VDT_Pointer :
            begin
                if( _Value._PChar = nil ) then
                begin
                    Get_Unsigned_Integer16 := 0 ;
                end else
                begin
                    val( strpas( _Value._PChar ), Res, Dummy ) ;
                    Get_Unsigned_Integer16 := Res ;
                end ;
            end ;
        else Get_Unsigned_Integer16 := 0 ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Unsigned_Integer16( I : uint16 ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Integer ;
    _Size := 2 ;
    _Value._int16 := I ;
    _Value_String := '' ;
    _Type_Information._Numeric_Information._Signed := False ;
    _Type_Information._Numeric_Information._Fixed := True ;
    _Type_Information._Numeric_Information._Mantissa := _Size * 8 ;
    _Type_Information._Numeric_Information._Exponent := 0 ;
    _Type_Information._Family := Family ;
end ;


{$IFDEF DELPHI}
function TSSimple_Generic_Value.Get_Unsigned_Integer32 : uint32 ;

var Dummy : integer ;
    Res : uint32 ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_Unsigned_Integer32 := 1 ;
            end else
            begin
                Get_Unsigned_Integer32 := 0 ;
            end ;
        VDT_Integer : Get_Unsigned_Integer32 := _Value._uint32 ;
        VDT_Real : Get_Unsigned_Integer32 := uint32( trunc( _Value._Extended ) and $FFFFFFFF ) ;
        VDT_String :
            begin
                val( _Value_String, Res, Dummy ) ;
                Get_Unsigned_Integer32 := Res ;
            end ;
        VDT_Char :
            begin
                val( _Value._Char, Res, Dummy ) ;
                Get_Unsigned_Integer32 := Res ;
            end ;
        VDT_Pointer :
            begin
                if( _Value._PChar = nil ) then
                begin
                    Get_Unsigned_Integer32 := 0 ;
                end else
                begin
                    val( strpas( _Value._PChar ), Res, Dummy ) ;
                    Get_Unsigned_Integer32 := Res ;
                end ;
            end ;
        else Get_Unsigned_Integer32 := 0 ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Unsigned_Integer32( I : uint32 ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Integer ;
    _Size := 8 ;
    _Value._int32 := I ;
    _Value_String := '' ;
    _Type_Information._Numeric_Information._Signed := False ;
    _Type_Information._Numeric_Information._Fixed := True ;
    _Type_Information._Numeric_Information._Mantissa := _Size * 8 ;
    _Type_Information._Numeric_Information._Exponent := 0 ;
    _Type_Information._Family := Family ;
end ;
{$ENDIF}


function TSSimple_Generic_Value.Get_Single : single ;

var Dummy : integer ;
    Res : single ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_Single := -1.0 ;
            end else
            begin
                Get_Single := 0.0 ;
            end ;
        VDT_Integer : Get_Single := _Value._int64 ;
        VDT_Real : Get_Single := _Value._Extended ;
        VDT_String :
            begin
                val( _Value_String, Res, Dummy ) ;
                Get_Single := Res ;
            end ;
        VDT_Char :
            begin
                val( _Value._Char, Res, Dummy ) ;
                Get_Single := Res ;
            end ;
        VDT_Pointer :
            begin
                if( _Value._PChar = nil ) then
                begin
                    Get_Single := 0 ;
                end else
                begin
                    val( strpas( _Value._PChar ), Res, Dummy ) ;
                    Get_Single := Res ;
                end ;
            end ;
        else Get_Single := 0.0 ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Single( S : single ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Real ;
    _Size := 4 ;
    _Value._single := S ;
    _Value_String := '' ;
    _Type_Information._Numeric_Information._Signed := True ;
    _Type_Information._Numeric_Information._Fixed := False ;
    _Type_Information._Numeric_Information._Mantissa := 23 ;
    _Type_Information._Numeric_Information._Exponent := 8 ;
    _Type_Information._Family := Family ;
end ;


function TSSimple_Generic_Value.Get_Double : double ;

var Dummy : integer ;
    Res : double ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_double := -1.0 ;
            end else
            begin
                Get_double := 0.0 ;
            end ;
        VDT_Integer : Get_double := _Value._int64 ;
        VDT_Real : Get_double := _Value._Extended ;
        VDT_String :
            begin
                val( _Value_String, Res, Dummy ) ;
                Get_double := Res ;
            end ;
        VDT_Char :
            begin
                val( _Value._Char, Res, Dummy ) ;
                Get_double := Res ;
            end ;
        VDT_Pointer :
            begin
                if( _Value._PChar = nil ) then
                begin
                    Get_double := 0 ;
                end else
                begin
                    val( strpas( _Value._PChar ), Res, Dummy ) ;
                    Get_double := Res ;
                end ;
            end ;
        else Get_double := 0.0 ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Double( D : double ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Real ;
    _Size := 8 ;
    _Value._Double := D ;
    _Value_String := '' ;
    _Type_Information._Numeric_Information._Signed := True ;
    _Type_Information._Numeric_Information._Fixed := False ;
    _Type_Information._Numeric_Information._Mantissa := 52 ;
    _Type_Information._Numeric_Information._Exponent := 11 ;
    _Type_Information._Family := Family ;
end ;


function TSSimple_Generic_Value.Get_Extended : extended ;

var Dummy : integer ;
    Res : Extended ;

begin
    case _Type of
        VDT_Boolean :
            if( Get_Boolean ) then
            begin
                Get_Extended := -1.0 ;
            end else
            begin
                Get_Extended := 0.0 ;
            end ;
        VDT_Integer : Get_Extended := _Value._int64 ;
        VDT_Real : Get_Extended := _Value._Extended ;
        VDT_String :
            begin
                val( _Value_String, Res, Dummy ) ;
                Get_Extended := Res ;
            end ;
        VDT_Char :
            begin
                val( _Value._Char, Res, Dummy ) ;
                Get_Extended := Res ;
            end ;
        VDT_Pointer :
            begin
                if( _Value._PChar = nil ) then
                begin
                    Get_Extended := 0 ;
                end else
                begin
                    val( strpas( _Value._PChar ), Res, Dummy ) ;
                    Get_Extended := Res ;
                end ;
            end ;
        else Get_Extended := 0.0 ;
    end ;
end ;


procedure TSSimple_Generic_Value.Set_Extended( E : extended ) ;

begin
    fillchar( _Value, sizeof( _Value ), 0 ) ;
    _Type := VDT_Real ;
    _Size := 10 ;
    _Value._Extended := E ;
    _Value_String := '' ;
    _Type_Information._Numeric_Information._Signed := True ;
    _Type_Information._Numeric_Information._Fixed := False ;
    _Type_Information._Numeric_Information._Mantissa := 64 ;
    _Type_Information._Numeric_Information._Exponent := 15 ;
    _Type_Information._Family := Family ;
end ;



{$IFDEF DELPHI}
type TSGeneric_Number_Wrapper = object( TSGeneric_Number )
                                    private
                                        _Value : TGeneric_Number ;
                                        _Value1 : TCOM_Generic_Number ;

                                    public
                                        constructor Init( V : TGeneric_Number ) ;
                                        constructor Init_COM( V : TCOM_Generic_Number ) ;
                                        function Fixed : boolean ; virtual ;
                                        function Signed : boolean ; virtual ;
                                        function Mantissa_Size : longint ;
                                            virtual ;
                                        function Exponent_Size : longint ;
                                            virtual ;
                                        function Fractional_Size : longint ;
                                            virtual ;
                                end ;
     PSGeneric_Number_Wrapper = ^TSGeneric_Number_Wrapper ;

constructor TSGeneric_Number_Wrapper.Init( V : TGeneric_Number ) ;

begin
    inherited Init ;

    _Value := V ;
    _Value1 := nil ;
end ;


constructor TSGeneric_Number_Wrapper.Init_COM( V : TCOM_Generic_Number ) ;

begin
    inherited Init ;

    _Value := nil ;
    _Value1 := V ;
end ;


function TSGeneric_Number_Wrapper.Fixed : boolean ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Fixed ;
    end else
    begin
        Result := _Value.Fixed ;
    end ;
end ;


function TSGeneric_Number_Wrapper.Signed : boolean ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Signed ;
    end else
    begin
        Result := _Value.Signed ;
    end ;
end ;


function TSGeneric_Number_Wrapper.Mantissa_Size : longint ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Mantissa_Size ;
    end else
    begin
        Result := _Value.Mantissa_Size ;
    end ;
end ;


function TSGeneric_Number_Wrapper.Exponent_Size : longint ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Exponent_Size ;
    end else
    begin
        Result := _Value.Exponent_Size ;
    end ;
end ;


function TSGeneric_Number_Wrapper.Fractional_Size : longint ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Fractional_Size ;
    end else
    begin
        Result := _Value.Fractional_Size ;
    end ;
end ;



type TSGeneric_Character_Wrapper = object( TSGeneric_Character )
                                       private
                                           _Value : TGeneric_Character ;
                                           _Value1 : TCOM_Generic_Character ;

                                       public
                                           constructor Init( V : TGeneric_Character ) ;
                                           constructor Init_COM( V : TCOM_Generic_Character ) ;

                                           function Max_Length : longint ;
                                               virtual ;
                                           function Fixed : boolean ; virtual ;
                                           function Length : longint ; virtual ;
                                           function Logical_Length : longint ;
                                               virtual ;
                                   end ;
     PSGeneric_Character_Wrapper = ^TSGeneric_Character_Wrapper ;

constructor TSGeneric_Character_Wrapper.Init( V : TGeneric_Character ) ;

begin
    inherited Init ;

    _Value := V ;
    _Value1 := nil ;
end ;


constructor TSGeneric_Character_Wrapper.Init_COM( V : TCOM_Generic_Character ) ;

begin
    inherited Init ;

    _Value := nil ;
    _Value1 := V ;
end ;


function TSGeneric_Character_Wrapper.Max_Length : longint ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Max_Length ;
    end else
    begin
        Result := _Value.Max_Length ;
    end ;
end ;


function TSGeneric_Character_Wrapper.Fixed : boolean ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Fixed ;
    end else
    begin
        Result := _Value.Fixed ;
    end ;
end ;


function TSGeneric_Character_Wrapper.Length : longint ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Length ;
    end else
    begin
        Result := _Value.Length ;
    end ;
end ;


function TSGeneric_Character_Wrapper.Logical_Length : longint ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Logical_Length ;
    end else
    begin
        Result := _Value.Logical_Length ;
    end ;
end ;



type TSType_Information_Wrapper = object( TSType_Information )
                                      private
                                          _Value : TType_Information ;
                                          _Value1 : TCOM_Type_Information ;

                                      public
                                          constructor Init( V : TType_Information ) ;
                                          constructor Init_COM( V : TCOM_Type_Information ) ;

                                          function Is_Composite : boolean ;
                                              virtual ;
                                          function Is_Collection : boolean ;
                                              virtual ;
                                          function Memory_Type : TMemory_Type ;
                                              virtual ;

                                          function Parameter : PSGeneric_Parameter ;
                                              virtual ;
                                          function Numeric_Information : PSGeneric_Number ;
                                              virtual ;
                                          function Character_Information : PSGeneric_Character ;
                                              virtual ;
                                          function Composite_Information : PSGeneric_Composite ;
                                              virtual ;
                                          function Collection_Information : PSGeneric_Collection ;
                                              virtual ;
                                          function Method : PSGeneric_Method ;
                                              virtual ;
                                          function _Property : PSProperty_Information ;
                                              virtual ;
                                          function Address : longint ; virtual ;

                                          function Range_Limited : boolean ;
                                              virtual ;
                                          function Low_Range : PSGeneric_Value ;
                                              virtual ;
                                          function High_Range : PSGeneric_Value ;
                                              virtual ;

                                          function Value_Name( Value : longint ) : string ;
                                              virtual ;
                                  end ;
     PSType_Information_Wrapper = ^TSType_Information_Wrapper ;

constructor TSType_Information_Wrapper.Init( V : TType_Information ) ;

begin
    inherited Init ;

    _Value := V ;
    _Value1 := nil
end ;


constructor TSType_Information_Wrapper.Init_COM( V : TCOM_Type_Information ) ;

begin
    inherited Init ;

    _Value := nil ;
    _Value1 := V ;
end ;


function TSType_Information_Wrapper.Is_Composite : boolean ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Is_Composite ;
    end else
    begin
        Result := _Value.Is_Composite ;
    end ;
end ;


function TSType_Information_Wrapper.Is_Collection : boolean ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Is_Collection ;
    end else
    begin
        Result := _Value.Is_Collection ;
    end ;
end ;


function TSType_Information_Wrapper.Memory_Type : TMemory_Type ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Memory_Type ;
    end else
    begin
        Result := _Value.Memory_Type ;
    end ;
end ;


function TSType_Information_Wrapper.Parameter : PSGeneric_Parameter ;

begin
    Result := nil ;
end ;


function TSType_Information_Wrapper.Numeric_Information : PSGeneric_Number ;

var Temp : PSGeneric_Number_Wrapper ;

begin
    if( _Value = nil ) then
    begin
        Temp := new( PSGeneric_Number_Wrapper, Init_COM( _Value1.Numeric_Information ) ) ;
    end else
    begin
        Temp := new( PSGeneric_Number_Wrapper, Init( _Value.Numeric_Information ) ) ;
    end ;
    Temp^.Attach ;
    Result := Temp ;
end ;


function TSType_Information_Wrapper.Character_Information : PSGeneric_Character ;

var Temp : PSGeneric_Character_Wrapper ;

begin
    if( _Value = nil ) then
    begin
        Temp := new( PSGeneric_Character_Wrapper, Init_COM( _Value1.Character_Information ) ) ;
    end else
    begin
        Temp := new( PSGeneric_Character_Wrapper, Init( _Value.Character_Information ) ) ;
    end ;
    Temp^.Attach ;
    Result := Temp ;
end ;


function TSType_Information_Wrapper.Composite_Information : PSGeneric_Composite ;

begin
    Result := nil ;
end ;


function TSType_Information_Wrapper.Collection_Information : PSGeneric_Collection ;

begin
    Result := nil ;
end ;


function TSType_Information_Wrapper.Method : PSGeneric_Method ;

begin
    Result := nil ;
end ;


function TSType_Information_Wrapper._Property : PSProperty_Information ;

begin
    Result := nil ;
end ;


function TSType_Information_Wrapper.Address : longint ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Address ;
    end else
    begin
        Result := _Value.Address ;
    end ;
end ;


function TSType_Information_Wrapper.Range_Limited : boolean ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Range_Limited ;
    end else
    begin
        Result := _Value.Range_Limited ;
    end ;
end ;


function TSType_Information_Wrapper.Low_Range : PSGeneric_Value ;

begin
    Result := nil ;
end ;


function TSType_Information_Wrapper.High_Range : PSGeneric_Value ;

begin
    Result := nil ;
end ;


function TSType_Information_Wrapper.Value_Name( Value : longint ) : string ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Value_Name( Value ) ;
    end else
    begin
        Result := _Value.Value_Name( Value ) ;
    end ;
end ;



type TGeneric_Value_Wrapper = object( TSGeneric_Value )
                                         private
                                             _Value : TGeneric_Value ;
                                             _Value1 : TCOM_Generic_Value ;

                                         public
                                             constructor Init( V : TGeneric_Value ) ;
                                             constructor Init_COM( V : TCOM_Generic_Value ) ;

                                             function Family : TData_Type_Family ;
                                                 virtual ;
                                             function Data_Type : TValue_Data_Type ;
                                                 virtual ;
                                             function User_Data_Type_Name : string ;
                                                 virtual ;
                                             function Name : string ; virtual ;
                                             function Type_Information : PSType_Information ;
                                                 virtual ;

                                             function Constant : boolean ;
                                                 virtual ;
                                             function Size : longint ; virtual ;
                                             function Total_Size : longint ;
                                                 virtual ;

                                             function Dereference : PSGeneric_Value ;
                                                 virtual ;

                                             function Can_Convert( Typ : TValue_Data_Type ) : boolean ;
                                                 virtual ;
                                             function Convert( Class_Type : PSGeneric_Value ;
                                                 var Loss : boolean ) : PSGeneric_Value ;
                                                 virtual ;

                                             function Clone : PSGeneric_Value ;
                                                 virtual ;

                                             procedure Simple_Cast( Typ : TValue_Data_Type ) ;
                                                 virtual ;
                                             procedure Cast( Typ : PSGeneric_Value ) ;
                                                 virtual ;

                                             procedure Deserialize( Value : PChar ) ;
                                                 virtual ;
                                             function Serialize : PChar ;
                                                 virtual ;

                                             procedure Set_Null ; virtual ;
                                             
                                             procedure Get_Undefined( Buffer : pointer ;
                                                 Len : longint ) ; virtual ;
                                             procedure Set_Undefined( Buffer : pointer ;
                                                 Len : longint ) ; virtual ;

                                             function Get_Boolean : boolean ;
                                                 virtual ;
                                             procedure Set_Boolean( B : boolean ) ;
                                                 virtual ;

                                             function Get_ByteBool : bytebool ;
                                                 virtual ;
                                             procedure Set_ByteBool( B : bytebool ) ;
                                                 virtual ;

                                             function Get_WordBool : wordbool ;
                                                 virtual ;
                                             procedure Set_WordBool( B : wordbool ) ;
                                                 virtual ;

                                             function Get_LongBool : longbool ;
                                                 virtual ;
                                             procedure Set_LongBool( B : longbool ) ;
                                                 virtual ;

                                             function Get_Char : char ;
                                                 virtual ;
                                             procedure Set_Char( C : char ) ;
                                                 virtual ;

                                             function Get_String : string ;
                                                 virtual ;
                                             procedure Set_String( const S : string ) ;
                                                 virtual ;

                                             function Get_PChar : PChar ;
                                                 virtual ;
                                             procedure Set_PChar( P : PChar ) ;
                                                 virtual ;

                                             function Get_Integer8 : int8 ;
                                                 virtual ;
                                             procedure Set_Integer8( I : int8 ) ;
                                                 virtual ;

                                             function Get_Integer16 : int16 ;
                                                 virtual ;
                                             procedure Set_Integer16( I : int16 ) ;
                                                 virtual ;

                                             function Get_Integer32 : int32 ;
                                                 virtual ;
                                             procedure Set_Integer32( I : int32 ) ;
                                                 virtual ;

                                             function Get_Unsigned_Integer8 : uint8 ;
                                                 virtual ;
                                             procedure Set_Unsigned_Integer8( I : uint8 ) ;
                                                 virtual ;

                                             function Get_Unsigned_Integer16 : uint16 ;
                                                 virtual ;
                                             procedure Set_Unsigned_Integer16( I : uint16 ) ;
                                                 virtual ;

{$IFDEF DELPHI}
                                             function Get_Unsigned_Integer32 : uint32 ;
                                                 virtual ;
                                             procedure Set_Unsigned_Integer32( I : uint32 ) ;
                                                virtual ;
{$ENDIF}

                                             function Get_Single : single ;
                                                 virtual ;
                                             procedure Set_Single( S : single ) ;
                                                 virtual ;

                                             function Get_Double : double ;
                                                 virtual ;
                                             procedure Set_Double( D : double ) ;
                                                 virtual ;

                                             function Get_Extended : extended ;
                                                 virtual ;
                                             procedure Set_Extended( E : extended ) ;
                                                 virtual ;
                                     end ;
     PGeneric_Value_Wrapper = ^TGeneric_Value_Wrapper ;

constructor TGeneric_Value_Wrapper.Init( V : TGeneric_Value ) ;

begin
    inherited Init ;

    _Value := V ;
    _Value1 := nil ;
end ;


constructor TGeneric_Value_Wrapper.Init_COM( V : TCOM_Generic_Value ) ;

begin
    inherited Init ;

    _Value := nil ;
    _Value1 := V ;
end ;


function TGeneric_Value_Wrapper.Family : TData_Type_Family ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Family ;
    end else
    begin
        Result := _Value.Family ;
    end ;
end ;


function TGeneric_Value_Wrapper.Data_Type : TValue_Data_Type ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Data_Type ;
    end else
    begin
        Result := _Value.Data_Type ;
    end ;
end ;


function TGeneric_Value_Wrapper.User_Data_Type_Name : string ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.User_Data_Type_Name ;
    end else
    begin
        Result := _Value.User_Data_Type_Name ;
    end ;
end ;


function TGeneric_Value_Wrapper.Name : string ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Name ;
    end else
    begin
        Result := _Value.Name ;
    end ;
end ;


function TGeneric_Value_Wrapper.Type_Information : PSType_Information ;

var Temp : PSType_Information_Wrapper ;

begin
    if( _Value = nil ) then
    begin
        Temp := new( PSType_Information_Wrapper, Init_COM( _Value1.Type_Information ) ) ;
    end else
    begin
        Temp := new( PSType_Information_Wrapper, Init( _Value.Type_Information ) ) ;
    end ;
    Result := Temp ;
end ;


function TGeneric_Value_Wrapper.Constant : boolean ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Constant ;
    end else
    begin
        Result := _Value.Constant ;
    end ;
end ;


function TGeneric_Value_Wrapper.Size : longint ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Size ;
    end else
    begin
        Result := _Value.Size ;
    end ;
end ;


function TGeneric_Value_Wrapper.Total_Size : longint ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Total_Size ;
    end else
    begin
        Result := _Value.Total_Size ;
    end ;
end ;


function TGeneric_Value_Wrapper.Dereference : PSGeneric_Value ;

var Temp : PGeneric_Value_Wrapper ;

begin
    if( _Value = nil ) then
    begin
        Temp := new( PGeneric_Value_Wrapper, Init_COM( _Value1.Dereference ) ) ;
    end else
    begin
        Temp := new( PGeneric_Value_Wrapper, Init( _Value.Dereference ) ) ;
    end ;
    Temp^.Attach ;
    Result := Temp ;
end ;


function TGeneric_Value_Wrapper.Can_Convert( Typ : TValue_Data_Type ) : boolean ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Can_Convert( Typ ) ;
    end else
    begin
        Result := _Value.Can_Convert( Typ ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Convert( Class_Type : PSGeneric_Value ;
    var Loss : boolean ) : PSGeneric_Value ;

var Temp : TSimple_Generic_Value ;
    Temp1 : TCOM_Simple_Generic_Value ;
    Work : TGeneric_Value ;
    Work1 : TCOM_Generic_Value ;

begin
    if( _Value = nil ) then
    begin
        Temp1 := TCOM_Simple_Generic_Value.Create ;
        Temp1.Attach ;
        Work1 := _Value1.Convert( Temp1, Loss ) ;
        if( Work1 = nil ) then
        begin
            Result := nil ;
        end else
        begin
            Result := new( PGeneric_Value_Wrapper, Init_COM( Work1 ) ) ;
        end ;
        Temp1.Detach ;
    end else
    begin
        Temp := TSimple_Generic_Value.Create ;
        Temp.Attach ;
        Work := _Value.Convert( Temp, Loss ) ;
        if( Work = nil ) then
        begin
            Result := nil ;
        end else
        begin
            Result := new( PGeneric_Value_Wrapper, Init( Work ) ) ;
        end ;
        Temp.Detach ;
    end ;
end ;


function Dynamic_To_Static_Generic_Value( Value : TGeneric_Value ) : PSSimple_Generic_Value ;

begin
    Result := PSSimple_Generic_Value( new( PGeneric_Value_Wrapper, Init( Value ) ) ) ;
end ;


function COM_To_Static_Generic_Value( Value : TCOM_Generic_Value ) : PSSimple_Generic_Value ;

begin
    Result := PSSimple_Generic_Value( new( PGeneric_Value_Wrapper, Init_COM( Value ) ) ) ;
end ;


function TGeneric_Value_Wrapper.Clone : PSGeneric_Value ;

begin
    if( _Value = nil ) then
    begin
        Result := COM_To_Static_Generic_Value( _Value1.Clone ) ;
    end else
    begin
        Result := Dynamic_To_Static_Generic_Value( _Value.Clone ) ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Simple_Cast( Typ : TValue_Data_Type ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Simple_Cast( Typ ) ;
    end else
    begin
        _Value.Simple_Cast( Typ ) ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Cast( Typ : PSGeneric_Value ) ;

var Temp : TSimple_Generic_Value ;
    Temp1 : TCOM_Simple_Generic_Value ;

begin
    if( _Value = nil ) then
    begin
        Temp1 := TCOM_Simple_Generic_Value.Create ;
        Temp1.Attach ;
        _Value1.Cast( Temp1 ) ;
        Temp1.Detach ;
    end else
    begin
        Temp := TSimple_Generic_Value.Create ;
        Temp.Attach ;
        _Value.Cast( Temp ) ;
        Temp.Detach ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Deserialize( Value : PChar ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Deserialize( Value ) ;
    end else
    begin
        _Value.Deserialize( Value ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Serialize : PChar ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Serialize ;
    end else
    begin
        Result := _Value.Serialize ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Null ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Null ;
    end else
    begin
        _Value.Set_Null ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Get_Undefined( Buffer : pointer ;
    Len : longint ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Get_Undefined( Buffer, Len ) ;
    end else
    begin
        _Value.Get_Undefined( Buffer, Len ) ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Undefined( Buffer : pointer ;
    Len : longint ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Undefined( Buffer, Len ) ;
    end else
    begin
        _Value.Set_Undefined( Buffer, Len ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_Boolean : boolean ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Boolean ;
    end else
    begin
        Result := _Value.Get_Boolean ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Boolean( B : boolean ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Boolean( B ) ;
    end else
    begin
        _Value.Set_Boolean( B ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_ByteBool : bytebool ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_ByteBool ;
    end else
    begin
        Result := _Value.Get_ByteBool ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_ByteBool( B : bytebool ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_ByteBool( B ) ;
    end else
    begin
        _Value.Set_ByteBool( B ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_WordBool : wordbool ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_WordBool ;
    end else
    begin
        Result := _Value.Get_WordBool ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_WordBool( B : wordbool ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_WordBool( B ) ;
    end else
    begin
        _Value.Set_WordBool( B ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_LongBool : longbool ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_LongBool ;
    end else
    begin
        Result := _Value.Get_LongBool ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_LongBool( B : longbool ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_LongBool( B ) ;
    end else
    begin
        _Value.Set_LongBool( B ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_Char : char ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Char ;
    end else
    begin
        Result := _Value.Get_Char ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Char( C : char ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Char( C ) ;
    end else
    begin
        _Value.Set_Char( C ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_String : string ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_String ;
    end else
    begin
        Result := _Value.Get_String ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_String( const S : string ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_String( S ) ;
    end else
    begin
        _Value.Set_String( S ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_PChar : PChar ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_PChar ;
    end else
    begin
        Result := _Value.Get_PChar ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_PChar( P : PChar ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_PChar( P ) ;
    end else
    begin
        _Value.Set_PChar( P ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_Integer8 : int8 ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Integer8 ;
    end else
    begin
        Result := _Value.Get_Integer8 ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Integer8( I : int8 ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Integer8( I ) ;
    end else
    begin
        _Value.Set_Integer8( I ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_Integer16 : int16 ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Integer16 ;
    end else
    begin
        Result := _Value.Get_Integer16 ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Integer16( I : int16 ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Integer16( I ) ;
    end else
    begin
        _Value.Set_Integer16( I ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_Integer32 : int32 ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Integer32 ;
    end else
    begin
        Result := _Value.Get_Integer32 ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Integer32( I : int32 ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Integer32( I ) ;
    end else
    begin
        _Value.Set_Integer32( I ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_Unsigned_Integer8 : uint8 ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Unsigned_Integer8 ;
    end else
    begin
        Result := _Value.Get_Unsigned_Integer8 ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Unsigned_Integer8( I : uint8 ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Unsigned_Integer8( I ) ;
    end else
    begin
        _Value.Set_Unsigned_Integer8( I ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_Unsigned_Integer16 : uint16 ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Unsigned_Integer16 ;
    end else
    begin
        Result := _Value.Get_Unsigned_Integer16 ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Unsigned_Integer16( I : uint16 ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Unsigned_Integer16( I ) ;
    end else
    begin
        _Value.Set_Unsigned_Integer16( I ) ;
    end ;
end ;


{$IFDEF DELPHI}
function TGeneric_Value_Wrapper.Get_Unsigned_Integer32 : uint32 ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Unsigned_Integer32 ;
    end else
    begin
        Result := _Value.Get_Unsigned_Integer32 ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Unsigned_Integer32( I : uint32 ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Unsigned_Integer32( I ) ;
    end else
    begin
        _Value.Set_Unsigned_Integer32( I ) ;
    end ;
end ;
{$ENDIF}


function TGeneric_Value_Wrapper.Get_Single : single ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Single ;
    end else
    begin
        Result := _Value.Get_Single ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Single( S : single ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Single( S ) ;
    end else
    begin
        _Value.Set_Single( S ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_Double : double ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Double ;
    end else
    begin
        Result := _Value.Get_Double ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Double( D : double ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Double( D ) ;
    end else
    begin
        _Value.Set_Double( D ) ;
    end ;
end ;


function TGeneric_Value_Wrapper.Get_Extended : extended ;

begin
    if( _Value = nil ) then
    begin
        Result := _Value1.Get_Extended ;
    end else
    begin
        Result := _Value.Get_Extended ;
    end ;
end ;


procedure TGeneric_Value_Wrapper.Set_Extended( E : extended ) ;

begin
    if( _Value = nil ) then
    begin
        _Value1.Set_Extended( E ) ;
    end else
    begin
        _Value.Set_Extended( E ) ;
    end ;
end ;


{ TSimple_Generic_Number methods... }

procedure TSimple_Generic_Number.Attach ;

begin
    inc( _Reference_Count ) ;
end ;


procedure TSimple_Generic_Number.Detach ;

begin
    dec( _Reference_Count ) ;
    if( _Reference_Count <= 0 ) then
    begin
        Free ;
    end ;
end ;


function TSimple_Generic_Number.Fixed : boolean ;

begin
    if( _Number = nil ) then
    begin
        Result := _Fixed ;
    end else
    begin
	Result := _Number.Fixed ;
    end ;
end ;


function TSimple_Generic_Number.Signed : boolean ;

begin
    if( _Number = nil ) then
    begin
        Result := _Signed ;
    end else
    begin
	Result := _Number.Signed ;
    end ;
end ;


function TSimple_Generic_Number.Mantissa_Size : longint ;

begin
    if( _Number = nil ) then
    begin
        Result := _Mantissa ;
    end else
    begin
	Result := _Number.Mantissa_Size ;
    end ;
end ;


function TSimple_Generic_Number.Exponent_Size : longint ;

begin
    if( _Number = nil ) then
    begin
        Result := _Exponent ;
    end else
    begin
	Result := _Number.Exponent_Size ;
    end ;
end ;


function TSimple_Generic_Number.Fractional_Size : longint ;

begin
    if( _Number = nil ) then
    begin
        Result := 0 ;
    end else
    begin
	Result := _Number.Fractional_Size ;
    end ;
end ;



{ TCOM_Simple_Generic_Number methods... }

procedure TCOM_Simple_Generic_Number.Attach ;

begin
    inc( _Reference_Count ) ;
end ;


procedure TCOM_Simple_Generic_Number.Detach ;

begin
    dec( _Reference_Count ) ;
    if( _Reference_Count <= 0 ) then
    begin
        Free ;
    end ;
end ;


function TCOM_Simple_Generic_Number.Fixed : boolean ;

begin
    if( _Number = nil ) then
    begin
        Result := _Fixed ;
    end else
    begin
	Result := _Number.Fixed ;
    end ;
end ;


function TCOM_Simple_Generic_Number.Signed : boolean ;

begin
    if( _Number = nil ) then
    begin
        Result := _Signed ;
    end else
    begin
	Result := _Number.Signed ;
    end ;
end ;


function TCOM_Simple_Generic_Number.Mantissa_Size : longint ;

begin
    if( _Number = nil ) then
    begin
        Result := _Mantissa ;
    end else
    begin
	Result := _Number.Mantissa_Size ;
    end ;
end ;


function TCOM_Simple_Generic_Number.Exponent_Size : longint ;

begin
    if( _Number = nil ) then
    begin
        Result := _Exponent ;
    end else
    begin
	Result := _Number.Exponent_Size ;
    end ;
end ;


function TCOM_Simple_Generic_Number.Fractional_Size : longint ;

begin
    if( _Number = nil ) then
    begin
        Result := 0 ;
    end else
    begin
	Result := _Number.Fractional_Size ;
    end ;
end ;



{ TSimple_Generic_Character methods... }

procedure TSimple_Generic_Character.Attach ;

begin
    inc( _Reference_Count ) ;
end ;


procedure TSimple_Generic_Character.Detach ;

begin
    dec( _Reference_Count ) ;
    if( _Reference_Count <= 0 ) then
    begin
        Free ;
    end ;
end ;


function TSimple_Generic_Character.Max_Length : longint ;

begin
    if( _Char = nil ) then
    begin
        Result := _Max ;
    end else
    begin
        Result := _Char^.Max_Length ;
    end ;
end ;


function TSimple_Generic_Character.Fixed : boolean ;

begin
    if( _Char = nil ) then
    begin
        Result := _Fixed ;
    end else
    begin
        Result := _Char^.Fixed ;
    end ;
end ;


function TSimple_Generic_Character.Length : longint ;

begin
    if( _Char = nil ) then
    begin
        Result := _Length ;
    end else
    begin
        Result := _Char^.Length ;
    end ;
end ;


function TSimple_Generic_Character.Logical_Length : longint ;

begin
    if( _Char = nil ) then
    begin
        Result := Length ;
    end else
    begin
        Result := _Char^.Logical_Length ;
    end ;
end ;



{ TCOM_Simple_Generic_Character methods... }

procedure TCOM_Simple_Generic_Character.Attach ;

begin
    inc( _Reference_Count ) ;
end ;


procedure TCOM_Simple_Generic_Character.Detach ;

begin
    dec( _Reference_Count ) ;
    if( _Reference_Count <= 0 ) then
    begin
        Free ;
    end ;
end ;


function TCOM_Simple_Generic_Character.Max_Length : longint ;

begin
    if( _Char = nil ) then
    begin
        Result := _Max ;
    end else
    begin
        Result := _Char^.Max_Length ;
    end ;
end ;


function TCOM_Simple_Generic_Character.Fixed : boolean ;

begin
    if( _Char = nil ) then
    begin
        Result := _Fixed ;
    end else
    begin
        Result := _Char^.Fixed ;
    end ;
end ;


function TCOM_Simple_Generic_Character.Length : longint ;

begin
    if( _Char = nil ) then
    begin
        Result := _Length ;
    end else
    begin
        Result := _Char^.Length ;
    end ;
end ;


function TCOM_Simple_Generic_Character.Logical_Length : longint ;

begin
    if( _Char = nil ) then
    begin
        Result := Length ;
    end else
    begin
        Result := _Char^.Logical_Length ;
    end ;
end ;



{ TSimple_Type_Information methods... }

procedure TSimple_Type_Information.Attach ;

begin
    inc( _Reference_Count ) ;
end ;


procedure TSimple_Type_Information.Detach ;

begin
    dec( _Reference_Count ) ;
    if( _Reference_Count <= 0 ) then
    begin
        Free ;
    end ;
end ;


function TSimple_Type_Information.Is_Composite : boolean ;

begin
    Result := _Type^.Is_Composite ;
end ;


function TSimple_Type_Information.Is_Collection : boolean ;

begin
    Result := _Type^.Is_Collection ;
end ;


function TSimple_Type_Information.Memory_Type : TMemory_Type ;

begin
    Result := _Type^.Memory_Type ;
end ;


function TSimple_Type_Information.Parameter : TGeneric_Parameter ;

begin
    Result := nil ;
end ;


function TSimple_Type_Information.Numeric_Information : TGeneric_Number ;

var Temp : PSGeneric_Number ;

begin
    Temp := _Type.Numeric_Information ;
    if( Temp = nil ) then
    begin
        Result := nil ;
    end else
    begin
        Result := TSimple_Generic_Number.Create ;
        TSimple_Generic_Number( Result )._Number := Temp ;
    end ;
end ;


function TSimple_Type_Information.Character_Information : TGeneric_Character ;

var Temp : PSGeneric_Character ;

begin
    Temp := _Type.Character_Information ;
    if( Temp = nil ) then
    begin
        Result := nil ;
    end else
    begin
        Result := TSimple_Generic_Character.Create ;
        TSimple_Generic_Character( Result )._Char := Temp ;
    end ;
end ;


function TSimple_Type_Information.Composite_Information : TGeneric_Composite ;

begin
    Result := nil ;
end ;


function TSimple_Type_Information.Collection_Information : TGeneric_Collection ;

begin
    Result := nil ;
end ;


function TSimple_Type_Information.Method : TGeneric_Method ;

begin
    Result := nil ;
end ;


function TSimple_Type_Information._Property : TProperty_Information ;

begin
    Result := nil ;
end ;


function TSimple_Type_Information.Address : longint ;

begin
    Result := _Type^.Address ;
end ;


function TSimple_Type_Information.Range_Limited : boolean ;

begin
    Result := _Type^.Range_Limited ;
end ;


function TSimple_Type_Information.Low_Range : TGeneric_Value ;

begin
    Result := nil ;
end ;


function TSimple_Type_Information.High_Range : TGeneric_Value ;

begin
    Result := nil ;
end ;


function TSimple_Type_Information.Value_Name( Value : longint ) : string ;

begin
    Result := _Type^.Value_Name( Value ) ;
end ;



{ TCOM_Simple_Type_Information methods... }

function TCOM_Simple_Type_Information.Is_Composite : boolean ;

begin
    Result := _Type^.Is_Composite ;
end ;


function TCOM_Simple_Type_Information.Is_Collection : boolean ;

begin
    Result := _Type^.Is_Collection ;
end ;


function TCOM_Simple_Type_Information.Memory_Type : TMemory_Type ;

begin
    Result := _Type^.Memory_Type ;
end ;


function TCOM_Simple_Type_Information.Parameter : TCOM_Generic_Parameter ;

begin
    Result := nil ;
end ;


function TCOM_Simple_Type_Information.Numeric_Information : TCOM_Generic_Number ;

var Temp : PSGeneric_Number ;

begin
    Temp := _Type.Numeric_Information ;
    if( Temp = nil ) then
    begin
        Result := nil ;
    end else
    begin
        Result := TCOM_Simple_Generic_Number.Create ;
        TCOM_Simple_Generic_Number( Result )._Number := Temp ;
    end ;
end ;


function TCOM_Simple_Type_Information.Character_Information : TCOM_Generic_Character ;

var Temp : PSGeneric_Character ;

begin
    Temp := _Type.Character_Information ;
    if( Temp = nil ) then
    begin
        Result := nil ;
    end else
    begin
        Result := TCOM_Simple_Generic_Character.Create ;
        TCOM_Simple_Generic_Character( Result )._Char := Temp ;
    end ;
end ;


function TCOM_Simple_Type_Information.Composite_Information : TCOM_Generic_Composite ;

begin
    Result := nil ;
end ;


function TCOM_Simple_Type_Information.Collection_Information : TCOM_Generic_Collection ;

begin
    Result := nil ;
end ;


function TCOM_Simple_Type_Information.Method : TCOM_Generic_Method ;

begin
    Result := nil ;
end ;


function TCOM_Simple_Type_Information._Property : TCOM_Property_Information ;

begin
    Result := nil ;
end ;


function TCOM_Simple_Type_Information.Address : longint ;

begin
    Result := _Type^.Address ;
end ;


function TCOM_Simple_Type_Information.Range_Limited : boolean ;

begin
    Result := _Type^.Range_Limited ;
end ;


function TCOM_Simple_Type_Information.Low_Range : TCOM_Generic_Value ;

begin
    Result := nil ;
end ;


function TCOM_Simple_Type_Information.High_Range : TCOM_Generic_Value ;

begin
    Result := nil ;
end ;


function TCOM_Simple_Type_Information.Value_Name( Value : longint ) : string ;

begin
    Result := _Type^.Value_Name( Value ) ;
end ;



{ TSimple_Generic_Value methods... }

{ API... }

procedure TSimple_Generic_Value.Attach ;

begin
    inc( _Reference_Count ) ;
end ;


procedure TSimple_Generic_Value.Detach ;

begin
    dec( _Reference_Count ) ;
    if( _Reference_Count <= 0 ) then
    begin
        _Value.Done ;
        Free ;
    end ;
end ;


function TSimple_Generic_Value.Family : TData_Type_Family ;

begin
    Result := _Value^.Family ;
end ;


function TSimple_Generic_Value.Data_Type : TValue_Data_Type ;

begin
    Result := _Value^.Data_Type ;
end ;


function TSimple_Generic_Value.User_Data_Type_Name : string ;

begin
    Result := _Value^.User_Data_Type_Name ;
end ;


function TSimple_Generic_Value.Name : string ;

begin
    Result := _Value^.Name ;
end ;


function TSimple_Generic_Value.Type_Information : TType_Information ;

begin
    Result := TSimple_Type_Information.Create ;
    TSimple_Type_Information( Result )._Type := PSSimple_Type_Information( _Value.Type_Information ) ;
end ;


function TSimple_Generic_Value.Constant : boolean ;

begin
    Result := _Value.Constant ;
end ;


function TSimple_Generic_Value.Size : longint ;

begin
    Result := _Value.Size ;
end ;


function TSimple_Generic_Value.Total_Size : longint ;

begin
    Result := _Value.Total_Size ;
end ;


function TSimple_Generic_Value.Dereference : TGeneric_Value ;

var Temp : PSGeneric_Value ;

begin
    Temp := _Value.Dereference ;
    if( Temp = nil ) then
    begin
        Result := nil ;
    end else
    begin
        Result := nil ; { Right now we don't support dereference }
    end ;
end ;


function TSimple_Generic_Value.Can_Convert( Typ : TValue_Data_Type ) : boolean ;

begin
    Result := _Value.Can_Convert( Typ ) ;
end ;


function TSimple_Generic_Value.Convert( Class_Type : TGeneric_Value ;
    var Loss : boolean ) : TGeneric_Value ;

begin
    if( Class_Type = nil ) then
    begin
        Result := nil ;
    end else
    begin
        Class_Type.Attach ;
        Result := TSimple_Generic_Value.Create ;
        TSimple_Generic_Value( Result )._Value :=
            _Value.Convert( Dynamic_To_Static_Generic_Value( Class_Type ), Loss ) ;
        Class_Type.Detach ;
    end ;
end ;


function TSimple_Generic_Value.Clone : TGeneric_Value ;

begin
    Result := TSimple_Generic_Value.Create ;
    TSimple_Generic_Value( Result )._Value := _Value.Clone ;
end ;


procedure TSimple_Generic_Value.Simple_Cast( Typ : TValue_Data_Type ) ;

begin
    _Value.Simple_Cast( Typ ) ;
end ;


procedure TSimple_Generic_Value.Cast( Typ : TGeneric_Value ) ;

begin
    if( Typ <> nil ) then
    begin
        Typ.Attach ;
        _Value.Cast( Dynamic_To_Static_Generic_Value( Typ ) ) ;
        Typ.Detach ;
    end ;
end ;


procedure TSimple_Generic_Value.Deserialize( Value : PChar ) ;

begin
    _Value.Deserialize( Value ) ;
end ;


function TSimple_Generic_Value.Serialize : PChar ;

begin
    Result := _Value.Serialize ;
end ;


procedure TSimple_Generic_Value.Set_Null ;

begin
    _Value.Set_Null ;
end ;


procedure TSimple_Generic_Value.Get_Undefined( Buffer : pointer ;
    Len : longint ) ;

begin
    _Value.Get_Undefined( Buffer, Len ) ;
end ;


procedure TSimple_Generic_Value.Set_Undefined( Buffer : pointer ;
    Len : longint ) ;

begin
    _Value.Set_Undefined( Buffer, Len ) ;
end ;


function TSimple_Generic_Value.Get_Boolean : boolean ;

begin
    Result := _Value.Get_Boolean ;
end ;


procedure TSimple_Generic_Value.Set_Boolean( B : boolean ) ;

begin
    _Value.Set_Boolean( B ) ;
end ;


function TSimple_Generic_Value.Get_ByteBool : bytebool ;

begin
    Result := _Value.Get_ByteBool ;
end ;


procedure TSimple_Generic_Value.Set_ByteBool( B : bytebool ) ;

begin
    _Value.Set_ByteBool( B ) ;
end ;


function TSimple_Generic_Value.Get_WordBool : wordbool ;

begin
    Result := _Value.Get_WordBool ;
end ;


procedure TSimple_Generic_Value.Set_WordBool( B : wordbool ) ;

begin
    _Value.Set_WordBool( B ) ;
end ;


function TSimple_Generic_Value.Get_LongBool : longbool ;

begin
    Result := _Value.Get_LongBool ;
end ;


procedure TSimple_Generic_Value.Set_LongBool( B : longbool ) ;

begin
    _Value.Set_LongBool( B ) ;
end ;


function TSimple_Generic_Value.Get_Char : char ;

begin
    Result := _Value.Get_Char ;
end ;


procedure TSimple_Generic_Value.Set_Char( C : char ) ;

begin
    _Value.Set_Char( C ) ;
end ;


function TSimple_Generic_Value.Get_String : string ;

begin
    Result := _Value.Get_String ;
end ;


procedure TSimple_Generic_Value.Set_String( const S : string )  ;

begin
    _Value.Set_String( S ) ;
end ;


function TSimple_Generic_Value.Get_PChar : PChar  ;

begin
    Result := _Value.Get_PChar ;
end ;


procedure TSimple_Generic_Value.Set_PChar( P : PChar ) ;

begin
    _Value.Set_PChar( P ) ;
end ;


function TSimple_Generic_Value.Get_Integer8 : int8 ;

begin
    Result := _Value.Get_Integer8 ;
end ;


procedure TSimple_Generic_Value.Set_Integer8( I : int8 ) ;

begin
    _Value.Set_Integer8( I ) ;
end ;


function TSimple_Generic_Value.Get_Integer16 : int16 ;

begin
    Result := _Value.Get_Integer16 ;
end ;


procedure TSimple_Generic_Value.Set_Integer16( I : int16 ) ;

begin
    _Value.Set_Integer16( I ) ;
end ;


function TSimple_Generic_Value.Get_Integer32 : int32 ;

begin
    Result := _Value.Get_Integer32 ;
end ;


procedure TSimple_Generic_Value.Set_Integer32( I : int32 ) ;

begin
    _Value.Set_Integer32( I ) ;
end ;


function TSimple_Generic_Value.Get_Integer64 : int64 ;

begin
    Result := _Value.Get_Integer64 ;
end ;


procedure TSimple_Generic_Value.Set_Integer64( I : int64 ) ;

begin
    _Value.Set_Integer64( I ) ;
end ;


function TSimple_Generic_Value.Get_Unsigned_Integer8 : uint8 ;

begin
    Result := _Value.Get_Unsigned_Integer8 ;
end ;


procedure TSimple_Generic_Value.Set_Unsigned_Integer8( I : uint8 ) ;

begin
    _Value.Set_Unsigned_Integer8( I ) ;
end ;


function TSimple_Generic_Value.Get_Unsigned_Integer16 : uint16 ;

begin
    Result := _Value.Get_Unsigned_Integer16 ;
end ;


procedure TSimple_Generic_Value.Set_Unsigned_Integer16( I : uint16 ) ;

begin
    _Value.Set_Unsigned_Integer16( I ) ;
end ;


function TSimple_Generic_Value.Get_Unsigned_Integer32 : uint32 ;

begin
    Result := _Value.Get_Unsigned_Integer32 ;
end ;


procedure TSimple_Generic_Value.Set_Unsigned_Integer32( I : uint32 ) ;

begin
    _Value.Set_Unsigned_Integer32( I ) ;
end ;


function TSimple_Generic_Value.Get_Single : single ;

begin
    Result := _Value.Get_Single ;
end ;


procedure TSimple_Generic_Value.Set_Single( S : single ) ;

begin
    _Value.Set_Single( S ) ;
end ;


function TSimple_Generic_Value.Get_Double : double ;

begin
    Result := _Value.Get_Double ;
end ;


procedure TSimple_Generic_Value.Set_Double( D : double ) ;

begin
    _Value.Set_Double( D ) ;
end ;


function TSimple_Generic_Value.Get_Extended : extended ;

begin
    Result := _Value.Get_Extended ;
end ;


procedure TSimple_Generic_Value.Set_Extended( E : extended ) ;

begin
    _Value.Set_Extended( E ) ;
end ;



{ TCOM_Simple_Generic_Value methods... }

{ API... }

procedure TCOM_Simple_Generic_Value.Attach ;

begin
    inc( _Reference_Count ) ;
end ;


procedure TCOM_Simple_Generic_Value.Detach ;

begin
    dec( _Reference_Count ) ;
    if( _Reference_Count <= 0 ) then
    begin
        _Value.Done ;
        Free ;
    end ;
end ;


function TCOM_Simple_Generic_Value.Family : TData_Type_Family ;

begin
    Result := _Value^.Family ;
end ;


function TCOM_Simple_Generic_Value.Data_Type : TValue_Data_Type ;

begin
    Result := _Value^.Data_Type ;
end ;


function TCOM_Simple_Generic_Value.User_Data_Type_Name : string ;

begin
    Result := _Value^.User_Data_Type_Name ;
end ;


function TCOM_Simple_Generic_Value.Name : string ;

begin
    Result := _Value^.Name ;
end ;


function TCOM_Simple_Generic_Value.Type_Information : TCOM_Type_Information ;

begin
    Result := TCOM_Simple_Type_Information.Create ;
    TCOM_Simple_Type_Information( Result )._Type := PSSimple_Type_Information( _Value.Type_Information ) ;
end ;


function TCOM_Simple_Generic_Value.Constant : boolean ;

begin
    Result := _Value.Constant ;
end ;


function TCOM_Simple_Generic_Value.Size : longint ;

begin
    Result := _Value.Size ;
end ;


function TCOM_Simple_Generic_Value.Total_Size : longint ;

begin
    Result := _Value.Total_Size ;
end ;


function TCOM_Simple_Generic_Value.Dereference : TCOM_Generic_Value ;

var Temp : PSGeneric_Value ;

begin
    Temp := _Value.Dereference ;
    if( Temp = nil ) then
    begin
        Result := nil ;
    end else
    begin
        Result := nil ; { Right now we don't support dereference }
    end ;
end ;


function TCOM_Simple_Generic_Value.Can_Convert( Typ : TValue_Data_Type ) : boolean ;

begin
    Result := _Value.Can_Convert( Typ ) ;
end ;


function TCOM_Simple_Generic_Value.Convert( Class_Type : TCOM_Generic_Value ;
    var Loss : boolean ) : TCOM_Generic_Value ;

begin
    if( Class_Type = nil ) then
    begin
        Result := nil ;
    end else
    begin
        Class_Type.Attach ;
        Result := TCOM_Simple_Generic_Value.Create ;
        TCOM_Simple_Generic_Value( Result )._Value :=
            _Value.Convert( COM_To_Static_Generic_Value( Class_Type ), Loss ) ;
        Class_Type.Detach ;
    end ;
end ;


function TCOM_Simple_Generic_Value.Clone : TCOM_Generic_Value ;

begin
    Result := TCOM_Simple_Generic_Value.Create ;
    TCOM_Simple_Generic_Value( Result )._Value := _Value.Clone ;
end ;


procedure TCOM_Simple_Generic_Value.Simple_Cast( Typ : TValue_Data_Type ) ;

begin
    _Value.Simple_Cast( Typ ) ;
end ;


procedure TCOM_Simple_Generic_Value.Cast( Typ : TCOM_Generic_Value ) ;

begin
    if( Typ <> nil ) then
    begin
        Typ.Attach ;
        _Value.Cast( COM_To_Static_Generic_Value( Typ ) ) ;
        Typ.Detach ;
    end ;
end ;


procedure TCOM_Simple_Generic_Value.Deserialize( Value : PChar ) ;

begin
    _Value.Deserialize( Value ) ;
end ;


function TCOM_Simple_Generic_Value.Serialize : PChar ;

begin
    Result := _Value.Serialize ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Null ;

begin
    _Value.Set_Null ;
end ;


procedure TCOM_Simple_Generic_Value.Get_Undefined( Buffer : pointer ;
    Len : longint ) ;

begin
    _Value.Get_Undefined( Buffer, Len ) ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Undefined( Buffer : pointer ;
    Len : longint ) ;

begin
    _Value.Set_Undefined( Buffer, Len ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Boolean : boolean ;

begin
    Result := _Value.Get_Boolean ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Boolean( B : boolean ) ;

begin
    _Value.Set_Boolean( B ) ;
end ;


function TCOM_Simple_Generic_Value.Get_ByteBool : bytebool ;

begin
    Result := _Value.Get_ByteBool ;
end ;


procedure TCOM_Simple_Generic_Value.Set_ByteBool( B : bytebool ) ;

begin
    _Value.Set_ByteBool( B ) ;
end ;


function TCOM_Simple_Generic_Value.Get_WordBool : wordbool ;

begin
    Result := _Value.Get_WordBool ;
end ;


procedure TCOM_Simple_Generic_Value.Set_WordBool( B : wordbool ) ;

begin
    _Value.Set_WordBool( B ) ;
end ;


function TCOM_Simple_Generic_Value.Get_LongBool : longbool ;

begin
    Result := _Value.Get_LongBool ;
end ;


procedure TCOM_Simple_Generic_Value.Set_LongBool( B : longbool ) ;

begin
    _Value.Set_LongBool( B ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Char : char ;

begin
    Result := _Value.Get_Char ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Char( C : char ) ;

begin
    _Value.Set_Char( C ) ;
end ;


function TCOM_Simple_Generic_Value.Get_String : string ;

begin
    Result := _Value.Get_String ;
end ;


procedure TCOM_Simple_Generic_Value.Set_String( const S : string )  ;

begin
    _Value.Set_String( S ) ;
end ;


function TCOM_Simple_Generic_Value.Get_PChar : PChar  ;

begin
    Result := _Value.Get_PChar ;
end ;


procedure TCOM_Simple_Generic_Value.Set_PChar( P : PChar ) ;

begin
    _Value.Set_PChar( P ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Integer8 : int8 ;

begin
    Result := _Value.Get_Integer8 ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Integer8( I : int8 ) ;

begin
    _Value.Set_Integer8( I ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Integer16 : int16 ;

begin
    Result := _Value.Get_Integer16 ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Integer16( I : int16 ) ;

begin
    _Value.Set_Integer16( I ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Integer32 : int32 ;

begin
    Result := _Value.Get_Integer32 ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Integer32( I : int32 ) ;

begin
    _Value.Set_Integer32( I ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Integer64 : int64 ;

begin
    Result := _Value.Get_Integer64 ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Integer64( I : int64 ) ;

begin
    _Value.Set_Integer64( I ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Unsigned_Integer8 : uint8 ;

begin
    Result := _Value.Get_Unsigned_Integer8 ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Unsigned_Integer8( I : uint8 ) ;

begin
    _Value.Set_Unsigned_Integer8( I ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Unsigned_Integer16 : uint16 ;

begin
    Result := _Value.Get_Unsigned_Integer16 ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Unsigned_Integer16( I : uint16 ) ;

begin
    _Value.Set_Unsigned_Integer16( I ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Unsigned_Integer32 : uint32 ;

begin
    Result := _Value.Get_Unsigned_Integer32 ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Unsigned_Integer32( I : uint32 ) ;

begin
    _Value.Set_Unsigned_Integer32( I ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Single : single ;

begin
    Result := _Value.Get_Single ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Single( S : single ) ;

begin
    _Value.Set_Single( S ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Double : double ;

begin
    Result := _Value.Get_Double ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Double( D : double ) ;

begin
    _Value.Set_Double( D ) ;
end ;


function TCOM_Simple_Generic_Value.Get_Extended : extended ;

begin
    Result := _Value.Get_Extended ;
end ;


procedure TCOM_Simple_Generic_Value.Set_Extended( E : extended ) ;

begin
    _Value.Set_Extended( E ) ;
end ;
{$ENDIF}


procedure Initialize_Unit ;

    procedure Allow( V1, V2 : TValue_Data_Type ) ;

    begin
        Supported_Conversion[ V1, V2 ] := True ;
        Supported_Conversion[ V2, V1 ] := True ;
    end ;

var Loop : TValue_Data_Type ;

begin
    fillchar( Supported_Conversion, sizeof( Supported_Conversion ), 0 ) ;
    for Loop := low( Loop ) to high( Loop ) do
    begin
        case Loop of
            VDT_Empty,
            VDT_Null,
            VDT_Boolean,
            VDT_Integer,
            VDT_Real,
            VDT_String,
            VDT_Char : Supported_Conversion[ Loop, Loop ] := True ;
        end ;
    end ;
    Allow( VDT_String, VDT_Char ) ;
    Allow( VDT_Integer, VDT_Real ) ;
    Allow( VDT_Boolean, VDT_String ) ;
    Allow( VDT_Boolean, VDT_Integer ) ;
    Allow( VDT_Boolean, VDT_Real ) ;
    Allow( VDT_Boolean, VDT_Char ) ;
    Allow( VDT_String, VDT_Real ) ;
    Allow( VDT_Char, VDT_Real ) ;
    Allow( VDT_String, VDT_Integer ) ;
    Allow( VDT_Char, VDT_Integer ) ;
end ; { Initialize_Unit }


begin
    Initialize_Unit ;
end.

