{
        Program Name    : LOADARRA
        Package Name    : LB
        Purpose         : Loads an array from a file containing a table
        Institution     : Conroy & Conroy Co.
        Date written    : 06-Oct-1995
        Written by      : Alan Conroy
        Version         : 1.0

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

      *******************************************************
      *                                                     *
      *      M O D I F I C A T I O N   H I S T O R Y        *
      *                                                     *
      *    DATE        BY            REASON                 *
      *                                                     *
      *                                                     *
      *******************************************************

      *******************************************************
      *                                                     *
      *          P R O G R A M   P U R P O S E              *
      *                                                     *
      *******************************************************

	This function will load an array from a table in a file.  The
	format of the file is:
		t	c1	c2	c3...
		r1	x	x	x
		r2	x	x	x
		r3	x	x	x

	where "t" indicates a tab, and all columns are separated with
	tabs (ASCII 9).  r1-r3... indicates a single value, which is the
	first element of a two-dimensional array.  c1-c3... are single
	values or a range.  They can have the form: n1-n2 for a range from
	n1 to n2, inclusive, n1+, which means all value from n1 and up,
	or just n1 for a single value.  "x" are the values for the array
	with those elements.
}

unit LoadArra ;

Interface

uses Structur ;

type PUnbounded_Static_Array = ^Unbounded_Static_Array ;

type t2D_Unbounded_Array = object
				_Array : PUnbounded_Static_Array ;

				constructor Init( Name : string ) ;
				destructor Done ; virtual ;

				procedure Put2( X, Y, Value : integer ) ;
				function Get2( X, Y : integer ) : integer ;
			   end ;
     p2D_Unbounded_Array = ^t2D_Unbounded_Array ;

implementation

uses ASCIIDef ;

procedure Parse_Range( S : String ; Position : integer ; var Low, High : integer ) ;
{ Parse Positionth range specification in S and return its values. }

var Dummy, Dummy1 : integer ;

begin
    Low := 32767 ;
    High := 32767 ;
    while( Position > 1 ) do
    begin
	Dummy := pos( HT, S ) ;
	if( Dummy = 0 ) then { Element does not exist }
	begin
	    exit ;
	end ;
	S := copy( S, Dummy + 1, 254 ) ;
	Position := Position - 1 ;
    end ;

    { Make sure we are only processing this element }
    Dummy := pos( HT, S ) ;
    if( Dummy > 0 ) then
    begin
	S := copy( S, 1, Dummy - 1 ) ;
    end ;

    { Check for "+" }
    if( S[ length( S ) ] = '+' ) then
    begin
	S := copy( S, 1, length( S ) - 1 ) ;
    end ;

    { Check for "-" }
    if( S[ 1 ] = '-' ) then
    begin
	S := copy( S, 2, 254 ) ;
    end ;

    { Check for range }
    Dummy := pos( '-', S ) ;
    if( Dummy > 0 ) then
    begin
	val( copy( S, 1, Dummy - 1 ), Low, Dummy1 ) ;
	val( copy( S, Dummy + 1, 254 ), High, Dummy1 ) ;
	exit ;
    end ;
    val( S, Low, Dummy ) ;
    High := Low ;
end  ;



constructor t2D_Unbounded_Array.init( Name : string ) ;

var F : Text ;
    S, S1, Line1 : string ;
    Dummy, Low, High, Loop : integer ;
    This, This_Low, This_High : integer ;
    Child_Array : PUnbounded_Static_Array ;
    Column_Low, Column_High : integer ; { Column range (major offset) }
    Row_Low, Row_High : integer ; { Row range }
    Value : integer ;

begin
    { Setup... }
    assign( F, Name ) ;
    {$I-}
    reset( F ) ;
    {$I+}
    if( IOResult <> 0 ) then
    begin
	fail ;
    end ;

    { Create array of arrays... }
    S := '' ;
    while( length( S ) = 0 ) do
    begin
	{$I-}
	readln( F, S ) ;
	{$I+}
	if( IOResult <> 0 ) then
	begin
	    close( F ) ;
	    fail ;
	end ;
	if( copy( S, 1, 1 ) = '!' ) then { Skip comments }
	begin
	    S := '' ;
	end ;
    end ;
    if( S[ 1 ] <> HT ) then
    begin
	close( F ) ;
	Fail ;
    end ;
    S := copy( S, 2, 254 ) ; { Trim tab }
    Line1 := S ;
    Parse_Range( S, 1, Low, High ) ;
    if( Low = 32767 ) then
    begin
	close( F ) ;
	Fail ;
    end ;
    Column_Low := High ; { Lowest unique column }
    Row_High := High ;
    Dummy := 2 ;
    while( High < 32767 ) do
    begin
	Row_High := Low ; { Save new high value }
	Parse_Range( S, Dummy, Low, High ) ;
	Dummy := Dummy + 1 ;
    end ;
    if( Low = 32767 ) then { Failure - EOL }
    begin
	Column_High := Row_High ;
    end else
    begin
	Column_High := Low ; { Highest unique value }
    end ;

    { Find out the range of rows }
    Row_Low := 32767 ;
    Row_High := 0 ;
    while( not EOF( F ) ) do
    begin
	readln( F, S ) ;
	Parse_Range( S, 1, Low, High ) ;
	if( Low < Row_Low ) then
	begin
	    Row_Low := Low ;
	end ;
	if( High > Row_High ) then
	begin
	    Row_High := High ;
	end ;
    end ;

    { Create the root array (columns) }
    _Array := nil ;
    new( _Array, Init( Row_Low, Row_High, 4, True ) ) ;
    if( _Array = nil ) then
    begin
	Fail ;
    end ;

    { Now create and populate the child arrays... }

    { Skip past first line }
    reset( F ) ;
    S := '' ;
    while( length( S ) = 0 ) do
    begin
	readln( F, S ) ;
	if( copy( S, 1, 1 ) = '!' ) then { Skip comments }
	begin
	    S := '' ;
	end ;
    end ;

    { Create the children arrays (rows) }
    while( not EOF( F ) ) do
    begin
	readln( F, S ) ;
	if( copy( S1, 1, 1 ) = '!' ) then { Skip comments }
	begin
	    continue ;
	end ;
	if( S1 = '' ) then
	begin
	    break ; { Done with table }
	end ;

	{ Create the children arrays }
	Parse_Range( S, 1, Low, High ) ;
	for Loop := Low to High do
	begin
	    Child_Array := nil ;
	    new( Child_Array, Init( Column_Low, Column_High, sizeof( integer ), False ) ) ;
	    _Array^.Put( Loop, Child_Array ) ;
	end ;

	{ Populate the children arrays }
	This_High := This_Low ;
	Dummy := 2 ;
	while( This_High < Column_High ) do
	begin
	    Parse_Range( Line1, Dummy, This_Low, This_High ) ;
	    for Loop := Low to High do
	    begin
		_Array^.Get( Loop, Child_Array ) ;
		Parse_Range( S, Dummy, Value, Value ) ;
		for This := This_Low to This_High do
		begin
		    Child_Array^.Put( This, Value ) ;
		end ;
	    end ;
	    inc( Dummy ) ;
	end ;
    end ;
    close( F ) ;
end ;


destructor t2D_Unbounded_Array.Done ;

begin
    dispose( _Array, Done ) ;
end ;


procedure t2D_Unbounded_Array.Put2( X, Y, Value : integer ) ;

var Child_Array : PUnbounded_Static_Array ;

begin
    _Array^.Get( X, Child_Array ) ;
    Child_Array^.Put( Y, Value ) ;
end ;


function t2D_Unbounded_Array.Get2( X, Y : integer ) : integer ;

var Child_Array : PUnbounded_Static_Array ;
    Dummy : integer ;

begin
    if( X < _Array^.Low ) then
    begin
	X := _Array^.Low ;
    end ;
    if( X > _Array^.High ) then
    begin
	X := _Array^.High ;
    end ;
    _Array^.Get( X, Child_Array ) ;
    if( Y < Child_Array^.Low ) then
    begin
	Y := Child_Array^.Low ;
    end ;
    if( Y > Child_Array^.High ) then
    begin
	Y := Child_Array^.High ;
    end ;
    Child_Array^.Get( Y, Dummy ) ;
    Get2 := Dummy ;
end ;


end.
