{$N+}
{$E+}
unit MStores ;
{
        Program Name : MStores
        Package Name : Subroutine Library - Layer 1
        Purpose      : Managed Storage objects
        Institution  : Conroy & Conroy Co.
        Date Written : 1999
        Written By   : Alan Conroy
        Version      : 1.1B

        Copyright (C) 1999-2005 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *  1-Nov-2002    EAC         Use TObject from Common.       *
        * 13-Nov-2002    EAC         Rename TAllocator to           *
        *                            TManaged_Store.                *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TManaged_Store is the base (mostly-) abstract class of all allocation
        objects.  It descends from TStore and provides a managed store of data
        (allocations and deallocations, reuse of deallocated storage, etc).

          If your program is experiencing problems possibly related to maanaged
        stores, you can set the DEBUG compiler conditional and all managed
        store allocations, deallocations, reads, and writes will be checked for
        correctness.
}

interface

{$I EDEFINES.INC}

uses { Borland... }
{$IFDEF Debug}
     Classes, { TList }
{$ENDIF}

     { Subroutine Library - Layer 1... }
{$IFDEF Debug}
     CommonUt, { TInteger_List }
{$ENDIF}
     _DebugIn, { PSDebug_Interface }
     _Stores, { TStore_Address }
     Standard, { TInteger64_List }
     Stores, { TBase_Store }
     _UEHDefs ; { TUEC }

type TManaged_Store_Statistics = record
                                     Bytes_Allocated : longint ; { Sum total size of allocations }
                                     Bytes_Deallocated : longint ; { Sum total size of deallocations }
                                     Allocations : longint ; { Count of allocations }
                                     Deallocations : longint ; { Count of deallocations }
                                     Reallocations : longint ; { Count of physical reallocations }
                                     Reallocations_In_Place : longint ;
                                     { Count of reallocations not requiring physical reallocation }
                                     Root_Writes : longint ; { Count of writes to root }
                                     Root_Reads : longint ; { Count of reads from root }
                                     Copies : longint ; { Copy operation count }
                                     Bytes_Copied : longint ; { Bytes copied via Copy method }
                                     Fills : longint ; { Fill operation count }
                                     Bytes_Filled : longint ; { Bytes modified via Fill method }
                                     Extensions : longint ; { Number of extension operations }
                                     { Note: Allocations - Extensions = allocations
                                       not requiring file extensions
                                       (reallocations). }
                                 end ; { TManaged_Store_Statistics }

type TSManaged_Store = object( TSBase_Store )
{$IFDEF Debug}
                           public { Instance data... }
                              Allocations : TList ;
                              _Sizes : TInteger_List ;
                              IDs : TInteger_List ;
                              Allocation_Count : integer ;
{$ENDIF}

                           public { API... }
                               Statistics : TManaged_Store_Statistics ;
{$IFDEF Debug}
                               function Check_Access( PTR, Count : TStore_Address ) : boolean ;
                               procedure Log_Allocation( PTR, Size : TStore_Address ) ;
                               procedure Log_Deallocation( PTR, Size : TStore_Address ) ;
{$ENDIF}

                           public { Constructors and destructors... }
                               constructor Init ;
                               destructor Done ; virtual ;

                           public { Overrides... }
                               function Max_Storage : TStore_Size ; virtual ;

                           public { overrides... }
                               function Read_Data( var Data ; Address, _Size : TStore_Address ;
                                   var UEC : TUEC ) : TStore_Address ; virtual ;

                               function Write_Data( var Data ;
                                   Address, _Size : TStore_Address ;
                                   var UEC : TUEC ) : TStore_Address ; virtual ;

                           public { API... }
                               function Allocate( Size : TStore_Address ) : TStore_Address ;
                                   virtual ; { Allocate space}
                               function Allocate_At( Offset, Size : TStore_Address ) : boolean ;
                                   virtual ; { Allocate space at specific location }
                               procedure Copy( Source, Destination : TStore_Address ;
                                   Count : integer ) ; virtual ; { Copy data }
                               procedure Deallocate( PTR, Size : TStore_Address ) ;
                                   virtual ;
                               { Deallocate space }

                               procedure Fill( PTR : TStore_Address ; Count : integer ;
                                   Value : byte ) ; virtual ;
                               function Reallocate( PTR, Old, New : TStore_Address ) : TStore_Address ;
                                   virtual ;
                               procedure Read( PTR, Size : TStore_Address ;
                                   var Buffer ) ; virtual ; { Read data }
                               procedure Write( PTR, Size : TStore_Address ;
                                   var Buffer ) ; virtual ; { Write data }
                               function SpaceAvail : TStore_Address ; virtual ;
                               function XSpaceAvail : TStore_Address ; virtual ;
                               function MaxSpace : TStore_Address ; virtual ;
                               procedure Get_Root( var Buf ) ; virtual ;
                               procedure Set_Root( var Buf ) ; virtual ;
                               function Debugger : PSDebug_Interface ; virtual ;
                         end ; { TSManaged_Store }
     PSManaged_Store = ^TSManaged_Store ;

{$IFDEF DELPHI}
type TSManaged_Store64 = object( TSBase_Store64 )
  {$IFDEF Debug}
                           private { Instance data... }
                              Allocations : TInteger64_List ;
                              _Sizes : TInteger64_List ;
                              IDs : TInteger_List ;
                              Allocation_Count : integer ;
  {$ENDIF}

                           public { API... }
                               Statistics : TManaged_Store_Statistics ;
  {$IFDEF Debug}
                               function Check_Access( PTR, Count : TStore_Address64 ) : boolean ;
                               procedure Log_Allocation( PTR, Size : TStore_Address64 ) ;
                               procedure Log_Deallocation( PTR, Size : TStore_Address64 ) ;
  {$ENDIF}

                           public { Constructors and destructors... }
                               constructor Init ;
                               destructor Done ; virtual ;

                           public { Overrides... }
                               function Max_Storage : TStore_Size64 ; virtual ;

                               function Read_Data( var Data ;
                                   Address, _Size : TStore_Address64 ;
                                   var UEC : TUEC ) : TStore_Address64 ;
                                   virtual ;

                               function Write_Data( var Data ;
                                   Address, _Size : TStore_Address64 ;
                                   var UEC : TUEC ) : TStore_Address64 ;
                                   virtual ;

                           public { API... }
                               function Allocate( Size : TStore_Address64 ) : TStore_Address64 ;
                                   virtual ; { Allocate space}
                               function Allocate_At( Offset, Size : TStore_Address64 ) : boolean ;
                                   virtual ; { Allocate space at specific location }
                               procedure Copy( Source, Destination : TStore_Address64 ;
                                   Count : integer ) ; virtual ; { Copy data }
                               procedure Deallocate( PTR, Size : TStore_Address64 ) ;
                                   virtual ;
                               { Deallocate space }
                               procedure Fill( PTR : TStore_Address64 ; Count : integer ;
                                   Value : byte ) ; virtual ;
                               function Reallocate( PTR, Old, New : TStore_Address64 ) : TStore_Address64 ;
                                   virtual ;
                               procedure Read( PTR, Size : TStore_Address64 ;
                                   var Buffer ) ; virtual ; { Read data }
                               procedure Write( PTR, Size : TStore_Address64 ;
                                   var Buffer ) ; virtual ; { Write data }
                               function SpaceAvail : TStore_Address64 ;
                                   virtual ;
                               function XSpaceAvail : TStore_Address64 ;
                                   virtual ;
                               function MaxSpace : TStore_Address64 ; virtual ;
                               procedure Get_Root( var Buf ) ; virtual ;
                               procedure Set_Root( var Buf ) ; virtual ;
                               function Debugger : PSDebug_Interface ; virtual ;
                         end ; { TSManaged_Store64 }
     PSManaged_Store64 = ^TSManaged_Store64 ;

type TManaged_Store = class( TBase_Store )
                          public { Instance data... }
                              Statistics : TManaged_Store_Statistics ;

                          public { Overrides... }
                              function Max_Storage : TStore_Size ; override ;

                              function Read_Data( var Data ;
                                  Address, _Size : TStore_Address ;
                                  var UEC : TUEC ) : TStore_Address ; override ;

                              function Write_Data( var Data ;
                                  Address, _Size : TStore_Address ;
                                  var UEC : TUEC ) : TStore_Address ; override ;

                              function Debugger : TCOM_Debug_Interface ;
                                  override ;

                          public { API... }
                              function Allocate( Size : TStore_Address ) : TStore_Address ;
                                  virtual ; { Allocate space}
                              function Allocate_At( Offset, Size : TStore_Address ) : boolean ;
                                  virtual ; { Allocate space at specific location }
                              procedure Copy( Source, Destination : TStore_Address ;
                                  Count : integer ) ; virtual ; { Copy data }
                              procedure Deallocate( PTR, Size : TStore_Address ) ;
                                  virtual ;
                              { Deallocate space }
                              procedure Fill( PTR : TStore_Address ; Count : integer ;
                                  Value : byte ) ; virtual ;
                              function Reallocate( PTR, Old, New : TStore_Address ) : TStore_Address ;
                                  virtual ;
                              procedure Read( PTR, Size : TStore_Address ;
                                  var Buffer ) ; virtual ; { Read data }
                              procedure Write( PTR, Size : TStore_Address ;
                                  var Buffer ) ; virtual ; { Write data }
                              function SpaceAvail : TStore_Address ; virtual ;
                              function XSpaceAvail : TStore_Address ; virtual ;
                              function MaxSpace : TStore_Address ; virtual ;
                              procedure Get_Root( var Buf ) ; virtual ;
                              procedure Set_Root( var Buf ) ; virtual ;
                        end ; { TManaged_Store }

type TManaged_Store64 = class( TBase_Store64 )
                            public { Instance data... }
                                Statistics : TManaged_Store_Statistics ;

                            public { Overrides... }
                                function Max_Storage : TStore_Size64 ;
                                    override ;

                                function Read_Data( var Data ;
                                    Address, _Size : TStore_Address64 ;
                                    var UEC : TUEC ) : TStore_Address64 ;
                                    override ;

                                function Write_Data( var Data ;
                                    Address, _Size : TStore_Address64 ;
                                    var UEC : TUEC ) : TStore_Address64 ;
                                    override ;

                                function Debugger : TCOM_Debug_Interface ;
                                    override ;

                            public { API... }
                                function Allocate( Size : TStore_Address64 ) : TStore_Address64 ;
                                    virtual ; { Allocate space}
                                function Allocate_At( Offset, Size : TStore_Address64 ) : boolean ;
                                    virtual ; { Allocate space at specific location }
                                procedure Copy( Source, Destination : TStore_Address64 ;
                                    Count : integer ) ; virtual ; { Copy data }
                                procedure Deallocate( PTR, Size : TStore_Address64 ) ;
                                    virtual ;
                                { Deallocate space }
                                procedure Fill( PTR : TStore_Address64 ; Count : integer ;
                                    Value : byte ) ; virtual ;
                                function Reallocate( PTR, Old, New : TStore_Address64 ) : TStore_Address64 ;
                                    virtual ;
                                procedure Read( PTR, Size : TStore_Address64 ;
                                    var Buffer ) ; virtual ; { Read data }
                                procedure Write( PTR, Size : TStore_Address64 ;
                                    var Buffer ) ; virtual ; { Write data }
                                function SpaceAvail : TStore_Address64 ;
                                    virtual ;
                                function XSpaceAvail : TStore_Address64 ;
                                    virtual ;
                                function MaxSpace : TStore_Address64 ; virtual ;
                                procedure Get_Root( var Buf ) ; virtual ;
                                procedure Set_Root( var Buf ) ; virtual ;
                          end ; { TManaged_Store64 }

type TCOM_Managed_Store = class( TStandard_COM_Store )
                              public { Instance data... }
                                  Statistics : TManaged_Store_Statistics ;

                              public { Overrides... }
                                  function Max_Storage : TStore_Size ;
                                      override ;

                                  function Read_Data( var Data ;
                                      Address, _Size : TStore_Address ;
                                      var UEC : TUEC ) : TStore_Address ;
                                          override ;

                                  function Write_Data( var Data ;
                                      Address, _Size : TStore_Address ;
                                      var UEC : TUEC ) : TStore_Address ;
                                      override ;

                                  function Debugger : TCOM_Debug_Interface ;
                                      override ;

                              public { API... }
                                  function Allocate( Size : TStore_Address ) : TStore_Address ;
                                      virtual ; stdcall ; { Allocate space}
                                  function Allocate_At( Offset, Size : TStore_Address ) : boolean ;
                                      virtual ; stdcall ; { Allocate space at specific location }
                                  procedure Copy( Source, Destination : TStore_Address ;
                                      Count : integer ) ; virtual ; stdcall ; { Copy data }
                                  procedure Deallocate( PTR, Size : TStore_Address ) ;
                                      virtual ; stdcall ;
                                  { Deallocate space }
                                  procedure Fill( PTR : TStore_Address ; Count : integer ;
                                      Value : byte ) ; virtual ; stdcall ;
                                  function Reallocate( PTR, Old, New : TStore_Address ) : TStore_Address ;
                                      virtual ; stdcall ;
                                  procedure Read( PTR, Size : TStore_Address ;
                                      var Buffer ) ; virtual ; stdcall ; { Read data }
                                  procedure Write( PTR, Size : TStore_Address ;
                                      var Buffer ) ; virtual ; stdcall ; { Write data }
                                  function SpaceAvail : TStore_Address ;
                                      virtual ; stdcall ;
                                  function XSpaceAvail : TStore_Address ;
                                      virtual ; stdcall ;
                                  function MaxSpace : TStore_Address ;
                                      virtual ; stdcall ;
                                  procedure Get_Root( var Buf ) ;
                                      virtual ; stdcall ;
                                  procedure Set_Root( var Buf ) ;
                                      virtual ; stdcall ;
                          end ; { TCOM_Managed_Store }

type TCOM_Managed_Store64 = class( TStandard_COM_Store64 )
                                public { Instance data... }
                                    Statistics : TManaged_Store_Statistics ;

                                public { Overrides... }
                                    function Max_Storage : TStore_Size64 ;
                                        override ;

                                    function Read_Data( var Data ;
                                        Address, _Size : TStore_Address64 ;
                                        var UEC : TUEC ) : TStore_Address64 ;
                                        override ;

                                    function Write_Data( var Data ;
                                        Address, _Size : TStore_Address64 ;
                                        var UEC : TUEC ) : TStore_Address64 ;
                                        override ;

                                    function Debugger : TCOM_Debug_Interface ;
                                        override ;

                                public { API... }
                                    function Allocate( Size : TStore_Address64 ) : TStore_Address64 ;
                                        virtual ; stdcall ; { Allocate space}
                                    function Allocate_At( Offset, Size : TStore_Address64 ) : boolean ;
                                        virtual ; stdcall ; { Allocate space at specific location }
                                    procedure Copy( Source, Destination : TStore_Address64 ;
                                        Count : integer ) ; virtual ; stdcall ; { Copy data }
                                    procedure Deallocate( PTR, Size : TStore_Address64 ) ;
                                        virtual ; stdcall ; { Deallocate space }
                                    procedure Fill( PTR : TStore_Address64 ; Count : integer ;
                                        Value : byte ) ; virtual ; stdcall ;
                                    function Reallocate( PTR, Old, New : TStore_Address64 ) : TStore_Address64 ;
                                        virtual ; stdcall ;
                                    procedure Read( PTR, Size : TStore_Address64 ;
                                        var Buffer ) ; virtual ; stdcall ; { Read data }
                                    procedure Write( PTR, Size : TStore_Address64 ;
                                        var Buffer ) ; virtual ; stdcall ; { Write data }
                                    function SpaceAvail : TStore_Address64 ;
                                        virtual ; stdcall ;
                                    function XSpaceAvail : TStore_Address64 ;
                                        virtual ; stdcall ;
                                    function MaxSpace : TStore_Address64 ;
                                        virtual ; stdcall ;
                                    procedure Get_Root( var Buf ) ;
                                        virtual ; stdcall ;
                                    procedure Set_Root( var Buf ) ;
                                        virtual ; stdcall ;
                            end ; { TCOM_Managed_Store64 }
{$ENDIF}

{$IFDEF Test}
procedure Test_Unit ;
{$ENDIF}


implementation

uses { Subroutine Library - Layer 0... }
     _Common, { Abstract_Error }

     { Subroutine Library - Layer 1... }
     _App ; { Application_Manager }

{ TSManaged_Store methods... }

{ Constructors and destructors... }

constructor TSManaged_Store.Init ;

begin
    inherited Init ;
{$IFDEF Debug}
    Allocations := TList.Create ;
    _Sizes := TInteger_List.Create ;
    IDs := TInteger_List.Create ;
    Allocation_Count := 0 ;
{$ENDIF}
end ;


destructor TSManaged_Store.Done ;

begin
    inherited Done ;
{$IFDEF Debug}
    Allocations.Free ;
    Allocations := nil ;
    _Sizes.Free ;
    _Sizes := nil ;
    IDs.Free ;
    IDs := nil ;
{$ENDIF}
end ;


{ Overrides... }

function TSManaged_Store.Max_Storage : TStore_Size ;

begin
    Max_Storage := MaxSpace ;
end ;


{$IFDEF Debug}
{ Debug support... }

function TSManaged_Store.Check_Access( PTR, Count : TStore_Address ) : boolean ;

var Loop : integer ;
    Address, Size : integer ;

begin
    for Loop := 0 to Allocations.Count - 1 do
    begin
        Address := integer( Allocations[ Loop ] ) ;
        Size := integer( _Sizes[ Loop ] ) ;
        if( ( PTR >= Address ) and ( PTR + Count <= Address + Size ) ) then
        begin
            Check_Access := True ;
            exit ;
        end ;
    end ;
    Check_Access := False ;
end ;


procedure TSManaged_Store.Log_Allocation( PTR, Size : TStore_Address ) ;

begin
    Allocations.Add( pointer( Ptr ) ) ;
    _Sizes.Add( Size ) ;
    IDs.Add( Allocation_Count ) ;
    inc( Allocation_Count ) ;
end ;


procedure TSManaged_Store.Log_Deallocation( PTR, Size : TStore_Address ) ;

var Index : integer ;

begin
    Index := Allocations.IndexOf( pointer( PTR ) ) ;
    if( Index = -1 ) then { Attempt to deallocate unallocated memory }
    begin
        if( Application_Manager = nil ) then
        begin
            halt ; { UNTESTED }
        end else
        begin
            Application_Manager^.Fatal_Exception( 'Deallocation of unallocated store' ) ;
            exit ;
        end ;
    end ;
    Allocations.Delete( Index ) ;
    _Sizes.Delete( Index ) ;
    IDs.Delete( Index ) ;
end ;
{$ENDIF}


{ API... }

function TSManaged_Store.Allocate( Size : TStore_Address ) : TStore_Address ;

begin
    Allocate := 0 ;
end ;


function TSManaged_Store.Allocate_At( Offset, Size : TStore_Address ) : boolean ;

begin
    Allocate_At := False ;
end ;


procedure TSManaged_Store.Copy( Source, Destination : TStore_Address ; Count : integer ) ;
{ Copy data }

var Buffer : array[ 0..32767 ] of char ;

begin
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Read( Source, sizeof( Buffer ), Buffer ) ;
            Write( Destination, sizeof( Buffer ), Buffer ) ;
            Source := Source + sizeof( Buffer ) ;
            Destination := Destination + Sizeof( Buffer ) ;
        end else
        begin
            Read( Source, Count, Buffer ) ;
            Write( Destination, Count, Buffer ) ;
            break ; { All done }
        end ;
        dec( Count, sizeof( Buffer ) ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Copies ) ;
    inc( Statistics.Bytes_Copied, Count ) ;
end ;


procedure TSManaged_Store.Deallocate( PTR, Size : TStore_Address ) ;

begin
    Abstract_Error ;
end ;


procedure TSManaged_Store.Fill( PTR : TStore_Address ; Count : integer ; Value : byte ) ;

var Buffer : array[ 0..32767 ] of char ;

begin
    if( Count > sizeof( Buffer ) ) then
    begin
        fillchar( Buffer, sizeof( Buffer ), Value ) ;
    end else
    begin
        fillchar( Buffer, Count, Value ) ;
    end ;
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Write( PTR, sizeof( Buffer ), Buffer ) ;
        end else
        begin
            Write( PTR, Count, Buffer ) ;
        end ;
        Count := Count - sizeof( Buffer ) ;
        PTR := PTR + sizeof( Buffer ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Fills ) ;
    inc( Statistics.Bytes_Filled, Count ) ;
end ;


function TSManaged_Store.Reallocate( PTR, Old, New : TStore_Address ) : TStore_Address ;
{ This function does a reallocation operation by allocating new memory, copying
  it and then deallocating the old.  Descendants should override this default
  method if they have a more efficient reallocation method. }

var Dummy : longint ;

begin
    Dummy := Allocate( New ) ;
    Reallocate := Dummy ;
    if( Dummy <> 0 ) then { Something was allocated }
    begin
        { Copy old data to new location... }
        if( New > Old ) then
        begin
            New := Old ; { Amount to move }
        end ;
        Copy( PTR, Dummy, New ) ;

        { All done with old data }
        Deallocate( PTR, Old ) ;
    end ;

    { Update statistics }
    inc( Statistics.Reallocations ) ;
end ;


function TSManaged_Store.Read_Data( var Data ; Address, _Size : TStore_Address ;
    var UEC : TUEC ) : TStore_Address ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Read( Address, _Size, Data ) ;
    Read_Data := _Size ;
end ;


function TSManaged_Store.Write_Data( var Data ;
    Address, _Size : TStore_Address ; var UEC : TUEC ) : TStore_Address ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Write( Address, _Size, Data ) ;
    Write_Data := _Size ;
end ;


procedure TSManaged_Store.Read( PTR, Size : TStore_Address ; var Buffer ) ;

begin
    Abstract_Error ;
end ;


procedure TSManaged_Store.Write( PTR, Size : TStore_Address ; var Buffer ) ;

begin
end ;


function TSManaged_Store.SpaceAvail : TStore_Address ;

begin
    SpaceAvail := 0 ;
end ;


function TSManaged_Store.XSpaceAvail : TStore_Address ;

begin
    XSpaceAvail := 0 ;
end ;


function TSManaged_Store.MaxSpace : TStore_Address ;

begin
    MaxSpace := 0 ;
end ;


procedure TSManaged_Store.Get_Root( var Buf ) ;

begin
end ;


procedure TSManaged_Store.Set_Root( var Buf ) ;

begin
end ;


function TSManaged_Store.Debugger : PSDebug_Interface ;

begin
    Debugger := nil ;
end ;



{$IFDEF DELPHI}
{ TSManaged_Store64 methods... }

{ Constructors and destructors... }

constructor TSManaged_Store64.Init ;

begin
    inherited Init ;
  {$IFDEF Debug}
    Allocations := TInteger64_List.Create ;
    _Sizes := TInteger64_List.Create ;
    IDs := TInteger_List.Create ;
    Allocation_Count := 0 ;
  {$ENDIF}
end ;


destructor TSManaged_Store64.Done ;

begin
    inherited Done ;
  {$IFDEF Debug}
    Allocations.Free ;
    Allocations := nil ;
    _Sizes.Free ;
    _Sizes := nil ;
    IDs.Free ;
    IDs := nil ;
  {$ENDIF}
end ;


{ Overrides... }

function TSManaged_Store64.Max_Storage : TStore_Size64 ;

begin
    Max_Storage := MaxSpace ;
end ;


{$IFDEF Debug}
{ Debug support... }

function TSManaged_Store64.Check_Access( PTR, Count : TStore_Address64 ) : boolean ;

var Loop : integer ;
    Address, Size : int64 ;

begin
    for Loop := 0 to Allocations.Count - 1 do
    begin
        Address := Allocations[ Loop ] ;
        Size := integer( _Sizes[ Loop ] ) ;
        if( ( PTR >= Address ) and ( PTR + Count <= Address + Size ) ) then
        begin
            Check_Access := True ;
            exit ;
        end ;
    end ;
    Check_Access := False ;
end ;


procedure TSManaged_Store64.Log_Allocation( PTR, Size : TStore_Address64 ) ;

begin
    Allocations.Add( Ptr ) ;
    _Sizes.Add( Size ) ;
    IDs.Add( Allocation_Count ) ;
    inc( Allocation_Count ) ;
end ;


procedure TSManaged_Store64.Log_Deallocation( PTR, Size : TStore_Address64 ) ;

var Index : integer ;

begin
    Index := Allocations.IndexOf( PTR ) ;
    if( Index = -1 ) then
    begin
        if( Application_Manager = nil ) then
        begin
            halt ; { UNTESTED }
        end else
        begin
            Application_Manager^.Fatal_Exception( 'Deallocation of unallocated store' ) ;
            exit ;
        end ;
    end ;
    Allocations.Delete( Index ) ;
    _Sizes.Delete( Index ) ;
    IDs.Delete( Index ) ;
end ;
{$ENDIF}


{ API... }

function TSManaged_Store64.Allocate( Size : TStore_Address64 ) : TStore_Address64 ;

begin
    Allocate := 0 ;
end ;


function TSManaged_Store64.Allocate_At( Offset, Size : TStore_Address64 ) : boolean ;

begin
    Allocate_At := False ;
end ;


procedure TSManaged_Store64.Copy( Source, Destination : TStore_Address64 ; Count : integer ) ;
{ Copy data }

var Buffer : array[ 0..32767 ] of char ;

begin
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Read( Source, sizeof( Buffer ), Buffer ) ;
            Write( Destination, sizeof( Buffer ), Buffer ) ;
            Source := Source + sizeof( Buffer ) ;
            Destination := Destination + Sizeof( Buffer ) ;
        end else
        begin
            Read( Source, Count, Buffer ) ;
            Write( Destination, Count, Buffer ) ;
            break ; { All done }
        end ;
        dec( Count, sizeof( Buffer ) ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Copies ) ;
    inc( Statistics.Bytes_Copied, Count ) ;
end ;


procedure TSManaged_Store64.Deallocate( PTR, Size : TStore_Address64 ) ;

begin
    Abstract_Error ;
end ;


procedure TSManaged_Store64.Fill( PTR : TStore_Address64 ; Count : integer ;
    Value : byte ) ;

var Buffer : array[ 0..32767 ] of char ;

begin
    if( Count > sizeof( Buffer ) ) then
    begin
        fillchar( Buffer, sizeof( Buffer ), Value ) ;
    end else
    begin
        fillchar( Buffer, Count, Value ) ;
    end ;
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Write( PTR, sizeof( Buffer ), Buffer ) ;
        end else
        begin
            Write( PTR, Count, Buffer ) ;
        end ;
        Count := Count - sizeof( Buffer ) ;
        PTR := PTR + sizeof( Buffer ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Fills ) ;
    inc( Statistics.Bytes_Filled, Count ) ;
end ;


function TSManaged_Store64.Reallocate( PTR, Old, New : TStore_Address64 ) : TStore_Address64 ;
{ This function does a reallocation operation by allocating new memory, copying
  it and then deallocating the old.  Descendants should override this default
  method if they have a more efficient reallocation method. }

var Dummy : longint ;

begin
    Dummy := Allocate( New ) ;
    Reallocate := Dummy ;
    if( Dummy <> 0 ) then { Something was allocated }
    begin
        { Copy old data to new location... }
        if( New > Old ) then
        begin
            New := Old ; { Amount to move }
        end ;
        Copy( PTR, Dummy, New ) ;

        { All done with old data }
        Deallocate( PTR, Old ) ;
    end ;

    { Update statistics }
    inc( Statistics.Reallocations ) ;
end ;


function TSManaged_Store64.Read_Data( var Data ; Address, _Size : TStore_Address64 ;
    var UEC : TUEC ) : TStore_Address64 ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Read( Address, _Size, Data ) ;
    Read_Data := _Size ;
end ;


function TSManaged_Store64.Write_Data( var Data ;
    Address, _Size : TStore_Address64 ; var UEC : TUEC ) : TStore_Address64 ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Write( Address, _Size, Data ) ;
    Write_Data := _Size ;
end ;


procedure TSManaged_Store64.Read( PTR, Size : TStore_Address64 ; var Buffer ) ;

begin
    Abstract_Error ;
end ;


procedure TSManaged_Store64.Write( PTR, Size : TStore_Address64 ; var Buffer ) ;

begin
end ;


function TSManaged_Store64.SpaceAvail : TStore_Address64 ;

begin
    SpaceAvail := 0 ;
end ;


function TSManaged_Store64.XSpaceAvail : TStore_Address64 ;

begin
    XSpaceAvail := 0 ;
end ;


function TSManaged_Store64.MaxSpace : TStore_Address64 ;

begin
    MaxSpace := 0 ;
end ;


procedure TSManaged_Store64.Get_Root( var Buf ) ;

begin
end ;


procedure TSManaged_Store64.Set_Root( var Buf ) ;

begin
end ;


function TSManaged_Store64.Debugger : PSDebug_Interface ;

begin
    Debugger := nil ;
end ;


{ TManaged_Store methods... }

{ Overrides... }

function TManaged_Store.Max_Storage : TStore_Size ;

begin
    Max_Storage := MaxSpace ;
end ;


{ API... }

function TManaged_Store.Allocate( Size : TStore_Address ) : TStore_Address ;

begin
    Allocate := 0 ;
end ;


function TManaged_Store.Allocate_At( Offset, Size : TStore_Address ) : boolean ;

begin
    Allocate_At := False ;
end ;


procedure TManaged_Store.Copy( Source, Destination : TStore_Address ; Count : integer ) ;
{ Copy data }

var Buffer : array[ 0..32767 ] of char ;

begin
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Read( Source, sizeof( Buffer ), Buffer ) ;
            Write( Destination, sizeof( Buffer ), Buffer ) ;
            Source := Source + sizeof( Buffer ) ;
            Destination := Destination + Sizeof( Buffer ) ;
        end else
        begin
            Read( Source, Count, Buffer ) ;
            Write( Destination, Count, Buffer ) ;
            break ; { All done }
        end ;
        dec( Count, sizeof( Buffer ) ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Copies ) ;
    inc( Statistics.Bytes_Copied, Count ) ;
end ;


procedure TManaged_Store.Deallocate( PTR, Size : TStore_Address ) ;

begin
    Abstract_Error ;
end ;


procedure TManaged_Store.Fill( PTR : TStore_Address ; Count : integer ; Value : byte ) ;

var Buffer : array[ 0..32767 ] of char ;

begin
    if( Count > sizeof( Buffer ) ) then
    begin
        fillchar( Buffer, sizeof( Buffer ), Value ) ;
    end else
    begin
        fillchar( Buffer, Count, Value ) ;
    end ;
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Write( PTR, sizeof( Buffer ), Buffer ) ;
        end else
        begin
            Write( PTR, Count, Buffer ) ;
        end ;
        Count := Count - sizeof( Buffer ) ;
        PTR := PTR + sizeof( Buffer ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Fills ) ;
    inc( Statistics.Bytes_Filled, Count ) ;
end ;


function TManaged_Store.Reallocate( PTR, Old, New : TStore_Address ) : TStore_Address ;
{ This function does a reallocation operation by allocating new memory, copying
  it and then deallocating the old.  Descendants should override this default
  method if they have a more efficient reallocation method. }

var Dummy : longint ;

begin
    Dummy := Allocate( New ) ;
    Reallocate := Dummy ;
    if( Dummy <> 0 ) then { Something was allocated }
    begin
        { Copy old data to new location... }
        if( New > Old ) then
        begin
            New := Old ; { Amount to move }
        end ;
        Copy( PTR, Dummy, New ) ;

        { All done with old data }
        Deallocate( PTR, Old ) ;
    end ;

    { Update statistics }
    inc( Statistics.Reallocations ) ;
end ;


function TManaged_Store.Read_Data( var Data ; Address, _Size : TStore_Address ;
    var UEC : TUEC ) : TStore_Address ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Read( Address, _Size, Data ) ;
    Read_Data := _Size ;
end ;


function TManaged_Store.Write_Data( var Data ;
    Address, _Size : TStore_Address ; var UEC : TUEC ) : TStore_Address ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Write( Address, _Size, Data ) ;
    Write_Data := _Size ;
end ;


procedure TManaged_Store.Read( PTR, Size : TStore_Address ; var Buffer ) ;

begin
    Abstract_Error ;
end ;


procedure TManaged_Store.Write( PTR, Size : TStore_Address ; var Buffer ) ;

begin
end ;


function TManaged_Store.SpaceAvail : TStore_Address ;

begin
    SpaceAvail := 0 ;
end ;


function TManaged_Store.XSpaceAvail : TStore_Address ;

begin
    XSpaceAvail := 0 ;
end ;


function TManaged_Store.MaxSpace : TStore_Address ;

begin
    MaxSpace := 0 ;
end ;


procedure TManaged_Store.Get_Root( var Buf ) ;

begin
end ;


procedure TManaged_Store.Set_Root( var Buf ) ;

begin
end ;


function TManaged_Store.Debugger : TCOM_Debug_Interface ;

begin
    Debugger := nil ;
end ;



{ TManaged_Store64 methods... }

{ Overrides... }

function TManaged_Store64.Max_Storage : TStore_Size64 ;

begin
    Max_Storage := MaxSpace ;
end ;


{ API... }

function TManaged_Store64.Allocate( Size : TStore_Address64 ) : TStore_Address64 ;

begin
    Allocate := 0 ;
end ;


function TManaged_Store64.Allocate_At( Offset, Size : TStore_Address64 ) : boolean ;

begin
    Allocate_At := False ;
end ;


procedure TManaged_Store64.Copy( Source, Destination : TStore_Address64 ; Count : integer ) ;
{ Copy data }

var Buffer : array[ 0..32767 ] of char ;

begin
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Read( Source, sizeof( Buffer ), Buffer ) ;
            Write( Destination, sizeof( Buffer ), Buffer ) ;
            Source := Source + sizeof( Buffer ) ;
            Destination := Destination + Sizeof( Buffer ) ;
        end else
        begin
            Read( Source, Count, Buffer ) ;
            Write( Destination, Count, Buffer ) ;
            break ; { All done }
        end ;
        dec( Count, sizeof( Buffer ) ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Copies ) ;
    inc( Statistics.Bytes_Copied, Count ) ;
end ;


procedure TManaged_Store64.Deallocate( PTR, Size : TStore_Address64 ) ;

begin
    Abstract_Error ;
end ;


procedure TManaged_Store64.Fill( PTR : TStore_Address64 ; Count : integer ;
    Value : byte ) ;

var Buffer : array[ 0..32767 ] of char ;

begin
    if( Count > sizeof( Buffer ) ) then
    begin
        fillchar( Buffer, sizeof( Buffer ), Value ) ;
    end else
    begin
        fillchar( Buffer, Count, Value ) ;
    end ;
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Write( PTR, sizeof( Buffer ), Buffer ) ;
        end else
        begin
            Write( PTR, Count, Buffer ) ;
        end ;
        Count := Count - sizeof( Buffer ) ;
        PTR := PTR + sizeof( Buffer ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Fills ) ;
    inc( Statistics.Bytes_Filled, Count ) ;
end ;


function TManaged_Store64.Reallocate( PTR, Old, New : TStore_Address64 ) : TStore_Address64 ;
{ This function does a reallocation operation by allocating new memory, copying
  it and then deallocating the old.  Descendants should override this default
  method if they have a more efficient reallocation method. }

var Dummy : longint ;

begin
    Dummy := Allocate( New ) ;
    Reallocate := Dummy ;
    if( Dummy <> 0 ) then { Something was allocated }
    begin
        { Copy old data to new location... }
        if( New > Old ) then
        begin
            New := Old ; { Amount to move }
        end ;
        Copy( PTR, Dummy, New ) ;

        { All done with old data }
        Deallocate( PTR, Old ) ;
    end ;

    { Update statistics }
    inc( Statistics.Reallocations ) ;
end ;


function TManaged_Store64.Read_Data( var Data ; Address, _Size : TStore_Address64 ;
    var UEC : TUEC ) : TStore_Address64 ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Read( Address, _Size, Data ) ;
    Read_Data := _Size ;
end ;


function TManaged_Store64.Write_Data( var Data ;
    Address, _Size : TStore_Address64 ; var UEC : TUEC ) : TStore_Address64 ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Write( Address, _Size, Data ) ;
    Write_Data := _Size ;
end ;


procedure TManaged_Store64.Read( PTR, Size : TStore_Address64 ; var Buffer ) ;

begin
    Abstract_Error ;
end ;


procedure TManaged_Store64.Write( PTR, Size : TStore_Address64 ; var Buffer ) ;

begin
end ;


function TManaged_Store64.SpaceAvail : TStore_Address64 ;

begin
    SpaceAvail := 0 ;
end ;


function TManaged_Store64.XSpaceAvail : TStore_Address64 ;

begin
    XSpaceAvail := 0 ;
end ;


function TManaged_Store64.MaxSpace : TStore_Address64 ;

begin
    MaxSpace := 0 ;
end ;


procedure TManaged_Store64.Get_Root( var Buf ) ;

begin
end ;


procedure TManaged_Store64.Set_Root( var Buf ) ;

begin
end ;


function TManaged_Store64.Debugger : TCOM_Debug_Interface ;

begin
    Debugger := nil ;
end ;


{ TCOM_Managed_Store methods... }

{ Overrides... }

function TCOM_Managed_Store.Max_Storage : TStore_Size ;

begin
    Max_Storage := MaxSpace ;
end ;


{ API... }

function TCOM_Managed_Store.Allocate( Size : TStore_Address ) : TStore_Address ;

begin
    Allocate := 0 ;
end ;


function TCOM_Managed_Store.Allocate_At( Offset, Size : TStore_Address ) : boolean ;

begin
    Allocate_At := False ;
end ;


procedure TCOM_Managed_Store.Copy( Source, Destination : TStore_Address ; Count : integer ) ;
{ Copy data }

var Buffer : array[ 0..32767 ] of char ;

begin
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Read( Source, sizeof( Buffer ), Buffer ) ;
            Write( Destination, sizeof( Buffer ), Buffer ) ;
            Source := Source + sizeof( Buffer ) ;
            Destination := Destination + Sizeof( Buffer ) ;
        end else
        begin
            Read( Source, Count, Buffer ) ;
            Write( Destination, Count, Buffer ) ;
            break ; { All done }
        end ;
        dec( Count, sizeof( Buffer ) ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Copies ) ;
    inc( Statistics.Bytes_Copied, Count ) ;
end ;


procedure TCOM_Managed_Store.Deallocate( PTR, Size : TStore_Address ) ;

begin
    Abstract_Error ;
end ;


procedure TCOM_Managed_Store.Fill( PTR : TStore_Address ; Count : integer ; Value : byte ) ;

var Buffer : array[ 0..32767 ] of char ;

begin
    if( Count > sizeof( Buffer ) ) then
    begin
        fillchar( Buffer, sizeof( Buffer ), Value ) ;
    end else
    begin
        fillchar( Buffer, Count, Value ) ;
    end ;
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Write( PTR, sizeof( Buffer ), Buffer ) ;
        end else
        begin
            Write( PTR, Count, Buffer ) ;
        end ;
        Count := Count - sizeof( Buffer ) ;
        PTR := PTR + sizeof( Buffer ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Fills ) ;
    inc( Statistics.Bytes_Filled, Count ) ;
end ;


function TCOM_Managed_Store.Reallocate( PTR, Old, New : TStore_Address ) : TStore_Address ;
{ This function does a reallocation operation by allocating new memory, copying
  it and then deallocating the old.  Descendants should override this default
  method if they have a more efficient reallocation method. }

var Dummy : longint ;

begin
    Dummy := Allocate( New ) ;
    Reallocate := Dummy ;
    if( Dummy <> 0 ) then { Something was allocated }
    begin
        { Copy old data to new location... }
        if( New > Old ) then
        begin
            New := Old ; { Amount to move }
        end ;
        Copy( PTR, Dummy, New ) ;

        { All done with old data }
        Deallocate( PTR, Old ) ;
    end ;

    { Update statistics }
    inc( Statistics.Reallocations ) ;
end ;


function TCOM_Managed_Store.Read_Data( var Data ; Address, _Size : TStore_Address ;
    var UEC : TUEC ) : TStore_Address ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Read( Address, _Size, Data ) ;
    Read_Data := _Size ;
end ;


function TCOM_Managed_Store.Write_Data( var Data ;
    Address, _Size : TStore_Address ; var UEC : TUEC ) : TStore_Address ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Write( Address, _Size, Data ) ;
    Write_Data := _Size ;
end ;


procedure TCOM_Managed_Store.Read( PTR, Size : TStore_Address ; var Buffer ) ;

begin
    Abstract_Error ;
end ;


procedure TCOM_Managed_Store.Write( PTR, Size : TStore_Address ; var Buffer ) ;

begin
end ;


function TCOM_Managed_Store.SpaceAvail : TStore_Address ;

begin
    SpaceAvail := 0 ;
end ;


function TCOM_Managed_Store.XSpaceAvail : TStore_Address ;

begin
    XSpaceAvail := 0 ;
end ;


function TCOM_Managed_Store.MaxSpace : TStore_Address ;

begin
    MaxSpace := 0 ;
end ;


procedure TCOM_Managed_Store.Get_Root( var Buf ) ;

begin
end ;


procedure TCOM_Managed_Store.Set_Root( var Buf ) ;

begin
end ;


function TCOM_Managed_Store.Debugger : TCOM_Debug_Interface ;

begin
    Debugger := nil ;
end ;



{ TCOM_Managed_Store64 methods... }

{ Overrides... }

function TCOM_Managed_Store64.Max_Storage : TStore_Size64 ;

begin
    Max_Storage := MaxSpace ;
end ;


{ API... }

function TCOM_Managed_Store64.Allocate( Size : TStore_Address64 ) : TStore_Address64 ;

begin
    Allocate := 0 ;
end ;


function TCOM_Managed_Store64.Allocate_At( Offset, Size : TStore_Address64 ) : boolean ;

begin
    Allocate_At := False ;
end ;


procedure TCOM_Managed_Store64.Copy( Source, Destination : TStore_Address64 ; Count : integer ) ;
{ Copy data }

var Buffer : array[ 0..32767 ] of char ;

begin
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Read( Source, sizeof( Buffer ), Buffer ) ;
            Write( Destination, sizeof( Buffer ), Buffer ) ;
            Source := Source + sizeof( Buffer ) ;
            Destination := Destination + Sizeof( Buffer ) ;
        end else
        begin
            Read( Source, Count, Buffer ) ;
            Write( Destination, Count, Buffer ) ;
            break ; { All done }
        end ;
        dec( Count, sizeof( Buffer ) ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Copies ) ;
    inc( Statistics.Bytes_Copied, Count ) ;
end ;


procedure TCOM_Managed_Store64.Deallocate( PTR, Size : TStore_Address64 ) ;

begin
    Abstract_Error ;
end ;


procedure TCOM_Managed_Store64.Fill( PTR : TStore_Address64 ; Count : integer ;
    Value : byte ) ;

var Buffer : array[ 0..32767 ] of char ;

begin
    if( Count > sizeof( Buffer ) ) then
    begin
        fillchar( Buffer, sizeof( Buffer ), Value ) ;
    end else
    begin
        fillchar( Buffer, Count, Value ) ;
    end ;
    while( Count > 0 ) do
    begin
        if( Count > sizeof( Buffer ) ) then
        begin
            Write( PTR, sizeof( Buffer ), Buffer ) ;
        end else
        begin
            Write( PTR, Count, Buffer ) ;
        end ;
        Count := Count - sizeof( Buffer ) ;
        PTR := PTR + sizeof( Buffer ) ;
    end ;

    { Update statistics... }
    inc( Statistics.Fills ) ;
    inc( Statistics.Bytes_Filled, Count ) ;
end ;


function TCOM_Managed_Store64.Reallocate( PTR, Old, New : TStore_Address64 ) : TStore_Address64 ;
{ This function does a reallocation operation by allocating new memory, copying
  it and then deallocating the old.  Descendants should override this default
  method if they have a more efficient reallocation method. }

var Dummy : longint ;

begin
    Dummy := Allocate( New ) ;
    Reallocate := Dummy ;
    if( Dummy <> 0 ) then { Something was allocated }
    begin
        { Copy old data to new location... }
        if( New > Old ) then
        begin
            New := Old ; { Amount to move }
        end ;
        Copy( PTR, Dummy, New ) ;

        { All done with old data }
        Deallocate( PTR, Old ) ;
    end ;

    { Update statistics }
    inc( Statistics.Reallocations ) ;
end ;


function TCOM_Managed_Store64.Read_Data( var Data ; Address, _Size : TStore_Address64 ;
    var UEC : TUEC ) : TStore_Address64 ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Read( Address, _Size, Data ) ;
    Read_Data := _Size ;
end ;


function TCOM_Managed_Store64.Write_Data( var Data ;
    Address, _Size : TStore_Address64 ; var UEC : TUEC ) : TStore_Address64 ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
    Write( Address, _Size, Data ) ;
    Write_Data := _Size ;
end ;


procedure TCOM_Managed_Store64.Read( PTR, Size : TStore_Address64 ; var Buffer ) ;

begin
    Abstract_Error ;
end ;


procedure TCOM_Managed_Store64.Write( PTR, Size : TStore_Address64 ; var Buffer ) ;

begin
end ;


function TCOM_Managed_Store64.SpaceAvail : TStore_Address64 ;

begin
    SpaceAvail := 0 ;
end ;


function TCOM_Managed_Store64.XSpaceAvail : TStore_Address64 ;

begin
    XSpaceAvail := 0 ;
end ;


function TCOM_Managed_Store64.MaxSpace : TStore_Address64 ;

begin
    MaxSpace := 0 ;
end ;


procedure TCOM_Managed_Store64.Get_Root( var Buf ) ;

begin
end ;


procedure TCOM_Managed_Store64.Set_Root( var Buf ) ;

begin
end ;


function TCOM_Managed_Store64.Debugger : TCOM_Debug_Interface ;

begin
    Debugger := nil ;
end ;
{$ENDIF}


{$IFDEF Test}
procedure Error ;

begin
    writeln( 'Error' ) ;
    halt ;
end ;


  {$IFDEF Debug}
procedure Test_Logging ;

var OK : boolean ;
    Test : TSManaged_Store ;
    Test64 : TSManaged_Store64 ;

begin
    Test.Init ;
    if( Test.Check_Access( 0, 10 ) ) then
    begin
        Error ;
    end ;
    Test.Log_Allocation( 0, 10 ) ;
    if( not Test.Check_Access( 0, 10 ) ) then
    begin
        Error ;
    end ;
    if( Test.Check_Access( 0, 16 ) ) then
    begin
        Error ;
    end ;
    Test.Log_Deallocation( 0, 10 ) ;
  {$IFDEF DELPHI}
    try
        Test.Log_Deallocation( 0, 10 ) ;
        OK := False ;
    except
        OK := True ;
    end ;
    if( not OK ) then
    begin
        Error ;
    end ;
  {$ENDIF}
    Test.Done ;

    Test64.Init ;
    if( Test64.Check_Access( 0, 10 ) ) then
    begin
        Error ;
    end ;
    Test64.Log_Allocation( 0, 10 ) ;
    if( not Test64.Check_Access( 0, 10 ) ) then
    begin
        Error ;
    end ;
    if( Test64.Check_Access( 0, 16 ) ) then
    begin
        Error ;
    end ;
    Test64.Log_Deallocation( 0, 10 ) ;
  {$IFDEF DELPHI}
    try
        Test64.Log_Deallocation( 0, 10 ) ;
        OK := False ;
    except
        OK := True ;
    end ;
    if( not OK ) then
    begin
        Error ;
    end ;
  {$ENDIF}
    Test64.Done ;
end ;
  {$ENDIF}

procedure Test_Unit ;

begin
  {$IFDEF Debug}
    Test_Logging ;
  {$ENDIF}
end ;
{$ENDIF}



end.
