{
	      Program Name : SMUBIOS
	      Package Name : SMU
	      Purpose      : Screen Management Utility I/O routines
	      Institution  : Conroy & Conroy
	      Date Written : 21-Jul-98
	      Written By   : Alan Conroy
	      Version      : 2.0

              Copyright (C) 1998 by Alan Conroy.  Released to the public domain.

	      TO THE GLORY OF GOD THROUGH JESUS CHRIST

	*********************************************************
	*                                                       *
	*        M O D I F I C A T I O N   H I S T O R Y        *
	*                                                       *
	*     DATE      BY             REASON                   *
	*                                                       *
	*********************************************************

	*********************************************************
	*                                                       *
	*           P R O G R A M   P U R P O S E               *
	*                                                       *
	*********************************************************

	  These routines provide SMU Basic Input/Output Services.
}

unit SMUBIOS ;

interface

{$I EDEFINES.INC}

{$IFDEF WINDOWS}
{$DEFINE VER70}
{$ENDIF}

{$IFDEF DELPHI}
{$DEFINE WINDOWS}
{$ENDIF}

{$IFDEF CONSOLE}
{$DEFINE WIN32}
{$ENDIF}
{$IFDEF DELPHI}
{$DEFINE WIN32}
{$ENDIF}

uses { C&C... }
     Colors, { TRGB }
{$IFDEF WINDOWS}
     Windows,
     WinAPI, { GetDC }
{$ENDIF}
     TypeDefs ; { max_Memory }

{ Video modes... }
const VM_PC_Graphics = 5 ;
      VM_Text = 7 ;

{ Hardware models... }
const { 0 is a special value meaning uninitialized }
      Model_Unknown = 1 ; { Use ANSI }
      Model_Rainbow = 2 ; { DEC Rainbow }
      Model_PC_Monochrome = 3 ;
      Model_PC_Color_Text = 4 ; {CGA/EGA/VGA/SVGA in text mode}
      Model_PC_Graphics = 5 ; {CGA/EGA/VGA/SVGA in graphics mode}
      Model_Hercules = 6 ;

type Window_Colors = array[ 0..( Max_Memory div sizeof( TRGB ) ) - 1 ] of TRGB ;
     Window_Colors_Pointer = ^Window_Colors ;

procedure _Clear_Cursor( VR, VC, Background, Foreground, Attribute : byte ) ;

function _Get_Screen( Row, Col : Integer ; Default : byte ) : Byte ;

procedure IBM_Pos( X, Y : Integer ) ; { Position cursor relative to screen }

{ Set cursor position }
procedure _Set_Cursor( X, Y : byte ; Foreground, Background : byte ) ;

{ Set the hardware model }
procedure Set_Model( X : integer ) ;

{ Set the video mode to the specified value. }
procedure Set_Video_Mode( X : integer ) ;

{ Write text and attributes to specified line, starting column. }
procedure _Write_Screen( ID, L, C, Text_Height, Text_Width, Border : integer ;
    Driver : pointer ; const Screen_String, Attribute_String : string ;
    Backgrounds, Foregrounds : Window_Colors_Pointer ;
    Background_Offset, Foreground_Offset : longint ) ;

{$IFNDEF DELPHI}
{ Write character C to screen position X, Y. }
procedure Write_Text( X, Y : Byte ; C : Char ) ;
{$ENDIF}

{$IFNDEF DELPHI}
procedure Wait_Idle ; { Wait for GDC to be idle - awaiting commands }
{$ENDIF}

implementation

{$IFDEF DELPHI}
uses Graphics ;
{$ELSE}
uses WinDOS
    {$IFDEF WINDOWS}
    , WinTypes, WinProcs
    {$ELSE}
    , DOS, Graph
    {$ENDIF}
;
{$ENDIF}

{$IFNDEF DELPHI}
  {$IFNDEF WINDOWS}
type TRegisters = Registers ;
  {$ENDIF}
{$ENDIF}

const Max_Row = 24 ;
      Max_Column = 80 ;

{$IFNDEF DELPHI}
type Font_Array_Big = Array[ 0..2559 ] Of Byte ; {Font array}
     Font_Array_Small = Array[ 0..2047 ] Of Byte ; {Ditto}

{$I FONT.INC}

var Monochrome : Array[ 0..1999, 0..1 ] Of Byte Absolute $B000 : 0 ;
     Rainbow_Data : Array[ 0..4095 ] Of Byte Absolute $Ee00 : 0 ;
     Rainbow_Attributes : Array[ 0..4095 ] Of Byte Absolute $Ef00 : 0 ;
{$ENDIF}

var Video_Mode : integer ; { VM_* }
    Model : integer ; { Model_* }
    Row_Start : Array[ 1..Max_Row ] Of Integer ; {Start of rows in memory}

{$IFNDEF DELPHI}
procedure Wait_Idle ; { Wait for GDC to be idle - awaiting commands }

begin
    inline(
	$E4 / $56 /    {    IN   AL,56H ; FIFO Full?}
	$A8 / $02 /    {    TEST AL,2}
	$74 / $0B /    {    JZ   B2     ; Nope}
	$B9 / $8000 /  {    MOV  CX,8000H ; Maximum wait}
	$E4 / $56 /    {B0: IN   AL,56H ; FIFO full?}
	$A8 / $02 /    {    TEST AL,2}
	$74 / $02 /    {    JZ   B2     ; Nope}
	$E2 / $F8 /    {    LOOP B0     ; Check again}
	$B0 / $0D /    {B2: MOV  AL,0DH}
	$E6 / $57 /    {    OUT  57H,AL ; Turn screen on}
	$E4 / $56 /    {    IN   AL,56H ; Last command fill buffer?}
	$A8 / $02 /    {    TEST AL,2}
	$74 / $0B /    {    JZ   B4     ; Nope}
	$B9 / $8000 /  {    MOV  CX,8000H ; Maximum wait}
	$E4 / $56 /    {B3: IN   AL,56H ; FIFO full?}
	$A8 / $02 /    {    TEST AL,2}
	$75 / $02 /    {    JNZ  B4     ; Nope}
	$E2 / $F8 /    {    LOOP B3     ; Check again}
	$B8 / $040D /  {B4: MOV  AX,40DH}
	$E6 / $57 /    {    OUT  57H,AL ; Another screen-on command}
	$B9 / $8000 /  {    MOV  CX,8000H ; Maximum wait}
	$E4 / $56 /    {B5: IN   AL,56H ; FIFO full?}
	$84 / $E0 /    {    TEST AH,AL}
	$75 / $02 /    {    JNZ  B6     ; Nope}
	$E2 / $F8 /    {    LOOP B5     ; Check again}
	$90 ) ;       {B6: NOP         ; Done}
end ;
{$ENDIF}


procedure _Clear_Cursor( VR, VC, Background, Foreground, Attribute : byte ) ;

{$IFNDEF DELPHI}
var Z : integer ;
    Reg : TRegisters ;
{$ENDIF}

begin
{$IFNDEF DELPHI}
    If( Model <> Model_Rainbow ) then
    Begin
	If( Video_Mode <> VM_Text ) then {Non-text}
	Begin
	    Reg.Ax := $0800 ; {Read character at current cursor position}
	    Intr( $10, Reg ) ; {Get info}
	    Z := Reg.Ax ; {Get character}
	    Reg.Ax := $0900 Or Lo( Z ) ; {Character to write}
	    Reg.Cx := 1 ; {Multiplier}
	    Reg.Bx := ( Background Shl 4 ) + Foreground ;
	    Intr( $10, Reg ) ; {Reverse it now}
	End Else {Text mode}
	Begin
	    Reg.Ax := $0200 ;
	    Reg.Bx := 0 ;
	    Reg.Dx := 80 ;
	    Intr( $10, Reg ) ; {Make cursor vanish}
	End ;
	Exit ;
    End ;
    If( Video_Mode <> VM_Text ) then {Not text mode}
    begin
        _Set_Cursor( VR, VC, Foreground, Background ) ;
    end Else
    begin
        Mem[ $Ef00 : Row_Start[ Vr ] + Vc ] := Attribute ;
        {Set attribute}
    end ;
{$ENDIF}
end ;


function _Get_Screen( Row, Col : Integer ; Default : byte ) : Byte ;

{$IFNDEF DELPHI}
var Reg : TRegisters ;
{$ENDIF}

begin
{$IFDEF DELPHI}
    _Get_Screen := Default ;
{$ELSE}
    case Model of
        Model_Unknown : _Get_Screen := Default ;
        Model_Rainbow : _Get_Screen := Rainbow_Data[ Row_Start[ Row ] + Col - 1 ] ;
        Model_PC_Monochrome : _Get_Screen := Monochrome[ Row_Start[ Row ] + Col - 1, 0 ] ;
        Model_PC_Color_Text :
            begin
                Reg.Ax := $0800 ;
                Reg.Bx := 0 ;
                Intr( $10, Reg ) ; {Read character}
                _Get_Screen := Reg.Al ;
            end ;
	else _Get_Screen := 0 ; { Graphics }
    end ; { case Model of }
{$ENDIF}
end ;


Procedure IBM_Pos( X, Y : Integer ) ; { Position cursor relative to screen }

{$IFDEF CONSOLE}
var Point : COORD ;
{$ELSE}
  {$IFNDEF DELPHI}
var R : TRegisters ;
  {$ENDIF}
{$ENDIF}
                    
begin
{$IFDEF CONSOLE}
    Point.Y := X ;
    Point.X := Y ;
    SetConsoleCursorPosition( GetStdHandle( STD_OUTPUT_HANDLE ), Point ) ;
{$ELSE}
  {$IFNDEF DELPHI}
    fillchar( R, sizeof( R ), 0 ) ;
    R.AX := $0200 ; { Set cursor position }
    R.BX := 0 ; {Page}
    R.DH := X ;
    R.DL := Y ; {Position}
    Intr( $10, R ) ; {Call BIOS}
  {$ENDIF}
{$ENDIF}
end ;


procedure _Set_Cursor( X, Y : byte ; Foreground, Background : byte ) ;

{$IFNDEF DELPHI}
var Line_Count, Lines_So_Far : Integer ;
    Offset, Z, Rows : integer ;
    Reg : Tregisters ;
{$ELSE}
    {$IFDEF CONSOLE}
var Info : CONSOLE_CURSOR_INFO ;
    {$ENDIF}
{$ENDIF}

begin
    if( Video_Mode = VM_Text ) then { Text }
    Begin
        case Model Of
           Model_Unknown :
               Write( '[', X, ';', Y - 1, 'H' ) ;
{$IFNDEF DELPHI}
           Model_Rainbow :
               Begin
                   Offset := Row_Start[ X ] + Y ; {Offset to cursor position}
                   Mem[ $Ef00 : Offset ] :=
                       ( Mem[ $Ef00 : Offset ] or 1 ) and ( not 4 ) ;
                   { Reverse-video with blink }
               End ;
{$ENDIF}
            Model_PC_Monochrome..Model_PC_Graphics : {IBM PC}
                begin
    {$IFDEF CONSOLE}
                    GetConsoleCursorInfo( GetStdHandle( STD_OUTPUT_HANDLE ), Info ) ;
                    Info.bVisible := False ;
                    SetConsoleCursorInfo( GetStdHandle( STD_OUTPUT_HANDLE ), Info ) ;
    {$ENDIF}
                    Ibm_Pos( X - 1, Y - 1 ) ;
                end ;
        end ;
        exit ;
    end ; { if( Video_Mode = VM_Text ) }
{$IFNDEF DELPHI}
    If( Model <> Model_Rainbow ) Then {Not Rainbow}
    Begin
	If( Video_Mode <> VM_Text ) then {Non-text}
	Begin
	    Reg.Ax := $0900 ; {Read character}
	    Intr( $10, Reg ) ; {Get info}
	    Z := Reg.Ax ; {Get character}
	    Reg.Ax := $0900 Or Lo( Z ) ; {Character to write}
	    Reg.Cx := 1 ; {Multiplier}
	    Reg.Bx := Foreground * 16 + Background ;
	    Intr( $10, Reg ) ; {Reverse it now}
	End ;
	Exit ;
    End ;
    X := X - 1 ;
    Y := Y - 1 ; {Normalize row and column}
    If( Video_Mode < 4 ) then {Text mode}
    begin
	Rows := 10 ; {Lines per character}
    end Else
    begin
	Rows := 8 ; {Lines per character}
    end ;
    Z := X * Rows ;
    If Video_Mode = 6 Then Z := Z Shl 6 Else Z := Z Shl 5 ;
    Z := Z + ( Y Shr 1 ) ; {Z:=X*Rows*(either 64 or 32)+Y/2 - Get word address}
    Wait_Idle ;
    If Odd( Y ) Then
    Begin
	Port[ $54 ] := 00 ;
	Port[ $55 ] := $FF ;
    End Else
    Begin
	Port[ $54 ] := $FF ;
	Port[ $55 ] := 00 ;
    End ;
    {If Y is odd, then write in right half of word, else write in left half}

    Wait_Idle ;
    Port[ $53 ] := $EF ; {Select ALU/PS}
    Port[ $51 ] := $10 ; {Complement mode}
    Port[ $53 ] := $F7 ; {Select fgbg}
    Port[ $51 ] := ( Background Shl 4 ) + Foreground ;
    {Invert colors}
    Port[ $53 ] := $Bf ; {Select mode register}
    If Video_Mode = 6 Then Port[ $51 ] := $B1 Else Port[ $51 ] := $B0 ;
    {Word mode}
    Port[ $57 ] := $49 ; {Cursor command}
    Port[ $56 ] := Lo( Z ) ;
    Port[ $56 ] := Hi( Z ) ;
    Port[ $57 ] := $4A ; {Mask command}
    Port[ $56 ] := $Ff ;
    Port[ $56 ] := $Ff ; {Enable everything}
    Line_Count := Rows ;
    While Line_Count > 0 Do {While lines to write}
    Begin
	Lines_So_Far := 0 ;
	Port[ $53 ] := $Fe ; {Select write buffer}
	Port[ $51 ] := 0 ; {Clear counter}
	While ( Lines_So_Far < 8 ) And ( Line_Count > 0 ) Do
	Begin
	    Port[ $52 ] := 0 ;
	    Port[ $52 ] := 0 ;
	    Line_Count := Line_Count - 1 ;
	    Lines_So_Far := Lines_So_Far + 1
	    {Increment count of lines in write buffer}
	End ;
	Port[ $53 ] := $Fe ; {Select write buffer} ;
	Port[ $51 ] := 0 ; {Reset counter}
	Port[ $57 ] := $4C ; {FIGs command}
	Port[ $56 ] := 0 ; {Write down}
	Port[ $56 ] := Lines_So_Far - 1 ; {Number of words to write}
	Port[ $56 ] := 0 ;
	Port[ $57 ] := $22 ; {WDAT command}
	Port[ $56 ] := $Ff ;
	Port[ $56 ] := $Ff ;
	Port[ $57 ] := $22 ; {Issue dummy WDAT}
	While ( Port[ $56 ] And 8 ) = 8 Do ; {Wait 'till done}
    End ;

    Port[ $53 ] := $Bf ; {Select mode register}
    If Video_Mode = 6 Then Port[ $51 ] := $B3 Else Port[ $51 ] := $B2 ;
    {Restore mode}
    Port[ $54 ] := 0 ;
    Port[ $55 ] := 0 ;
    Wait_Idle ;
    Port[ $53 ] := $Ef ; {Select ALU/PS}
    Port[ $51 ] := 0 ; {Replace mode}
    Port[ $53 ] := $F7 ; {Select fgbg}
    Port[ $51 ] := ( Foreground Shl 4 ) + Background ;
    {Reinvert colors}
    Wait_Idle ;
{$ENDIF}
end ;


{ Set the hardware model }
procedure Set_Model( X : integer ) ;

var R : integer ;

begin
    Model := X ;
    case Model of
	 Model_Rainbow : {Rainbow}
             For R := 1 To 24 Do Row_Start[ R ] := ( R - 1 ) * 135 + 16 ;
	 Model_PC_Monochrome..Model_PC_Color_Text : {IBM PC in text mode}
             For R := 1 To 24 Do Row_Start[ R ] := ( R - 1 ) * 80 ;
    end ;
end ;


{ Set the video mode to the specified value. }
procedure Set_Video_Mode( X : integer ) ;

begin
    Video_Mode := X ;
end ;


procedure _Write_Screen( ID, L, C, Text_Height, Text_Width, Border : integer ;
    Driver : pointer ; const Screen_String, Attribute_String : string ;
    Backgrounds, Foregrounds : Window_Colors_Pointer ;
    Background_Offset, Foreground_Offset : longint ) ;

var Ll : string[ 3 ] ;
    Column : integer ;
{$IFNDEF DELPHI}
    Row, Char, Char1, Temp : integer ;
{$ENDIF}
    Len, X : Integer ;
{$IFNDEF DELPHI}
    Reg : TRegisters ;
{$ENDIF}
{$IFDEF WINDOWS}
    DC : HDC ;
{$ENDIF}

    {$IFDEF WINDOWS}
    procedure Windows_Write ;

    var Background, Foreground : TRGB ;

    begin
        DC := GetDC( ID ) ;
        SelectObject( DC, GetStockObject( ANSI_FIXED_FONT ) ) ;
        X := 1 ;
        Column := C ;
        while( Column <= C + Len - 1 ) do
        begin
        {$IFDEF VER70}
            if( ( ord( Attribute_String[ X ] ) and 1 ) = 0 ) then { Normal video }
            begin
                SetTextColor( DC, RGB( 0, 0, 0 ) ) ;
                SetBKColor( DC, RGB( 255, 255, 255 ) ) ;
            end else
            begin
                SetTextColor( DC, RGB( 255, 255, 255 ) ) ;
                SetBKColor( DC, RGB( 0, 0, 0 ) ) ;
            end ;
        {$ELSE}
            Background := Backgrounds[ Background_Offset + X - 1 ] ;
            Foreground := Foregrounds[ Foreground_Offset + X - 1 ] ;
            if( ( ord( Attribute_String[ X ] ) and 1 ) = 0 ) then { Normal video }
            begin
                SetTextColor( DC, RGB( Background.R, Background.G, Background.B ) ) ;
                SetBKColor( DC, RGB( Foreground.R, Foreground.G, Foreground.B ) ) ;
(*
                SetTextColor( DC, ColorToRGB( clWindowText ) ) ;
                SetBKColor( DC, ColorToRGB( clWindow ) ) ;
*)
            end else
            begin
                SetTextColor( DC, RGB( Foreground.R, Foreground.G, Foreground.B ) ) ;
                SetBKColor( DC, RGB( Background.R, Background.G, Background.B ) ) ;
(*
                SetTextColor( DC, ColorToRGB( clWindow ) ) ;
                SetBKColor( DC, ColorToRGB( clWindowText ) ) ;
*)
            end ;
        {$ENDIF}
            SetBKMode( DC, OPAQUE ) ;
            TextOut( DC,
                ( Column - 1 ) * Text_Width + Border,
                ( L - 1 ) * Text_Height + Border,
                @Screen_String[ X ], 1 ) ;
            X := X + 1 ;
            inc( Column ) ;
        end ;
        ReleaseDC( ID, DC ) ;
    end ;
    {$ENDIF}

    procedure Non_Windows_Write ;

    var CC : String[ 3 ] ;
{$IFDEF MSDOS}
        Temp : integer ;
{$ENDIF}
{$IFDEF CONSOLE}
        Point : COORD ;
        S : string ;
        X : DWord ;
{$ENDIF}

    begin
{$IFNDEF DELPHI}
        if( Driver <> nil ) then { A screen driver }
        begin
            Row := L ;
            Column := C ;
            X := 1 ;
            while( X <= Len ) do { For each character }
            begin
                Char := Ord( Screen_String[ X ] ) ;
                Char1 := Ord( Attribute_String[ X ] ) ;
                Inline(
                    $1E / {PUSH DS}
                    $55 / {PUSH BP}
                    $Bb / Driver /
                    {MOV BX,OFFSET Driver ; Address of driver}
                    $8A / $A6 / Row / {MOV AH,[BP].Row}
                    $8A / $86 / Column / {MOV AL,[BP].Column}
                    $8A / $96 / Char / {MOV DL,[BP].Char}
                    $8A / $B6 / Char1 / {MOV DH,[BP].Char1}
                    $Be / > $0000 / {MOV SI,0}
                    $Ff / $18 / {CALL FAR [BX+SI] ; Far call to routine}
                    $5D / {POP BP}
                    $1F {POP DS}
                ) ;
                Inc( X ) ;
            end ;
            Exit ;
        end ;
{$ENDIF}
        if( Model = Model_Unknown ) then { Unknown machine type - use ANSI.SYS }
        begin
            Str( L, Ll ) ;
            Str( C, Cc ) ;
            Write( '[' + Ll + ';' + Cc + 'H', Screen_String ) ;
            Exit ;
       end ;
       if( Model = Model_PC_Monochrome ) then {Monochrome PC}
       begin
{$IFDEF MSDOS}
           Temp := ( Row_Start[ L ] + C - 1 ) Shl 1 ; {Destination offset}
           Inline(
               $1E /                    {PUSH DS}
               $55 /                    {PUSH BP}
               $B9 / $B000 /            {MOV  CX,0B00H}
               $8E / $C1 /              {MOV  ES,CX}
               $8B / $Be / Temp /       {MOV  DI,[BP].Temp}
               $8B / $8E / Len /        {MOV  CX,[BP].Len}
               $Bb / Attribute_String / {MOV  BX,OFFSET Attribute_String}
               $Be / Screen_String /    {MOV  SI,OFFSET Screen_String}
               $46 /                    {INC  SI}
               $43 /                    {INC  BX}
                                        {10:}
               $8A / $24 /              {MOV  AH,[SI]}
               $26 / $88 / $25 /        {MOV ES:[DI],AH}
               $47 /                    {INC  DI}
               $8A / $27 /              {MOV AH,[BX]}
               $26 / $88 / $25 /        {MOV  ES:[DI],AH}
               $47 /                    {INC  DI}
               $46 /                    {INC  SI}
               $43 /                    {INC  BX}
               $E0 / $F0 /              {LOOPNZ 10}
               $5D /                    {POP  BP}
               $1F                      {POP  DS}
           ) ;
{$ELSE} {(IFDEF MSDOS)}
  {$IFDEF CONSOLE}
           Point.X := C - 1 ;
           Point.Y := L - 1 ;
           WriteConsoleOutputCharacter( GetStdHandle( STD_OUTPUT_HANDLE ),
               PChar( Screen_String ), length( Screen_String ), Point, X ) ;
           if( length( Attribute_String ) > 0 ) then
           begin
               S := '' ;
               for X := 1 to length( Attribute_String ) do
               begin
                  S := S + Attribute_String[ X ] + #0 ;
               end ;
               WriteConsoleOutputAttribute( GetStdHandle( STD_OUTPUT_HANDLE ),
                   PChar( S ), length( Attribute_String ), Point, X ) ;
            end ;
  {$ELSE}
    {$IFNDEF WIN32}
      {Use BIOS if protected mode}
           Dec( L ) ;
           Dec( C ) ;
           for Temp := 1 to Len do
           begin
               IBM_Pos( L, C ) ; {Position cursor}
               Inc( C ) ;
               Reg.Ah := $09 ;
               Reg.Al := Ord( Screen_String[ Temp ] ) ;
               Reg.Bh := 0 ;
               Reg.Bl := Ord( Attribute_String[ Temp ] ) and 127 ;
               Reg.Cx := 1 ;
               Intr( $10, Reg ) ; {Write character}
           end ;
      {$ENDIF}
   {$ENDIF}
{$ENDIF} {(IFDEF MSDOS)}
           exit ;
        end ;
        if( Model = Model_PC_Color_Text ) then {CGA/EGA/VGA/SVGA PC in text mode}
        begin
   {$IFDEF CONSOLE}
           Point.X := C - 1 ;
           Point.Y := L - 1 ;
           WriteConsoleOutputCharacter( GetStdHandle( STD_OUTPUT_HANDLE ),
               PChar( Screen_String ), length( Screen_String ), Point, X ) ;
           if( length( Attribute_String ) > 0 ) then
           begin
               S := '' ;
               for X := 1 to length( Attribute_String ) do
               begin
                  S := S + Attribute_String[ X ] + #0 ;
               end ;
               WriteConsoleOutputAttribute( GetStdHandle( STD_OUTPUT_HANDLE ),
                   PChar( S ), length( S ), Point, X ) ;
           end ;
   {$ELSE}
     {$IFNDEF WIN32}
            Dec( L ) ;
            Dec( C ) ;
            for Temp := 1 to Len do
            begin
                Ibm_Pos( L, C ) ; {Position cursor}
                Inc( C ) ;
                Reg.Ah := $09 ;
                Reg.Al := Ord( Screen_String[ Temp ] ) ;
                Reg.Bh := 0 ;
                Reg.Bl := Ord( Attribute_String[ Temp ] ) and 127 ;
                Reg.Cx := 1 ;
                Intr( $10, Reg ) ; {Write character}
            end ;
     {$ENDIF}
   {$ENDIF}
            Exit ;
        end ;
        if( Model = Model_PC_Graphics ) then { CGA/EGA/VGA/SVGA PC in graphics mode }
        begin
  {$IFDEF CONSOLE}
           Point.X := C - 1 ;
           Point.Y := L - 1 ;
           WriteConsoleOutputCharacter( GetStdHandle( STD_OUTPUT_HANDLE ),
               PChar( Screen_String ), length( Screen_String ), Point, X ) ;
  {$ELSE}
            Ibm_Pos( L - 1, C - 1 ) ; {Position cursor}
            C := ( C - 1 ) * Text_Width ;
            L := ( L - 1 ) * Text_Height ;
    {$IFNDEF WINDOWS}
            Outtextxy( C + Border, L + Border, Screen_String ) ; { Output text }
    {$ENDIF}
  {$ENDIF}
        end ;
{$IFNDEF DELPHI}
        if( Model = Model_Rainbow ) then {Rainbow}
        begin
            if( Video_Mode <> VM_Text ) then {If graphics mode}
            begin
                X := 1 ;
                while( X <= Len ) do {For each character}
                begin
                    Write_Text( L, C, Screen_String[ X ] ) ;
                    Inc( C ) ;
                    Inc( X ) ;
                end ;
                Exit ;
            end ;
            Temp := Row_Start[ L ] + C + 1 ; {Destination offset}
            Move( Screen_String[ 1 ], Rainbow_Data[ Temp ], Len ) ;
            {Write characters}
            Move( Attribute_String[ 1 ], Rainbow_Attributes[ Temp ], Len ) ;
            {Write attributes}
            Exit ;
        end ;
{$ENDIF}
      end ;

begin
    Len := Length( Screen_String ) ;
{$IFDEF WINDOWS}
    {$IFDEF CONSOLE}
    Non_Windows_Write ;
    {$ELSE}
    Windows_Write ;
    {$ENDIF}
{$ELSE}
    Non_Windows_Write ;
{$ENDIF}
end ;


{$IFNDEF DELPHI}
Procedure Write_Text( X, Y : Byte ; C : Char ) ;
{Write character C to location X,Y.  1,1 is home position}

Var Line_Count, Lines_So_Far, Offset, Z, Rows : Integer ;

begin
    X := X - 1 ;
    Y := Y - 1 ; {Normalize row and column}
    If( Video_Mode < 4 ) then {Text mode}
    begin
	Rows := 10 ; {Lines per character}
	Offset := Ord( C ) * Rows {Get offset of character in font table}
    end else
    begin
	Rows := 8 ; {Lines per character}
	Offset := Ord( C ) Shl 3 {Get offset of character in font table}
    end ;
    Z := X * Rows ;
    If Video_Mode = 6 then Z := Z Shl 6 else Z := Z Shl 5 ;
    Z := Z + ( Y Shr 1 ) ; {Z:=X*Rows*(either 64 or 32)+Y/2 - Get word address}
    Wait_Idle ;
    If Odd( Y ) then
    begin
	Port[ $54 ] := 00 ;
	Port[ $55 ] := $Ff
    end else
    begin
	Port[ $54 ] := $Ff ;
	Port[ $55 ] := 00
    end ;
    {If Y is odd, then write in right half of word, else write in left half}

    Port[ $53 ] := $Bf ; {Select mode register}
    If Video_Mode = 6 then Port[ $51 ] := $B1 else Port[ $51 ] := $B0 ;
    {Word mode}
    Port[ $57 ] := $49 ; {Cursor command}
    Port[ $56 ] := Lo( Z ) ;
    Port[ $56 ] := Hi( Z ) ;
    Port[ $57 ] := $4A ; {Mask command}
    Port[ $56 ] := $Ff ;
    Port[ $56 ] := $Ff ; {Enable everything}
    Line_Count := Rows ; {Lines to display}
    while Line_Count > 0 Do {while lines to write}
    begin
	Lines_So_Far := 0 ;
	Port[ $53 ] := $Fe ; {Select write buffer}
	Port[ $51 ] := 0 ; {Clear counter}
	while ( Lines_So_Far < 8 ) And ( Line_Count > 0 ) Do
	begin
	    If Rows = 10 then
	    begin
		Port[ $52 ] := Font0[ Offset ] ;
		Port[ $52 ] := Font0[ Offset ]
	    end else
	    begin
		Port[ $52 ] := Font1[ Offset ] ;
		Port[ $52 ] := Font1[ Offset ]
	    end ;
	    Offset := Offset + 1 ;
	    Line_Count := Line_Count - 1 ;
	    Lines_So_Far := Lines_So_Far + 1
	    {Increment count of lines in write buffer}
	end ;
	Port[ $53 ] := $Fe ; {Select write buffer} ;
	Port[ $51 ] := 0 ; {Reset counter}
	Port[ $57 ] := $4C ; {FIGs command}
	Port[ $56 ] := 0 ; {Write down}
	Port[ $56 ] := Lines_So_Far - 1 ; {Number of words to write}
	Port[ $56 ] := 0 ;
	Port[ $57 ] := $22 ; {WDAT command}
	Port[ $56 ] := $Ff ;
	Port[ $56 ] := $Ff ;
	Port[ $57 ] := $22 ; {Issue dummy WDAT}
	while ( Port[ $56 ] and 8 ) = 8 do ; {Wait 'till done}
    end ;

    Port[ $53 ] := $Bf ; {Select mode register}
    If Video_Mode = 6 then Port[ $51 ] := $B3 else Port[ $51 ] := $B2 ;
    {Restore mode}
    Port[ $54 ] := 0 ;
    Port[ $55 ] := 0
end ; { Write_Text }
{$ENDIF}



end.
