{$N+}
{$E+}
{
	Program Name : UStrings
	Package Name : LB
	Purpose      : String utility functions
	Institution  : Conroy & Conroy Co.
	Date Written : 01-Jan-1995
	Written By   : Alan Conroy
	Version      : 1.0G

        Copyright (C) 1995-2003 by Alan Conroy.  Released to the public domain.

	TO THE GLORY OF GOD THROUGH JESUS CHRIST

	*************************************************************
	*                                                           *
	*        M O D I F I C A T I O N   H I S T O R Y            *
	*                                                           *
	*    DATE        BY          REASON                         *
	*                                                           *
    	*  24-May-2002   EAC         Added Print_Edit.              *
    	*  24-Oct-2002   EAC         Added Boolean_To_String.       *
    	*  24-Oct-2002   EAC         Added TString, and             *
    	*                            TString_Translator.            *
    	*  17-Dec-2002   EAC         Added Trim_Quotes.             *
	*  10-Feb-2003   EAC         Added Remove_Termination and   *
    	*                            Ensure_Termination.            *
	*  06-Mar-2006   RBJ         Set the memory type to MT_HEAP *
	*                            for objects created on heap.   *
        *  25-Oct-2006   EAC         Added Tab_To_Space.            *
	*                                                           *
	*************************************************************

        LEVEL 1 CERTIFICATION ON 3-Oct-2003 by EAC.

	*************************************************************
	*                                                           *
	*        P R O G R A M   P U R P O S E                      *
	*                                                           *
	*************************************************************

	  This unit provides various string utility functions.
}

unit UStrings ;

interface

{$I EDEFINES.INC}

uses { Borland... }
{$IFDEF DELPHI}
     Classes, { TStringList }
{$ENDIF}

     { C&C... }
     Common, { TBase_Object }
     MStores, { PSManaged_Store }
     TypeDefs ; { PChar_Array }

type RComparison = record
			Bias : array[ 0..255 ] of integer ;
			Compensation : array[ 0..255 ] of integer ;
			Weight : array[ 0..255 ] of integer ;
		   end ;

function Compare_Strings( String1, String2 : string ;
    X : RComparison ) : double ;
function Extract( Original, Substring : string ;
    Start, Ending : integer ) : string ;
{ Extract all occurances of substring from the original string and return the
  result.  Only occurances between Start and Ending are extracted.  If
  Ending < 1, it  defaults to the entire length of the string. }

function Decode_Hex_String( const Value : string ) : string ;
{ Extract original string from encoded hex string }

{$IFDEF DELPHI}
function Decode_Text( S : string ) : TStringList ;
{$ENDIF}

function Encode_Hex_String( const Value : string ) : string ;
{ Convert a string to a list of hexadecimal values }

function Formatted_Hex_String( const Value : string ;
    Blocking : integer ) : string ;
{ Format binary data in a string as a series of hex digits, separated by spaces.
  Spaces are included after every Blocking digits. }

function Print_Edit( Value : String ) : String ;
{ Return string with only printable characters in it }

procedure FRight( var Z, C : string ; Position : Integer ) ;
function Repeating_String( Value : char ; Len : integer ) : string ;
function Reverse_String( Value : string ) : string ;
function Right( C : string ; Position : Integer ) : string ;
function Replace( A : string ; const Substring, Substitute : string ;
    Options : integer ) : string ;
function String_To_Boolean( S : string ) : boolean ;
function Substitute( A : string ; const B : string ) : string ;
function Tab_To_Space( S : string ; Tab_Stops : integer ) : string ;
function Trim_Quotes( S : string ) : string ;
function Remove_Termination( const S, T : string ) : string ;
function Ensure_Termination( const S, T : string ) : string ;
function String_To_BCD( S : string ) : string ;
function String_To_Packed_BCD( S : string ) : string ;

const { Char_Classes... }
      CC_Unknown = 1 ;
      CC_Alpha = 2 ;
      CC_Digit = 4 ;
      CC_Punctuation = 8 ;
      CC_Symbol = 16 ;
      CC_Control = 32 ;
      CC_Quote = 64 ;
      CC_Whitespace = 128 ;
      CC_Left = 256 ; { Left-side of matched characters (such as [ ( and { }
      CC_Currency = 512 ; { Currency symbol }

type PString = ^TString ;
     TString = object( TBase_Object ) { Generic string object }
               { Generic base class for all strings.  Strings share the these
                 traits: 1) Copy-on-write semantics.  Normally assigning strings
                 results in multiple TStrings pointing to the same physical
                 data.  However, when one string is changed, the shared data is
                 copied and then that copy is modified.
                 2) Sharable.  If the Shared flag is set, the copy-on-write
                 semantics are changed.  Any changes to a shared string change
                 the existing physical data.
                 3) Length and size.  The Length returns the number of characters
                 in the string.  Size returns the number of bytes used by the
                 physical data. }
                   public { Constructors and destructors... }
                       constructor Init ;

{$IFDEF Delphi}
                   protected { Instance support... }
{$ENDIF}
                       procedure Modify ; virtual ;
                       { Notice that string is about to be modified }

                   public { API... }
                       Shared : boolean ;

                       procedure Append( Source : PString ) ; virtual ;
                       { Append another TString }

                       procedure Append_Char( Source : longint ) ; virtual ;
                       { Append a character to end of the string }

                       procedure Append_String( Source : string ) ; virtual ;
                       { Append a string }

                       procedure Assign( Source : PString ) ; virtual ;
                       { Copy from another TString }

                       procedure RAssign( Source : PString ) ; virtual ;
                       { Copy from another TString, right-aligned }

                       function As_PChar : PChar ; virtual ;
                       function At( Index : integer ) : longint ; virtual ;
                       { Value at index }

                       procedure At_Put( Index, Source : longint ) ; virtual ;
                       function Compare( S : PString ; CS : boolean ) : TComparison ;
                           virtual ;
                       { Compare S to self, returning result of comparison.  CS
                         is True to do case-sensitive compare and false to do
                         case-insensitive compare.  Function returns one of the
                         following: EqualTo, LessThan, GreaterThan. }

                       function Copy( Start, _Length : longint ) : PString ;
                           virtual ; { Return specified portion of string }
                       function Create_Compatible_String : PString ; virtual ;
                       function Edit( Flags : longint ) : PString ;
                           virtual ;
                       (*
                         This function will perform various editing operations
                         on strings.  Flags has the following meanings:

                                     AND     1 = Normalize
                                     AND     2 = Remove all whitespace.
                                     AND     4 = Remove all non-displaying (control) characters.
                                     AND     8 = Remove leading whitespace.
                                     AND    16 = Reduce whitespace to one space.
                                     AND    32 = Convert lower to upper case.
                                     AND    64 = Convert '[' to '(' and ']' to ')'.
                                     AND   128 = Remove all trailing whitespace.
                                     AND   256 = Leave characters within quotes unmodified.
                                     AND   512 = Convert upper case to lower case.
                                     AND  1024 = Convert '[' to '{' and ']' to '}'.
                                     AND  2048 = Convert '(' to '[' and ')' to ']'.
                                     AND  4096 = Convert '(' to '{' and ')' to '}'.
                                     AND  8192 = Convert '{' to '(' and '}' to ')'.
                                     AND 16384 = Convert '{' to '[' and '}' to ']'.
                                     AND 32768- = reserved for future use
                       *)

                       function Fixed : boolean ; virtual ;
                       function From_Universal( X : longint ) : longint ;
                           virtual ; { Convert from universal to string-specific }
                       function Instr( Pos : longint ; S : PString ) : longint ;
                           virtual ;
                       function Length : longint ; virtual ;
                       { Length of string, in characters }

                       function Max_Length : longint ; virtual ;
                       { Maximum allowable length, in characters }

                       function Normalize( X : longint ) : longint ; virtual ;
                       procedure Set_Length( X : longint ) ; virtual ;
                       { Set length of string, in characters }

                       procedure Set_String( const Value : string ) ; virtual ;
                       { Set from string }

                       function Repeating_String( Value : longint ;
                           Len : longint ) : PString ; virtual ;
                       { Returns a compatible string of length Len, consisting
                         entirely of Value characters. }

                       function Right( Pos : longint ) : PString ; virtual ;
                       { Portion of string, beginning at Pos }

                       procedure RSet_String( Value : string ) ; virtual ;
                       { Set from string, right-aligned }

                       function Get_Store : PSManaged_Store ; virtual ;
                       { Return string's store. }

                       procedure Set_Store( S : PSManaged_Store ) ; virtual ;
                       { Place string contents on specified store. }

                       function Size : longint ; virtual ;
                       { Size of physical data, in bytes }

                       function Text : string ; virtual ;
                       function To_Universal( X : longint ) : longint ;
                           virtual ; { Convert to universal from string-specific }
		       function Class_Of( C : longint ) : longint ; virtual ;
                       function Upcase( X : longint ) : longint ; virtual ;
                       { Return uppercase version of X.  If X is not lowercase,
                         original value of X is returned. }
                       function Downcase( X : longint ) : longint ; virtual ;
                       { Return lowercase version of X.  If X is not uppercase,
                         original value of X is returned. }

                       function Facility : longint ; virtual ;
                       function Version : longint ; virtual ;
               end ; { TString }

      TReference_Storage = object( TBase_Object )
                               public { API... }
                                   Buffer : longint ; { Address of data (actual data if Size <= 4) }
                                   Size : longint ; { Size of data, in bytes }
                                   _Store : PSManaged_Store ; { Store for data }
                                   _PChar : PChar ;
                                   Cache_Data : boolean ; { True to cache small strings in Buffer }
                                   {TODO: Store must be memory heap if cache_data is True (for now)}

                               public { Constructors and destructors... }
                                   constructor Init ;
                                   destructor Done ; virtual ;

                               private { Internal utility routines... }
                                   function Store : PSManaged_Store ;

                               public { API... }
                                   procedure Move( var Old : longint ;
                                       Old_Size : longint ; var New : longint ;
                                       New_Size, Len : longint ) ; virtual ;
                                   function As_PChar : PChar ; virtual ;
                                   procedure Zero( Pos, Len : longint ) ;
                                       virtual ;
                                   function Get_Byte( Pos : longint ) : byte ;
                                       virtual ;
                                   procedure Set_Byte( D : byte ;
                                       Pos : longint ) ; virtual ;
                                   procedure Get_Data( var D ;
                                       Pos, Len : longint ) ; virtual ;
                                   procedure Set_Data( var D ;
                                       Pos, Len : longint ) ; virtual ;
                                   procedure Set_Size( S : longint ) ; virtual ;
                                   procedure Set_Store( S : PSManaged_Store ) ;
                                       virtual ;
                                   procedure Fill( Pos, Len, Value : longint ) ;
                                       virtual ;
                           end ; { TReference_Storage }
      PReference_Storage = ^TReference_Storage ;

      TASCII_String = object( TString ) { ASCII 8-bit }
{$IFDEF DELPHI}
                          protected { Instance data... }
{$ELSE}
                          public { Instance data... }
{$ENDIF}
                              _Data : PReference_Storage ;

                          public { Constructors and destructors... }
                              constructor Init ;
                              destructor Done ; virtual ;

{$IFDEF Delphi}
                          protected { Internal utility routines... }
{$ENDIF}
                              procedure Modify ; virtual ;

                          public { Overrides... }
                              procedure Append( Source : PString ) ; virtual ;
                              procedure Append_String( Source : string ) ;
                                  virtual ;
                              procedure Append_Char( Source : longint ) ;
                                  virtual ;
                              procedure Assign( Source : PString ) ; virtual ;
                              procedure RAssign( Source : PString ) ; virtual ;
                              function As_PChar : PChar ; virtual ;
                              function At( Index : integer ) : longint ;
                                  virtual ;
                              procedure At_Put( Index, Source : longint ) ;
                                  virtual ;
                              function Class_Of( C : longint ) : longint ;
                                  virtual ;
                              function Compare( S : PString ;
                                  CS : boolean ) : TComparison ; virtual ;
                              function Copy( Start, _Length : longint ) : PString ;
                                  virtual ;
                              function Create_Compatible_String : PString ;
                                  virtual ;
                              function Fixed : boolean ; virtual ;
                              function From_Universal( X : longint ) : longint ;
                                  virtual ;
                              function Instr( Pos : longint ; S : PString ) : longint ;
                                  virtual ;
                              function Length : longint ; virtual ;
                              function Max_Length : longint ; virtual ;
                              function Normalize( X : longint ) : longint ;
                                  virtual ;
                              function Get_Store : PSManaged_Store ; virtual ;
                              function Repeating_String( Value : longint ;
                                   Len : longint ) : PString ; virtual ;
                              procedure Set_Store( S : PSManaged_Store ) ; virtual ;
                              procedure Set_Length( X : longint ) ; virtual ;
                              procedure Set_String( const Value : string ) ; virtual ;
                              procedure RSet_String( Value : string ) ;
                                  virtual ;
                              function Text : string ; virtual ;
                              function To_Universal( X : longint ) : longint ;
                                  virtual ;
                              function Upcase( X : longint ) : longint ;
                                  virtual ;
                              function Downcase( X : longint ) : longint ;
                                  virtual ;

                              function Facility : longint ; virtual ;
                      end ;
      PASCII_String = ^TASCII_String ;

      TASCII7_String = object( TASCII_String ) { ASCII 7-bit }
                          public { Overrides... }
                              function Create_Compatible_String : PString ;
                                  virtual ;
                              procedure Append_String( Source : string ) ;
                                  virtual ;
                              function At( Index : integer ) : longint ;
                                  virtual ;
                              procedure At_Put( Index, Source : longint ) ;
                                  virtual ;
                              function Class_Of( C : longint ) : longint ;
                                  virtual ;
                              function Upcase( X : longint ) : longint ;
                                  virtual ;
                              function Downcase( X : longint ) : longint ;
                                  virtual ;
                              procedure Set_String( const Value : string ) ;
                                  virtual ;
                              procedure RSet_String( Value : string ) ;
                                  virtual ;
                              function Facility : longint ; virtual ;
                       end ;
      PASCII7_String = ^TASCII7_String ;

      TASCII_DOS_String = object( TASCII_String ) { DOS character set (OEM Code page 437) }
                              public { Overrides... }
                                  function Class_Of( C : longint ) : longint ;
                                      virtual ;
                                  function Create_Compatible_String : PString ;
                                      virtual ;
                                  function Upcase( X : longint ) : longint ;
                                      virtual ;
                                  function Downcase( X : longint ) : longint ;
                                      virtual ;
                                  function From_Universal( X : longint ) : longint ;
                                      virtual ;
                                  function Normalize( X : longint ) : longint ;
                                      virtual ;
                                  function To_Universal( X : longint ) : longint ;
                                      virtual ;
                                  function Facility : longint ; virtual ;
                          end ;
      PASCII_DOS_String = ^TASCII_DOS_String ;

      TASCII_Windows_String = object( TASCII_String ) { MS Windows character set (True Type) }
                                  public { Overrides... }
                                      function Class_Of( C : longint ) : longint ;
                                          virtual ;
                                      function Create_Compatible_String : PString ;
                                          virtual ;
                                      function Upcase( X : longint ) : longint ;
                                          virtual ;
                                      function Downcase( X : longint ) : longint ;
                                          virtual ;
                                      function From_Universal( X : longint ) : longint ;
                                          virtual ;
                                      function Normalize( X : longint ) : longint ;
                                          virtual ;
                                      function To_Universal( X : longint ) : longint ;
                                          virtual ;
                                      function Facility : longint ; virtual ;
                               end ;
      PASCII_Windows_String = ^TASCII_Windows_String ;

      TUTF32_String = object( TString ) { Unicode UTF-32 string }
                          public { Overrides... }
                              function Create_Compatible_String : PString ;
                                  virtual ;
                              function From_Universal( X : longint ) : longint ;
                                  virtual ;
                              function To_Universal( X : longint ) : longint ;
                                  virtual ;
                      end ;
      PUTF32_String = ^TUTF32_String ;

      TUTF8_String = object( TUTF32_String ) { Unicode UTF-8 string }
                          public { Overrides... }
                              function Create_Compatible_String : PString ;
                                  virtual ;
                              function From_Universal( X : longint ) : longint ;
                                  virtual ;
                              function To_Universal( X : longint ) : longint ;
                                  virtual ;
                     end ;
      PUTF8_String = ^TUTF8_String ;

      TUTF16_String = object( TUTF32_String ) { Unicode UTF-16 string }
                          public { Overrides... }
                              function Create_Compatible_String : PString ;
                                  virtual ;
                              function From_Universal( X : longint ) : longint ;
                                  virtual ;
                      end ;
      PUTF16_String = ^TUTF16_String ;

      TArray_String = object( TString ) { String implemented as a static char array }
                          private { Instance data... }
                              _Array : PChar_Array ;
                              _Low, _High : longint ;

                          public { Constructors and destructors... }
                              constructor Init( A : array of char ) ;
                      end ;


      TString_Translator_Base = object( TBase_Object )
                                    public { API... }
                                        function Translate_String( Value : PString ) : PString ;
                                            virtual ;
                                end ;
      PString_Translator_Base = ^TString_Translator_Base ;

      TString_Translator = object( TString_Translator_Base )
                               private { Instance data... }
                                   Translations : array[ 0..255 ] of PString ;

                               public { Constructors and destructors... }
                                   constructor Init ;

                               public { API... }
                                   procedure Set_Translation( Index : integer ;
                                       Value : PString ) ; virtual ;
                                   function Translate_String( Value : PString ) : PString ;
                                       virtual ;
                           end ;

function Temp_String( const S : string ) : PString ;

{$DEFINE TEST}
{$IFDEF TEST}
procedure Test_Unit ;
{$ENDIF}

implementation

uses { C&C... }
     _App, { Application_Manager }
     App, { make sure default application manager is loaded }
     ASCIIDef, { HT }
     CommonUt, { TInteger_List }
     _Common,  { MT_Heap }
     Compatib, { strpas }
     CVT,
     Heaps, { PSHeap }
     Instrs,
     Maths, { Min }
     Num1s, { Num1 }
     Standard, { Set_Length }
     _UEHDefs ; { TUEC }

const ASCII_CC : array[ 0..127 ] of integer =
         (
             CC_Control, { Null }
             CC_Control, { Start Of Heading }
             CC_Control, { Start Text }
             CC_Control, { End Text }
             CC_Control, { End Of Transmission }
             CC_Control, { Enquiry }
             CC_Control, { Acknowledge }
             CC_Control, { Bell }
             CC_Control, { Backspace }
             CC_Whitespace, { Horizontal Tab }
             CC_Control, { Line Feed }
             CC_Control, { Vertical Tab }
             CC_Control, { Form Feed }
             CC_Control, { Carriage return }
             CC_Control, { Shift Out }
             CC_Control, { Shift In }
             CC_Control, { Data Link Escape }
             CC_Control, { Direct Control 1 }
             CC_Control, { Direct Control 2 }
             CC_Control, { Direct Control 3 }
             CC_Control, { Direct Control 4 }
             CC_Control, { Negative Acknowledge }
             CC_Control, { Synchronous idle }
             CC_Control, { End Tranmission Block }
             CC_Control, { Cancel }
             CC_Control, { End of Medium }
             CC_Control, { Substitute }
             CC_Control, { Escape }
             CC_Control, { Form separator }
             CC_Control, { Group separator }
             CC_Control, { Record separator }
             CC_Control, { Unit separator }
             CC_Whitespace, { Space }
             CC_Punctuation, { ! }
             CC_Punctuation + CC_Quote, { " }
             CC_Symbol, { # }
             CC_Symbol or CC_Currency, { $ }
             CC_Symbol, { % }
             CC_Symbol, { & }
             CC_Punctuation + CC_Quote, { ' }
             CC_Symbol + CC_Left, { ( }
             CC_Symbol, { ) }
             CC_Symbol, { * }
             CC_Symbol, { + }
             CC_Punctuation, { , }
             CC_Punctuation, { - }
             CC_Punctuation, { . }
             CC_Punctuation, { / }
             CC_Digit, { 0 }
             CC_Digit, { 1 }
             CC_Digit, { 2 }
             CC_Digit, { 3 }
             CC_Digit, { 4 }
             CC_Digit, { 5 }
             CC_Digit, { 6 }
             CC_Digit, { 7 }
             CC_Digit, { 8 }
             CC_Digit, { 9 }
             CC_Punctuation, { : }
             CC_Punctuation, { ; }
             CC_Symbol + CC_Left, { < }
             CC_Symbol, { = }
             CC_Symbol, { > }
             CC_Punctuation, { ? }
             CC_Symbol, { @ }
             CC_Alpha, { A }
             CC_Alpha, { B }
             CC_Alpha, { C }
             CC_Alpha, { D }
             CC_Alpha, { E }
             CC_Alpha, { F }
             CC_Alpha, { G }
             CC_Alpha, { H }
             CC_Alpha, { I }
             CC_Alpha, { J }
             CC_Alpha, { K }
             CC_Alpha, { L }
             CC_Alpha, { M }
             CC_Alpha, { N }
             CC_Alpha, { O }
             CC_Alpha, { P }
             CC_Alpha, { Q }
             CC_Alpha, { R }
             CC_Alpha, { S }
             CC_Alpha, { T }
             CC_Alpha, { U }
             CC_Alpha, { V }
             CC_Alpha, { W }
             CC_Alpha, { X }
             CC_Alpha, { Y }
             CC_Alpha, { Z }
             CC_Symbol + CC_Left, { [ }
             CC_Symbol, { \ }
             CC_Symbol, { ] }
             CC_Symbol, { ^ }
             CC_Symbol, { _ }
             CC_Symbol, { ` }
             CC_Alpha, { a }
             CC_Alpha, { b }
             CC_Alpha, { c }
             CC_Alpha, { d }
             CC_Alpha, { e }
             CC_Alpha, { f }
             CC_Alpha, { g }
             CC_Alpha, { h }
             CC_Alpha, { i }
             CC_Alpha, { j }
             CC_Alpha, { k }
             CC_Alpha, { l }
             CC_Alpha, { m }
             CC_Alpha, { n }
             CC_Alpha, { o }
             CC_Alpha, { p }
             CC_Alpha, { q }
             CC_Alpha, { r }
             CC_Alpha, { s }
             CC_Alpha, { t }
             CC_Alpha, { u }
             CC_Alpha, { v }
             CC_Alpha, { w }
             CC_Alpha, { x }
             CC_Alpha, { y }
             CC_Alpha, { z }
             CC_Symbol + CC_Left, { { }
             CC_Symbol, { | }
             CC_Symbol, { }
             CC_Symbol, { ~ }
             CC_Control { Delete }
         ) ; { ASCII_CC }

{$IFDEF TEST}
var Test_Generic : boolean ; { True to generic test strings }
{$ENDIF}

{ TString methods.. }

{ Constructors and destructors... }

constructor TString.Init ;

begin
    inherited Init ;

    Shared := False ;
end ;


{ Instance support... }

procedure TString.Modify ;

begin
end ;


{ API... }

procedure TString.Append( Source : PString ) ;

var Len, Loop, Start : longint ;

begin
    if( ( Source <> nil ) and ( Source^.Length > 0 ) ) then
    begin
        Start := Length ; { Current string length } {UNTESTED}
        Set_Length( Length + Source^.Length ) ;
        Len := Length - Start ; { Amount of data to actually append }
        for Loop := 1 to Len do
        begin
            At_Put( Start + 1, Source^.At( Loop ) ) ; {UNTESTED}
            inc( Start ) ;
        end ;
    end ;
end ;


procedure TString.Append_Char( Source : longint ) ;

begin
    if( Length < Max_Length ) then { Cannot extend a string beyond its maximum length }
    begin
        Set_Length( Length + 1 ) ; {UNTESTED}
        At_Put( Length, Source ) ;
    end ;
end ;


procedure TString.Append_String( Source : string ) ;

var Len, Loop, Start : longint ;

begin
    if( system.length( Source ) > 0 ) then
    begin
        Start := Length ; { Current string length }
        Set_Length( Length + system.length( Source ) ) ;
        Len := Length - Start ; { Amount of data to actually append }
        for Loop := 1 to Len do
        begin
            At_Put( Start + 1, ord( Source[ Loop ] ) ) ; {UNTESTED}
            inc( Start ) ;
        end ;
    end ;
end ;


procedure TString.Assign( Source : PString ) ;

var Loop : integer ;

begin
    if( Source = nil ) then
    begin
        Set_Length( 0 ) ;
        exit ;
    end ;
    Modify ;
    Set_Length( Source^.Length ) ;
    for Loop := 1 to Length do
    begin
        At_Put( Loop, Source^.At( Loop ) ) ; {UNTESTED}
    end ;
end ;


procedure TString.RAssign( Source : PString ) ;

var _Source, _Destination : integer ;

begin
    if( ( Source <> nil ) and ( Source^.Length > 0 ) ) then
    begin
        _Source := Source^.Length - Length + 1 ;
        if( _Source < 1 ) then
        begin
            _Source := 1 ; {UNTESTED}
        end ;
        _Destination := Length - Source^.Length + 1 ;
        if( _Destination < 1 ) then
        begin
            _Destination := 1 ; {UNTESTED}
        end ;
        Modify ;
        while( _Destination <= Length ) do
        begin
            At_Put( _Destination, Source^.At( _Source ) ) ; {UNTESTED}
            inc( _Destination ) ;
            inc( _Source ) ;
        end ;
    end ;
end ;


function TString.As_PChar : PChar ;

begin
    As_PChar := nil ;
end ;


function TString.At( Index : integer ) : longint ;

begin
    At := 0 ;
end ;


procedure TString.At_Put( Index, Source : longint ) ;

begin
end ;


function TString.Compare( S : PString ; CS : boolean ) : TComparison ;

var Len, Loop : longint ;
    C1, C2 : longint ;

begin
    if( S = @self ) then { Compare to self }
    begin
        Compare := EqualTo ;
        exit ;
    end ;
    if( @self = nil ) then
    begin
        if( S^.Length = 0 ) then
        begin
            Compare := EqualTo ; {UNTESTED}
        end else
        begin
            Compare := LessThan ; {UNTESTED}
        end ;
        exit ;
    end ;
    if( S = nil ) then { S is nil and S1 is not }
    begin
        if( Length = 0 ) then
        begin
            Compare := EqualTo ;
        end else
        begin
            Compare := GreaterThan ;
        end ;
        exit ;
    end ;
    if( ( Length = 0 ) and ( S^.Length = 0 ) ) then { Both are null strings }
    begin
        Compare := EqualTo ;
        exit ;
    end ;

    Len := Min( Length, S^.Length ) ;
    for Loop := 1 to Len do
    begin
        C1 := At( Loop ) ;
        C2 := S^.At( Loop ) ;
        if( not CS ) then
        begin
            C1 := Upcase( C1 ) ;
            C2 := S^.Upcase( C2 ) ;
        end ;
        C1 := To_Universal( C1 ) ;
        C2 := S^.To_Universal( C2 ) ;
        if( C1 > C2 ) then
        begin
            Compare := GreaterThan ;
            exit ;
        end else
        if( C1 < C2 ) then
        begin
            Compare := LessThan ;
            exit ;
        end ;
    end ;

    { If we get here, the strings are equal up to the length of the shortest
      one. }
    if( Length = S^.Length ) then
    begin
        Compare := EqualTo ;
    end else
    if( Length > S^.Length ) then
    begin
        Compare := GreaterThan ;
    end else
    begin
        Compare := LessThan ;
    end ;
end ; { TString.Compare }


function TString.Copy( Start, _Length : longint ) : PString ;

var Loop : longint ;
    Res : PString ;

begin
    Res := Create_Compatible_String ;
    if( Start > Length ) then
    begin
        Copy := Res ; { Return null string }
        exit ;
    end ;
    if( Start + _Length - 1 > Length ) then
    begin
        _Length := Length - Start + 1 ; { Only copy what is there } {UNTESTED}
    end ;
    if( ( Start = 1 ) and ( _Length = Length ) ) then { Copy entire string }
    begin
        Res^.Assign( @self ) ; {UNTESTED}
    end else
    begin
        Res^.Set_Length( _Length ) ;
        for Loop := Start to Start + _Length - 1 do
        begin
            Res^.At_Put( Loop - Start + 1, At( Loop ) ) ; {UNTESTED}
        end ;
    end ;
    Copy := Res ;
end ;


function TString.Create_Compatible_String : PString ;

begin
    Create_Compatible_String := New( PString, Init ) ;
end ;


function TString.Edit( Flags : longint ) : PString ;

var _Class : longint ;
    Current_Char, Original_Char : longint ;
    Destination : PString ;
    Index : integer ; { Current character index within string }
    Last : longint ;
    Leading, Quotes, Space : boolean ;
    OK : boolean ;
    Quote_Type : longint ;

begin
    Destination := Create_Compatible_String ;
    Edit := Destination ;
    if( Length = 0 ) then { No edits on zero-length strings }
    begin
        Exit ;
    end ;
    if ( Flags And 544 ) = 544 then
    begin
        Flags := Flags And Not 512 ;
        { Disallow lower-case command if upper-case command is also present }
    end ;
    if ( Flags And 1088 ) = 1088 then
    begin
        Flags := Flags And Not 1024 ;
    end ;
    { Disallow [] to {}{ if [] to () }
    if ( Flags And 6144 ) = 6144 then
    begin
        Flags := Flags And Not 4096 ;
    end ;
    { Disallow () to {}{ if () to [] }
    if ( Flags And 24576 ) = 24576 then
    begin
        Flags := Flags And ( not 16384 ) ;
    end ;
    { Disallow {}{ to [] if {}{ to () }
    Space := False ; { No spaces }
    Last := 0 ;
    Quotes := False ; { No quotes yet }
    Quote_Type := 0 ;
    Index := 0 ;
    Leading := True ;
    while( Index < Length ) do
    begin
        inc( Index ) ; { Increment source string pointer }
        OK := True ; { This byte is OK - so far }
        Current_Char := At( Index ) ; { Current character }
        _Class := Class_Of( Current_Char ) ;
        if( not Quotes ) then
        begin
            Original_Char := Current_Char ; { Save original value }
            if( ( _Class and CC_Whitespace ) = 0 ) then { Non space }
            begin
                Leading := False ;
            end ;
            if( ( Flags and 1 ) = 1 ) then { Normalize }
            begin
                Current_Char := Normalize( Current_Char ) ;
                _Class := Class_Of( Current_Char ) ;
            end ;
            if( ( Flags and 2 ) = 2 ) then { Remove all whitespace }
            begin
                if( ( _Class and CC_Whitespace ) <> 0 ) then { Space }
                begin
                    OK := False ;
                end ;
            end ;
            if( ( Flags and 4 ) = 4 ) then { Ignore special values? }
            begin
                if( ( _Class and CC_Control ) <> 0 ) then
                begin
                    OK := False ;
                end ;
            end ;
            if( ( Flags and 8 ) = 8 ) then { Ignore leading whitespace }
            begin
                if( Leading ) then
                begin
                    OK := False ;
                end ;
            end ;
            if( ( Flags and 16 ) = 16 ) then { Reduce tabs/spaces to single space }
            begin
                if( ( _Class and CC_Whitespace ) <> 0 ) then
                begin
                    if( Space ) then
                    begin
                        OK := False ;
                    end else
                    begin
                        Space := True ;
                        Current_Char := ord( ASCIIDef.Space ) ;
                    end ;
                end ;
            end ;
            if( ( Flags and 32 ) = 32 ) then { Lower to upper case }
            begin
                Current_Char := Upcase( Current_Char ) ;
            end ;
            if( ( Flags and 512 ) = 512 ) then { Upper to lower case }
            begin
                Current_Char := DownCase( Current_Char ) ;
            end ;
            if( ( Flags and 64 ) = 64 ) then { Convert [] to () }
            begin
                if( Original_Char = ord( '[' ) ) then
                begin
                    Current_Char := ord( '(' ) ;
                end else
                if( Original_Char = ord( ']' ) ) then
                begin
                    Current_Char := ord( ')' ) ;
                end ;
            end ;
            if( ( Flags and 1024 ) = 1024 ) then { Convert [] to braces }
            begin
                if( ( Original_Char = ord( '[' ) ) or ( Original_Char = ord( ']' ) ) ) then
                begin
                    Current_Char := Original_Char + 32 ;
                end ;
            end ;
            if( ( Flags and 2048 ) = 2048 ) then { Convert () to [] }
            begin
                if( Original_Char = ord( '(' ) ) then
                begin
                    Current_Char := ord( '[' ) ;
                end else
                if( Original_Char = ord( ')' ) ) then
                begin
                    Current_Char := ord( ']' ) ;
                end ;
            end ;
            if( ( Flags and 4096 ) = 4096 ) then { Convert () to braces }
            begin
                if( Original_Char = ord( '(' ) ) then
                begin
                    Current_Char := ord( '{' ) ;
                end else
                if( Original_Char = ord( ')' ) ) then
                begin
                    Current_Char := ord( '}' ) ;
                end ;
            end ;
            if( ( Flags and 8192 ) = 8192 ) then { Convert braces to () }
            begin
                if( Original_Char = ord( '{' ) ) then
                begin
                    Current_Char := ord( '(' ) ;
                end else
                if( Original_Char = ord( '}' ) ) then
                begin
                    Current_Char := ord( ')' ) ;
                end ;
            end ;
            if( ( Flags and 16384 ) = 16384 ) then { Convert braces to [] }
            begin
                if( ( Original_Char = ord( '{' ) ) or ( Original_Char = ord( '}' ) ) ) then
                begin
                    Current_Char := Original_Char - 32 ;
                end ;
            end ;
        end ; { if( not Quotes ) }
        if( OK ) then
        begin
            Destination^.Append_Char( Current_Char ) ; { Build result }
            if( ( _Class and CC_Whitespace ) <> 0 ) then { Space }
            begin
                Space := True ;
            end ;
        end ;
        if( ( _Class and CC_Whitespace ) = 0 ) then { Non space }
        begin
            Space := False ;
            if( ( Flags and 256 ) = 256 ) then { Allow no alter within quotes }
            begin
                if( ( _Class and CC_Quote ) <> 0 ) then
                begin
                    if( not Quotes ) then
                    begin
                        Quote_Type := Current_Char ;
                    end ;
                    if( Quote_Type = Current_Char ) then
                    begin
                        Quotes := not Quotes ;
                    end ;
                end ;
            end ;
            Last := Destination^.Length ; { Last non-space character }
        end ;
    end ; { while( Index < Length ) }
    if( ( Flags and 128 ) = 128 ) then { Trim following spaces }
    begin
	if( not Quotes ) then { Not within quotes }
	begin
	    Destination^.Set_Length( Last ) ; { Last non-space character position }
	end ;
    end ;
    Edit := Destination ;
end ; { TString.Edit }


function TString.Fixed : boolean ;

begin
    Fixed := True ;
end ;


function TString.From_Universal( X : longint ) : longint ;

begin
    From_Universal := 0 ;
end ;


function TString.Instr( Pos : longint ; S : PString ) : longint ;

label Start ;

var Loop, Offset : longint ;
    Search_String : PString ;

begin
    { Setup... }
    Instr := 0 ;
    if( S = nil ) then
    begin
        exit ;
    end ;

    if( Pos < 1 ) then
    begin
        Pos := 1 ;
    end ;
    if( Facility = S^.Facility ) then
    begin
        Search_String := S ; {UNTESTED}
    end else
    begin
        Search_String := Create_Compatible_String ;
        Search_String^.Assign( S ) ;

        { Make sure that search string contents are compatible with our type of
          string. }
        for Loop := 1 to S^.Length do {UNTESTED:S^.Length<1}
        begin
            if( S^.To_Universal( S^.At( Loop ) ) <> Search_String^.To_Universal( Search_String^.At( Loop ) ) ) then {UNTESTED}
            begin
                dispose( Search_String, Done ) ;
                exit ;
            end ;
        end ;
    end ;

    { Match loop... }
Start:
    if( S^.Length + Pos - 1 > Length ) then {UNTESTED: S^.Length + Pos - 1 <= Length}
    begin
        Instr := 0 ;
        if( Facility <> S^.Facility ) then {UNTESTED: Facility = S^.Facility}
        begin
            dispose( Search_String, Done ) ; {UNTESTED}
        end ;
        exit ;
    end ;
    Offset := 1 ;
    Instr := Pos ;
    while(
           ( Pos <= Length )
           and
           ( Offset <= Search_String^.Length )
         ) do {UNTESTED}
    begin
        if( At( Pos ) <> Search_String^.At( Offset ) ) then {UNTESTED:=}
        begin
            Pos := Pos - Offset + 2 ; {UNTESTED}
            goto Start ;
        end ;
        inc( Pos ) ;
        inc( Offset ) ;
    end ;
    if( Facility <> S^.Facility ) then {UNTESTED:=}
    begin
        dispose( Search_String, Done ) ; {UNTESTED}
    end ;
end ; { TString.Instr }


function TString.Length : longint ;

begin
    Length := 0 ;
end ;


function TString.Max_Length : longint ;

begin
    Max_Length := 0 ;
end ;


function TString.Normalize( X : longint ) : longint ;

begin
    Normalize := X ;
end ;


function TString.Get_Store : PSManaged_Store ;

begin
    Get_Store := nil ;
end ;


procedure TString.Set_Store( S : PSManaged_Store ) ;

begin
end ;


function TString.Size : longint ;

begin
    Size := Length ;
end ;


procedure TString.Set_Length( X : longint ) ;

begin
    if( X <> Length ) then
    begin
        Modify ;
    end ;
end ;


function TString.Repeating_String( Value : longint ; Len : longint ) : PString ; {UNTESTED}

var Dummy : integer ;
    Res : PString ;

begin
    Res := Create_Compatible_String ;
    Res^.Set_Length( Len ) ;
    for Dummy := 1 to Res^.Length do
    begin
        Res^.At_Put( Dummy, Value ) ;
    end ;
    Repeating_String := Res ;
end ;


function TString.Right( Pos : longint ) : PString ;

begin
    Right := Copy( Pos, Length ) ;
end ;


procedure TString.RSet_String( Value : string ) ;

var Source, Destination : integer ;

begin
    Source := system.length( Value ) - Length + 1 ;
    if( Source < 1 ) then
    begin
        Source := 1 ; {UNTESTED}
    end ;
    Destination := Length - system.length( Value ) + 1 ;
    if( Destination < 1 ) then {UNTESTED: Destination >= 1}
    begin
        Destination := 1 ;
    end ;
    Modify ;
    while( Destination <= Length ) do
    begin
        At_Put( Destination, ord( Value[ Source ] ) ) ; {UNTESTED}
        inc( Destination ) ;
        inc( Source ) ;
    end ;
end ;


procedure TString.Set_String( const Value : string ) ;

var Loop : longint ;

begin
    Modify ;
    Set_Length( system.length( Value ) ) ;
    for Loop := 1 to system.length( Value ) do
    begin
        At_Put( Loop, ord( Value[ Loop ] ) ) ;
    end ;
end ;


function TString.Text : string ;

begin
    Text := '' ;
end ;


function TString.To_Universal( X : longint ) : longint ;

begin
    To_Universal := 0 ;
end ;


function TString.Class_Of( C : longint ) : longint ;

begin
    Class_Of := CC_Unknown ;
end ;



function TString.Upcase( X : longint ) : longint ;

begin
    Upcase := X ;
end ;


function TString.Downcase( X : longint ) : longint ;

begin
    Downcase := X ;
end ;


function TString.Facility : longint ;

begin
    Facility := 68 ;
end ;


function TString.Version : longint ;

begin
    Version := 10 ; { Version 1.0 }
end ;



{ TReference_Storage methods... }

{ Constructors and destructors... }

constructor TReference_Storage.Init ;

begin
    inherited Init ;

    Buffer := 0 ;
    Size := 0 ;
    _PChar := nil ;
    Cache_Data := True ;
end ;


destructor TReference_Storage.Done ;

begin
    Set_Size( 0 ) ;
    if( _Store <> nil ) then {UNTESTED: _Store = nil - This should never happen}
    begin
        _Store^.Detach ;
        _Store := nil ;
    end ;

    inherited Done ;
end ;


{ Internal utility routines... }

function TReference_Storage.Store : PSManaged_Store ;

begin
    if( _Store = nil ) then
    begin
        _Store := Application_Manager^.Get_Heap ;
        _Store^.Attach ;
    end ;
    Store := _Store ;
end ;


{ API... }

function TReference_Storage.As_PChar : PChar ;

var UEC : TUEC ;

begin
    if( ( _PChar = nil ) and ( Size > 0 ) ) then
    begin
        getmem( _PChar, Size ) ;
        if( ( Size <= sizeof( Buffer ) ) and Cache_Data ) then
        begin
            system.move( Buffer, _PChar[ 0 ], Size ) ;
        end else
        begin
            Store^.Read_Data( _PChar[ 0 ], Buffer, Size, UEC ) ;
        end ;
    end ;
    As_PChar := _PChar ;
end ;


{ Parameters: Old = source data address
              Old_Size = source data size, in bytes
              New = destination data address
              New_Size = destination data size, in bytes
              Len = Amount of data to move from Old to New, in bytes. }
procedure TReference_Storage.Move( var Old : longint ; Old_Size : longint ;
    var New : longint ; New_Size, Len : longint ) ;

begin
    if( Len > New_Size ) then
    begin
        Len := New_Size ;
    end ;
    if( Len > Old_Size ) then
    begin
        Len := Old_Size ;
    end ;
    if( ( Old_Size <= sizeof( Buffer ) ) and Cache_Data ) then
    begin
        if( ( New_Size <= sizeof( Buffer ) ) and Cache_Data ) then
        begin
            system.move( Old, New, Len ) ;
        end else
        begin
            Store^.Write( New, Len, Old ) ;
        end ;
    end else
    begin
        if( ( New_Size <= sizeof( Buffer ) ) and Cache_Data ) then
        begin
            Store^.Read( Old, Len, New ) ;
        end else
        begin
            Store^.Copy( Old, New, Len ) ;
        end ;
    end ;
    if( _PChar <> nil ) then
    begin
        freemem( _PChar, Size ) ;
        _PChar := nil ;
    end ;
end ; { TReference_Storage.Move }


procedure TReference_Storage.Zero( Pos, Len : longint ) ;

begin
    if( ( Size <= sizeof( Buffer ) ) and Cache_Data ) then
    begin
        fillchar( PChar( @Buffer )[ Pos ], 0, Len ) ;
    end else
    begin
        Store^.Fill( Buffer + Pos, Len, 0 ) ;
    end ;
    if( _PChar <> nil ) then
    begin
        fillchar( _PChar[ Pos ], Len, 0 ) ; {UNTESTED: This should never happen}
    end ;
end ;


function TReference_Storage.Get_Byte( Pos : longint ) : byte ;

var Res : byte ;
    UEC : TUEC ;

begin
    if( ( Size <= sizeof( Buffer ) ) and Cache_Data ) then
    begin
        system.move( PChar( @Buffer )[ Pos ], Res, 1 ) ;
    end else
    begin
        Store^.Read_Data( Res, Buffer + Pos, 1, UEC ) ;
    end ;
    Get_Byte := Res ;
end ;


procedure TReference_Storage.Set_Byte( D : byte ; Pos : longint ) ;

var UEC : TUEC ;

begin
    if( ( Size <= sizeof( Buffer ) ) and Cache_Data ) then
    begin
        system.move( D, PChar( @Buffer )[ Pos ], 1 ) ;
    end else
    begin
        Store^.Write_Data( D, Buffer + Pos, 1, UEC ) ;
    end ;
    if( _PChar <> nil ) then
    begin
        _PChar[ Pos ] := char( D ) ;
    end ;
end ;


procedure TReference_Storage.Get_Data( var D ; Pos, Len : longint ) ;

var UEC : TUEC ;

begin
    if( ( Size <= sizeof( Buffer ) ) and Cache_Data ) then
    begin
        system.move( PChar( @Buffer )[ Pos ], D, Len ) ;
    end else
    begin
        Store^.Read_Data( D, Buffer + Pos, Len, UEC ) ;
    end ;
end ;


procedure TReference_Storage.Set_Data( var D ; Pos, Len : longint ) ;

var UEC : TUEC ;

begin
    if( ( Size <= sizeof( Buffer ) ) and Cache_Data ) then
    begin
        system.move( D, PChar( @Buffer )[ Pos ], Len ) ;
    end else
    begin
        Store^.Write_Data( D, Buffer + Pos, Len, UEC ) ;
    end ;
    if( _PChar <> nil ) then
    begin
        Store^.Read_Data( _PChar[ 0 ], Buffer, Size, UEC ) ;
    end ;
end ;


procedure TReference_Storage.Set_Size( S : longint ) ;

var Old : integer ;{sizeof( Buffer )}

begin
    if( S = Size ) then
    begin
        exit ; { No change in size }
    end ;
    if( _PChar <> nil ) then
    begin
        freemem( _PChar, Size ) ;
        _PChar := nil ;
    end ;
    if( ( Buffer <> 0 ) and ( Size > sizeof( Buffer ) ) and Cache_Data ) then
    begin
        if( ( S <= sizeof( Buffer ) ) and Cache_Data ) then
        begin
            if( ( Size > sizeof( Buffer ) ) and Cache_Data ) then
            begin
                Store^.Deallocate( Buffer, Size ) ;
            end ;
            Buffer := 0 ;
        end else
        begin
            if( ( Size > sizeof( Buffer ) ) and Cache_Data ) then
            begin
                Buffer := Store^.Reallocate( Buffer, Size, S ) ;
            end else
            begin
                Buffer := Store^.Allocate( S ) ;
            end ;
        end ;
    end else
    begin
        if( ( S > sizeof( Buffer ) ) or ( not Cache_Data ) ) then {UNTESTED: S<=0 - This situation should never happen}
        begin
            Old := Buffer ;
            Buffer := Store^.Allocate( S ) ;
            if( ( Size <= sizeof( Buffer ) ) and Cache_Data ) then { Converting from in-place to store }
            begin
                Store^.Write( Buffer, Size, Old ) ;
            end else
            if( Size > 0 ) then
            begin
                if( S < Size ) then
                begin
                    Store^.Copy( Old, Buffer, S ) ;
                end else
                begin
                    Store^.Copy( Old, Buffer, Size ) ;
                end ;
                Store^.Deallocate( Old, Size ) ;
            end ;
        end ;
    end ;
    Size := S ;
end ; { TReference_Storage.Set_Size }


procedure TReference_Storage.Fill( Pos, Len, Value : longint ) ; { UNTESTED }

begin
    if( Pos < 0 ) then
    begin
        Len := Len + Pos ;
        Pos := 0 ;
    end ;
    if( Pos >= Size ) then
    begin
        exit ;
    end ;
    if( Pos + Len > Size ) then
    begin
        Len := Size - Pos + 1 ;
    end ;
    if( Len < 1 ) then
    begin
        exit ;
    end ;
    if( ( Size <= sizeof( Buffer ) ) and Cache_Data ) then
    begin
        fillchar( PChar( @Buffer )[ Pos ], Value, Len ) ;
    end else
    begin
        Store^.Fill( Buffer + Pos, Len, Value ) ;
    end ;
end ;


procedure TReference_Storage.Set_Store( S : PSManaged_Store ) ;

begin
    Set_Size( 0 ) ;
    if( _Store <> nil ) then
    begin
        _Store^.Detach ;
    end ;
    _Store := S ;
    if( _Store <> nil ) then
    begin
        _Store^.Attach ;
    end ;
end ;



{ TASCII_String methods... }

{ Constructors and destructors... }

constructor TASCII_String.Init ;

begin
    inherited Init ;

    _Data := nil ;
end ;


destructor TASCII_String.Done ;

begin
    if( _Data <> nil ) then
    begin
        _Data^.Detach ;
        _Data := nil ;
    end ;

    inherited Done ;
end ;


{ Internal utility routines... }

procedure TASCII_String.Modify ;

var Old : PReference_Storage ;

begin
    if(
        ( not Shared )
        and
        ( _Data <> nil )
        and
        ( _Data^.Reference_Count > 1 )
      ) then { If others refer to this unshared data }
    begin
        { Create our own local copy... }
        Old := _Data ;
        _Data := new( PReference_Storage, Init ) ;
        _Data.Set_Memory_Type( MT_Heap ) ;
        _Data^.Set_Store( Old^._Store ) ;
        _Data^.Set_Size( Old^.Size ) ;
        _Data^.Move( Old^.Buffer, Old^.Size, _Data^.Buffer, _Data^.Size, Old^.Size ) ;
        Old^.Detach ;
        _Data^.Attach ;
    end ;
end ;


{ Overrides... }

procedure TASCII_String.Append( Source : PString ) ;

var Len, Loop, Start : longint ;
    P : longint ;

begin
    if( ( Source <> nil ) and ( Source^.Length > 0 ) ) then
    begin
        Start := Length ; { Current string length }
        Set_Length( Length + Source^.Length ) ;
        Len := Length - Start ; { Amount of data to actually append }
        if(
             ( Facility <> Source^.Facility )
             or
             ( Get_Store <> Source^.Get_Store )
             or
             ( ( _Data^.Size <= sizeof( _Data^.Buffer ) ) and _Data^.Cache_Data )
             or
             ( ( PASCII_String( Source )^._Data^.Size <= sizeof( _Data^.Buffer ) ) and _Data^.Cache_Data )
{$IFDEF TEST}
             or
             Test_Generic
{$ENDIF}
          ) then
        begin
            for Loop := 1 to Len do
            begin
                _Data^.Set_Byte( From_Universal( Source^.To_Universal( Source^.At( Loop ) ) ), Start ) ;
                inc( Start ) ;
            end ;
        end else
        begin
            P := _Data^.Buffer + Start ;
            _Data^.Move( PASCII_String( Source )^._Data^.Buffer,
                PASCII_String( Source )^._Data^.Size, P,
                _Data^.Size, Source^.Length ) ;
        end ;
    end ;
end ;


procedure TASCII_String.Append_Char( Source : longint ) ;

begin
    if( Length < Max_Length ) then { Cannot extend a string beyond its maximum length } {UNTESTED:ELSE}
    begin
        Set_Length( Length + 1 ) ;
        _Data^.Set_Byte( Source, _Data^.Size - 2 ) ;
    end ;
end ;


procedure TASCII_String.Append_String( Source : string ) ;

var Start : longint ;

begin
    if( system.length( Source ) > 0 ) then
    begin
        Start := Length ; { Current string length }
        Set_Length( Length + system.length( Source ) ) ;
        _Data^.Set_Data( Source[ 1 ], Start, Length - Start ) ;
    end ;
end ;


procedure TASCII_String.Assign( Source : PString ) ;

var Loop : integer ;

begin
    if( Source = nil ) then
    begin
        Set_Length( 0 ) ;
        exit ;
    end ;
    Modify ;
    if(
         ( Facility <> Source^.Facility )
{$IFDEF TEST}
         or
         Test_Generic
{$ENDIF}
      ) then
    begin
        Set_Length( Source^.Length ) ;
        for Loop := 1 to Length do
        begin
            _Data^.Set_Byte( From_Universal( Source^.To_Universal( Source^.At( Loop ) ) ), Loop - 1 ) ;
        end ;
    end else
    begin
        if( _Data <> nil ) then
        begin
            _Data^.Detach ;
        end ;
        _Data := PASCII_String( Source )^._Data ;
        if( _Data <> nil ) then {UNTESTED: _Data=nil}
        begin
            _Data^.Attach ;
        end ;
    end ;
end ;


procedure TASCII_String.RAssign( Source : PString ) ;

var _Source, _Destination : integer ;
    Dst, Src : longint ;

begin
    if( ( Source <> nil ) and ( Source^.Length > 0 ) ) then
    begin
        _Source := Source^.Length - Length + 1 ;
        if( _Source < 1 ) then
        begin
            _Source := 1 ;
        end ;
        _Destination := Length - Source^.Length + 1 ;
        if( _Destination < 1 ) then
        begin
            _Destination := 1 ;
        end ;
        Modify ;
        if(
             ( Facility <> Source^.Facility )
             or
             ( Get_Store <> Source^.Get_Store )
             or
             ( ( _Data^.Size <= sizeof( _Data^.Buffer ) ) and _Data^.Cache_Data )
             or
             ( ( PASCII_String( Source )^._Data^.Size <= sizeof( _Data^.Buffer ) ) and _Data^.Cache_Data )
{$IFDEF TEST}
             or
             Test_Generic
{$ENDIF}
          ) then
        begin
            while( _Destination <= Length ) do
            begin
                At_Put( _Destination, From_Universal( Source^.To_Universal( Source^.At( _Source ) ) ) ) ;
                inc( _Destination ) ;
                inc( _Source ) ;
            end ;
        end else
        begin
            if( Source^.Length = Length ) then
            begin
                if( _Data <> nil ) then {UNTESTED: _Data = nil}
                begin
                    _Data^.Detach ;
                end ;
                _Data := PASCII_String( Source )^._Data ;
                _Data^.Attach ;
            end else
            begin
                Dst := _Data^.Buffer + _Destination - 1 ;
                Src := PASCII_String( Source )^._Data^.Buffer + _Source - 1 ;
                _Data^.Move( Src,
                    PASCII_String( Source )^._Data^.Size,
                    Dst, _Data^.Size, Length - _Destination + 1 ) ;
            end ;
        end ;
    end ;
end ; { TASCII_String.RAssign }


function TASCII_String.As_PChar : PChar ;

begin
    As_PChar := _Data^.As_PChar ;
end ;


function TASCII_String.At( Index : integer ) : longint ;

begin
    if( ( Index < 1 ) or ( Index > Length ) ) then
    begin
        At := 0 ;
    end else
    begin
        At := _Data^.Get_Byte( Index - 1 ) ;
    end ;
end ;


procedure TASCII_String.At_Put( Index, Source : longint ) ;

begin
    if( ( Index > 0 ) and ( Index <= Length ) ) then
    begin
        if( _Data^.Get_Byte( Index - 1 ) <> Source ) then
        begin
            Modify ;
            _Data^.Set_Byte( Source, Index - 1 ) ;
        end ;
    end ;
end ;


function TASCII_String.Class_Of( C : longint ) : longint ;

begin
    if( ( C < 0 ) or ( C > 255 ) ) then
    begin
        Class_Of := CC_Unknown ;
    end else
    begin
        Class_Of := ASCII_CC[ C and 127 ] ;
    end ;
end ;


function TASCII_String.Upcase( X : longint ) : longint ;

begin
    if( ( ( X and 127 ) >= ord( 'a' ) ) and ( ( X and 127 ) <= ord( 'z' ) ) ) then
    begin
        X := X - 32 ;
    end ;
    Upcase := X ;
end ;


function TASCII_String.Downcase( X : longint ) : longint ;

begin
    if( ( ( X and 127 ) >= ord( 'A' ) ) and ( ( X and 127 ) <= ord( 'Z' ) ) ) then
    begin
        X := X + 32 ;
    end ;
    Downcase := X ;
end ;


function TASCII_String.Compare( S : PString ; CS : boolean ) : TComparison ;

var Len, Loop : longint ;
    C1, C2 : longint ;

begin
    if( @self = S ) then { Compare with self }
    begin
        Compare := EqualTo ;
        exit ;
    end ;
    if( ( @self = nil ) or ( S = nil ) or ( S^.Facility <> Facility ) ) then
    begin
        Compare := inherited Compare( S, CS ) ;
        exit ;
    end ;
    if( ( Length = 0 ) and ( S^.Length = 0 ) ) then { Both are null strings }
    begin
        Compare := EqualTo ;
        exit ;
    end ;

    Len := Min( Length, S^.Length ) ;
    for Loop := 1 to Len do
    begin
        C1 := _Data^.Get_Byte( Loop - 1 ) ;
        C2 := PASCII_String( S )^._Data^.Get_Byte( Loop - 1 ) ;
        if( not CS ) then
        begin
            C1 := Upcase( C1 ) ;
            C2 := Upcase( C2 ) ;
        end ;
        if( C1 > C2 ) then
        begin
            Compare := GreaterThan ;
            exit ;
        end else
        if( C1 < C2 ) then
        begin
            Compare := LessThan ;
            exit ;
        end ;
    end ;

    { If we get here, the strings are equal up to the length of the shortest
      one. }
    if( Length = S^.Length ) then
    begin
        Compare := EqualTo ;
    end else
    if( Length > S^.Length ) then
    begin
        Compare := GreaterThan ;
    end else
    begin
        Compare := LessThan ;
    end ;
end ; { TASCII_String.Compare }


function TASCII_String.Copy( Start, _Length : longint ) : PString ;

var _Destination, _Source : longint ;
    Dst, Src : longint ;
    Res : PASCII_String ;

begin
    Res := PASCII_String( Create_Compatible_String ) ;
    if( Start > Length ) then
    begin
        Copy := Res ; { Return null string }
        exit ;
    end ;
    if( Start + _Length - 1 > Length ) then
    begin
        _Length := Length - Start + 1 ; { Only copy what is there }
    end ;
    if( ( Start = 1 ) and ( _Length = Length ) ) then { Copy entire string }
    begin
        Res^.Assign( @self ) ;
    end else
    begin
        Res^.Set_Store( _Data^._Store ) ;
        Res^.Set_Length( _Length ) ;
        if( _Length > 0 ) then
        begin
            if(
                ( ( _Data^.Size <= sizeof( _Data^.Buffer ) ) and _Data^.Cache_Data )
                or
                ( ( _Length + 1 <= sizeof( _Data^.Buffer ) ) and _Data^.Cache_Data )
              ) then
            begin
                _Source := Start - 1 ;
                _Destination := 0 ;
                while( _Destination <= _Length ) do
                begin
                    Res^.At_Put( _Destination, At( _Source ) ) ;
                    inc( _Destination ) ;
                    inc( _Source ) ;
                end ;
            end else
            begin
                Src := _Data^.Buffer + Start - 1 ;
                Dst := Res^._Data^.Buffer ;
                _Data^.Move( Src, _Data^.Size, Dst, Res^._Data^.Size, _Length ) ;
            end ;
        end ;
    end ;
    Copy := Res ;
end ; { TASCII_String.Copy }


function TASCII_String.Create_Compatible_String : PString ;

var S : PASCII_String ;

begin
    S := New( PASCII_String, Init ) ;
    S^.Set_Memory_Type( MT_Heap ) ;
    Create_Compatible_String := S ;
end ;


function TASCII_String.Fixed : boolean ;

begin
    Fixed := False ;
end ;


function TASCII_String.From_Universal( X : longint ) : longint ;

begin
    if( ( X < 0 ) or ( X > 127 ) ) then
    begin
        From_Universal := 0 ;
    end else
    begin
        From_Universal := X ;
    end ;
end ;


function TASCII_String.Instr( Pos : longint ; S : PString ) : longint ;

label Start ;

var Loop, Offset : longint ;
    Search_String : PASCII_String ;

begin
    { Setup... }
    Instr := 0 ;
    if( ( S = nil ) or ( S^.Length = 0 ) ) then
    begin
        exit ;
    end ;

    if( Pos < 1 ) then
    begin
        Pos := 1 ;
    end ;
    if(
        ( Facility = S^.Facility )
{$IFDEF TEST}
         and
         ( not Test_Generic )
{$ENDIF}
      ) then
    begin
        Search_String := PASCII_String( S ) ;
    end else
    begin
        Search_String := PASCII_String( Create_Compatible_String ) ;
        Search_String^.Assign( S ) ;

        { Make sure that search string contents are compatible with our type of
          string. }
        for Loop := 1 to S^.Length do
        begin
            if( S^.To_Universal( S^.At( Loop ) ) <> Search_String^.To_Universal( Search_String^.At( Loop ) ) ) then
            begin
                dispose( Search_String, Done ) ; {UNTESTED}
                exit ;
            end ;
        end ;
    end ;

    { Match loop... }
Start:
    if( S^.Length + Pos - 1 > Length ) then
    begin
        Instr := 0 ;
        if( Facility <> S^.Facility ) then
        begin
            dispose( Search_String, Done ) ;
        end ;
        exit ;
    end ;
    Offset := 1 ;
    Instr := Pos ;
    while(
           ( Pos <= Length )
           and
           ( Offset <= Search_String^.Length )
         ) do
    begin
        if( _Data^.Get_Byte( Pos - 1 ) <> Search_String^._Data^.Get_Byte( Offset - 1 ) ) then
        begin
            Pos := Pos - Offset + 2 ;
            goto Start ;
        end ;
        inc( Pos ) ;
        inc( Offset ) ;
    end ;
    if(
        ( Facility <> S^.Facility )
{$IFDEF TEST}
        or
        Test_Generic
{$ENDIF}
      ) then
    begin
        dispose( Search_String, Done ) ;
    end ;
end ; { TASCII_String.Instr }


function TASCII_String.Length : longint ;

begin
    if( _Data = nil ) then
    begin
        Length := 0 ;
    end else
    begin
        Length := _Data^.Size - 1 ;
    end ;
end ;


function TASCII_String.Max_Length : longint ;

begin
    Max_Length := Max_Memory - 1 ;
end ;


function TASCII_String.Normalize( X : longint ) : longint ;

begin
    Normalize := X and 127 ;
end ;


function TASCII_String.Get_Store : PSManaged_Store ;

begin
    if( _Data = nil ) then
    begin
        Get_Store := nil ;
    end else
    begin
        Get_Store := _Data^._Store ;
    end ;
end ;


procedure TASCII_String.Set_Store( S : PSManaged_Store ) ;

begin
    if( _Data = nil ) then
    begin
        _Data := new( PReference_Storage, Init ) ;
        _Data.Set_Memory_Type( MT_Heap ) ;
        _Data^.Attach ;
    end ;
    if( S <> _Data^._Store ) then
    begin
        Set_Length( 0 ) ; { Changing store deletes old string }
        _Data^.Set_Store( S ) ;
    end ;
end ;


procedure TASCII_String.Set_Length( X : longint ) ;

var Old_Size : longint ;

begin
    if( X < 0 ) then
    begin
        X := 0 ;
    end ;
    if( ( X = 0 ) and ( _Data = nil ) ) then
    begin
        exit ;
    end ;
    if( X > Max_Length ) then
    begin
        X := Max_Length ; {UNTESTED}
    end ;
    if( ( _Data = nil ) or ( _Data^.Size <> X + 1 ) ) then { Change in size }
    begin
        Modify ;
        if( _Data = nil ) then
        begin
            _Data := new( PReference_Storage, Init ) ;
            _Data.Set_Memory_Type( MT_Heap ) ;
            _Data^.Attach ;
        end ;
        Old_Size := _Data^.Size ;
        _Data^.Set_Size( X + 1 ) ;
        if( X > Old_Size ) then { Extending the data }
        begin
            _Data^.Zero( Old_Size, X - Old_Size + 1 ) ; { Zero-fill extended portion }
        end else
        begin
            _Data^.Set_Byte( 0, X ) ; { Make sure terminating character is null }
        end ;
    end ;
end ; { TASCII_String.Set_Length }


procedure TASCII_String.Set_String( const Value : string ) ;

var P : PChar ;

begin
    Modify ;
    Set_Length( system.length( Value ) ) ;
    if( system.length( Value ) > 0 ) then
    begin
{$IFDEF DELPHI}
        P := PChar( Value ) ;
{$ELSE}
        P := @Value[ 1 ] ;
{$ENDIF}
        _Data^.Set_Data( P^, 0, system.length( Value ) ) ;
    end ;
end ;


function TASCII_String.Repeating_String( Value : longint ; Len : longint ) : PString ; {UNTESTED}

var Res : PString ;

begin
    Res := Create_Compatible_String ;
    Res^.Set_Length( Len ) ;
    if( PASCII_String( Res )^._Data <> nil ) then
    begin
        PASCII_String( Res )^._Data^.Fill( 0, Res^.Length, Value ) ;
    end ;
    Repeating_String := Res ;
end ;


procedure TASCII_String.RSet_String( Value : string ) ;

var P : PChar ;
    Source, Destination : integer ;

begin
    if( system.length( Value ) = 0 ) then { Empty string - do nothing }
    begin
        exit ;
    end ;
    Source := system.length( Value ) - Length + 1 ;
    if( Source < 1 ) then
    begin
        Source := 1 ;
    end ;
    Destination := Length - system.length( Value ) + 1 ;
    if( Destination < 1 ) then
    begin
        Destination := 1 ;
    end ;
    Modify ;
{$IFDEF DELPHI}
    P := PChar( Value ) ;
    P := P + Source - 1 ;
{$ELSE}
    P := @Value[ Source - 1 ] ;
{$ENDIF}
    _Data^.Set_Data( P^, Destination - 1, system.length( Value ) - Source + 1 ) ;
end ;


function TASCII_String.Text : string ;

var P : PChar ;
{$IFNDEF DELPHI}
    Result : string ;
{$ENDIF}

begin
{$IFDEF DELPHI}
    setlength( Result, Length ) ;
{$ELSE}
    Result[ 0 ] := char( Length ) ;
{$ENDIF}
    if( system.length( Result ) > 0 ) then
    begin
{$IFDEF DELPHI}
        P := PChar( Result ) ;
{$ELSE}
        P := @Result[ 1 ] ;
{$ENDIF}
        _Data^.Get_Data( P^, 0, system.length( Result ) ) ;
    end ;
{$IFNDEF DELPHI}
    Text := Result ;
{$ENDIF}
end ;


function TASCII_String.To_Universal( X : longint ) : longint ;

begin
    if( ( X < 0 ) or ( X > 127 ) ) then
    begin
        To_Universal := 0 ;
    end else
    begin
        To_Universal := X ;
    end ;
end ;


function TASCII_String.Facility : longint ;

begin
    Facility := 69 ;
end ;



{ TASCII7_String methods... }

{ Overrides... }

function TASCII7_String.Create_Compatible_String : PString ;

var S : PASCII_String ;

begin
    S := New( PASCII7_String, Init ) ;
    S^.Set_Memory_Type( MT_Heap ) ;
    Create_Compatible_String := S ;
end ;


procedure TASCII7_String.Append_String( Source : string ) ;

var Loop : longint ;

begin
    for Loop := 1 to system.length( Source ) do
    begin
        Source[ Loop ] := char( ord( Source[ Loop ] ) and 127 ) ;
    end ;
    inherited Append_String( Source ) ;
end ;


function TASCII7_String.At( Index : integer ) : longint ;

begin
    if( ( Index < 1 ) or ( Index > Length ) ) then
    begin
        At := 0 ;
    end else
    begin
        At := _Data^.Get_Byte( Index - 1 ) and 127 ;
    end ;
end ;


procedure TASCII7_String.At_Put( Index, Source : longint ) ;

begin
    if( ( Index > 0 ) and ( Index <= Length ) ) then
    begin
        Source := Source and 127 ;
        if( _Data^.Get_Byte( Index - 1 ) <> Source ) then
        begin
            Modify ;
            _Data^.Set_Byte( Source, Index - 1 ) ;
        end ;
    end ;
end ;


function TASCII7_String.Class_Of( C : longint ) : longint ;

begin
    if( ( C < 0 ) or ( C > 127 ) ) then
    begin
        Class_Of := CC_Unknown ;
    end else
    begin
        Class_Of := ASCII_CC[ C and 127 ] ;
    end ;
end ;


function TASCII7_String.Upcase( X : longint ) : longint ;

begin
    if( ( X >= ord( 'a' ) ) and ( X <= ord( 'z' ) ) ) then
    begin
        X := X - 32 ;
    end ;
    Upcase := X ;
end ;


function TASCII7_String.Downcase( X : longint ) : longint ;

begin
    if( ( X >= ord( 'A' ) ) and ( X <= ord( 'Z' ) ) ) then
    begin
        X := X + 32 ;
    end ;
    Downcase := X ;
end ;


procedure TASCII7_String.Set_String( const Value : string ) ;

var Loop : integer ;
    S : string ;

begin
    S := Value ;
    for Loop := 1 to system.length( Value ) do
    begin
        S[ Loop ] := char( ord( Value[ Loop ] ) and 127 ) ;
    end ;
    inherited Set_String( Value ) ;
end ;


procedure TASCII7_String.RSet_String( Value : string ) ;

var Loop : integer ;

begin
    for Loop := 1 to system.length( Value ) do
    begin
        Value[ Loop ] := char( ord( Value[ Loop ] ) and 127 ) ;
    end ;
    inherited RSet_String( Value ) ;
end ;


function TASCII7_String.Facility : longint ;

begin
    Facility := 71 ;
end ;


{ TASCII_DOS_String methods... }

{ Overrides... }

function TASCII_DOS_String.Class_Of( C : longint ) : longint ;

begin
    case C of
        1..8 : Class_Of := CC_Symbol ;
        9 : Class_Of := CC_Digit ;
        10..13 : Class_Of := CC_Alpha ;
        14..18 : Class_Of := CC_Symbol ;
        19 : Class_Of := CC_Punctuation ;
        20..29 : Class_Of := CC_Symbol ;
        30..31 : Class_Of := CC_Alpha ;
        127 : Class_Of := CC_Symbol ;
        128..154 : Class_Of := CC_Alpha ;
        155..157 : Class_Of := CC_Symbol or CC_Currency ;
        158..159 : Class_Of := CC_Symbol ;
        160..165 : Class_Of := CC_Alpha ;
        166..167 : Class_Of := CC_Symbol ;
        168 : Class_Of := CC_Punctuation ;
        169..223 : Class_Of := CC_Symbol ;
        224..235 : Class_Of := CC_Alpha ;
        236 : Class_Of := CC_Symbol ;
        237..254 : Class_Of := CC_Symbol ;
        else Class_Of := inherited Class_Of( C ) ;
    end ;
end ;


function TASCII_DOS_String.Create_Compatible_String : PString ;

var S : PASCII_String ;

begin
    S := New( PASCII_DOS_String, Init ) ;
    S^.Set_Memory_Type( MT_Heap ) ;
    Create_Compatible_String := S ;
end ;


function TASCII_DOS_String.Facility : longint ;

begin
    Facility := 72 ;
end ;


function TASCII_DOS_String.From_Universal( X : longint ) : longint ;

begin
    case X of
        $263A : X := 1 ;
        $263B : X := 2 ;
        $2665 : X := 3 ;
        $2666 : X := 4 ;
        $2663 : X := 5 ;
        $2660 : X := 6 ;
        $25CF : X := 7 ;
        $25D8 : X := 8 ;
        $25CB : X := 9 ;
        $2642 : X := 11 ;
        $2640 : X := 12 ;
        $266A : X := 13 ;
        $266C : X := 14 ;
        $25B8 : X := 16 ;
        $25C2 : X := 17 ;
        $2195 : X := 18 ;
        $203C : X := 19 ;
        $B6 : X := 20 ;
        $A7 : X := 21 ;
        $2596 : X := 22 ;
        $21A8 : X := 23 ;
        $2191 : X := 24 ;
        $2193 : X := 25 ;
        $2192 : X := 26 ;
        $2190 : X := 27 ;
        $221F : X := 28 ;
        $2194 : X := 29 ;
        $25B4 : X := 30 ;
        $25BE : X := 31 ;
        $C7 : X := $80 ;
        $FC : X := $81 ;
        $E9 : X := $82 ;
        $E2 : X := $83 ;
        $E4 : X := $84 ;
        $E0 : X := $85 ;
        $E5 : X := $86 ;
        $E7 : X := $87 ;
        $EA : X := $88 ;
        $EB : X := $89 ;
        $E8 : X := $8A ;
        $EF : X := $8B ;
        $EE : X := $8C ;
        $EC : X := $8D ;
        $C4 : X := $8E ;
        $C5 : X := $8F ;
        $C9 : X := $90 ;
        $E6 : X := $91 ;
        $C6 : X := $92 ;
        $F4 : X := $93 ;
        $F6 : X := $94 ;
        $F2 : X := $95 ;
        $FB : X := $96 ;
        $F9 : X := $97 ;
        $FF : X := $98 ;
        $D6 : X := $99 ;
        $DC : X := $9A ;
        $A2 : X := $9B ;
        $A3 : X := $9C ;
        $A5 : X := $9D ;
        $20A7 : X := $9E ;
        $192 : X := $9F ;
        $E1 : X := $A0 ;
        $ED : X := $A1 ;
        $F3 : X := $A2 ;
        $FA : X := $A3 ;
        $F1 : X := $A4 ;
        $D1 : X := $A5 ;
        $AA : X := $A6 ;
        $BA : X := $A7 ;
        $BF : X := $A8 ;
        $2310 : X := $A9 ;
        $AC : X := $AA ;
        $BD : X := $AB ;
        $BC : X := $AC ;
        $A1 : X := $AD ;
        $AB : X := $AE ;
        $BB : X := $AF ;
        $2591 : X := $B0 ;
        $2592 : X := $B1 ;
        $2593 : X := $B2 ;
        $2502 : X := $B3 ;
        $2524 : X := $B4 ;
        $2561 : X := $B5 ;
        $2562 : X := $B6 ;
        $2556 : X := $B7 ;
        $2555 : X := $B8 ;
        $2563 : X := $B9 ;
        $2551 : X := $BA ;
        $2557 : X := $BB ;
        $255D : X := $BC ;
        $255C : X := $BD ;
        $255B : X := $BE ;
        $2510 : X := $BF ;
        $2514 : X := $C0 ;
        $2534 : X := $C1 ;
        $252C : X := $C2 ;
        $251C : X := $C3 ;
        $2500 : X := $C4 ;
        $253C : X := $C5 ;
        $255E : X := $C6 ;
        $255F : X := $C7 ;
        $255A : X := $C8 ;
        $2554 : X := $C9 ;
        $2569 : X := $CA ;
        $2566 : X := $CB ;
        $2560 : X := $CC ;
        $2550 : X := $CD ;
        $256C : X := $CE ;
        $2567 : X := $CF ;
        $2568 : X := $D0 ;
        $2564 : X := $D1 ;
        $2565 : X := $D2 ;
        $2559 : X := $D3 ;
        $2558 : X := $D4 ;
        $2552 : X := $D5 ;
        $2553 : X := $D6 ;
        $256B : X := $D7 ;
        $256A : X := $D8 ;
        $2518 : X := $D9 ;
        $250C : X := $DA ;
        $2588 : X := $DB ;
        $2584 : X := $DC ;
        $258C : X := $DD ;
        $2590 : X := $DE ;
        $2580 : X := $DF ;
        $3B1 : X := $E0 ;
        $DF : X := $E1 ;
        $393 : X := $E2 ;
        $3C0 : X := $E3 ;
        $3A3 : X := $E4 ;
        $3C3 : X := $E5 ;
        $B5 : X := $E6 ;
        $3C4 : X := $E7 ;
        $3A6 : X := $E8 ;
        $398 : X := $E9 ;
        $3A9 : X := $EA ;
        $3B4 : X := $EB ;
        $221E : X := $EC ;
        $3C6 : X := $ED ;
        $3B5 : X := $EE ;
        $2229 : X := $EF ;
        $2261 : X := $F0 ;
        $B1 : X := $F1 ;
        $2265 : X := $F2 ;
        $2264 : X := $F3 ;
        $2320 : X := $F4 ;
        $2321 : X := $F5 ;
        $F7 : X := $F6 ;
        $2248 : X := $F7 ;
        $B0 : X := $F8 ;
        $2219 : X := $F9 ;
        $B7 : X := $FA ;
        $221A : X := $FB ;
        $207F : X := $FC ;
        $B2 : X := $FD ;
        $25A0 : X := $FE ;
        else X := inherited From_Universal( X ) ;
    end ;
    From_Universal := X ;
end ;


function TASCII_DOS_String.Normalize( X : longint ) : longint ;

begin
    case X of
        128 : Normalize := 67 ;
        131..134 : Normalize := 97 ;
        135 : Normalize := 63 ;
        136..138 : Normalize := 101 ;
        139..141 : Normalize := 105 ;
        142..143 : Normalize := 65 ;
        144 : Normalize := 69 ;
        147..149 : Normalize := 111 ;
        150..151 : Normalize := 117 ;
        152 : Normalize := 121 ;
        153 : Normalize := 79 ;
        154 : Normalize := 85 ;
        160 : Normalize := 97 ;
        161 : Normalize := 105 ;
        162 : Normalize := 111 ;
        163 : Normalize := 117 ;
        164 : Normalize := 110 ;
        165 : Normalize := 78 ;
        else Normalize := X ;
    end ;
end ;


function TASCII_DOS_String.To_Universal( X : longint ) : longint ;

begin
    case X of
        1 : X := $263A ;
        2 : X := $263B ;
        3 : X := $2665 ;
        4 : X := $2666 ;
        5 : X := $2663 ;
        6 : X := $2660 ;
        7 : X := $25CF ; {?}
        8 : X := $25D8 ;
        9 : X := $25CB ;
{        10 : X := $ ;}
        11 : X := $2642 ;
        12 : X := $2640 ;
        13 : X := $266A ;
        14 : X := $266C ;
{        15 : X := $ ;}
        16 : X := $25B8 ;
        17 : X := $25C2 ;
        18 : X := $2195 ;
        19 : X := $203C ;
        20 : X := $B6 ;
        21 : X := $A7 ;
        22 : X := $2596 ; {?}
        23 : X := $21A8 ;
        24 : X := $2191 ;
        25 : X := $2193 ;
        26 : X := $2192 ;
        27 : X := $2190 ;
        28 : X := $221F ; {?}
        29 : X := $2194 ;
        30 : X := $25B4 ;
        31 : X := $25BE ;
        $80 : X := $C7 ;
        $81 : X := $FC ;
        $82 : X := $E9 ;
        $83 : X := $E2 ;
        $84 : X := $E4 ;
        $85 : X := $E0 ;
        $86 : X := $E5 ;
        $87 : X := $E7 ;
        $88 : X := $EA ;
        $89 : X := $EB ;
        $8A : X := $E8 ;
        $8B : X := $EF ;
        $8C : X := $EE ;
        $8D : X := $EC ;
        $8E : X := $C4 ;
        $8F : X := $C5 ;
        $90 : X := $C9 ;
        $91 : X := $E6 ;
        $92 : X := $C6 ;
        $93 : X := $F4 ;
        $94 : X := $F6 ;
        $95 : X := $F2 ;
        $96 : X := $FB ;
        $97 : X := $F9 ;
        $98 : X := $FF ;
        $99 : X := $D6 ;
        $9A : X := $DC ;
        $9B : X := $A2 ;
        $9C : X := $A3 ;
        $9D : X := $A5 ;
        $9E : X := $20A7 ;
        $9F : X := $192 ;
        $A0 : X := $E1 ;
        $A1 : X := $ED ;
        $A2 : X := $F3 ;
        $A3 : X := $FA ;
        $A4 : X := $F1 ;
        $A5 : X := $D1 ;
        $A6 : X := $AA ;
        $A7 : X := $BA ;
        $A8 : X := $BF ;
        $A9 : X := $2310 ;
        $AA : X := $AC ;
        $AB : X := $BD ;
        $AC : X := $BC ;
        $AD : X := $A1 ;
        $AE : X := $AB ;
        $AF : X := $BB ;
        $B0 : X := $2591 ;
        $B1 : X := $2592 ;
        $B2 : X := $2593 ;
        $B3 : X := $2502 ;
        $B4 : X := $2524 ;
        $B5 : X := $2561 ;
        $B6 : X := $2562 ;
        $B7 : X := $2556 ;
        $B8 : X := $2555 ;
        $B9 : X := $2563 ;
        $BA : X := $2551 ;
        $BB : X := $2557 ;
        $BC : X := $255D ;
        $BD : X := $255C ;
        $BE : X := $255B ;
        $BF : X := $2510 ;
        $C0 : X := $2514 ;
        $C1 : X := $2534 ;
        $C2 : X := $252C ;
        $C3 : X := $251C ;
        $C4 : X := $2500 ;
        $C5 : X := $253C ;
        $C6 : X := $255E ;
        $C7 : X := $255F ;
        $C8 : X := $255A ;
        $C9 : X := $2554 ;
        $CA : X := $2569 ;
        $CB : X := $2566 ;
        $CC : X := $2560 ;
        $CD : X := $2550 ;
        $CE : X := $256C ;
        $CF : X := $2567 ;
        $D0 : X := $2568 ;
        $D1 : X := $2564 ;
        $D2 : X := $2565 ;
        $D3 : X := $2559 ;
        $D4 : X := $2558 ;
        $D5 : X := $2552 ;
        $D6 : X := $2553 ;
        $D7 : X := $256B ;
        $D8 : X := $256A ;
        $D9 : X := $2518 ;
        $DA : X := $250C ;
        $DB : X := $2588 ;
        $DC : X := $2584 ;
        $DD : X := $258C ;
        $DE : X := $2590 ;
        $DF : X := $2580 ;
        $E0 : X := $3B1 ;
        $E1 : X := $DF ;
        $E2 : X := $393 ;
        $E3 : X := $3C0 ;
        $E4 : X := $3A3 ;
        $E5 : X := $3C3 ;
        $E6 : X := $B5 ;
        $E7 : X := $3C4 ;
        $E8 : X := $3A6 ;
        $E9 : X := $398 ;
        $EA : X := $3A9 ;
        $EB : X := $3B4 ;
        $EC : X := $221E ;
        $ED : X := $3C6 ;
        $EE : X := $3B5 ;
        $EF : X := $2229 ;
        $F0 : X := $2261 ;
        $F1 : X := $B1 ;
        $F2 : X := $2265 ;
        $F3 : X := $2264 ;
        $F4 : X := $2320 ;
        $F5 : X := $2321 ;
        $F6 : X := $F7 ;
        $F7 : X := $2248 ;
        $F8 : X := $B0 ;
        $F9 : X := $2219 ;
        $FA : X := $B7 ;
        $FB : X := $221A ;
        $FC : X := $207F ;
        $FD : X := $B2 ;
        $FE : X := $25A0 ;
        else X := inherited To_Universal( X ) ;
    end ;
    To_Universal := X ;
end ;


function TASCII_DOS_String.Upcase( X : longint ) : longint ;

begin
    if( X = 164 ) then
    begin
        X := 165 ;
    end else
    if( X = 135 ) then
    begin
        X := 128 ;
    end else
    if( X = 129 ) then
    begin
        X := 154 ;
    end else
    if( X = 132 ) then
    begin
        X := 142 ;
    end else
    if( X = 134 ) then
    begin
        X := 143 ;
    end else
    if( X = 130 ) then
    begin
        X := 144 ;
    end else
    if( X = 145 ) then
    begin
        X := 146 ;
    end else
    if( X = 148 ) then
    begin
        X := 153 ;
    end else
    begin
        X := inherited Upcase( X ) ;
    end ;
    Upcase := X ;
end ; { TASCII_DOS_String.Upcase }


function TASCII_DOS_String.Downcase( X : longint ) : longint ;

begin
    if( X = 165 ) then
    begin
        X := 164 ;
    end else
    if( X = 128 ) then
    begin
        X := 135 ;
    end else
    if( X = 154 ) then
    begin
        X := 129 ;
    end else
    if( X = 142 ) then
    begin
        X := 132 ;
    end else
    if( X = 143 ) then
    begin
        X := 134 ;
    end else
    if( X = 144 ) then
    begin
        X := 130 ;
    end else
    if( X = 146 ) then
    begin
        X := 145 ;
    end else
    if( X = 153 ) then
    begin
        X := 148 ;
    end else
    begin
        X := inherited Downcase( X ) ;
    end ;
    Downcase := X ;
end ;



{ TArray_String methods... }

{ Constructors and destructors... }

constructor TArray_String.Init( A : array of char ) ;

begin
    _Array := @A ;
    _Low := low( A ) ;
    _High := high( A ) ;
end ;



{ TASCII_Windows_String methods... }

{ Overrides... }

function TASCII_Windows_String.Class_Of( C : longint ) : longint ;

begin
    case C of
        128 : Class_Of := CC_Symbol ;
        129 : Class_Of := CC_Unknown ;
        130..132 : Class_Of := CC_Symbol ;
        133 : Class_Of := CC_Punctuation ;
        135..137 : Class_Of := CC_Symbol ;
        138 : Class_Of := CC_Alpha ;
        139 : Class_Of := CC_Symbol ;
        140 : Class_Of := CC_Alpha ;
        141 : Class_Of := CC_Unknown ;
        142 : Class_Of := CC_Alpha ;
        143..144 : Class_Of := CC_Unknown ;
        145 : Class_Of := CC_Punctuation or CC_Left ;
        146 : Class_Of := CC_Punctuation ;
        147 : Class_Of := CC_Punctuation or CC_Left ;
        148 : Class_Of := CC_Punctuation ;
        149..153 : Class_Of := CC_Symbol ;
        154 : Class_Of := CC_Alpha ;
        155 : Class_Of := CC_Symbol ;
        156 : Class_Of := CC_Alpha ;
        157 : Class_Of := CC_Unknown ;
        158..159 : Class_Of := CC_Alpha ;
        160 : Class_Of := CC_Whitespace ;
        161 : Class_Of := CC_Punctuation or CC_Left ;
        162..190 : Class_Of := CC_Symbol ;
        191 : Class_Of := CC_Punctuation or CC_Left ;
        192..214 : Class_Of := CC_Alpha ;
        215 : Class_Of := CC_Symbol ;
        216..246 : Class_Of := CC_Alpha ;
        247 : Class_Of := CC_Symbol ;
        248..255 : Class_Of := CC_Alpha ;
        else Class_Of := inherited Class_Of( C ) ;
    end ;
end ; { TASCII_Windows_String.Class_Of }


function TASCII_Windows_String.Create_Compatible_String : PString ;

var S : PASCII_String ;

begin
    S := New( PASCII_Windows_String, Init ) ;
    S^.Set_Memory_Type( MT_Heap );
    Create_Compatible_String := S ;
end ;


function TASCII_Windows_String.Facility : longint ;

begin
    Facility := 73 ;
end ;


function TASCII_Windows_String.From_Universal( X : longint ) : longint ;

begin
    case X of
        0..127 :; { Leave alone }
        8364 : X := 128 ;
        806 : X := 130 ;
        10765 : X := 131 ;
        750 : X := 132 ;
        8230 : X := 133 ;
        8224 : X := 134 ;
        8225 : X := 135 ;
        770 : X := 136 ;
        8240 : X := 137 ;
        352 : X := 138 ;
        8249 : X := 139 ;
        338 : X := 140 ;
        381 : X := 142 ;
        8216 : X := 145 ;
        8217 : X := 146 ;
        8220 : X := 147 ;
        8221 : X := 148 ;
        8226 : X := 149 ;
        8212 : X := 150 ;
        8213 : X := 151 ;
        771 : X := 152 ;
        8482 : X := 153 ;
        353 : X := 154 ;
        8250 : X := 155 ;
        339 : X := 156 ;
        382 : X := 158 ;
        376 : X := 159 ;
	160..255 :; { Leave alone }
        else X := inherited To_Universal( X ) ;
    end ;
    From_Universal := X ;
end ;


function TASCII_Windows_String.Normalize( X : longint ) : longint ;

begin
    case X of
        138 : Normalize := 83 ;
        142 : Normalize := 90 ;
        154 : Normalize := 115 ;
        158 : Normalize := 122 ;
        159 : Normalize := 89 ;
        192..197 : Normalize := 65 ;
        199 : Normalize := 67 ;
        200..203 : Normalize := 69 ;
        204..207 : Normalize := 73 ;
        208 : Normalize := 68 ;
        209 : Normalize := 78 ;
        210..214 : Normalize := 79 ;
        217..220 : Normalize := 85 ;
        221 : Normalize := 89 ;
        224..229 : Normalize := 97 ;
        231 : Normalize := 99 ;
        232..235 : Normalize := 101 ;
        236..239 : Normalize := 105 ;
        240 : Normalize := 111 ;
        241 : Normalize := 110 ;
        242..246 : Normalize := 111 ;
        249..252 : Normalize := 117 ;
        253 : Normalize := 121 ;
        255 : Normalize := 121 ;
        else Normalize := inherited Normalize( X ) ;
    end ;
end ;


function TASCII_Windows_String.To_Universal( X : longint ) : longint ;

begin
    case X of
        0..127 :; { Leave alone }
        128 : X := 8364 ; { currency }
        130 : X := 806 ;
        131 : X := 10765 ;
        132 : X := 750 ;
        133 : X := 8230 ;
        134 : X := 8224 ;
        135 : X := 8225 ;
        136 : X := 770 ;
        137 : X := 8240 ;
        138 : X := 352 ;
        139 : X := 8249 ;
        140 : X := 338 ;
        142 : X := 381 ;
        145 : X := 8216 ;
        146 : X := 8217 ;
        147 : X := 8220 ;
        148 : X := 8221 ;
        149 : X := 8226 ;
        150 : X := 8212 ;
        151 : X := 8213 ;
        152 : X := 771 ;
        153 : X := 8482 ;
        154 : X := 353 ;
        155 : X := 8250 ;
        156 : X := 339 ;
        158 : X := 382 ;
        159 : X := 376 ;
	160..255 :; { Leave alone }
        else X := inherited To_Universal( X ) ;
    end ;
    To_Universal := X ;
end ;


function TASCII_Windows_String.Upcase( X : longint ) : longint ;

begin
    case X of
        154 : X := 138 ;
        156 : X := 140 ;
        158 : X := 142 ;
        255 : X := 159 ;
        224..239 : X := X + 192 - 224 ;
        241..246 : X := X + 209 - 241 ;
        248..254 : X := X + 216 - 248 ;
        else X := inherited Upcase( X ) ;
    end ;
    Upcase := X ;
end ;


function TASCII_Windows_String.Downcase( X : longint ) : longint ;

begin
    case X of
        138 : X := 154 ;
        140 : X := 156 ;
        142 : X := 158 ;
        159 : X := 255 ;
        192..207 : X := X + 224 - 192 ;
        209..214 : X := X + 241 - 209 ;
        216..222 : X := X + 248 - 216 ;
        else X := inherited Downcase( X ) ;
    end ;
    Downcase := X ;
end ;



type TUTF8 = array[ 0..7 ] of byte ;

{ TUTF8_String methods... }

{ Overrides... }

function TUTF8_String.Create_Compatible_String : PString ;

var S : PUTF8_String ;

begin
    S := New( PUTF8_String, Init ) ;
    S^.Set_Memory_Type( MT_Heap );
    Create_Compatible_String := S ;
end ;


function TUTF8_String.From_Universal( X : longint ) : longint ;

var R : longint ;
    Res : TUTF8 ;
    _Size : integer ;

begin
    fillchar( Res, sizeof( Res ), 0 ) ;
    if( ( X >= 0 ) and ( X <= $7F ) ) then
    begin
        _Size := 1 ;
    end else
    if( ( X >= $80 ) and ( X <= $7FF ) ) then
    begin
        _Size := 2 ;
    end else
    if( ( X >= $800 ) and ( X <= $FFFF ) ) then
    begin
        _Size := 3 ;
    end else
    if( ( X >= $10000 ) and ( X <= $1FFFFF ) ) then
    begin
        _Size := 4 ;
    end else
    if( ( X >= $200000 ) and ( X <= $3FFFFFF ) ) then
    begin
        _Size := 5 ;
    end else
    begin
        _Size := 6 ;
    end ;
    case _Size of
        2 : begin
                Res[ 0 ] := $C0 ;
                Res[ 1 ] := $80 ;
            end ;
        3 : begin
                Res[ 0 ] := $E0 ;
                Res[ 1 ] := $80 ;
                Res[ 2 ] := $80 ;
            end ;
        4 : begin
                Res[ 0 ] := $F0 ;
                Res[ 1 ] := $80 ;
                Res[ 2 ] := $80 ;
                Res[ 3 ] := $80 ;
            end ;
        5 : begin
                Res[ 0 ] := $F8 ;
                Res[ 1 ] := $80 ;
                Res[ 2 ] := $80 ;
                Res[ 3 ] := $80 ;
                Res[ 4 ] := $80 ;
            end ;
        6 : begin
                Res[ 0 ] := $FC ;
                Res[ 1 ] := $80 ;
                Res[ 2 ] := $80 ;
                Res[ 3 ] := $80 ;
                Res[ 4 ] := $80 ;
                Res[ 5 ] := $80 ;
            end ;
    end ;
    R := 0 ;
    while( _Size > 0 ) do
    begin
        dec( _Size ) ;
        R := ( R shr 8 ) or Res[ _Size ] or ( X and $3F ) ;
        X := X shr 6 ;
    end ;
    From_Universal := R ;
end ; { TUTF8_String.From_Universal }


function TUTF8_String.To_Universal( X : longint ) : longint ;

var Dummy, Loop : integer ;
    Res : TUTF8 ;
    R : longint ;
    _Size : integer ;

begin
    if( ( X >= 0 ) and ( X <= $7F ) ) then
    begin
        To_Universal := X ;
    end else
    begin
        fillchar( Res, sizeof( Res ), 0 ) ;
        Res[ 0 ] := X shr 24 ;
        Dummy := 24 ;
        while( Res[ 0 ] = 0 ) do
        begin
            Dummy := Dummy - 8 ;
            Res[ 0 ] := X shr Dummy ;
        end ;
        if( Dummy > 0 ) then
        begin
            Res[ 1 ] := X shr Dummy ;
            Dummy := Dummy - 8 ;
            if( Dummy > 0 ) then
            begin
                Res[ 2 ] := X shr Dummy ;
                Dummy := Dummy - 8 ;
                if( Dummy > 0 ) then
                begin
                    Res[ 3 ] := X shr Dummy ;
                end ;
            end ;
        end ;
        case Res[ 0 ] of
            $C0..$D7 : _Size := 2 ;
            $E0..$EF : _Size := 3 ;
            $F0..$F7 : _Size := 4 ;
            $F8..$FB : _Size := 5 ;
            else _Size := 6 ;
        end ;

        { Now build up result... }
        case _Size of
            2 : R := Res[ 0 ] and $1F ;
            3 : R := Res[ 0 ] and $F ;
            4 : R := Res[ 0 ] and 7 ;
            5 : R := Res[ 0 ] and 3 ;
            6 : R := Res[ 0 ] and 1 ;
            else R := 0 ;
        end ;
        for Loop := 1 to _Size - 1 do
        begin
            R := ( R shl 6 ) or Res[ Loop ] ;
        end ;
        To_Universal := R ;
    end ;
end ; { TUTF8_String.To_Universal }



{ TUTF16_String methods... }

{ Overrides... }

function TUTF16_String.Create_Compatible_String : PString ;

var S : PUTF16_String ;

begin
    S := New( PUTF16_String, Init ) ;
    S^.Set_Memory_Type( MT_Heap );
    Create_Compatible_String := S ;
end ;


function TUTF16_String.From_Universal( X : longint ) : longint ;

begin
    if( ( X < 0 ) or ( X > $FFFF ) ) then
    begin
        From_Universal := 0 ;
    end else
    begin
        From_Universal := X ;
    end ;
end ;



{ TUTF32_String methods... }

{ Overrides... }

function TUTF32_String.Create_Compatible_String : PString ;

var S : PUTF32_String ;

begin
    S := New( PUTF32_String, Init ) ;
    S^.Set_Memory_Type( MT_Heap ) ;
    Create_Compatible_String := S ;
end ;


function TUTF32_String.From_Universal( X : longint ) : longint ;

begin
    From_Universal := X ;
end ;


function TUTF32_String.To_Universal( X : longint ) : longint ;

begin
    To_Universal := X ;
end ;



{ TString_Translator_Base methods... }

{ API... }

function TString_Translator_Base.Translate_String( Value : PString ) : PString ;

var S : PString ;

begin
    S := new( PString, Init ) ;
    S.Set_Memory_Type( MT_Heap ) ;
    Translate_String := S ; { We are an abstract base class, always return null string }
end ;


{ TString_Translator methods... }

{ Constructors and destructors... }

constructor TString_Translator.Init ;

var Loop : integer ;

begin
    for Loop := 0 to 255 do
    begin
        Translations[ Loop ] := nil ;
    end ;
end ;


{ API... }

procedure TString_Translator.Set_Translation( Index : integer ;
    Value : PString ) ;

begin
    if( Translations[ Index ] = nil ) then
    begin
        Translations[ Index ] := new( PString, Init ) ;
        Translations[ Index ].Set_Memory_Type( MT_Heap ) ;
    end ;
    Translations[ Index ]^.Assign( Value ) ;
end ;


function TString_Translator.Translate_String( Value : PString ) : PString ;

var Index, Loop : integer ;
    P, Res : PString ;

begin
    Res := new( PString, Init ) ;
    Res.Set_Memory_Type( MT_Heap  );
    for Loop := 1 to Value^.Length do
    begin
        P := Value^.Copy( Loop, 1 ) ;
        Index := P^.At( Loop ) ;
        if( ( Index > 255 ) or ( Translations[ Index ] = nil ) ) then
        begin
            Res^.Append( P ) ;
        end else
        begin
            Res^.Append( Translations[ Loop ] ) ;
        end ;
        dispose( P, Done ) ;
    end ;
    Translate_String := Res ;
end ;


{ Unit API... }

function Compare_Strings( String1, String2 : string ;
    X : RComparison ) : double ;
{ This function will return a comparison value that is the percentage of
  closeness of the two strings (between 0 and 1). }

var A, B, C, D : integer ; { Word variables }
    Matches, Match1, Match2, Self_Match, Total_Match : real ;

begin
    { Make sure compenstation values are valid }
    for A := 0 to 255 do
    begin
	B := X.Weight[ A ] ;
	C := B + X.Bias[ A ] ;
	if( C < B ) then
	begin
	    B := C ;
	end ;
	if( X.Compensation[ A ] > B ) then
	begin
	    X.Compensation[ A ] := B ;
	end ;
    end ;

    { Find length of longest string... }
    A := length( String1 ) ;
    B := length( String2 ) ;
    if( B < A ) then
    begin
	A := B ;
    end ;

    { Find forward match... }
    Matches := 0 ;
    Match1 := 0 ;
    Match2 := 0 ;
    for B := 1 to A do
    begin
	Match1 := Match1 + Matches ;
	D := ord( String1[ B ] ) ;
	if( String1[ B ] = String2[ B ] ) then
	begin
	    Match1 := Match1 + X.Weight[ D ] ;
	    Matches := Matches + X.Weight[ D ] ;
	end ;
    end ;

    { Find reverse match... }
    Matches := 0 ;
    for B := A downto 1 do
    begin
	Match2 := Match2 + Matches ;
	D := ord( String1[ B ] ) ;
	if( string1[ B ] = String2[ B ] ) then
	begin
	    Match2 := Match2 + X.Weight[ D ] + X.Bias[ D ] ;
	    Matches := Matches + X.Weight[ D ]  + X.Bias[ D ] ;
	end ;
    end ;

    { Determine compensation }
    Total_Match := Match1 + Match2 ;
    Match1 := 0 ;
    Match2 := 0 ;
    C := 0 ;
    D := 0 ;
    for A := 0 to 255 do
    begin
	for B := 1 to length( String1 ) do
	begin
	    if( String1[ B ] = char( A ) ) then
	    begin
		inc( C ) ;
	    end ;
	end ;
	for B := 1 to length( String2 ) do
	begin
	    if( String2[ B ] = char( A ) ) then
	    begin
		inc( D ) ;
	    end ;
	end ;
	if( C < 0 ) then
	begin
	    C := 0 ;
	end ;
	if( D < 0 ) then
	begin
	    D := 0 ;
	end ;
	C := C * X.Compensation[ A ] ;
	D := D * X.Compensation[ A ] ;
	Match1 := Match1 + C ;
	Match2 := Match2 + D ;
    end ; { for A := 0 to 255 }

    { Calculate total match... }
    Match1 := Match1 * ( length( String1 ) + 1 ) ;
    Match2 := Match2 * ( length( String2 ) + 1 ) ;
    Total_Match := Total_Match + Match1 + Match2 ;

    { Determine string1 match to itself }
    Matches := 0 ;
    Match1 := 0 ;
    Match2 := 0 ;
    for C := 1 to length( String1 ) do
    begin
	Match1 := Match1 + Matches ;
	D := ord( String1[ C ] ) ;
	Match1 := Match1 + X.Weight[ D ] ;
	Matches := Matches + X.Weight[ D ] ;
    end ;

    { Determine string1 match to itself (reverse)... }
    Matches := 0 ;
    for C := length( String1 ) downto 1 do
    begin
	Match2 := Match2 + Matches ;
	D := ord( string1[ C ] ) ;
	Match2 := Match2 + X.Weight[ D ] + X.Bias[ D ] ;
	Matches := Matches + X.Weight[ D ] + X.Bias[ D ] ;
    end ;
    Self_Match := Match1 + Match2 ;

    { Determine string2 match to itself }
    for C := 1 to length( String2 ) do
    begin
	Match1 := Match1 + Matches ;
	D := ord( String2[ C ] ) ;
	Match1 := Match1 + X.Weight[ D ] ;
	Matches := Matches + X.Weight[ D ] ;
    end ;

    { Determine string2 match to itself (reversed)... }
    for C := length( String2 ) downto 1 do
    begin
	Match2 := Match2 + Matches ;
	D := ord( string2[ C ] ) ;
	Match2 := Match2 + X.Weight[ D ] + X.Bias[ D ] ;
	Matches := Matches + X.Weight[ D ] + X.Bias[ D ] ;
    end ;
    Self_Match := Self_Match + Match1 + Match2 ;
    Compare_Strings := 2.0 * Total_Match / Self_Match ;
end ; { Compare_Strings }


function Extract( Original, Substring : string ; Start, Ending : integer ) : string ;
{ Extract all occurances of substring from the original string and return the
  result.  Only occurances between Start and Ending are extracted.  If
  Ending < 1, it  defaults to the entire length of the string. }

var Dummy : integer ;
    Work : String ;

begin
    Work := Original ;
    if( Length( Substring ) > 0 ) then
    begin
	if( Ending < 1 ) then
	begin
	    Ending := length( Original ) ;
        end ;
        Work := Original ;
        Dummy := 1 ;
        while( ( Dummy > 0 ) and ( Dummy < Ending ) ) do
        begin
            Dummy := instr( Start, Work, Substring ) ;
            if( ( Dummy > 0 ) and ( Dummy <= Ending ) ) then
            begin
                { Remove substring }
                Work := copy( Work, 1, Dummy - 1 ) +
                    copy( Work, Dummy + length( Substring ), 254 ) ;
                Ending := Ending - length( Substring ) ; { Adjust end }
            end ;
	end ;
    end ;
    Extract := Work ; { Return result }
end ;


function Decode_Hex_String( const Value : string ) : string ;
{ Extract original string from encoded hex string }

var C : byte ;
    Loop : integer ;
    Res : string ;

begin
    Set_Length( Res, length( Value ) div 2 ) ;
    for Loop := 1 to length( Res ) do
    begin
        C := ( pos( Res[ ( Loop - 1 ) * 2 + 1 ], '0123456789ABCDEF' ) - 1 ) or
            ( ( pos( Res[ ( Loop - 1 ) * 2 + 2 ], '0123456789ABCDEF' ) - 1 ) shl 4 ) ;
        Res[ Loop ] := char( C ) ;
    end ;
    Decode_Hex_String := Res ;
end ;


{$IFDEF DELPHI}
function Decode_Text( S : string ) : TStringList ;

var Count : integer ;
    List : TInteger_List ;
    Loop : integer ;
    Work : string ;

begin
    { Setup... }
    Result := TStringList.Create ;
    List := TInteger_List.Create ;

    { Do simple decodings... }
    Loop := pos( '\/', S ) ;
    while( Loop <> 0 ) do
    begin
        S := copy( S, 1, Loop - 1 ) + 'V' + copy( S, Loop + 2, length( S ) ) ;
        Loop := pos( '\/', S ) ;
    end ;

    Loop := pos( 'VV', S ) ;
    while( Loop <> 0 ) do
    begin
        S := copy( S, 1, Loop - 1 ) + 'W' + copy( S, Loop + 2, length( S ) ) ;
        Loop := pos( 'VV', S ) ;
    end ;

    for Loop := 1 to length( S ) do
    begin
        case S[ Loop ] of
            '1' : S[ Loop ] := 'L' ;
            '2' : S[ Loop ] := 'S' ;
            '3' : S[ Loop ] := 'E' ;
            '4' : S[ Loop ] := 'A' ;
            '5' : S[ Loop ] := 'S' ;
            '6' : List.Add( Loop ) ;
            '8' : S[ Loop ] := 'B' ;
            '9' : List.Add( Loop ) ;
            '0' : S[ Loop ] := 'O' ;
            '|' : List.Add( Loop ) ;
            '/' : S[ Loop ] := '7' ;
            '{' : S[ Loop ] := 'C' ;
            '[' : S[ Loop ] := 'C' ;
            '(' : S[ Loop ] := 'C' ;
            '$' : S[ Loop ] := 'S' ;
            '&' : S[ Loop ] := 'G' ;
            '@' : S[ Loop ] := 'A' ;
            '*' : S[ Loop ] := 'O' ;
        end ;
    end ;

    if( ( List.Count = 0 ) or ( List.Count > 31 ) ) then { No more to decode, or too much to decode }
    begin
        Result.Add( S ) ;
        List.Free ;
        exit ;
    end ;

    { Handle complex decoding... }
    Count := Bit_Values[ List.Count ] - 1 ; { Number of combinations }
    while( Count >= 0 ) do
    begin
        for Loop := 0 to List.Count - 1 do
        begin
            Work := S ;
            if( ( ( 1 shl Loop ) and 1 ) = 0 ) then
            begin
		case S[ List[ Loop ] ] of
                    '|' : Work[ List[ Loop ] ] := 'L' ;
                    '9' : Work[ List[ Loop ] ] := 'G' ;
                    '6' : Work[ List[ Loop ] ] := 'G' ;
                end ;
            end else
            begin
		case S[ List[ Loop ] ] of
                    '|' : Work[ List[ Loop ] ] := 'I' ;
                    '9' : Work[ List[ Loop ] ] := 'P' ;
                    '6' : Work[ List[ Loop ] ] := 'B' ;
                end ;
            end ;
            Result.Add( Work ) ;
        end ;
    end ;

    { Cleanup... }
    List.Free ;
end ; { Decode_Text }
{$ENDIF}


function Encode_Hex_String( const Value : string ) : string ; {UNTESTED}

var I : integer ;
    Loop : integer ;
    Res : string ;

begin
    Set_Length( Res, Length( Value ) * 2 ) ;
    for Loop := 1 to length( Value ) do
    begin
        I := FHex( ord( Value[ Loop ] ) ) ;
        move( I, Res[ ( Loop - 1 ) * 2 + 1 ], 2 ) ;
    end ;
    Encode_Hex_String := Res ;
end ;


function Formatted_Hex_String( const Value : string ;
    Blocking : integer ) : string ; {UNTESTED}

var Res, S, Work : string ;

begin
    if( Blocking < 1 ) then
    begin
        Blocking := 1 ;
    end ;
    S := Encode_Hex_String( Value ) ;
    Res := '' ;
    while( length( S ) > 0 ) do
    begin
        Work := Reverse_String( copy( S, 1, Blocking ) ) ;
        S := copy( S, Blocking + 1, length( S ) ) ;
        if( length( Res ) > 0 ) then
        begin
            Res := Res + ' ' ;
        end ;
        Res := Res + Work ;
    end ;
    Formatted_Hex_String := Res ;
end ;


function Print_Edit( Value : String ) : String ;
{ Return string with only printable characters in it }

var A : String ;
    B, C : Integer ;

begin
    A := '' ;
    for B := 1 to Length( Value ) do
    begin
        C := Ord( Value[ B ] ) ;
        C := C and 127 ;
        if( ( C < 127 ) and ( C > 31 ) ) then
	begin
	    A := A + Chr( C ) ;
	end ;
    end ;
    Print_Edit := A ; { Return value }
end ;


procedure FRight( var Z, C : string ; Position : Integer ) ;

begin
    if( Position < 1 ) then
    begin
	Position := 1 ;
    end ;
    if( Position > Length( C ) ) then
    begin
	Z := '' ;
    end else
    begin
	Z := Copy( C, Position, Length( C ) - Position + 1 ) ;
    end ;
end ;


function Repeating_String( Value : char ; Len : integer ) : string ; {UNTESTED}

var S : string ;

begin
    Set_Length( S, Len ) ;
    fillchar( S[ 1 ], length( S ), ord( Value ) ) ;
    Repeating_String := S ;
end ;


function Reverse_String( Value : string ) : string ; {UNTESTED}

var L, H : integer ;
    LC, HC : char ;

begin
    L := 1 ;
    H := length( Value ) ;
    while( L < H ) do
    begin
        LC := Value[ L ] ;
        HC := Value[ H ] ;
        Value[ L ] := HC ;
        Value[ H ] := LC ;
        inc( L ) ;
        dec( H ) ;
    end ;
    Reverse_String := Value ;
end ;


function Right( C : string ; Position : Integer ) : string ;

var A : string ;

begin
    Fright( A, C, Position ) ;
    Right := A ;
end ;


function Replace( A : string ; const Substring, Substitute : string ;
    Options : integer ) : string ;

var Dummy : integer ;

begin
    if( Substring = Substitute ) then { Nop }
    begin
        Replace := A ;
        exit ;
    end ;
    Dummy := pos( Substring, A ) ;
    while( Dummy > 0 ) do
    begin
        A := copy( A, 1, Dummy - 1 ) + Substitute + copy( A, Dummy + length( Substring ), length( A ) ) ;
        Dummy := Dummy + length( Substitute ) ;
        Dummy := instr( Dummy, A, Substring ) ;
    end ;
    Replace := A ;
end ;


function String_To_Boolean( S : string ) : boolean ;

begin
    String_To_Boolean := False ;
    if( length( S ) = 0 ) then
    begin
	exit ;
    end ;
    S := upcase( S[ 1 ] ) ;
    if( ( S = 'T' ) or ( S = '1' ) or ( S = 'Y' ) or ( S = 'X' ) ) then
    begin
	String_To_Boolean := True ;
    end ;
end ;


function Substitute( A : string ; const B : string ) : string ;
{ Translate A based on B.  Returns a string which is A, but with each character
  converted to characters in B based on ASCII values.  For example, if A[ 1 ]
  is "A", then A[ 1 ] is set to B[ 65 ]. }

var Dummy : integer ;
    I : integer ;

begin
    for Dummy := 1 to length( A ) do
    begin
        I := ord( A[ Dummy ] ) + 1 ;
	if( I <= length( B ) ) then
        begin
            A[ Dummy ] := B[ I ] ;
        end ;
    end ;
    Substitute := A ;
end ; { Substitute }


function Tab_To_Space( S : string ; Tab_Stops : integer ) : string ;

var Dummy, Loop : integer ;

begin
    for Loop := 1 to length( S ) do
    begin
        if( S[ Loop ] = HT ) then
        begin
            S[ Loop ] := ' ' ;
            Dummy := Loop + 1 ;
            while( ( Dummy mod Tab_Stops ) <> 0 ) do
            begin
                insert( ' ', S, Dummy ) ;
                inc( Dummy ) ;
            end ;
        end ;
    end ;
    Tab_To_Space := S ;
end ;


function Trim_Quotes( S : string ) : string ;

begin
    if( ( copy( S, 1, 1 ) = '"' ) or ( copy( S, 1, 1 ) = #39 ) ) then { A quoted string }
    begin
        if( copy( S, length( S ), 1 ) = copy( S, 1, 1 ) ) then
        begin
            S := copy( S, 1, length( S ) - 1 ) ; { Trim trailing quote }
        end ;
        S := copy( S, 2, length( S ) ) ; { Trim leading quote }
    end ;
    Trim_Quotes := S ;
end ;


function Remove_Termination( const S, T : string ) : string ;

begin
    if( copy( S, length( S ) - length( T ) + 1, length( T ) ) = T ) then
    begin
        Remove_Termination := copy( S, 1, length( S ) - length( T ) ) ;
    end else
    begin
        Remove_Termination := S ;
    end ;
end ;


function Ensure_Termination( const S, T : string ) : string ;

begin
    if( copy( S, length( S ) - length( T ) + 1, length( T ) ) = T ) then
    begin
        Ensure_Termination := S ;
    end else
    begin
        Ensure_Termination := S + T ;
    end ;
end ;


var Temp_Strings : array[ 0..9 ] of PASCII_String ;
{$IFDEF DELPHI}
    Temp_String_Current : integer = -1 ;
{$ELSE}
    Temp_String_Current : integer ;
{$ENDIF}


// procedure to free the temp strngs
procedure Dispose_Temp_Strings ;

var Loop : integer ;

begin
    for Loop := low( Temp_Strings ) to high( Temp_Strings ) do
    begin
        if Temp_Strings[ Loop ] <> nil then
        begin
            dispose( Temp_Strings[ Loop ], Done ) ;
        end ;
    end ;
end ;


function Temp_String( const S : string ) : PString ;

begin
    if( ( Temp_String_Current < 0 ) or ( Temp_String_Current > high( Temp_Strings ) ) ) then
    begin
        // dispose of temp stings if we hit the condition - should not happen 
        if( Temp_String_Current > high( Temp_Strings ) ) then
        begin
            Dispose_Temp_Strings ;
        end ;

        Temp_String_Current := 0 ;
        fillchar( Temp_Strings, sizeof( Temp_Strings ), 0 ) ;
    end ;
    if( Temp_Strings[ Temp_String_Current ] <> nil ) then
    begin
        dispose( Temp_Strings[ Temp_String_Current ], Done ) ;
    end ;
    Temp_Strings[ Temp_String_Current ] := new( PASCII_String, Init ) ;
    Temp_Strings[ Temp_String_Current ].Set_Memory_Type( MT_Heap ) ;
    Temp_Strings[ Temp_String_Current ]^.Set_String( S ) ;
    Temp_String := Temp_Strings[ Temp_String_Current ] ;
    inc( Temp_String_Current ) ;
    if( Temp_String_Current > high( Temp_Strings ) ) then
    begin
        Temp_String_Current := 0 ;
    end ;
end ;


{$IFDEF TEST}
procedure Test_String ;

    procedure Error ;

    begin
        writeln( 'Error' ) ;
        halt ;
    end ;

var Dummy : integer ;
    Temp : TString ;
    Test : TString ;
    Test1 : PString ;

begin
    Test.Init ;

{$IFNDEF DELPHI}
    Dummy :=
{$ENDIF}
        Test.Version ;
{$IFNDEF DELPHI}
    Dummy :=
{$ENDIF}
        Test.Facility ;
    for Dummy := -1 to 256 do
    begin
        if( Test.Class_Of( Dummy ) <> CC_Unknown ) then
        begin
            Error ;
        end ;
        if( Test.To_Universal( Dummy ) <> 0 ) then
        begin
            Error ;
        end ;
        if( Test.From_Universal( Dummy ) <> 0 ) then
        begin
            Error ;
        end ;
        if( Test.UpCase( Dummy ) <> Dummy ) then
        begin
            Error ;
        end ;
        if( Test.DownCase( Dummy ) <> Dummy ) then
        begin
            Error ;
        end ;
        if( Test.Normalize( Dummy ) <> Dummy ) then
        begin
            Error ;
        end ;
    end ;
    if( Test.Get_Store <> nil ) then
    begin
        Error ;
    end ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( not Test.Fixed ) then
    begin
        Error ;
    end ;
    if( Test.Max_Length <> 0 ) then
    begin
        Error ;
    end ;

    Test.Set_String( '' ) ;
    Test.Append_String( '' ) ;
    Test.Assign( nil ) ;
    Test.Set_String( '1234567890' ) ;
    if( Test.As_PChar <> nil ) then
    begin
        Error ;
    end ;
    Test.RSet_String( 'ABC' ) ;
    Test.Append_Char( 32 ) ;
    Test.Append_String( '*()' ) ;
    Test.At_Put( -1, 0 ) ;
    Test.At_Put( 32767, 0 ) ;
    if( Test.At( -1 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.At( 32767 ) <> 0 ) then
    begin
        Error ;
    end ;

    for Dummy := 1 to 9 do
    begin
        Test.At_Put( Dummy, 48 + Dummy ) ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Set_String( '1' ) ;

    Test1 := Test.EDIT(1);
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(2) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(4) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(8) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(16) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(32) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(64) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(128) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(256) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(512) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(1024) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(2048) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(4096) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(8192) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(16384) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(-1) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Edit( 0 ) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;

    Test.Append( nil ) ;
    Temp.Init ;
    Temp.Set_String( '123' ) ;
    Test.Append( @Temp ) ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;

    Test.Assign( nil ) ;
    Test.Assign( @Temp ) ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;

    Test.RAssign( nil ) ;
    Test.RAssign( @Temp ) ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;
    Temp.Done ;
    Test1 := Test.Copy( 1, 0 ) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Copy( -1, 0 ) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Copy( 100, 10 ) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := new( PASCII_String, Init ) ;
    Test1.Set_Memory_Type( MT_Heap ) ;
    Test1^.Set_String( 'e' ) ;
    if( Test.Instr( 0, Test1 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( -1, Test1 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( 10, Test1 ) <> 0 ) then
    begin
        Error ;
    end ;
    Test1^.Set_String( 'fg' ) ;
    if( Test.Instr( 0, Test1 ) <> 0 ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    if( Test.Instr( 1, nil ) <> 0 ) then
    begin
        Error ;
    end ;

    if( Test.Compare( nil, True ) <> EqualTo ) then
    begin
        Error ;
    end ;
    Temp.Init ;
    Temp.Set_String( '' ) ;
    if( Test.Compare( @Temp, False ) <> EqualTo ) then
    begin
        Error ;
    end ;
    if( Test.Compare( @Test, False ) <> EqualTo ) then
    begin
        Error ;
    end ;
    Temp.Done ;

    Test.Set_Length( 0 ) ;
    Test.Set_Length( -1 ) ;
    Test.Done ;

    Test.Init ;
    Test.Done ;
end ; { Test_String }


procedure Test_ASCII_String_Work ;

    procedure Error ;

    begin
        writeln( 'Error' ) ;
        halt ;
    end ;

var Dummy : integer ;
    Temp : TASCII_String ;
    Test : TASCII_String ;
    Test1 : PString ;
    Heap : PSHeap ;

begin
    Test.Init ;
    if( Test.Get_Store <> nil ) then
    begin
        Error ;
    end ;

{$IFNDEF DELPHI}
    Dummy :=
{$ENDIF}
        Test.Version ;
{$IFNDEF DELPHI}
    Dummy :=
{$ENDIF}
        Test.Facility ;
    for Dummy := 0 to 127 do
    begin
        if( Test.Class_Of( Dummy ) <> ASCII_CC[ Dummy ] ) then
        begin
            Error ;
        end ;
        if( Test.Class_Of( Dummy + 128 ) <> ASCII_CC[ Dummy ] ) then
        begin
            Error ;
        end ;
    end ;
    if( Test.Class_Of( -1 ) <> CC_Unknown ) then
    begin
        Error ;
    end ;
    if( Test.Class_Of( 256 ) <> CC_Unknown ) then
    begin
        Error ;
    end ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Fixed ) then
    begin
        Error ;
    end ;
    if( Test.Max_Length <> Max_Memory - 1 ) then
    begin
        Error ;
    end ;
    if( Test.To_Universal( 1024 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.To_Universal( 32 ) <> 32 ) then
    begin
        Error ;
    end ;
    if( Test.From_Universal( 1024 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.From_Universal( 32 ) <> 32 ) then
    begin
        Error ;
    end ;

    Test.Set_String( '' ) ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Fixed ) then
    begin
        Error ;
    end ;

    Test.Append_String( '' ) ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Fixed ) then
    begin
        Error ;
    end ;

    Test.Assign( nil ) ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Fixed ) then
    begin
        Error ;
    end ;

    Test.Set_String( '1234567890' ) ;
    Test1 := Test.Right( 3 ) ;
    if( Test1^.Text <> '34567890' ) then
    begin
        Error ;
    end ;
    if( strpas( Test1^.As_PChar ) <> '34567890' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 8 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    if( Test.Text <> '1234567890' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '1234567890' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 10 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    for Dummy := 1 to 10 do
    begin
        if( char( Test.At( Dummy ) ) <> copy( '1234567890', Dummy, 1 ) ) then
        begin
            Error ;
        end ;
    end ;

    Test.RSet_String( 'ABC' ) ;
    if( Test.Text <> '1234567ABC' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '1234567ABC' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 10 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Append_Char( 32 ) ;
    if( Test.Text <> '1234567ABC ' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '1234567ABC ' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 11 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Append_String( '*()' ) ;
    if( Test.Text <> '1234567ABC *()' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '1234567ABC *()' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 14 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    Test.At_Put( -1, 0 ) ;
    Test.At_Put( 32767, 0 ) ;
    if( Test.At( -1 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.At( 32767 ) <> 0 ) then
    begin
        Error ;
    end ;

    for Dummy := 1 to 9 do
    begin
        Test.At_Put( Dummy, 48 + Dummy ) ;
    end ;
    if( Test.Text <> '123456789C *()' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '123456789C *()' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 14 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Set_Length( 10 ) ;
    if( Test.Text <> '123456789C' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '123456789C' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 10 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Set_Length( 15 ) ;
    if( Test.Text <> '123456789C'#0#0#0#0#0 ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '123456789C' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 15 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    if( Test.Upcase( Test.Downcase( ord( 'A' ) ) ) <> ord( 'A' ) ) then
    begin
        Error ;
    end ;
    if( Test.Upcase( 32 ) <> 32 ) then
    begin
        exit ;
    end ;
    if( Test.Downcase( 32 ) <> 32 ) then
    begin
        exit ;
    end ;

    Test.Set_String( '1' ) ;
    Test.RSet_String( 'ABC' ) ;
    if( Test.Text <> 'C' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> 'C' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 1 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Set_String( '  Hi there "Hi", []{}       () ' ) ;
    Test1 := Test.EDIT(1);
    if( Test1^.Text <> '  Hi there "Hi", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(2) ;
    if( Test1^.Text <> 'Hithere"Hi",[]{}()' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(4) ;
    if( Test1^.Text <> '  Hi there "Hi", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(8) ;
    if( Test1^.Text <> 'Hi there "Hi", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(16) ;
    if( Test1^.Text <> ' Hi there "Hi", []{} () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(32) ;
    if( Test1^.Text <> '  HI THERE "HI", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(64) ;
    if( Test1^.Text <> '  Hi there "Hi", (){}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(128) ;
    if( Test1^.Text <> '  Hi there "Hi", []{}       ()' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(256) ;
    if( Test1^.Text <> '  Hi there "Hi", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(512) ;
    if( Test1^.Text <> '  hi there "hi", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(1024) ;
    if( Test1^.Text <> '  Hi there "Hi", {}{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(2048) ;
    if( Test1^.Text <> '  Hi there "Hi", []{}       [] ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(4096) ;
    if( Test1^.Text <> '  Hi there "Hi", []{}       {} ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(8192) ;
    if( Test1^.Text <> '  Hi there "Hi", []()       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(16384) ;
    if( Test1^.Text <> '  Hi there "Hi", [][]       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(-1) ;
    if( Test1^.Text <> 'HITHERE"Hi",()()[]' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test.Set_String( '   Hi there "Hi", []{}       ()" ' ) ;
    Test1 := Test.EDIT(128) ;
    if( Test1^.Text <> '   Hi there "Hi", []{}       ()"' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test.Set_String( '   Hi there "H'#39'i", []{}       ()" ' ) ;
    Test1 := Test.EDIT(128) ;
    if( Test1^.Text <> '   Hi there "H'#39'i", []{}       ()"' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test.Set_String( '   Hi there "H'#39'i", []{}       ()" ' ) ;
    Test1 := Test.EDIT(128 or 256) ;
    if( Test1^.Text <> '   Hi there "H'#39'i", []{}       ()" ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test.Set_String( '   Hi there "H'#39'i", []{}       ()" ' ) ;
    Test1 := Test.EDIT(128 or 256) ;
    if( Test1^.Text <> '   Hi there "H'#39'i", []{}       ()" ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test.Set_String( '   Hi there "Hi", []{}       ()"'#4 ) ;
    Test1 := Test.EDIT(4) ;
    if( Test1^.Text <> '   Hi there "Hi", []{}       ()"' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;

    Test1 := Test.Edit( 0 ) ;
    if( Test.Text <> Test1^.Text ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;

    Test.Append( nil ) ;
    Temp.Init ;
    Temp.Set_String( '123' ) ;
    Test.Append( @Temp ) ;
    if( Test.Text <> '   Hi there "Hi", []{}       ()"'#4'123' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '   Hi there "Hi", []{}       ()"'#4'123' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 36 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Assign( nil ) ;
    Test.Assign( @Temp ) ;
    if( Test.Text <> '123' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '123' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 3 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    Test.Append_Char( 32 ) ;
    if( Test.Text <> '123 ' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '123 ' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    if( Temp.Text <> '123' ) then
    begin
        Error ;
    end ;
    if( strpas( Temp.As_PChar ) <> '123' ) then
    begin
        Error ;
    end ;
    if( Temp.Length <> 3 ) then
    begin
        Error ;
    end ;
    if( Temp.Size <> Temp.Length ) then
    begin
        Error ;
    end ;

    Temp.Set_String( 'AB' ) ;
    Test.RAssign( nil ) ;
    Test.RAssign( @Temp ) ;
    if( Test.Text <> '12AB' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '12AB' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    if( Temp.Text <> 'AB' ) then
    begin
        Error ;
    end ;
    if( strpas( Temp.As_PChar ) <> 'AB' ) then
    begin
        Error ;
    end ;
    if( Temp.Length <> 2 ) then
    begin
        Error ;
    end ;
    if( Temp.Size <> Temp.Length ) then
    begin
        Error ;
    end ;
    Temp.Set_String( '&*()' ) ;
    Test.RAssign( @Temp ) ;
    if( Test.Text <> '&*()' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '&*()' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    if( Temp.Text <> '&*()' ) then
    begin
        Error ;
    end ;
    if( strpas( Temp.As_PChar ) <> '&*()' ) then
    begin
        Error ;
    end ;
    if( Temp.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Temp.Size <> Temp.Length ) then
    begin
        Error ;
    end ;
    Temp.Set_String( 'abcdefg' ) ;
    Test.RAssign( @Temp ) ;
    if( Test.Text <> 'defg' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> 'defg' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    if( Temp.Text <> 'abcdefg' ) then
    begin
        Error ;
    end ;
    if( strpas( Temp.As_PChar ) <> 'abcdefg' ) then
    begin
        Error ;
    end ;
    if( Temp.Length <> 7 ) then
    begin
        Error ;
    end ;
    if( Temp.Size <> Temp.Length ) then
    begin
        Error ;
    end ;
    Temp.Done ;
    Test1 := Test.Copy( 1, 0 ) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Copy( 100, 10 ) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Copy( 2, 100 ) ;
    if( Test1^.Text <> 'efg' ) then
    begin
        Error ;
    end ;
    if( strpas( Test1^.As_PChar ) <> 'efg' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 3 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Copy( 1, 2 ) ;
    if( Test1^.Text <> 'de' ) then
    begin
        Error ;
    end ;
    if( strpas( Test1^.As_PChar ) <> 'de' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 2 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Copy( 1, 4 ) ;
    if( Test1^.Text <> 'defg' ) then
    begin
        Error ;
    end ;
    if( strpas( Test1^.As_PChar ) <> 'defg' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    if( Test.Instr( 0, Temp_String( 'e' ) ) <> 2 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( -1, Temp_String( 'e' ) ) <> 2 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( 10, Temp_String( 'e' ) ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( 0, Temp_String( 'fg' ) ) <> 3 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( 1, Temp_String( 'abcdefg' ) ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( 1, nil ) <> 0 ) then
    begin
        Error ;
    end ;

    if( Test.Compare( nil, True ) <> GreaterThan ) then
    begin
        Error ;
    end ;
    if( Test.Compare( Temp_String( 'DEFG' ), True ) <> GreaterThan ) then
    begin
        Error ;
    end ;
    if( Test.Compare( Temp_String( 'DEFG' ), False ) <> EqualTo ) then
    begin
        Error ;
    end ;
    if( Test.Compare( Temp_String( 'defgh' ), False ) <> LessThan ) then
    begin
        Error ;
    end ;
    if( Test.Compare( Temp_String( 'def' ), False ) <> GreaterThan ) then
    begin
        Error ;
    end ;
    if( Test.Compare( Temp_String( 'e' ), False ) <> LessThan ) then
    begin
        Error ;
    end ;
    Test.Set_String( '' ) ;
    if( Test.Compare( Temp_String( '' ), False ) <> EqualTo ) then
    begin
        Error ;
    end ;
    if( Test.Compare( @Test, False ) <> EqualTo ) then
    begin
        Error ;
    end ;

    Test.Set_Length( 0 ) ;
    Test.Set_Length( -1 ) ;
    Test.Done ;

    Test.Init ;
    Heap := new( PSHeap, Init ) ;
    Heap.Set_Memory_Type( MT_Heap ) ;
    Test.Set_Store( Heap ) ;
    Test.Set_Store( Application_Manager^.Get_Heap ) ;
    Test.Set_Store( Application_Manager^.Get_Heap ) ;
    Test.Done ;
end ; { Test_ASCII_String_Work }


procedure Test_ASCII_String ;

begin
    Test_Generic := True ;
    Test_ASCII_String_Work ;
    Test_Generic := False ;
    Test_ASCII_String_Work ;
end ;


procedure Test_ASCII7_String ;

    procedure Error ;

    begin
        writeln( 'Error' ) ;
        halt ;
    end ;

var Dummy : integer ;
    Temp : TASCII7_String ;
    Test : TASCII7_String ;
    Test1 : PString ;
    Heap : PSHeap ;

begin
    Test.Init ;
    if( Test.Get_Store <> nil ) then
    begin
        Error ;
    end ;

{$IFNDEF DELPHI}
    Dummy :=
{$ENDIF}
        Test.Version ;
{$IFNDEF DELPHI}
    Dummy :=
{$ENDIF}
        Test.Facility ;
    for Dummy := 0 to 127 do
    begin
        if( Test.Class_Of( Dummy ) <> ASCII_CC[ Dummy ] ) then
        begin
            Error ;
        end ;
        if( Test.Class_Of( Dummy + 128 ) <> CC_Unknown ) then
        begin
            Error ;
        end ;
    end ;
    if( Test.Class_Of( -1 ) <> CC_Unknown ) then
    begin
        Error ;
    end ;
    if( Test.Class_Of( 256 ) <> CC_Unknown ) then
    begin
        Error ;
    end ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Fixed ) then
    begin
        Error ;
    end ;
    if( Test.Max_Length <> Max_Memory - 1 ) then
    begin
        Error ;
    end ;
    if( Test.To_Universal( 1024 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.To_Universal( 32 ) <> 32 ) then
    begin
        Error ;
    end ;
    if( Test.From_Universal( 1024 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.From_Universal( 32 ) <> 32 ) then
    begin
        Error ;
    end ;

    Test.Set_String( '' ) ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Fixed ) then
    begin
        Error ;
    end ;

    Test.Append_String( '' ) ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Fixed ) then
    begin
        Error ;
    end ;

    Test.Assign( nil ) ;
    if( Test.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Fixed ) then
    begin
        Error ;
    end ;

    Test.Set_String( '1234567890' ) ;
    Test1 := Test.Right( 3 ) ;
    if( Test1^.Text <> '34567890' ) then
    begin
        Error ;
    end ;
    if( strpas( Test1^.As_PChar ) <> '34567890' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 8 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    if( Test.Text <> '1234567890' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '1234567890' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 10 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    for Dummy := 1 to 10 do
    begin
        if( char( Test.At( Dummy ) ) <> copy( '1234567890', Dummy, 1 ) ) then
        begin
            Error ;
        end ;
    end ;

    Test.RSet_String( 'ABC' ) ;
    if( Test.Text <> '1234567ABC' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '1234567ABC' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 10 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Append_Char( 32 ) ;
    if( Test.Text <> '1234567ABC ' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '1234567ABC ' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 11 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Append_String( '*()' ) ;
    if( Test.Text <> '1234567ABC *()' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '1234567ABC *()' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 14 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    Test.At_Put( -1, 0 ) ;
    Test.At_Put( 32767, 0 ) ;
    if( Test.At( -1 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.At( 32767 ) <> 0 ) then
    begin
        Error ;
    end ;

    for Dummy := 1 to 9 do
    begin
        Test.At_Put( Dummy, 48 + Dummy ) ;
    end ;
    if( Test.Text <> '123456789C *()' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '123456789C *()' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 14 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Set_Length( 10 ) ;
    if( Test.Text <> '123456789C' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '123456789C' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 10 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Set_Length( 15 ) ;
    if( Test.Text <> '123456789C'#0#0#0#0#0 ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '123456789C' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 15 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    if( Test.Upcase( Test.Downcase( ord( 'A' ) ) ) <> ord( 'A' ) ) then
    begin
        Error ;
    end ;
    if( Test.Upcase( 32 ) <> 32 ) then
    begin
        exit ;
    end ;
    if( Test.Downcase( 32 ) <> 32 ) then
    begin
        exit ;
    end ;

    Test.Set_String( '1' ) ;
    Test.RSet_String( 'ABC' ) ;
    if( Test.Text <> 'C' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> 'C' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 1 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Set_String( '  Hi there "Hi", []{}       () ' ) ;
    Test1 := Test.EDIT(1);
    if( Test1^.Text <> '  Hi there "Hi", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(2) ;
    if( Test1^.Text <> 'Hithere"Hi",[]{}()' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(4) ;
    if( Test1^.Text <> '  Hi there "Hi", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(8) ;
    if( Test1^.Text <> 'Hi there "Hi", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(16) ;
    if( Test1^.Text <> ' Hi there "Hi", []{} () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(32) ;
    if( Test1^.Text <> '  HI THERE "HI", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(64) ;
    if( Test1^.Text <> '  Hi there "Hi", (){}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(128) ;
    if( Test1^.Text <> '  Hi there "Hi", []{}       ()' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(256) ;
    if( Test1^.Text <> '  Hi there "Hi", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(512) ;
    if( Test1^.Text <> '  hi there "hi", []{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(1024) ;
    if( Test1^.Text <> '  Hi there "Hi", {}{}       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(2048) ;
    if( Test1^.Text <> '  Hi there "Hi", []{}       [] ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(4096) ;
    if( Test1^.Text <> '  Hi there "Hi", []{}       {} ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(8192) ;
    if( Test1^.Text <> '  Hi there "Hi", []()       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(16384) ;
    if( Test1^.Text <> '  Hi there "Hi", [][]       () ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.EDIT(-1) ;
    if( Test1^.Text <> 'HITHERE"Hi",()()[]' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test.Set_String( '   Hi there "Hi", []{}       ()" ' ) ;
    Test1 := Test.EDIT(128) ;
    if( Test1^.Text <> '   Hi there "Hi", []{}       ()"' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test.Set_String( '   Hi there "H'#39'i", []{}       ()" ' ) ;
    Test1 := Test.EDIT(128) ;
    if( Test1^.Text <> '   Hi there "H'#39'i", []{}       ()"' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test.Set_String( '   Hi there "H'#39'i", []{}       ()" ' ) ;
    Test1 := Test.EDIT(128 or 256) ;
    if( Test1^.Text <> '   Hi there "H'#39'i", []{}       ()" ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test.Set_String( '   Hi there "H'#39'i", []{}       ()" ' ) ;
    Test1 := Test.EDIT(128 or 256) ;
    if( Test1^.Text <> '   Hi there "H'#39'i", []{}       ()" ' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test.Set_String( '   Hi there "Hi", []{}       ()"'#4 ) ;
    Test1 := Test.EDIT(4) ;
    if( Test1^.Text <> '   Hi there "Hi", []{}       ()"' ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;

    Test1 := Test.Edit( 0 ) ;
    if( Test.Text <> Test1^.Text ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;

    Test.Append( nil ) ;
    Temp.Init ;
    Temp.Set_String( '123' ) ;
    Test.Append( @Temp ) ;
    if( Test.Text <> '   Hi there "Hi", []{}       ()"'#4'123' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '   Hi there "Hi", []{}       ()"'#4'123' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 36 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;

    Test.Assign( nil ) ;
    Test.Assign( @Temp ) ;
    if( Test.Text <> '123' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '123' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 3 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    Test.Append_Char( 32 ) ;
    if( Test.Text <> '123 ' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '123 ' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    if( Temp.Text <> '123' ) then
    begin
        Error ;
    end ;
    if( strpas( Temp.As_PChar ) <> '123' ) then
    begin
        Error ;
    end ;
    if( Temp.Length <> 3 ) then
    begin
        Error ;
    end ;
    if( Temp.Size <> Temp.Length ) then
    begin
        Error ;
    end ;

    Temp.Set_String( 'AB' ) ;
    Test.RAssign( nil ) ;
    Test.RAssign( @Temp ) ;
    if( Test.Text <> '12AB' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '12AB' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    if( Temp.Text <> 'AB' ) then
    begin
        Error ;
    end ;
    if( strpas( Temp.As_PChar ) <> 'AB' ) then
    begin
        Error ;
    end ;
    if( Temp.Length <> 2 ) then
    begin
        Error ;
    end ;
    if( Temp.Size <> Temp.Length ) then
    begin
        Error ;
    end ;
    Temp.Set_String( '&*()' ) ;
    Test.RAssign( @Temp ) ;
    if( Test.Text <> '&*()' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> '&*()' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    if( Temp.Text <> '&*()' ) then
    begin
        Error ;
    end ;
    if( strpas( Temp.As_PChar ) <> '&*()' ) then
    begin
        Error ;
    end ;
    if( Temp.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Temp.Size <> Temp.Length ) then
    begin
        Error ;
    end ;
    Temp.Set_String( 'abcdefg' ) ;
    Test.RAssign( @Temp ) ;
    if( Test.Text <> 'defg' ) then
    begin
        Error ;
    end ;
    if( strpas( Test.As_PChar ) <> 'defg' ) then
    begin
        Error ;
    end ;
    if( Test.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Test.Size <> Test.Length ) then
    begin
        Error ;
    end ;
    if( Temp.Text <> 'abcdefg' ) then
    begin
        Error ;
    end ;
    if( strpas( Temp.As_PChar ) <> 'abcdefg' ) then
    begin
        Error ;
    end ;
    if( Temp.Length <> 7 ) then
    begin
        Error ;
    end ;
    if( Temp.Size <> Temp.Length ) then
    begin
        Error ;
    end ;
    Temp.Done ;
    Test1 := Test.Copy( 1, 0 ) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Copy( 100, 10 ) ;
    if( Test1^.Text <> '' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 0 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Copy( 2, 100 ) ;
    if( Test1^.Text <> 'efg' ) then
    begin
        Error ;
    end ;
    if( strpas( Test1^.As_PChar ) <> 'efg' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 3 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Copy( 1, 2 ) ;
    if( Test1^.Text <> 'de' ) then
    begin
        Error ;
    end ;
    if( strpas( Test1^.As_PChar ) <> 'de' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 2 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    Test1 := Test.Copy( 1, 4 ) ;
    if( Test1^.Text <> 'defg' ) then
    begin
        Error ;
    end ;
    if( strpas( Test1^.As_PChar ) <> 'defg' ) then
    begin
        Error ;
    end ;
    if( Test1^.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( Test1^.Size <> Test1^.Length ) then
    begin
        Error ;
    end ;
    dispose( Test1, Done ) ;
    if( Test.Instr( 0, Temp_String( 'e' ) ) <> 2 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( -1, Temp_String( 'e' ) ) <> 2 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( 10, Temp_String( 'e' ) ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( 0, Temp_String( 'fg' ) ) <> 3 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( 1, Temp_String( 'abcdefg' ) ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Instr( 1, nil ) <> 0 ) then
    begin
        Error ;
    end ;

    if( Test.Compare( nil, True ) <> GreaterThan ) then
    begin
        Error ;
    end ;
    if( Test.Compare( Temp_String( 'DEFG' ), True ) <> GreaterThan ) then
    begin
        Error ;
    end ;
    if( Test.Compare( Temp_String( 'DEFG' ), False ) <> EqualTo ) then
    begin
        Error ;
    end ;
    if( Test.Compare( Temp_String( 'defgh' ), False ) <> LessThan ) then
    begin
        Error ;
    end ;
    if( Test.Compare( Temp_String( 'def' ), False ) <> GreaterThan ) then
    begin
        Error ;
    end ;
    if( Test.Compare( Temp_String( 'e' ), False ) <> LessThan ) then
    begin
        Error ;
    end ;
    Test.Set_String( '' ) ;
    if( Test.Compare( Temp_String( '' ), False ) <> EqualTo ) then
    begin
        Error ;
    end ;
    if( Test.Compare( @Test, False ) <> EqualTo ) then
    begin
        Error ;
    end ;

    Test.Set_Length( 0 ) ;
    Test.Set_Length( -1 ) ;
    Test.Done ;

    Test.Init ;
    Heap := new( PSHeap, Init ) ;
    Heap.Set_Memory_Type( MT_Heap ) ;
    Test.Set_Store( Heap ) ;
    Test.Set_Store( Application_Manager^.Get_Heap ) ;
    Test.Set_Store( Application_Manager^.Get_Heap ) ;
    Test.RSet_String( '' ) ;
    Test.Done ;
end ; { Test_ASCII7_String }


procedure Test_Unit ;

var Comparison : RComparison ;
    Loop : integer ;

begin
    { Replace test... }
    if( Replace( '', '$', 'A', 0 ) <> '' ) then
    begin
        writeln( 'Replace failure' ) ;
        halt ;
    end ;
    if( Replace( '1$1', '$', 'A', 0 ) <> '1A1' ) then
    begin
        writeln( 'Replace failure' ) ;
        halt ;
    end ;
    if( Replace( '$1', '$', 'A', 0 ) <> 'A1' ) then
    begin
        writeln( 'Replace failure' ) ;
        halt ;
    end ;
    if( Replace( '1$', '$', 'A', 0 ) <> '1A' ) then
    begin
        writeln( 'Replace failure' ) ;
        halt ;
    end ;
    if( Replace( '1$2$3$', '$', 'ABC', 0 ) <> '1ABC2ABC3ABC' ) then
    begin
        writeln( 'Replace failure' ) ;
        halt ;
    end ;

    { Boolean_To_String test... }
    if( Boolean_To_String( True ) <> 'true' ) then
    begin
        writeln( 'Boolean_To_String failure' ) ;
        halt ;
    end ;
    if( Boolean_To_String( False ) <> 'false' ) then
    begin
        writeln( 'Boolean_To_String failure' ) ;
        halt ;
    end ;

    { Right test... }
    if( Right( '123456789', 4 ) <> '456789' ) then
    begin
        writeln( 'Right failure' ) ;
        halt ;
    end ;
    if( Right( '123456789', 14 ) <> '' ) then
    begin
        writeln( 'Right failure' ) ;
        halt ;
    end ;
    if( Right( '123456789', 0 ) <> '123456789' ) then
    begin
        writeln( 'Right failure' ) ;
        halt ;
    end ;

    { Pointer_To_String test... }
    if( Pointer_To_String( nil ) <> 'nil' ) then
    begin
        writeln( 'Pointer_To_String( nil ) failed' ) ;
        halt ;
    end ;
    if( Pointer_To_String( pointer( $10 ) ) <> '16 ($10)' ) then
    begin
        writeln( 'Pointer_To_String failed' ) ;
        halt ;
    end ;

    { String_To_Boolean test... }
    if( String_To_Boolean( 'F' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;
    if( String_To_Boolean( 'f' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;
    if( String_To_Boolean( '' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;
    if( String_To_Boolean( 'Z' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;
    if( not String_To_Boolean( 'T' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;
    if( not String_To_Boolean( 't' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;
    if( not String_To_Boolean( 'Y' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;
    if( not String_To_Boolean( 'y' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;
    if( not String_To_Boolean( '1' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;
    if( not String_To_Boolean( 'x' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;
    if( not String_To_Boolean( 'X' ) ) then
    begin
        writeln( 'String_To_Boolean failed' ) ;
        halt ;
    end ;

    { Extract test... }
    if( Extract( 'ABCDEFG', '', 1, 1 ) <> 'ABCDEFG' ) then
    begin
        writeln( 'Extract failed' ) ;
        halt ;
    end ;
    if( Extract( 'ABCDEFG', 'B', 1, 1 ) <> 'ABCDEFG' ) then
    begin
        writeln( 'Extract failed' ) ;
        halt ;
    end ;
    if( Extract( 'ABCDEFG', 'B', 1, -1 ) <> 'ACDEFG' ) then
    begin
        writeln( 'Extract failed' ) ;
        halt ;
    end ;
    if( Extract( 'ABCDEFG', 'B', 1, 4 ) <> 'ACDEFG' ) then
    begin
        writeln( 'Extract failed' ) ;
        halt ;
    end ;

    { Print_Edit test... }
    if( Print_Edit( '' ) <> '' ) then
    begin
        writeln( 'Print_Edit failed' ) ;
        halt ;
    end ;
    if( Print_Edit( ' '#2 ) <> ' ' ) then
    begin
        writeln( 'Print_Edit failed' ) ;
        halt ;
    end ;

    { Trim_Quotes test... }
    if( Trim_Quotes( 'ABC' ) <> 'ABC' ) then
    begin
        writeln( 'Trim_Quotes failed' ) ;
        halt ;
    end ;
    if( Trim_Quotes( '"ABC' ) <> 'ABC' ) then
    begin
        writeln( 'Trim_Quotes failed' ) ;
        halt ;
    end ;
    if( Trim_Quotes( '"ABC"' ) <> 'ABC' ) then
    begin
        writeln( 'Trim_Quotes failed' ) ;
        halt ;
    end ;

    { Remove_Termination test... }
    if( Remove_Termination( 'A=', '=' ) <> 'A' ) then
    begin
        writeln( 'Remove_Termination failed' ) ;
        halt ;
    end ;
    if( Remove_Termination( 'A=', '-' ) <> 'A=' ) then
    begin
        writeln( 'Remove_Termination failed' ) ;
        halt ;
    end ;

    { Ensure_Termination test... }
    if( Ensure_Termination( 'A=', '=' ) <> 'A=' ) then
    begin
        writeln( 'Remove_Termination failed' ) ;
        halt ;
    end ;
    if( Ensure_Termination( 'A=', '-' ) <> 'A=-' ) then
    begin
        writeln( 'Remove_Termination failed' ) ;
        halt ;
    end ;

    { Compare_Strings test... }
    for Loop := 0 to 255 do
    begin
        Comparison.Bias[ Loop ] := 1 ;
        Comparison.Compensation[ Loop ] := 1 ;
        Comparison.Weight[ Loop ] := 1 ;
    end ;
    if( int( Compare_Strings( 'ABCD', 'ABCD', Comparison ) * 1000 ) <> 89529.0 ) then
    begin
        writeln( 'Compare_Strings failed' ) ;
        halt ;
    end ;
    if( int( Compare_Strings( 'ABCD', 'ABC', Comparison ) * 1000 ) <> 90192.0 ) then
    begin
        writeln( 'Compare_Strings failed' ) ;
        halt ;
    end ;
    if( int( Compare_Strings( 'ABCD', 'EFGH', Comparison ) * 1000 ) <> 88235.0 ) then
    begin
        writeln( 'Compare_Strings failed' ) ;
        halt ;
    end ;
    for Loop := 0 to 255 do
    begin
        Comparison.Compensation[ Loop ] := 100 ;
    end ;
    if( int( Compare_Strings( 'EFGH', 'ABCD', Comparison ) * 1000 ) <> 88235.0 ) then
    begin
        writeln( 'Compare_Strings failed' ) ;
        halt ;
    end ;
    for Loop := 0 to 255 do
    begin
        Comparison.Bias[ Loop ] := -100 ;
    end ;
    if( int( Compare_Strings( 'EFGH', 'ABCD', Comparison ) * 1000 ) <> 650.0 ) then
    begin
        writeln( 'Compare_Strings failed' ) ;
        halt ;
    end ;

    { Substitute test... }
    if( Substitute( ' !"', '12345678901234567890123456789012ABC' ) <> 'ABC' ) then
    begin
        writeln( 'Substitute failed' ) ;
        halt ;
    end ;
    if( Substitute( ' !"', '' ) <> ' !"' ) then
    begin
        writeln( 'Substitute failed' ) ;
        halt ;
    end ;

    { Base string class test }
    Test_String ;

    { ASCII String test }
    Test_ASCII_String ;

    { TASCII7 String test }
    Test_ASCII7_String ;
end ; { Test_Unit }
{$ENDIF}


function BCD( C : char ) : integer ;

begin
    case C of
        '-' : BCD := 13 ; { other valid = 11 }
        '+' : BCD := 12 ; { other valid = 10,14,15 }
        '0'..'9' : BCD := ord( C ) - 48 ;
        else BCD := 15 ;
    end ;
end ;


function String_To_BCD( S : string ) : string ;

var Dummy : integer ;
    _Res : string ;

begin
    { Setup }
    _Res := '' ;

    { Convert }
    for Dummy := 1 to length( S ) do
    begin
        _Res := _Res + chr( BCD( S[ Dummy ] ) ) ;
    end ;
    String_To_BCD := _Res ;
end ;


function String_To_Packed_BCD( S : string ) : string ;

var Current, Dummy : integer ;
    _Res : string ;

begin
    { Setup... }
    _Res := '' ;
    Current := 0 ;

    { Make sure string is even bytes in length... }
    if( odd( length( S ) ) ) then
    begin
        if( ( copy( S, 1, 1 ) = '-' ) or ( copy( S, 1, 1 ) = '+' ) ) then
        begin
            S := copy( S, 1, 1 ) + '0' + copy( S, 2, length( S ) ) ;
        end else
        begin
            S := '0' + S ;
        end ;
    end ;

    { Convert }
    for Dummy := 1 to length( S ) do
    begin
        if( odd( Dummy ) ) then
        begin
            Current := BCD( S[ Dummy ] ) shl 4 ;
        end else
        begin
            Current := Current or BCD( S[ Dummy ] ) ;
            _Res := _Res + chr( Current ) ;
        end ;
    end ;
    String_To_Packed_BCD := _Res ;
end ; { String_To_Packed_BCD }


{$IFNDEF DELPHI}
begin
    Temp_String_Current := -1 ;

{$ELSE}

initialization

finalization
    // dispose of temp stings 
    Dispose_Temp_Strings ;

{$ENDIF}

end.
