/* 
              Program Name : XLATE
              Package Name : LB
              Purpose      : Translate filename based on wildcard spec.
              Institution  : Conroy & Conroy Co.
              Date Written : 04-Apr-1992
              Written By   : Alan Conroy
              Version      : 2.0

	      Copyright (C) 1992 by Alan Conroy.  Released to the public domain.

		TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *********************************************************
        *                                                       *
        *        M O D I F I C A T I O N   H I S T O R Y        *
        *                                                       *
        *     DATE      BY             REASON                   *
        *                                                       *
        *********************************************************

        *********************************************************
        *                                                       *
        *           P R O G R A M   P U R P O S E               *
        *                                                       *
        *********************************************************

	  These functions handle matching wildcards.

        *********************************************************
        *                                                       *
        *            F I L E   D E S C R I P T I O N S          *
        *                                                       *
        *********************************************************

 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <dir.h>

#include "xlate.h"
#include "standard.h"

static char *Resolve_Wildcard( char *Passed_Ref_Spec, char *Passed_In_Spec,
    char *Passed_Out_Spec )
{

int Dummy ;
char Ref_Spec[ 256 ], In_Spec[ 256 ] , Out_Spec[ 256 ], Result[ 256 ] ;
char Ref[ 256 ], Outp[ 256 ] ; /* Work variable */

    /* Make local copies of passed strings */
    strcpy( Ref_Spec, Passed_Ref_Spec ) ;
    strcpy( In_Spec, Passed_In_Spec ) ;
    strcpy( Out_Spec, Passed_Out_Spec ) ;

    Result[ 0 ] = 0 ;

    if( strcmp( Ref_Spec, "\\..." ) == 0 || strcmp( Ref_Spec, "\\...\\" ) == 0 )
    {
	strcpy( Ref_Spec, "\\*" ) ;
    } ; /* Handle special directory wildcard */

    Dummy = Instr( 1, Ref_Spec, "*" ) ;
    if( Dummy == 0 )
    {
	Dummy = strlen( Ref_Spec ) + 1 ;
    } ;
    strcpy( Ref, Left( Ref_Spec, Dummy - 1 ) ) ;
    strcpy( Ref_Spec, &Ref_Spec[ Dummy - 1 ] ) ;
    Dummy = Instr( 1, Out_Spec, "*" ) ;
    if( Dummy == 0 )
    {
	Dummy = strlen( Out_Spec ) + 1 ;
    } ;
    strcpy( Outp, Left( Out_Spec, Dummy - 1 ) ) ;
    strcpy( Out_Spec, &Out_Spec[ Dummy - 1 ] ) ;
    Dummy = 0 ;
    while( Dummy < strlen( Outp ) )
    {
	if( Outp[ Dummy ] == '?' )
	{
	    strcat( Result, Left( In_Spec, 1 ) ) ;
	}
	else
	{
	    if( Dummy >= strlen( Ref ) )
	    {
		strcat( Result, Mid( Outp, Dummy + 1, 1 ) ) ;
	    }
	    else
	    {
		if( Ref[ Dummy ] == In_Spec[ 0 ] )
		{
		    strcat( Result, Mid( Outp, Dummy + 1, 1 ) ) ;
		}
		else
		{
		    strcat( Result, Left( In_Spec, 1 ) ) ;
		} ;
	    } ;
	} ;
	if( Dummy < strlen( Ref ) )
	{
	    strcpy( In_Spec, &In_Spec[ 1 ] ) ;
	} ;
	/* One less character to worry about */
	Dummy = Dummy + 1 ; /* Try next character */
    } ;
    if( strcmp( Out_Spec, "*" ) == 0 )
    {
	strcat( Result, In_Spec ) ; /* Wildcard rest of name */
    } ;
    return Result ; /* Return result */
} ;


static int Count_Dir( char *Passed_X )
{

int C, D ;
char X[ 256 ] ;

    C = 0 ;
    strcpy( X, Passed_X ) ;
    if( X[ strlen( X ) - 1 ] != '\\' )
    {
	strcat( X, "\\" ) ;
    } ;
    for( D = 0; D < strlen( X ); D++ )
    {
	if( X[ D ] == '\\' )
	{
	    C = C + 1 ;
	} ;
    } ;
    return C - 1 ;
} ;


static char *Resolve_Dir( char *Passed_Ref_Spec, char *Passed_In_Spec,
    char *Passed_Out_Spec )
{

int D, Dummy ;
char Result[ 256 ] ;
char Ref[ 256 ], Inp[ 256 ], Outp[ 256 ] ; /* Work variable */
char Ref_Spec[ 256 ], In_Spec[ 256 ], Out_Spec[ 256 ] ;
int In_Count, Out_Count ;
int W ;

    /* Make local copies of passed strings */
    strcpy( Ref_Spec, Passed_Ref_Spec ) ;
    strcpy( In_Spec, Passed_In_Spec ) ;
    strcpy( Out_Spec, Passed_Out_Spec ) ;

    Result[ 0 ] = 0 ;
    In_Count = Count_Dir( In_Spec ) ;
    Out_Count = Count_Dir( Out_Spec ) ;

    Dummy = 1 ; /* Start past first backslash */
    W = False ; /* No wildcards found thus far */
    while( In_Count < Out_Count )
    {
        if( Out_Spec[ Dummy ] == '*' || Out_Spec[ Dummy ] == '?' )
	{
	    W = True ;
	} ;
        if( Out_Spec[ Dummy ] == '\\' )
        {
            D = Instr( 1, Out_Spec, "\\...\\" ) ;
            if( D != 0 && D <= Dummy ) 
	    {
		W = True ;
	    } ;
            if( W )
	    {
		Out_Count = In_Count ; /* Terminate loop immediately */
	    }
	    else
	    {
		strcat( Result, Left( Out_Spec, Dummy ) ) ;
		strcpy( Out_Spec, &Out_Spec[ Dummy ] ) ;
		Dummy = 0 ; /* Start over */
	    } ;
            Out_Count = Out_Count - 1 ; /* One less output directory */
        } ;
        Dummy = Dummy + 1 ; /* Check next character */
    } ;

Loop:
    if( strlen( Ref_Spec ) == 0 || strlen( Out_Spec ) == 0 || strlen( In_Spec ) == 0 )
    {
	if( strlen( Out_Spec ) != 0 && strcmp( Out_Spec, "\\...\\" ) != 0 )
	{
	    strcat( Result, Out_Spec ) ;
	} ;
	if( Result[ strlen( Result ) - 1 ] != '\\' )
	{
	    strcat( Result, "\\" ) ;
	    /* Directory paths must end with a backslash */
	} ;
	return Result ;
    } ;
    if( strcmp( Ref_Spec, "\\...\\" ) == 0 )
    {
	strcpy( Ref, Ref_Spec ) ;
    }
    else
    {
	Dummy = Instr( 1, &Ref_Spec[ 1 ], "\\" ) ;
	if( Dummy == 0 )
	{
	    Dummy = strlen( Ref_Spec ) ;
	} ;
	Dummy = Dummy + 1 ;
	strcpy( Ref, Left( Ref_Spec, Dummy - 1 ) ) ;
	/* Current reference directory to use */
	strcpy( Ref_Spec, &Ref_Spec[ Dummy - 1 ] ) ;
    } ;
    if( strcmp( Out_Spec, "\\...\\" ) == 0 )
    {
	strcpy( Outp, Out_Spec ) ;
    }
    else
    {
	Dummy = Instr( 1, &Out_Spec[ 1 ], "\\" ) ;
	if( Dummy == 0 )
	{
	    Dummy = strlen( Out_Spec ) ;
	} ;
	Dummy = Dummy + 1 ;
	strcpy( Outp, Left( Out_Spec, Dummy - 1 ) ) ;
	/* Current output directory to use */
	strcpy( Out_Spec, &Out_Spec[ Dummy - 1 ] ) ;
    } ;
    Dummy = Instr( 1, &In_Spec[ 1 ], "\\" ) ;
    if( Dummy == 0 )
    {
	Dummy = strlen( In_Spec ) ;
    } ;
    Dummy = Dummy + 1 ;
    strcpy( Inp, Left( In_Spec, Dummy - 1 ) ) ;
    /* Current input directory to use */
    strcpy( In_Spec, &In_Spec[ Dummy -1 ] ) ;

    if( strcmp( Outp, "\\...\\" ) == 0 ) /* Wildcard subdirectories */
    {
	strcat( Result, Inp ) ;
    }
    else
    {
	strcat( Result, Resolve_Wildcard( Ref, Inp, Outp ) ) ;
    } ;
    goto Loop ; /* Check next directory in path */
} ;


char *Wc_Xlate( char *Passed_Reference, char *Passed_Filename,
    char *Passed_Wildcard )
/* Translate a wildcard specification and an input file to an output spec. */
{

char Reference[ 256 ], Filename[ 256 ], Wildcard[ 256 ] ;
int Dummy ; /* Dummy variable */
char Cd[ 128 ] ; /* Current device/directory */
char Result[ 128 ] ; /* Resulting translation */
char Work[ 256 ] ; /* Work variable */
char Ref_Dev[ 3 ], In_Dev[ 3 ], Out_Dev[ 3 ] ; /* Input/output device */
char Ref_Dir[ 128 ], In_Dir[ 128 ], Out_Dir[ 128 ] ;
/* Input/output directory path */
char Ref_Name[ 10 ], In_Name[ 10 ], Out_Name[ 10 ] ; /* Input/output filename */
char Ref_Ext[ 5 ], In_Ext[ 5 ], Out_Ext[ 5 ] ; /* Input/output extension */

    /* Make local copies of passed strings */
    if( Passed_Reference == NULL )
    {
	Reference[ 0 ] = 0 ;
    }
    else
    {
	strcpy( Reference, Passed_Reference ) ;
    } ;
    if( Passed_Filename == NULL )
    {
	Filename[ 0 ] = 0 ;
    }
    else
    {
	strcpy( Filename, Passed_Filename ) ;
    }
    if( Passed_Wildcard == NULL )
    {
	Wildcard[ 0 ] = 0 ;
    }
    else
    {
	strcpy( Wildcard, Passed_Wildcard ) ;
    } ;

    /* Set up */
    Result[ 0 ] = 0 ; /* Nothing yet */
    getcurdir( 0, Cd ) ; /* Get current disk/directory */
    if( Cd[ strlen( Cd ) - 1 ] != '\\' )
    {
	strcat( Cd, "\\" ) ; /* Must terminate with "\" */
    } ;

    /* Break reference, filename, and wildcard into component pieces */
    _splitpath( Reference, Ref_Dev, Ref_Dir, Ref_Name, Ref_Ext ) ;
    _splitpath( Filename, In_Dev, In_Dir, In_Name, In_Ext ) ;
    _splitpath( Wildcard, Out_Dev, Out_Dir, Out_Name, Out_Ext ) ;
    
    if( Out_Dev[ 0 ] == 0 ) /* No device in wildcard */
    {
	Result[ 0 ] = Cd[ 0 ] ;
	Result[ 1 ] = Cd[ 1 ] ;
	Result[ 2 ] = 0 ; /* Default to current */
    }
    else
    {
	if( strcmp( Out_Dev, "*:" ) == 0 ) /* Wildcard output device */
	{
	    strcpy( Result, In_Dev ) ;
	}
	else
	{
	    strcpy( Result, Out_Dev ) ;
	} ;
    } ;
    if( Result[ 0 ] != Cd[ 0 ] )
    {
	getcurdir( Result[ 0 ] - 64, Cd ) ;
	/* Get current directory for output disk */
    } ;

    if( Out_Dir[ 0 ] == 0 ) /* No directory in wildcard */
    {
	strcat( Result, &Cd[ 2 ] ) ;
    }
    else
    {
	strcat( Result, Resolve_Dir( Ref_Dir, In_Dir, Out_Dir ) ) ;
    } ;

    if( Out_Name[ 0 ] == 0 ) /* No filename in wildcard */
    {
	strcat( Result, In_Name ) ;
    }
    else
    {
	strcat( Result, Resolve_Wildcard( Ref_Name, In_Name, Out_Name ) ) ;
    } ;
    if( Out_Ext[ 0 ] == 0 ) /* No extension in wildcard */
    {
	strcat( Result, In_Ext ) ;
    }
    else
    {
	strcat( Result, Resolve_Wildcard( Ref_Ext, In_Ext, Out_Ext ) ) ;
    } ;
    return Result ; /* Return result */
} ;


static int Match_Wildcard( char *Passed_Ref_Spec, char *Passed_In_Spec )
{

int Dummy ;
char Ref[ 256 ], Inp[ 256 ] ; /* Work variables */
char In_Spec[ 256 ], Ref_Spec[ 256 ] ;

    /* Make local copies of passed strings */
    strcpy( Ref_Spec, Passed_Ref_Spec ) ;
    strcpy( In_Spec, Passed_In_Spec ) ;

    if( strcmp( Ref_Spec, "\\..." ) == 0 || strcmp( Ref_Spec, "\\...\\" ) == 0 )
    {
	strcpy( Ref_Spec, "\\*" ) ;
	/* Handle special directory wildcard */
    }

    if( strcmp( Ref_Spec, "*" ) == 0 )
    {
	return True ;
    } ;
    Dummy = Instr( 1, Ref_Spec, "*" ) ;
    if( Dummy == 0 )
    {
	Dummy = strlen( Ref_Spec ) + 1 ;
    } ;
    strcpy( Ref, Left( Ref_Spec, Dummy - 1 ) ) ;
    strcpy( Ref_Spec, &Ref_Spec[ Dummy - 1 ] ) ;

    Dummy = Instr( 1, In_Spec, "*" ) ;
    if( Dummy == 0 )
    {
	Dummy = strlen( In_Spec ) + 1 ;
    } ;
    strcpy( Inp, Left( In_Spec, Dummy - 1 ) ) ;
    strcpy( In_Spec, &In_Spec[ Dummy - 1 ] ) ;

    if( strlen( Inp ) < strlen( Ref ) )
    {
	return False ;
    } ;
    Dummy = 0 ;
    while( Dummy < strlen( Ref ) )
    {
	if( Ref[ Dummy ] != '?' && Ref[ Dummy ] != Inp[ Dummy ] )
	{
	    return False ;
	} ;
	Dummy = Dummy + 1 ; /* Try next character */
    } ;
    strcpy( Inp, &Inp[ strlen( Ref ) ] ) ; /* Get remaining characters */
    if( strlen( Inp ) > 0 && strcmp( Ref_Spec, "*" ) != 0 )
    {
	return False ;
    } ;

    return True ; /* Everything matched */
} ;


static int Match_Dir( char *Passed_Ref_Spec, char *Passed_In_Spec )
{

int Dummy ;
char Ref[ 256 ], Inp[ 256 ] ; /* Work variables */
char Ref_Spec[ 256 ], In_Spec[ 256 ] ;

    /* Make local copies of passed strings */
    strcpy( Ref_Spec, Passed_Ref_Spec ) ;
    strcpy( In_Spec, Passed_In_Spec ) ;

Loop:
    if( ( strlen( Ref_Spec ) == 0 || strcmp( Ref_Spec, "\\...\\" ) == 0 ) &&
	strlen( In_Spec ) == 0 )
    {
	return True ;
    } ;
    if( strcmp( Ref_Spec, "\\...\\" ) == 0 ) /* Match everything else */
    {
	return True ;
    } ;
    Dummy = Instr( 1, &Ref_Spec[ 1 ], "\\" ) ;
    if( Dummy == 0 )
    {
	Dummy = strlen( Ref_Spec ) ;
    } ;
    Dummy = Dummy + 1 ;
    strcpy( Ref, Left( Ref_Spec, Dummy - 1 ) ) ;
    /* Current reference directory to use */
    strcpy( Ref_Spec, &Ref_Spec[ Dummy - 1 ] ) ;

    Dummy = Instr( 1, &In_Spec[ 1 ], "\\" ) ;
    if( Dummy == 0 )
    {
	Dummy = strlen( In_Spec ) ;
    } ;
    Dummy = Dummy + 1 ;
    strcpy( Inp, Left( In_Spec, Dummy - 1 ) ) ; /* Current input directory to use */
    strcpy( In_Spec, &In_Spec[ Dummy - 1 ] ) ;

    if( !Match_Wildcard( Ref, Inp ) )
    {
	return False ;
    } ;
    goto Loop ; /* Check next directory in path */
} ;


int Wc_Match( char *Passed_Wildcard, char *Passed_Filename )
/* Compare a filename and a wildcard specification and return TRUE if a match. */
{

char Wildcard[ 256 ], Filename[ 256 ] ;
int Dummy ; /* Dummy variable */
char Work[ 256 ] ; /* Work variable */
char Ref_Dev[ 3 ], In_Dev[ 3 ] ; /* Input/output device */
char Ref_Dir[ 128 ], In_Dir[ 128 ] ; /* Input/output directory path */
char Ref_Name[ 10 ], In_Name[ 10 ] ; /* Input/output filename */
char Ref_Ext[ 5 ], In_Ext[ 5 ] ; /* Input/output extension */

    /* Make local copies of passed strings */
    strcpy( Wildcard, Passed_Wildcard ) ;
    strcpy( Filename, Passed_Filename ) ;

    /* Break filename and wildcard into component pieces */
    _splitpath( Filename, In_Dev, In_Dir, In_Name, In_Ext ) ;
    _splitpath( Wildcard, Ref_Dev, Ref_Dir, Ref_Name, Ref_Ext ) ;

    if( strcmp( Ref_Dev, "*:" ) != 0 && strcmp( Ref_Dev, In_Dev ) != 0 )
    /* Device mismatch */
    {
	return False ;
    } ;

    if( !Match_Dir( Ref_Dir, In_Dir ) )
    {
	return False ;
    } ;
    if( !Match_Wildcard( Ref_Name, In_Name ) )
    {
	return False ;
    } ;
    if( !Match_Wildcard( Ref_Ext, In_Ext ) )
    {
	return False ;
    } ;

    return True ; /* Everything matched */
} ;
