{$E+}
{$N+}
{
        Program Name : _OS
        Package Name : LB
        Purpose      : Operating system interface
        Institution  : Conroy & Conroy Co.
        Date Written : 6-Nov-2002
        Written By   : Alan Conroy
        Version      : 1.0

	Copyright (C) 2002-2005 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TOperating_System is an abstract Operating System interface.
}

unit _OS ;

interface

uses Common, { TBase_Object }
     ComInter, { TCommon_COM_Interface }
     _DebugIn, { PDebug_Interface }
     _DevMan, { TSDevice_Manager }
     _Types, { int64 }
     _UEHDefs ; { TUEC }

{$IFDEF DELPHI}
type TOperating_System = class( TBase_Delphi_Object )
			     public { API... }
                                 { Identification... }
				 function Name : string ; virtual ; abstract ; { O/S name }
				 function Version : string ;
                                     virtual ; abstract ; { O/S version }


                                 { Date/Time support... }
                                 procedure Get_Date( var Year, Month, Day : word ) ;
                                     virtual ; abstract ;
                                 procedure Get_Time( var Hour, Minute, Second, Hundreths : word ) ;
                                     virtual ; abstract ;
				 function Timestamp : int64 ;
                                     virtual ; abstract ;
				 { Current date/time in Sirius format }


                                 { Process support... }
                                 function PID : longint ; virtual ; abstract ;
                                 procedure Reschedule ; virtual ; abstract ;
                                 { Reschedules process so other tasks can get
                                   some runtime. }

                                 procedure Spawn( const Path, Parameters : string ) ;
                                     virtual ; abstract ; { Spawns a process. }


                                 { Device support... }
                                 function Device_Manager : TDevice_Manager ;


                                 { I/O... }
                                 procedure Beep( Typ : integer ) ;
                                     virtual ; abstract ;
                                 { Beep the user. }

                                 procedure Cancel_Typeahead ;
                                    virtual ; abstract ;
                                 { Cancel any pending keyboard input }

                                 function Get_Char( Wait : boolean ;
                                     var Valid : boolean ) : char ;
                                     virtual ; abstract ;

                                 function Pending_Input : boolean ;
                                     virtual ; abstract ;
                                 { Returns true if any user input is pending }

                                 procedure Print( T : string ) ;
                                     virtual ; abstract ;
                                 { Print the text on the default printer }


                                 { Error processing... }
                                 procedure Last_Error( var UEC : TUEC ) ;
                                     virtual ; abstract ;

				 function Error_Text( const UEC : TUEC ;
                                     const Language : string ) : string ;
                                     virtual ; abstract ;


                                 { Files and pathing... }
                                 function Application_Path : string ;
                                     virtual ; abstract ;
                                 { Returns application path with terminating
                                   delimiter. }

                                 function File_List_Delimiter : string ;
                                     virtual ; abstract ;
                                 { Delimiter used to separate files in a list. }

                                 function Get_Temp_Path : string ;
                                     virtual ; abstract ;
                                 { Returns path(s) for temporary files.  If
                                   there are multiple paths, they are delimited
                                   by the File_List_Delimiter. }

                                 function Swap_File_List : string ;
                                     virtual ; abstract ;
                                 { Returns list of swap files.  Returns Null if
                                   no swap files.  If more than one, they are
                                   delimited by the File_List_Delimiter. }

                                 function System_Path : string ;
                                     virtual ; abstract ;
                                 { Path(s) to the system directory.  Returns
                                   null string if not supported. }


                                 { Misc... }
                                 function Command_Line : string ;
                                     virtual ; abstract ;
                                 { Returns command line used to start program. }

                                 function Debugger : TCOM_Debug_Interface ;
                                     virtual ; abstract ;
                                 { Returns a debugger for the operating system }

                                 function Get_Symbol( const X : string ) : string ;
                                     virtual ; abstract ;
                                 { Return contents of OS/Shell symbol whose name
                                   is in X.  Returns null if not defined. }

                                 function Switch_Delimiter_Count : integer ;
                                     virtual ; abstract ;
                                 { Returns number of switch delimiters. }

                                 function Switch_Delimiters( Index : integer ) : string ;
                                     virtual ; abstract ;
                                 { Returns switch delimiter for Index. }

                                 function Get_Clipboard( Path : string ) : string ;
                                     virtual ; abstract ;
                                 { Get clipboard contents in path. }

                                 procedure Set_Clipboard( Path, Value : string ) ;
                                     virtual ; abstract ;
                                 { Get clipboard contents for path. }

                                 function Max_File_Size : int64 ; virtual ;
                                 { Return the largest that a file could be on a
                                   device with infinite space, using the file
                                   system with the smallest max size files
                                   supported by the O/S. }
                         end ; { TOperating_System }

type TCOM_Operating_System = class( TCommon_COM_Interface )
                                 public { API... }
                                     { Identification... }
                                     function Name : PChar ;
                                         virtual ; stdcall ; abstract ; { O/S name }
                                     function Version : PChar ;
                                         virtual ; stdcall ; abstract ;
                                     { O/S version }


                                     { Date/Time support... }
                                     procedure Get_Date( var Year, Month, Day : word ) ;
                                         virtual ; stdcall ; abstract ;
                                     procedure Get_Time( var Hour, Minute, Second, Hundreths : word ) ;
                                         virtual ; stdcall ; abstract ;
                                     function Timestamp : int64 ;
                                         virtual ; stdcall ; abstract ;
                                     { Current date/time in Sirius format }


                                     { Process support... }
                                     function PID : longint ;
                                         virtual ; stdcall ; abstract ;
                                     procedure Reschedule ;
                                         virtual ; stdcall ; abstract ;
                                     { Reschedules process so other tasks can
                                       get some runtime. }

                                     procedure Spawn( Path, Parameters : PChar ) ;
                                         virtual ; stdcall ; abstract ; { Spawns a process. }


                                     { Device support... }
                                     function Device_Manager : TCOM_Device_Manager ;


                                     { I/O... }
                                     procedure Beep( Typ : integer ) ;
                                         virtual ; stdcall ; abstract ;
                                     { Beep the user. }

                                     procedure Cancel_Typeahead ;
                                         virtual ; stdcall ; abstract ;
                                     { Cancel any pending keyboard input }

                                     function Get_Char( Wait : boolean ;
                                         var Valid : boolean ) : char ;
                                         virtual ; stdcall ; abstract ;

                                     function Pending_Input : boolean ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns true if any user input is pending }

                                     procedure Print( T : PChar ) ;
                                         virtual ; stdcall ; abstract ;
                                     { Print the text on the default printer }


                                     { Error processing... }
                                     procedure Last_Error( var UEC : TUEC ) ;
                                         virtual ; stdcall ; abstract ;

                                     function Error_Text( const UEC : TUEC ;
                                         Language : PChar ) : PChar ;
                                         virtual ; stdcall ; abstract ;


                                     { Misc... }
                                     function Application_Path : PChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns application path with terminating
                                       delimiter }

                                     function Command_Line : PChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns command line used to start program. }

                                     function Debugger : TCOM_Debug_Interface ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns a debugger for the operating system }

                                     function Get_Symbol( X : PChar ) : PChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Return contents of OS/Shell symbol.  Returns
                                       null if not defined. }

                                     function Switch_Delimiter_Count : integer ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns number of switch delimiters. }

                                     function Switch_Delimiters( Index : integer ) : PChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns switch delimiter for Index. }

                                     function File_List_Delimiter : PChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Delimiter used to separate files in a list. }

                                     function Get_Temp_Path : PChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns path(s) for temporary files.  If
                                       there are multiple paths, they are delimited
                                       by the File_List_Delimiter. }

                                     function System_Path : PChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Path(s) to the system directory.  Returns
                                       null string if not supported. }

                                     function Swap_File_List : PChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns list of swap files.  Returns Null if
                                       no swap files. }

                                     function Get_Clipboard( Path : string ) : string ;
                                         virtual ; stdcall ; abstract ;
                                     { Get clipboard contents in path. }

                                     procedure Set_Clipboard( Path, Value : string ) ;
                                         virtual ; stdcall ; abstract ;
                                     { Get clipboard contents for path. }

                                     function Max_File_Size : int64 ; virtual ;
                                     { Return the largest that a file could be
                                       on a device with infinite space, using
                                       the file system with the smallest max
                                       size files supported by the O/S. }
                             end ; { TCOM_Operating_System }
{$ENDIF}

type TSOperating_System = object( TBase_Object )
			     public { API... }
                                 { Identification... }
				 function Name : string ; virtual ; { O/S name }
				 function Version : string ; virtual ;
				 { O/S version }


                                 { Date/Time support... }
                                 procedure Get_Date( var Year, Month, Day : word ) ;
                                     virtual ;
                                 procedure Get_Time( var Hour, Minute, Second, Hundreths : word ) ;
                                     virtual ;
				 function Timestamp : int64 ; virtual ;
				 { Current date/time in Sirius format }


                                 { Process support... }
                                 function PID : longint ; virtual ;
                                 procedure Reschedule ; virtual ;
                                 { Reschedules process so other tasks can get
                                   some runtime. }

                                 procedure Spawn( const Path, Parameters : string ) ;
                                     virtual ; { Spawns a process. }


                                 { Device support... }
                                 function Device_Manager : PSDevice_Manager ;


                                 { I/O... }
                                 procedure Beep( Typ : integer ) ; virtual ;
                                 { Beep the user. }

                                 procedure Cancel_Typeahead ; virtual ;
                                 { Cancel any pending keyboard input }

                                 function Get_Char( Wait : boolean ;
                                     var Valid : boolean ) : char ;
                                     virtual ;

                                 function Pending_Input : boolean ; virtual ;
                                 { Returns true if any user input is pending }

                                 procedure Print( T : string ) ; virtual ;
                                 { Print the text on the default printer }


                                 { Error processing... }
                                 procedure Last_Error( var UEC : TUEC ) ;
                                     virtual ;

                                 function Error_Text( const UEC : TUEC ;
                                     const Language : string ) : string ;
                                     virtual ;


                                 { Misc... }
                                 function Application_Path : string ; virtual ;
                                 { Returns application path with terminating
                                   delimiter }

                                 function Command_Line : string ; virtual ;
                                 { Returns command line used to start program. }
{$IFDEF DELPHI}

                                 function Debugger : PSDebug_Interface ;
                                     virtual ;
                                 { Returns a debugger for the operating system }
{$ENDIF}

                                 function Get_Symbol( const X : string ) : string ;
                                     virtual ;
                                 { Return contents of OS/Shell symbol.  Returns
                                   null if not defined. }

                                 function Switch_Delimiter_Count : integer ;
                                     virtual ;
                                 { Returns number of switch delimiters. }

                                 function Switch_Delimiters( Index : integer ) : string ;
                                     virtual ;
                                 { Returns switch delimiter for Index. }

                                 function File_List_Delimiter : string ;
                                     virtual ;
                                 { Delimiter used to separate files in a list. }

                                 function Get_Temp_Path : string ; virtual ;
                                 { Returns path(s) for temporary files.  If
                                   there are multiple paths, they are delimited
                                   by the File_List_Delimiter. }

                                 function System_Path : string ;
                                     virtual ;
                                 { Path(s) to the system directory.  Returns
                                   null string if not supported. }

                                 function Swap_File_List : string ; virtual ;
                                 { Returns list of swap files.  Returns Null if
                                   no swap files. }

                                 function Get_Clipboard( Path : string ) : string ;
                                     virtual ;
                                 { Get clipboard contents in path. }

                                 procedure Set_Clipboard( Path, Value : string ) ;
                                     virtual ;
                                 { Get clipboard contents for path. }

                                 function Max_File_Size : int64 ; virtual ;
                                 { Return the largest that a file could be on a
                                   device with infinite space, using the file
                                   system with the smallest max size files
                                   supported by the O/S. }
                         end ; { TSOperating_System }
     PSOperating_System = ^TSOperating_System ;

implementation

uses ASCIIDef ; { NUL }

{ TOperating_System methods... }

{ API... }

function TSOperating_System.Name : string ; { O/S name }

begin
    Name := '' ;
end ;


function TSOperating_System.Version : string ; { O/S version }

begin
    Version := '' ;
end ;


function TSOperating_System.Application_Path : string ;

begin
    Application_Path := '' ;
end ;


procedure TSOperating_System.Beep( Typ : integer ) ;

begin
end ;


procedure TSOperating_System.Cancel_Typeahead ;

var B : boolean ;

begin
    while( Pending_Input ) do
    begin
        Get_Char( False, B ) ;
    end ;
end ;


function TSOperating_System.Command_Line : string ;

begin
    Command_Line := '' ;
end ;


{$IFDEF DELPHI}
function TSOperating_System.Debugger : PSDebug_Interface ;

begin
    Debugger := nil ;
end ;
{$ENDIF}


function TSOperating_System.Device_Manager : PSDevice_Manager ;

begin
    Device_Manager := new( PSDevice_Manager, Init ) ;
end ;


function TSOperating_System.Get_Char( Wait : boolean ;
    var Valid : boolean ) : char ;

begin
    Valid := False ;
    Get_Char := NUL ;
end ;


procedure TSOperating_System.Get_Date( var Year, Month, Day : word ) ;

begin
    Year := 0 ;
    Month := 0 ;
    Day := 0 ;
end ;


function TSOperating_System.Get_Symbol( const X : string ) : string ;

begin
    Get_Symbol := '' ;
end ;


procedure TSOperating_System.Get_Time( var Hour, Minute, Second, Hundreths : word ) ;

begin
    Hour := 0 ;
    Minute := 0 ;
    Second := 0 ;
    Hundreths := 0 ;
end ;


procedure TSOperating_System.Last_Error( var UEC : TUEC ) ;

begin
    fillchar( UEC, sizeof( UEC ), 0 ) ;
end ;


function TSOperating_System.Error_Text( const UEC : TUEC ;
    const Language : string ) : string ;

begin
    Error_Text := '';
end ;


function TSOperating_System.Pending_Input : boolean ;

begin
    Pending_Input := False ;
end ;


procedure TSOperating_System.Print( T : string ) ;

begin
end ;


function TSOperating_System.PID : longint ;

begin
    PID := 0 ;
end ;


procedure TSOperating_System.Reschedule ;

begin
end ;


procedure TSOperating_System.Spawn( const Path, Parameters : string ) ;

begin
end ;


function TSOperating_System.Switch_Delimiter_Count : integer ;

begin
    Switch_Delimiter_Count := 1 ;
end ;


function TSOperating_System.Switch_Delimiters( Index : integer ) : string ;

begin
    Switch_Delimiters := '/' ;
end ;


function TSOperating_System.File_List_Delimiter : string ;

begin
    File_List_Delimiter := ';' ;
end ;



function TSOperating_System.Get_Temp_Path : string ;

begin
    Get_Temp_Path := '' ;
end ;


function TSOperating_System.System_Path : string ;

begin
    System_Path := '' ;
end ;


function TSOperating_System.Swap_File_List : string ;

begin
    Swap_File_List := '' ;
end ;


function TSOperating_System.Timestamp : int64 ;

begin
    Timestamp := 0 ;
end ;


function TSOperating_System.Get_Clipboard( Path : string ) : string ;

begin
    Get_Clipboard := '' ;
end ;


procedure TSOperating_System.Set_Clipboard( Path, Value : string ) ;

begin
end ;


function TSOperating_System.Max_File_Size : int64 ;

begin
    Max_File_Size := 0 ;
end ;


end.
