/* 
        Program Name : CACHE
        Package Name : LB
        Purpose      : Cache control routines
        Institution  : Conroy & Conroy Co.
        Date Written : 18-Jun-1994
        Written By   : Alan Conroy
        Version      : 1.0

	Copyright (C) 1994 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

	  This unit provides a means of coordinating several different
	caches.
 */

#include <stdlib.h>
#include <alloc.h>
#include "cache.h"

static struct TCache_Record *PCache_Handlers = NULL ;
/* Root pointer to cache list */


void Add_Handler( void (*Routine)(int, int), long int Priority )
/* This routine adds a cache handler to the cache chain.  Routine is a pointer
   to the cache handler and Priority is the relative priority of this cache.
   The cache list is kept in ascending order by priority to make our job
   easier later. */
{

struct TCache_Record *Current, *Previous ;

    if( PCache_Handlers == NULL ) /* If first one, then it is simple */
    {
	PCache_Handlers = malloc( sizeof( struct TCache_Record ) ) ;
	PCache_Handlers->Next = NULL ;
	PCache_Handlers->Priority = Priority ;
	PCache_Handlers->Routine = Routine ;
	return ;
    } ;

    /* We must insert into list */
    Current = PCache_Handlers ;
    Previous = NULL ;
    while( Current != NULL ) /* Until end of list */
    {
	if( Current->Priority > Priority ) /* Found insertion point */
	{
	    if( Previous == NULL ) /* Insert at beginning of list */
	    {
		PCache_Handlers = malloc( sizeof( struct TCache_Record ) ) ;
		Previous = PCache_Handlers ;
	    }
	    else
	    {
		Previous->Next = malloc( sizeof( struct TCache_Record ) ) ;
		/* Do insertion */
		Previous = Previous->Next ;
	    } ;
	    Previous->Next = Current ;
	    Previous->Priority = Priority ;
	    Previous->Routine = Routine ;
	    return ;
	} ;
	Previous = Current ;
	Current = Current->Next ; /* Go to next list element */
    } ;

    /* We hit the end of the list, add to end. */
    Current = malloc( sizeof( struct TCache_Record ) ) ;
    Previous->Next = Current ;
    Current->Next = NULL ;
    Current->Priority = Priority ;
    Current->Routine = Routine ;
} ;


void Delete_Handler( void (*Routine)(int, int) )
/* This routine removes a cache handler from the cache handler list. */
{

struct TCache_Record *Current, *Previous ;

    Current = PCache_Handlers ;
    Previous = NULL ;
    while( Current != NULL ) /* Until end of list */
    {
	if( Current->Routine == Routine ) /* Found it */
	{
	    if( Previous == NULL ) /* First one in list */
	    {
		PCache_Handlers = Current->Next ;
	    }
	    else
	    {
		Previous->Next = Current->Next ;
	    } ;
	    free( Current ) ; /* Free up memory */
	    return ;
	} ;
	Previous = Current ;
	Current = Current->Next ; /* Try next entry */
    } ;
} ;


void Free_Cache( long int Siz, long int Priority, void (*Exclude)(int, int) )
/* This routine calls the appropriate cache handlers to free up Siz bytes
   of memory.  A specific handler can be excluded from this.  Calling this
   with Exclude of NULL, priority of maxint, and Siz of -1, will zero all
   caches. */
{

struct TCache_Record *Current ;
long int Dummy ;

    Current = PCache_Handlers ;
    while( Current != NULL ) /* Go through entire list */
    {
#ifndef __BCPLUSPLUS__
	if( coreleft() >= Siz && Siz >= 0 )
	{
	/* Enough memory is now available and we are not zeroing */
	    Current = NULL ; /* Terminate loop */
	}
	else
#endif
	{
	    if( Current->Priority <= Priority && Current->Routine != Exclude )
	    {
		Current->Routine( Siz, 0 ) ;
	    } ;
	} ;
	Current = Current->Next ; /* Follow link to next one */
    } ;
} ;
