/*
        Program Name : SCU
        Package Name : SCU
        Purpose      : SCU definitions
        Institution  : Conroy & Conroy Co.
        Date Written : 16-November-2001
        Written By   : Alan Conroy
        Version      : 1
        Revision     : 0
        Edit         :

	    Copyright (C) 2001 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

*/

#include "DebugInt.h"
#include "uec.h"

const SCUErr_Facility = 22 ;
const SCUErr_Success = 0 ;
const SCUErr_Value_Not_Found = 1 ;
const SCUErr_Invalid_Size = 2 ;
const SCUErr_Exists = 3 ; // Key or value already exists
const SCUErr_Alarm_Failure = 4 ; // Operation caused an alarm which user used to abort operation

class T_SCU
{
    public: // API...
        // Instance support...
        virtual TUEC __stdcall Initialize() = 0 ;
        virtual void __stdcall Terminate() = 0 ;

        virtual TUEC __stdcall Init( const char* Name ) = 0 ;
        virtual int __stdcall Version() = 0 ;

        // Low-level access...
        virtual TUEC __stdcall Write( const char* Path, const void* Data,
            const int Size ) = 0 ;
        virtual TUEC __stdcall Read( const char* Path, void* Data,
            const int Size ) = 0 ;
        virtual int __stdcall Read_Size( const char* Path ) = 0 ;
        virtual int __stdcall Read_Type( const char* Path ) = 0 ;
        virtual bool __stdcall Exists( const char* Path ) = 0 ;
        virtual bool __stdcall Key_Exists( const char* Path ) = 0 ;

        virtual char* __stdcall Lookup( const char* Path, const int DT ) = 0 ;

        virtual TUEC __stdcall Delete( const char* Path ) = 0 ;
        virtual TUEC __stdcall Create( const char* Path, const int DT ) = 0 ;
        virtual TUEC __stdcall Create_And_Write( const char* Path, const int DT,
                    const void* Data, const int Size ) = 0 ;
        virtual TUEC __stdcall Rename( const char* Old, const char* New ) = 0 ;

        // High level API...
        virtual TUEC __stdcall Set_Value( const char* User, const char* Path,
            const void* Data, const int Size ) = 0 ;
        virtual TUEC __stdcall Get_Value( const char* User, const char* Path,
            void* Data, const int Size ) = 0 ;
        virtual int __stdcall Value_Size( const char* User,
            const char* Path ) = 0 ;
        virtual int __stdcall Value_Type( const char* User,
            const char* Path ) = 0 ;
        virtual bool __stdcall Value_Exists( const char* User,
            const char* Path ) = 0 ;
        virtual TUEC __stdcall Delete_Value( const char* User,
            const char* Path ) = 0 ;
        virtual TUEC __stdcall Create_Value( const char* User, const char* Path,
            const int DT ) = 0 ;
        virtual TUEC __stdcall Verify() = 0 ;
        virtual TDebug_Interface* __stdcall Debugger() = 0 ;
        virtual TUEC __stdcall Read_Value( char* User, char* Path, int& DT,
            int& Size, void*& Buffer ) = 0;
};

const SCUDT_Undefined = 0 ;
const SCUDT_Key = 1 ;
const SCUDT_String = 2 ;
const SCUDT_Integer = 3 ; // 4-byte signed
const SCUDT_Boolean = 4 ;
const SCUDT_Float = 5 ; // Double-precision
const SCUDT_Binary = 6 ;
const SCUDT_All = -1 ; // All types

